/***************************************************************************
                          dialog_appearance.cpp  -  description
                             -------------------
    begin                : Sun Mar 26 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dialog_appearance.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CAppearanceDialog
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CAppearanceDialog::CAppearanceDialog(QWidget *parent, const char *name, WFlags f)
  : CAppearanceDialogUI(parent, name, TRUE, f)
{
  molecule = new CKGLMolecule(this, "dlgmolecule");
  molecule->DeleteMolecule();
  bgColorPrint = QColor(255, 255, 255);

  // connections
  // button clicked
  connect(acceptButton, SIGNAL(clicked()), SLOT(accept()));
  connect(cancelButton, SIGNAL(clicked()), SLOT(reject()));
  connect(applyButton, SIGNAL(clicked()), SLOT(applyClicked()));
  connect(resetButton, SIGNAL(clicked()), SLOT(resetClicked()));
  // other signals
  connect(colorFlatButton, SIGNAL(clicked()), SLOT(colorFlatButtonClicked()));
  connect(colorOpenGLButton, SIGNAL(clicked()), SLOT(colorOpenGLButtonClicked()));
  connect(colorPrintButton, SIGNAL(clicked()), SLOT(colorPrintButtonClicked()));
  connect(perspectiveSlider, SIGNAL(valueChanged(int)), SLOT(perspectiveChanged(int)));
  connect(atomSizeSlider, SIGNAL(valueChanged(int)), SLOT(radiusReductionChanged(int)));
  connect(labelButtonGroup, SIGNAL(clicked(int)), SLOT(labelChanged(int)));
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
CAppearanceDialog::~CAppearanceDialog()
{
}

/////////////////////////////////////////////////////////////////////////////
// slots
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// slot applyClicked
void CAppearanceDialog::applyClicked(void)
{
  applyButton->setEnabled(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// slot resetClicked
void CAppearanceDialog::resetClicked(void)
{
  molecule->SetLabelType((int)LB_NOLABEL);  
  molecule->SetPerspectiveFactor((float)DEFAULT_PERSPECTIVEFACTOR);
  molecule->SetRadiusFactor((float)DEFAULT_RADIUSFACTOR);
  molecule->SetBackgroundColor((QColor)DEFAULT_BGCOLOR);
  molecule->SetBackgroundColorGL(QColor((int)DEFAULT_GL_BGCOLOR_RED, (int)DEFAULT_GL_BGCOLOR_GREEN, (int)DEFAULT_GL_BGCOLOR_BLUE));
  bgColorPrint = QColor(255, 255, 255);

  initWidgets();
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot perspectiveChanged
void CAppearanceDialog::perspectiveChanged(int perspective)
{
  molecule->SetPerspectiveFactor((float)perspective / 10.0);
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot radiusReductionChanged
void CAppearanceDialog::radiusReductionChanged(int radiusReduction)
{
  molecule->SetRadiusFactor((float)radiusReduction / 100.0);
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot labelChanged
void CAppearanceDialog::labelChanged(int label)
{
  switch(label)
  {
    case 0: molecule->SetLabelType(LB_NOLABEL); break;
    case 1: molecule->SetLabelType(LB_USER); break;
    case 2: molecule->SetLabelType(LB_ORDINALNUMBER); break;
    case 3: molecule->SetLabelType(LB_ATOMSYMBOL); break;
    default: molecule->SetLabelType(LB_NOLABEL);
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot colorFlatButtonClicked
void CAppearanceDialog::colorFlatButtonClicked(void)
{
  QColor newColor;
  newColor = QColorDialog::getColor(molecule->GetBackgroundColor());
  if(newColor.isValid() && newColor != molecule->GetBackgroundColor())
  {
    molecule->SetBackgroundColor(newColor);
    sampleFlat->setPaletteBackgroundColor(molecule->GetBackgroundColor());
    update();
    applyButton->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot colorOpenGLButtonClicked
void CAppearanceDialog::colorOpenGLButtonClicked(void)
{
  QColor newColor;
  newColor = QColorDialog::getColor(molecule->GetBackgroundColorGL());
  if(newColor.isValid() && newColor != molecule->GetBackgroundColorGL())
  {
    molecule->SetBackgroundColorGL(newColor);
    sampleOpenGL->setPaletteBackgroundColor(molecule->GetBackgroundColorGL());
    sampleOpenGLAnaglyph->setPaletteBackgroundColor(((float)(molecule->GetBackgroundColorGL().red() +
                                                             molecule->GetBackgroundColorGL().green() +
                                                             molecule->GetBackgroundColorGL().blue()) / (float)255) > 1.5 ? QColor(255, 255, 255) : QColor(0, 0, 0));    
    update();
    applyButton->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot colorPrintButtonClicked
void CAppearanceDialog::colorPrintButtonClicked(void)
{
  QColor newColor;
  newColor = QColorDialog::getColor(bgColorPrint);
  if(newColor.isValid() && newColor != bgColorPrint)
  {
    bgColorPrint = newColor;
    samplePrint->setPaletteBackgroundColor(bgColorPrint);
    samplePrintAnaglyph->setPaletteBackgroundColor(((float)(bgColorPrint.red() +
                                                            bgColorPrint.green() +
                                                            bgColorPrint.blue()) / (float)255) > 1.5 ? QColor(255, 255, 255) : QColor(0, 0, 0));
    update();
    applyButton->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot initWidgets
void CAppearanceDialog::initWidgets(void)
{
  perspectiveSlider->setValue(int(molecule->GetPerspectiveFactor() * 10));
  atomSizeSlider->setValue(int(molecule->GetRadiusFactor() * 100));

  sampleFlat->setPaletteBackgroundColor(molecule->GetBackgroundColor());
  sampleOpenGL->setPaletteBackgroundColor(molecule->GetBackgroundColorGL());
  samplePrint->setPaletteBackgroundColor(bgColorPrint);  
  sampleOpenGLAnaglyph->setPaletteBackgroundColor(((float)(molecule->GetBackgroundColorGL().red() +
                                                           molecule->GetBackgroundColorGL().green() +
                                                           molecule->GetBackgroundColorGL().blue()) / (float)255) > 1.5 ? QColor(255, 255, 255) : QColor(0, 0, 0));
  samplePrintAnaglyph->setPaletteBackgroundColor(((float)(bgColorPrint.red() +
                                                          bgColorPrint.green() +
                                                          bgColorPrint.blue()) / (float)255) > 1.5 ? QColor(255, 255, 255) : QColor(0, 0, 0));
  switch(molecule->GetLabelType())
  {
    case LB_NOLABEL: labelButtonGroup->setButton(labelButtonGroup->id(labelRadioButton1)); break;
    case LB_USER: labelButtonGroup->setButton(labelButtonGroup->id(labelRadioButton2)); break;
    case LB_ORDINALNUMBER: labelButtonGroup->setButton(labelButtonGroup->id(labelRadioButton3)); break;
    case LB_ATOMSYMBOL: labelButtonGroup->setButton(labelButtonGroup->id(labelRadioButton4)); break;
    default: labelButtonGroup->setButton(labelButtonGroup->id(labelRadioButton1));
  }

  applyButton->setEnabled(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// methods
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// setMolecule
void CAppearanceDialog::setMolecule(CKGLMolecule *mol)
{
  *molecule = *mol;
  initWidgets();
}

/////////////////////////////////////////////////////////////////////////////
// setBGColorPrint
void CAppearanceDialog::setBGColorPrint(QColor colorPrint)
{
  bgColorPrint = colorPrint;
  initWidgets();
}

/////////////////////////////////////////////////////////////////////////////
// getMolecule
CKGLMolecule* CAppearanceDialog::getMolecule(void)
{
  return(molecule);
}

/////////////////////////////////////////////////////////////////////////////
// getBGColorPrint
QColor CAppearanceDialog::getBGColorPrint(void)
{
  return(bgColorPrint);
}
