/***************************************************************************
                          dialog_atoms.cpp  -  description
                             -------------------
    begin                : Sun Mar 12 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dialog_atoms.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CAtomsDialog
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CAtomsDialog::CAtomsDialog(QWidget *parent, const char *name, WFlags f)
  : CAtomsDialogUI(parent, name, TRUE, f)
{
  rotX = -40.0;
  rotY = -40.0;
  rotZ = -40.0;
  
  periodicsystem = new CKPeriodicSystem(this, "dlgperiodicsystem");
  periodicsystem->RestoreData();
  molecule = new CKGLMolecule(this, "dlgmolecule");
  molecule->DeleteMolecule();

  timer = new QTimer(this, "timer");
  timer->start(50, FALSE);

  glAtom = new CGLAtom(selectionFrameOpenGl, "glatom");
  glAtom->setGeometry(selectionFrameOpenGl->contentsRect());
  glAtom->setRadius(0.35);
  
  glAtomStructure = new CGLAtom(selectionGridFrameOpenGl, "glatomstructure");
  glAtomStructure->setGeometry(selectionGridFrameOpenGl->contentsRect());
  glAtomStructure->setRadius(0.35);
  glAtomStructure->setDrawStyle(GLU_LINE);
  glAtomStructure->setBgColor(QColor(0, 0, 0));
  glAtomStructure->setColor(QColor(0, 255, 0));  
  glAtomStructure->setRotation(rotX, rotY, rotZ);
    
  // connections
  // timer
  connect(timer, SIGNAL(timeout()), this, SLOT(rotateStructure()));
  // button clicked
  connect(acceptButton, SIGNAL(clicked()), SLOT(accept()));
  connect(cancelButton, SIGNAL(clicked()), SLOT(reject()));
  connect(applyButton, SIGNAL(clicked()), SLOT(applyClicked()));
  connect(resetButton, SIGNAL(clicked()), SLOT(resetClicked()));
  // other signals
  connect(colorButton, SIGNAL(clicked()), SLOT(colorButtonClicked()));
  connect(atomsListBox, SIGNAL(highlighted(int)), SLOT(atomsListboxChanged(int)));
  connect(tabWidget, SIGNAL(currentChanged(QWidget*)), SLOT(tabChanged(QWidget*)));
  connect(slicesSpinBox, SIGNAL(valueChanged(int)), SLOT(slicesChanged(int)));
  connect(stacksSpinBox, SIGNAL(valueChanged(int)), SLOT(stacksChanged(int)));
  connect(radiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(radiusLineEditChanged(const QString&)));;
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
CAtomsDialog::~CAtomsDialog()
{
  timer->stop();
}

/////////////////////////////////////////////////////////////////////////////
// events
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// paintEvent
void CAtomsDialog::paintEvent(QPaintEvent*)
{
  QPainter p;
  p.begin(selectionFrameFlat);
  p.eraseRect(selectionFrameFlat->rect());
  p.fillRect(selectionFrameFlat->rect(), QBrush(molecule->GetBackgroundColor()));
  p.setPen(QPen(Qt::black, 1, Qt::SolidLine));
  p.setBrush(QBrush(periodicsystem->GetColor(atomsListBox->currentItem() + 1)));
  p.drawEllipse(15, 15, 70, 70),
  p.end();
}

/////////////////////////////////////////////////////////////////////////////
// slots
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// slot applyClicked
void CAtomsDialog::applyClicked(void)
{
  applyButton->setEnabled(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// slot resetClicked
void CAtomsDialog::resetClicked(void)
{
  QString str_radius = "";
  periodicsystem->RestoreData();
  atomsListBox->setCurrentItem(0);
  str_radius.setNum(periodicsystem->GetRadius(atomsListBox->currentItem() + 1));
  radiusLineEdit->setText(str_radius);
  glAtomStructure->setSlices(DEFAULT_ATOMSLICES);
  glAtomStructure->setStacks(DEFAULT_ATOMSTACKS);
  glAtom->setColor(periodicsystem->GetColor(atomsListBox->currentItem() + 1));
  glAtom->setSlices(DEFAULT_ATOMSLICES);
  glAtom->setStacks(DEFAULT_ATOMSTACKS);
  slicesSpinBox->setValue(DEFAULT_ATOMSLICES);
  stacksSpinBox->setValue(DEFAULT_ATOMSTACKS);
  glAtomStructure->updateGL();
  glAtom->updateGL();
  update();
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot colorButtonClicked
void CAtomsDialog::colorButtonClicked(void)
{
  QColor newColor;
  newColor = QColorDialog::getColor(periodicsystem->GetColor(atomsListBox->currentItem() + 1));
  if(newColor.isValid() && newColor != periodicsystem->GetColor(atomsListBox->currentItem() + 1))
  {
    periodicsystem->SetColor(atomsListBox->currentItem() + 1, newColor);
    glAtom->setColor(periodicsystem->GetColor(atomsListBox->currentItem() + 1));
    glAtom->updateGL();
    update();
    applyButton->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot initWidgets
void CAtomsDialog::initWidgets(void)
{
  QString str_radius = "";
  fillAtomsListBox();
  str_radius.setNum(periodicsystem->GetRadius(atomsListBox->currentItem() + 1));
  radiusLineEdit->setText(str_radius);
  slicesSpinBox->setValue(glAtomStructure->getSlices());
  stacksSpinBox->setValue(glAtomStructure->getStacks());
  applyButton->setEnabled(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// slot fillAtomsListBox
void CAtomsDialog::fillAtomsListBox(void)
{
  int i = 0;
  QString item = "";

  // ListBox
  atomsListBox->clear();
  for(i = 1; i <= 109; i++)
  {
    item.setNum(i);
    item.insert(12 - item.length(), periodicsystem->GetSymbol(i));
    atomsListBox->insertItem(item);
  }
  atomsListBox->setCurrentItem(0);
}

/////////////////////////////////////////////////////////////////////////////
// slot rotateStructure
void CAtomsDialog::rotateStructure(void)
{
  if((rotY += 1.0) > 360.0) rotY -= 360.0;
  if((rotZ += 2.0) > 360.0) rotZ -= 360.0;
  glAtomStructure->setRotation(rotX, rotY, rotZ);
  glAtomStructure->updateGL();
}

/////////////////////////////////////////////////////////////////////////////
// slot atomsListboxChanged
void CAtomsDialog::atomsListboxChanged(int newIndex)
{
  QString str_radius = "";
  str_radius.setNum(periodicsystem->GetRadius(newIndex + 1));
  radiusLineEdit->setText(str_radius);
  glAtom->setColor(periodicsystem->GetColor(newIndex + 1));
  glAtom->updateGL();
  update();
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot radiusLineEditChanged
void CAtomsDialog::radiusLineEditChanged(const QString& newRadius)
{
  if((newRadius.toFloat() >= 0) && (newRadius.toFloat() <= 100))
    periodicsystem->SetRadius(atomsListBox->currentItem() + 1, newRadius.toFloat());
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tabChanged
void CAtomsDialog::tabChanged(QWidget*)
{
  update();
}

/////////////////////////////////////////////////////////////////////////////
// slot slicesChanged
void CAtomsDialog::slicesChanged(int newSlices)
{
  glAtomStructure->setSlices(newSlices);
  glAtom->setSlices(newSlices);
  glAtomStructure->updateGL();
  glAtom->updateGL();
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot stacksChanged
void CAtomsDialog::stacksChanged(int newStacks)
{
  glAtomStructure->setStacks(newStacks);
  glAtom->setStacks(newStacks);
  glAtomStructure->updateGL();
  glAtom->updateGL();
  applyButton->setEnabled(TRUE);
}  

/////////////////////////////////////////////////////////////////////////////
// methods
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// setPeriodicSystem
void CAtomsDialog::setPeriodicSystem(CKPeriodicSystem *ps)
{ 
  *periodicsystem = *ps;
  initWidgets();
}

/////////////////////////////////////////////////////////////////////////////
// setMolecule
void CAtomsDialog::setMolecule(CKGLMolecule *mol)
{
  *molecule = *mol;
  glAtomStructure->takeMoleculeProperties(molecule);
  glAtomStructure->setBgColor(QColor(0, 0, 0));
  glAtom->takeMoleculeProperties(molecule);
  initWidgets();
}

/////////////////////////////////////////////////////////////////////////////
// getMolecule
CKGLMolecule* CAtomsDialog::getMolecule(void)
{
  glAtomStructure->putMoleculeProperties(molecule);
  glAtom->putMoleculeProperties(molecule);
  for(int i = 0; i < molecule->GetNumberOfAtoms(); i++)
  {
    molecule->SetColor(i, periodicsystem->GetColor(molecule->GetOrdinalNumber(i)));
    molecule->SetRadius(i, periodicsystem->GetRadius(molecule->GetOrdinalNumber(i)));
  }  
  return(molecule);
}
