/***************************************************************************
                          dialog_bonds.cpp  -  description
                             -------------------
    begin                : Tue Mar 14 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dialog_bonds.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CBondsDialog
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CBondsDialog::CBondsDialog(QWidget *parent, const char *name, WFlags f)
  : CBondsDialogUI(parent, name, TRUE, f)
{
  rotX = -40.0;
  rotY = -40.0;
  rotZ = -40.0;

  molecule = new CKGLMolecule(this, "dlgmolecule");
  molecule->DeleteMolecule();

  timer = new QTimer(this, "timer");
  timer->start(50, FALSE);

  glBond = new CGLBond(selectionFrameOpenGl, "glbond");
  glBond->setGeometry(selectionFrameOpenGl->contentsRect());
  glBond->setRadius(0.1);
  glBond->setLength(0.8);
  glBond->enableSampleAtoms(0.15, QColor(255, 0, 0), DEFAULT_ATOMSLICES, DEFAULT_ATOMSTACKS);
  glBond->setRotation(-90.0, 45.0, 0.0);

  glBondStructure = new CGLBond(selectionGridFrameOpenGl, "glbondstructure");
  glBondStructure->setGeometry(selectionGridFrameOpenGl->contentsRect());
  glBondStructure->setRadius(1.0);
  glBondStructure->setLength(7.0);  
  glBondStructure->setDrawStyle(GLU_LINE);
  glBondStructure->setBgColor(QColor(0, 0, 0));
  glBondStructure->setColor(QColor(0, 255, 0));
  glBondStructure->setRotation(rotX, rotY, rotZ);
  
  // connections
  // timer
  connect(timer, SIGNAL(timeout()), this, SLOT(rotateStructure()));
  // button clicked
  connect(acceptButton, SIGNAL(clicked()), SLOT(accept()));
  connect(cancelButton, SIGNAL(clicked()), SLOT(reject()));
  connect(applyButton, SIGNAL(clicked()), SLOT(applyClicked()));
  connect(resetButton, SIGNAL(clicked()), SLOT(resetClicked()));
  // other signals
  connect(colorButton, SIGNAL(clicked()), SLOT(colorButtonClicked()));
  connect(tabWidget, SIGNAL(currentChanged(QWidget*)), SLOT(tabChanged(QWidget*)));
  connect(slicesSpinBox, SIGNAL(valueChanged(int)), SLOT(slicesChanged(int)));
  connect(radiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(radiusLineEditChanged(const QString&)));;
  connect(bondFudgeLineEdit, SIGNAL(textChanged(const QString&)), SLOT(bondFudgeLineEditChanged(const QString&)));;
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
CBondsDialog::~CBondsDialog()
{
}

/////////////////////////////////////////////////////////////////////////////
// events
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// paintEvent
void CBondsDialog::paintEvent(QPaintEvent*)
{
  QPainter p;
  p.begin(selectionFrameFlat);
  p.eraseRect(selectionFrameFlat->rect());
  p.fillRect(selectionFrameFlat->rect(), QBrush(molecule->GetBackgroundColor()));
  p.setPen(QPen(Qt::black, 1, Qt::SolidLine));
  p.setBrush(QBrush(Qt::red));
  p.drawEllipse(10, 65, 30, 30),
  p.drawEllipse(65, 10, 30, 30),
  p.setPen(QPen(molecule->GetBondColor(), 2));
  p.drawLine(36, 69, 70, 35);
  p.end();
}

/////////////////////////////////////////////////////////////////////////////
// slots
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// slot applyClicked
void CBondsDialog::applyClicked(void)
{
  applyButton->setEnabled(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// slot resetClicked
void CBondsDialog::resetClicked(void)
{
  QString str_radius = "";
  float bondradius = (float)DEFAULT_BONDRADIUS;  
  QString str_bondFudge = "";
  float bondfudge = (float)DEFAULT_BONDFUDGE;
  str_radius.setNum(bondradius);
  radiusLineEdit->setText(str_radius);
  molecule->SetBondRadius(bondradius);
  str_bondFudge.setNum(bondfudge);  
  bondFudgeLineEdit->setText(str_bondFudge);
  molecule->SetBondFudge(bondfudge);  
  molecule->SetBondColor((QColor)DEFAULT_BONDCOLOR);
  glBond->setColor((QColor)DEFAULT_BONDCOLOR);  
  glBondStructure->setSlices((int)DEFAULT_BONDSLICES);
  glBond->setSlices((int)DEFAULT_BONDSLICES); 
  slicesSpinBox->setValue((int)DEFAULT_BONDSLICES);  
  glBondStructure->updateGL();
  glBond->updateGL();  
  update();
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot colorButtonClicked
void CBondsDialog::colorButtonClicked(void)
{
  QColor newColor;
  newColor = QColorDialog::getColor(molecule->GetBondColor());
  if(newColor.isValid() && newColor != molecule->GetBondColor())
  {
    molecule->SetBondColor(newColor);
    glBond->setColor(molecule->GetBondColor());
    glBond->updateGL();
    update();
    applyButton->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot radiusLineEditChanged
void CBondsDialog::radiusLineEditChanged(const QString& newRadius)
{
  if((newRadius.toFloat() >= 0) && (newRadius.toFloat() <= 1.0))
    molecule->SetBondRadius(newRadius.toFloat());
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot bondFudgeLineEditChanged
void CBondsDialog::bondFudgeLineEditChanged(const QString& newBondFudge)
{
  if((newBondFudge.toFloat() >= 0) && (newBondFudge.toFloat() <= 10.0))
    molecule->SetBondFudge(newBondFudge.toFloat());
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot slicesChanged
void CBondsDialog::slicesChanged(int newSlices)
{
  glBondStructure->setSlices(newSlices);
  glBond->setSlices(newSlices);
  glBondStructure->updateGL();
  glBond->updateGL();
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot initWidgets
void CBondsDialog::initWidgets(void)
{
  QString str_radius = "";
  QString str_bondFudge = "";
  str_radius.setNum(molecule->GetBondRadius());
  radiusLineEdit->setText(str_radius);
  str_bondFudge.setNum(molecule->GetBondFudge());  
  bondFudgeLineEdit->setText(str_bondFudge);  
  slicesSpinBox->setValue(glBondStructure->getSlices());
  glBondStructure->updateGL();
  glBond->updateGL();
  applyButton->setEnabled(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tabChanged
void CBondsDialog::tabChanged(QWidget*)
{
  update();
}

/////////////////////////////////////////////////////////////////////////////
// slot rotateStructure
void CBondsDialog::rotateStructure(void)
{
  if((rotY += 1.0) > 360.0) rotY -= 360.0;
  if((rotZ += 2.0) > 360.0) rotZ -= 360.0;
  glBondStructure->setRotation(rotX, rotY, rotZ);
  glBondStructure->updateGL();
}

/////////////////////////////////////////////////////////////////////////////
// methods
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// setMolecule
void CBondsDialog::setMolecule(CKGLMolecule *mol)
{
  *molecule = *mol;
  glBondStructure->takeMoleculeProperties(molecule);
  glBondStructure->setBgColor(QColor(0, 0, 0));
  glBondStructure->setColor(QColor(0, 255, 0));
  glBondStructure->setRadius(0.1);
  glBondStructure->setLength(0.7);
  glBond->takeMoleculeProperties(molecule);
  glBond->setColor(molecule->GetBondColor());
  glBond->setRadius(0.05);
  glBond->setLength(0.8);
  glBond->enableSampleAtoms(0.15, QColor(255, 0, 0), molecule->GetAtomSlices(), molecule->GetAtomStacks());
  initWidgets();
}

/////////////////////////////////////////////////////////////////////////////
// getMolecule
CKGLMolecule* CBondsDialog::getMolecule(void)
{
  glBondStructure->putMoleculeProperties(molecule);
  glBond->putMoleculeProperties(molecule);
  return(molecule);
}

