/***************************************************************************
                          dialog_povray.cpp  -  description
                             -------------------
    begin                : Mon Mar 27 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dialog_povray.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CPOVRayDialog
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CPOVRayDialog::CPOVRayDialog(QWidget *parent, const char *name, WFlags f)
  : CPOVRayDialogUI(parent, name, TRUE, f)
{
  // Initialisations
  Light1 = new CPOVCoordinate(this, "Light1");
  Light2 = new CPOVCoordinate(this, "Light2");
  Light3 = new CPOVCoordinate(this, "Light3");
  CameraRight = new CPOVCoordinate(this, "CameraRight");
  CameraUp = new  CPOVCoordinate(this, "CameraUp");
  CameraDirection = new CPOVCoordinate(this, "CameraDirection");
  CameraLocation = new  CPOVCoordinate(this, "CameraLocation");
  CameraLookAt = new  CPOVCoordinate(this, "CameraLookAt");
  Light1->Reset();
  Light2->Reset();
  Light3->Reset();
  CameraRight->Reset();
  CameraUp->Reset();
  CameraDirection->Reset();
  CameraLocation->Reset();
  CameraLookAt->Reset();

  // connections
  // button clicked
  connect(acceptButton, SIGNAL(clicked()), SLOT(accept()));
  connect(cancelButton, SIGNAL(clicked()), SLOT(reject()));
  connect(applyButton, SIGNAL(clicked()), SLOT(applyClicked()));
  connect(resetButton, SIGNAL(clicked()), SLOT(resetClicked()));
  // other signals
  connect(tab1AtomColorButton, SIGNAL(clicked()), SLOT(tab1AtomColorButtonClicked()));
  connect(tab1AtomTextureButtonGroup, SIGNAL(clicked(int)), SLOT(tab1AtomTextureButtonGroupClicked(int)));
  connect(tab1AtomRadiusButtonGroup, SIGNAL(clicked(int)), SLOT(tab1AtomRadiusButtonGroupClicked(int)));
  connect(tab1AtomColorButtonGroup, SIGNAL(clicked(int)), SLOT(tab1AtomColorButtonGroupClicked(int)));
  connect(tab1AtomTextureLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab1AtomIncludeLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab1AtomRadiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));  
  connect(tab2BondColorButton, SIGNAL(clicked()), SLOT(tab2BondColorButtonClicked()));
  connect(tab2BondColorButtonGroup, SIGNAL(clicked(int)), SLOT(tab2BondColorButtonGroupClicked(int)));
  connect(tab2BondRadiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light1CheckBox, SIGNAL(stateChanged(int)), SLOT(checkBoxChanged(int)));
  connect(tab3Light2CheckBox, SIGNAL(stateChanged(int)), SLOT(checkBoxChanged(int)));
  connect(tab3Light3CheckBox, SIGNAL(stateChanged(int)), SLOT(checkBoxChanged(int)));
  connect(tab3Light1CheckBox, SIGNAL(stateChanged(int)), SLOT(tab3light1CheckBoxChanged(int)));
  connect(tab3Light2CheckBox, SIGNAL(stateChanged(int)), SLOT(tab3light2CheckBoxChanged(int)));
  connect(tab3Light3CheckBox, SIGNAL(stateChanged(int)), SLOT(tab3light3CheckBoxChanged(int)));
  connect(tab3ShadowlessCheckBox, SIGNAL(stateChanged(int)), SLOT(checkBoxChanged(int)));  
  connect(tab3Light1XLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light1YLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light1ZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light2XLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light2YLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light2ZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light3XLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light3YLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light3ZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4RightXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4RightYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4RightZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4UpXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4UpYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4UpZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4DirectionXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4DirectionYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4DirectionZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LocationXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LocationYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LocationZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LookAtXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LookAtYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LookAtZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));         
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
CPOVRayDialog::~CPOVRayDialog()
{
}

/////////////////////////////////////////////////////////////////////////////
// slot applyClicked
void CPOVRayDialog::applyClicked(void)
{
  applyButton->setEnabled(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// slot resetClicked
void CPOVRayDialog::resetClicked(void)
{
  // reset tab1
  tab1AtomTexture1RadioButton->setChecked(TRUE);
  tab1AtomTextureLineEdit->setEnabled(FALSE);
  tab1AtomIncludeLineEdit->setEnabled(FALSE);
  tab1AtomTextureLineEdit->setText("");
  tab1AtomIncludeLineEdit->setText("");
  tab1AtomColor1RadioButton->setChecked(TRUE);  
  tab1AtomColorButton->setEnabled(FALSE);
  tab1AtomColorFrame->setEnabled(FALSE);
  tab1AtomColorFrame->setPaletteBackgroundColor(QColor(128, 128, 128)); 
  tab1AtomRadius1RadioButton->setChecked(TRUE);
  tab1AtomRadiusLineEdit->setEnabled(FALSE);
  tab1AtomRadiusLineEdit->setText("");
  
  // reset tab2
  tab2BondColor2RadioButton->setChecked(TRUE);
  tab2BondColorButton->setEnabled(TRUE);
  tab2BondColorFrame->setEnabled(TRUE);
  tab2BondColorFrame->setPaletteBackgroundColor(QColor(204, 204, 204));
  tab2BondRadiusLineEdit->setText("0.1");

  // reset tab3
  tab3Light1XLineEdit->setText("35");
  tab3Light1YLineEdit->setText("35");
  tab3Light1ZLineEdit->setText("20");
  tab3Light1XLineEdit->setEnabled(TRUE);
  tab3Light1YLineEdit->setEnabled(TRUE);
  tab3Light1ZLineEdit->setEnabled(TRUE);  
  tab3Light2XLineEdit->setText("0");
  tab3Light2YLineEdit->setText("0");
  tab3Light2ZLineEdit->setText("25");
  tab3Light2XLineEdit->setEnabled(TRUE);
  tab3Light2YLineEdit->setEnabled(TRUE);
  tab3Light2ZLineEdit->setEnabled(TRUE);  
  tab3Light3XLineEdit->setText("");
  tab3Light3YLineEdit->setText("");
  tab3Light3ZLineEdit->setText("");
  tab3Light3XLineEdit->setEnabled(FALSE);
  tab3Light3YLineEdit->setEnabled(FALSE);
  tab3Light3ZLineEdit->setEnabled(FALSE);
  tab3Light1CheckBox->setChecked(TRUE);
  tab3Light2CheckBox->setChecked(TRUE);
  tab3Light3CheckBox->setChecked(FALSE);
  tab3ShadowlessCheckBox->setChecked(FALSE);

  // reset tab4
  tab4RightXLineEdit->setText("-1.33333");
  tab4RightYLineEdit->setText("0");
  tab4RightZLineEdit->setText("0");
  tab4UpXLineEdit->setText("0");
  tab4UpYLineEdit->setText("1");
  tab4UpZLineEdit->setText("0");
  tab4DirectionXLineEdit->setText("0");
  tab4DirectionYLineEdit->setText("0");
  tab4DirectionZLineEdit->setText("1");
  tab4LocationXLineEdit->setText("0");
  tab4LocationYLineEdit->setText("0");
  tab4LocationZLineEdit->setText("5");
  tab4LookAtXLineEdit->setText("0");
  tab4LookAtYLineEdit->setText("0");
  tab4LookAtZLineEdit->setText("0");

  // enable apply button
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab1AtomColorButtonClicked
void CPOVRayDialog::tab1AtomColorButtonClicked(void)
{
  QColor newColor;
  newColor = QColorDialog::getColor(tab1AtomColorFrame->paletteBackgroundColor());
  if(newColor.isValid() && newColor != tab1AtomColorFrame->paletteBackgroundColor())
  {
    tab1AtomColorFrame->setPaletteBackgroundColor(newColor);
    applyButton->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot tab1AtomTextureButtonGroupClicked
void CPOVRayDialog::tab1AtomTextureButtonGroupClicked(int id)
{
  switch(id)
  {
    case 3:
      tab1AtomTextureLineEdit->setEnabled(TRUE);    
      tab1AtomIncludeLineEdit->setEnabled(TRUE);
      break;
    default:
      tab1AtomTextureLineEdit->setEnabled(FALSE);
      tab1AtomIncludeLineEdit->setEnabled(FALSE);
      break;
  }
  applyButton->setEnabled(TRUE);  
}

/////////////////////////////////////////////////////////////////////////////
// slottab1AtomColorButtonGroupClicked
void CPOVRayDialog::tab1AtomColorButtonGroupClicked(int id)
{
  switch(id)
  {
    case 1:
      tab1AtomColorButton->setEnabled(TRUE);
      tab1AtomColorFrame->setEnabled(TRUE);
      break;
    default:
      tab1AtomColorButton->setEnabled(FALSE);
      tab1AtomColorFrame->setEnabled(FALSE);
      break;
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab1AtomRadiusButtonGroupClicked
void CPOVRayDialog::tab1AtomRadiusButtonGroupClicked(int id)
{
  switch(id)
  {
    case 1:
      tab1AtomRadiusLineEdit->setEnabled(TRUE);
      break;
    default:
      tab1AtomRadiusLineEdit->setEnabled(FALSE);
      break;
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab2BondColorButtonClicked
void CPOVRayDialog::tab2BondColorButtonClicked(void)
{
  QColor newColor;
  newColor = QColorDialog::getColor(tab2BondColorFrame->paletteBackgroundColor());
  if(newColor.isValid() && newColor != tab2BondColorFrame->paletteBackgroundColor())
  {
    tab2BondColorFrame->setPaletteBackgroundColor(newColor);
    applyButton->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot tab2BondColorButtonGroupClicked
void CPOVRayDialog::tab2BondColorButtonGroupClicked(int id)
{
  switch(id)
  {
    case 1:
      tab2BondColorButton->setEnabled(TRUE);
      break;
    default:
      tab2BondColorButton->setEnabled(FALSE);
      break;
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot checkBoxChanged
void CPOVRayDialog::checkBoxChanged(int)
{
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab3light1CheckBoxChanged
void CPOVRayDialog::tab3light1CheckBoxChanged(int state)
{
  if(state == 0)
  {
    tab3Light2CheckBox->setChecked(FALSE);
    tab3Light3CheckBox->setChecked(FALSE);
    tab3Light2CheckBox->setEnabled(FALSE);
    tab3Light3CheckBox->setEnabled(FALSE);
    tab3Light1XLineEdit->setEnabled(FALSE);
    tab3Light1YLineEdit->setEnabled(FALSE);
    tab3Light1ZLineEdit->setEnabled(FALSE);
    tab3Light2XLineEdit->setEnabled(FALSE);
    tab3Light2YLineEdit->setEnabled(FALSE);
    tab3Light2ZLineEdit->setEnabled(FALSE);
    tab3Light3XLineEdit->setEnabled(FALSE);
    tab3Light3YLineEdit->setEnabled(FALSE);
    tab3Light3ZLineEdit->setEnabled(FALSE);
  }
  else
  {
    tab3Light2CheckBox->setEnabled(TRUE);
    tab3Light1XLineEdit->setEnabled(TRUE);
    tab3Light1YLineEdit->setEnabled(TRUE);
    tab3Light1ZLineEdit->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot tab3light2CheckBoxChanged
void CPOVRayDialog::tab3light2CheckBoxChanged(int state)
{
  if(state == 0)
  {
    tab3Light3CheckBox->setChecked(FALSE);
    tab3Light3CheckBox->setEnabled(FALSE);
    tab3Light2XLineEdit->setEnabled(FALSE);
    tab3Light2YLineEdit->setEnabled(FALSE);
    tab3Light2ZLineEdit->setEnabled(FALSE);
    tab3Light3XLineEdit->setEnabled(FALSE);
    tab3Light3YLineEdit->setEnabled(FALSE);
    tab3Light3ZLineEdit->setEnabled(FALSE);
  }
  else
  {
    tab3Light3CheckBox->setEnabled(TRUE);
    tab3Light2XLineEdit->setEnabled(TRUE);
    tab3Light2YLineEdit->setEnabled(TRUE);
    tab3Light2ZLineEdit->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot tab3light3CheckBoxChanged
void CPOVRayDialog::tab3light3CheckBoxChanged(int state)
{
  if(state == 0)
  {
    tab3Light3XLineEdit->setEnabled(FALSE);
    tab3Light3YLineEdit->setEnabled(FALSE);
    tab3Light3ZLineEdit->setEnabled(FALSE);
  }
  else
  {
    tab3Light3XLineEdit->setEnabled(TRUE);
    tab3Light3YLineEdit->setEnabled(TRUE);
    tab3Light3ZLineEdit->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot lineEditChanged
void CPOVRayDialog::lineEditChanged(const QString&)
{
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// methods
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////
// setAlternateAtomRadius
void CPOVRayDialog::setAlternateAtomRadius(int newAlternateAtomRadius)
{
  if(newAlternateAtomRadius < 0) newAlternateAtomRadius = 0;
  if(newAlternateAtomRadius > 1) newAlternateAtomRadius = 1;
  tab1AtomRadiusButtonGroup->setButton(newAlternateAtomRadius);
  switch(newAlternateAtomRadius)
  {
    case 1:
      tab1AtomRadiusLineEdit->setEnabled(TRUE);
      break;
    default:
      tab1AtomRadiusLineEdit->setEnabled(FALSE);
      break;
  }
}

/////////////////////////////////////////////////
// setAlternateAtomColor
void CPOVRayDialog::setAlternateAtomColor(int newAlternateAtomColor)
{
  if(newAlternateAtomColor < 0) newAlternateAtomColor = 0;
  if(newAlternateAtomColor > 1) newAlternateAtomColor = 1;
  tab1AtomColorButtonGroup->setButton(newAlternateAtomColor);
  switch(newAlternateAtomColor)
  {
    case 1:
      tab1AtomColorButton->setEnabled(TRUE);
      tab1AtomColorFrame->setEnabled(TRUE);
      break;
    default:
      tab1AtomColorButton->setEnabled(FALSE);
      tab1AtomColorFrame->setEnabled(FALSE);
      break;
  } 
}

/////////////////////////////////////////////////
// setAtomColor
void CPOVRayDialog::setAtomColor(QColor newAtomColor)
{
  tab1AtomColorFrame->setPaletteBackgroundColor(newAtomColor);
}

/////////////////////////////////////////////////
// setAtomRadius
void CPOVRayDialog::setAtomRadius(float newAtomRadius)
{
  disconnect(tab1AtomRadiusLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab1AtomRadiusLineEdit->setText(QString::number(newAtomRadius));
  connect(tab1AtomRadiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setTextureNumber
void CPOVRayDialog::setTextureNumber(int newTextureNumber)
{
  if(newTextureNumber < 1 || newTextureNumber > 4) newTextureNumber = 1;
  tab1AtomTextureButtonGroup->setButton(newTextureNumber - 1);
  switch(newTextureNumber)
  {
    case 4:
      tab1AtomTextureLineEdit->setEnabled(TRUE);
      tab1AtomIncludeLineEdit->setEnabled(TRUE);
      break;
    default:
      tab1AtomTextureLineEdit->setEnabled(FALSE);
      tab1AtomIncludeLineEdit->setEnabled(FALSE);
      break;
  }
}

/////////////////////////////////////////////////
// setTexture
void CPOVRayDialog::setTexture(QString newTexture)
{
  disconnect(tab1AtomTextureLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab1AtomTextureLineEdit->setText(newTexture);
  connect(tab1AtomTextureLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setInclude
void CPOVRayDialog::setInclude(QString newInclude)
{
  disconnect(tab1AtomIncludeLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab1AtomIncludeLineEdit->setText(newInclude);
  connect(tab1AtomIncludeLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setBondRadius
void CPOVRayDialog::setBondRadius(float newBondRadius)
{
  disconnect(tab2BondRadiusLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab2BondRadiusLineEdit->setText(QString::number(newBondRadius));
  connect(tab2BondRadiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setAlternateBondColor
void CPOVRayDialog::setAlternateBondColor(int newAlternateBondColor)
{
  if(newAlternateBondColor < 0) newAlternateBondColor = 0;
  if(newAlternateBondColor > 1) newAlternateBondColor = 1;
  tab2BondColorButtonGroup->setButton(newAlternateBondColor);
  switch(newAlternateBondColor)
  {
    case 1:
      tab2BondColorButton->setEnabled(TRUE);
      tab2BondColorFrame->setEnabled(TRUE);
      break;
    default:
      tab2BondColorButton->setEnabled(FALSE);
      tab2BondColorFrame->setEnabled(FALSE);
      break;
  } 
}

/////////////////////////////////////////////////
// setBondColor
void CPOVRayDialog::setBondColor(QColor newBondColor)
{
  tab2BondColorFrame->setPaletteBackgroundColor(newBondColor);
}

/////////////////////////////////////////////////
// setLight1Enabled
void CPOVRayDialog::setLight1Enabled(int newLight1Enabled)
{
  disconnect(tab3Light1CheckBox, SIGNAL(stateChanged(int)), this, SLOT(checkBoxChanged(int)));
  tab3Light1CheckBox->setChecked((bool)newLight1Enabled);
  connect(tab3Light1CheckBox, SIGNAL(stateChanged(int)), SLOT(checkBoxChanged(int)));
}

/////////////////////////////////////////////////
// setLight2Enabled
void CPOVRayDialog::setLight2Enabled(int newLight2Enabled)
{
  disconnect(tab3Light2CheckBox, SIGNAL(stateChanged(int)), this, SLOT(checkBoxChanged(int)));
  tab3Light2CheckBox->setChecked((bool)newLight2Enabled);
  connect(tab3Light2CheckBox, SIGNAL(stateChanged(int)), SLOT(checkBoxChanged(int)));
}

/////////////////////////////////////////////////
// setLight3Enabled
void CPOVRayDialog::setLight3Enabled(int newLight3Enabled)
{
  disconnect(tab3Light3CheckBox, SIGNAL(stateChanged(int)), this, SLOT(checkBoxChanged(int)));
  tab3Light3CheckBox->setChecked((bool)newLight3Enabled);
  connect(tab3Light3CheckBox, SIGNAL(stateChanged(int)), SLOT(checkBoxChanged(int)));
}

/////////////////////////////////////////////////
// setShadowless
void CPOVRayDialog::setShadowless(int newShadowless)
{
  disconnect(tab3ShadowlessCheckBox, SIGNAL(stateChanged(int)), this, SLOT(checkBoxChanged(int)));
  tab3ShadowlessCheckBox->setChecked((bool)newShadowless);
  connect(tab3ShadowlessCheckBox, SIGNAL(stateChanged(int)), SLOT(checkBoxChanged(int)));
}

/////////////////////////////////////////////////
// setLight1
void CPOVRayDialog::setLight1(CPOVCoordinate* newLight1)
{
  disconnect(tab3Light1XLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab3Light1YLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab3Light1ZLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab3Light1XLineEdit->setText(QString::number(newLight1->GetX()));
  tab3Light1YLineEdit->setText(QString::number(newLight1->GetY()));
  tab3Light1ZLineEdit->setText(QString::number(newLight1->GetZ()));
  connect(tab3Light1XLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light1YLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light1ZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setLight2
void CPOVRayDialog::setLight2(CPOVCoordinate* newLight2)
{
  disconnect(tab3Light2XLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab3Light2YLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab3Light2ZLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab3Light2XLineEdit->setText(QString::number(newLight2->GetX()));
  tab3Light2YLineEdit->setText(QString::number(newLight2->GetY()));
  tab3Light2ZLineEdit->setText(QString::number(newLight2->GetZ()));
  connect(tab3Light2XLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light2YLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light2ZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setLight3
void CPOVRayDialog::setLight3(CPOVCoordinate* newLight3)
{
  disconnect(tab3Light3XLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab3Light3YLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab3Light3ZLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab3Light3XLineEdit->setText(QString::number(newLight3->GetX()));
  tab3Light3YLineEdit->setText(QString::number(newLight3->GetY()));
  tab3Light3ZLineEdit->setText(QString::number(newLight3->GetZ()));
  connect(tab3Light3XLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light3YLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3Light3ZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setCameraRight
void CPOVRayDialog::setCameraRight(CPOVCoordinate* newCameraRight)
{
  disconnect(tab4RightXLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4RightYLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4RightZLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab4RightXLineEdit->setText(QString::number(newCameraRight->GetX()));
  tab4RightYLineEdit->setText(QString::number(newCameraRight->GetY()));
  tab4RightZLineEdit->setText(QString::number(newCameraRight->GetZ()));
  connect(tab4RightXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4RightYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4RightZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setCameraUp
void CPOVRayDialog::setCameraUp(CPOVCoordinate* newCameraUp)
{
  disconnect(tab4UpXLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4UpYLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4UpZLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab4UpXLineEdit->setText(QString::number(newCameraUp->GetX()));
  tab4UpYLineEdit->setText(QString::number(newCameraUp->GetY()));
  tab4UpZLineEdit->setText(QString::number(newCameraUp->GetZ()));
  connect(tab4UpXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4UpYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4UpZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setCameraDirection
void CPOVRayDialog::setCameraDirection(CPOVCoordinate* newCameraDirection)
{
  disconnect(tab4DirectionXLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4DirectionYLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4DirectionZLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab4DirectionXLineEdit->setText(QString::number(newCameraDirection->GetX()));
  tab4DirectionYLineEdit->setText(QString::number(newCameraDirection->GetY()));
  tab4DirectionZLineEdit->setText(QString::number(newCameraDirection->GetZ()));
  connect(tab4DirectionXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4DirectionYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4DirectionZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setCameraLocation
void CPOVRayDialog::setCameraLocation(CPOVCoordinate* newCameraLocation)
{
  disconnect(tab4LocationXLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4LocationYLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4LocationZLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab4LocationXLineEdit->setText(QString::number(newCameraLocation->GetX()));
  tab4LocationYLineEdit->setText(QString::number(newCameraLocation->GetY()));
  tab4LocationZLineEdit->setText(QString::number(newCameraLocation->GetZ()));
  connect(tab4LocationXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LocationYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LocationZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setCameraLookAt
void CPOVRayDialog::setCameraLookAt(CPOVCoordinate* newCameraLookAt)
{
  disconnect(tab4LookAtXLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4LookAtYLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  disconnect(tab4LookAtZLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab4LookAtXLineEdit->setText(QString::number(newCameraLookAt->GetX()));
  tab4LookAtYLineEdit->setText(QString::number(newCameraLookAt->GetY()));
  tab4LookAtZLineEdit->setText(QString::number(newCameraLookAt->GetZ()));
  connect(tab4LookAtXLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LookAtYLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab4LookAtZLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// getAlternateAtomRadius
int CPOVRayDialog::getAlternateAtomRadius(void)
{
  return(tab1AtomRadius1RadioButton->isChecked() ? 0 : 1);
}

/////////////////////////////////////////////////
// getAlternateAtomColor
int CPOVRayDialog::getAlternateAtomColor(void)
{
  return(tab1AtomColor1RadioButton->isChecked() ? 0 : 1);
}

/////////////////////////////////////////////////
// getAtomColor
QColor CPOVRayDialog::getAtomColor(void)
{
  return(tab1AtomColorFrame->paletteBackgroundColor());
}

/////////////////////////////////////////////////
// getAtomRadius
float CPOVRayDialog::getAtomRadius(void)
{
  return(tab1AtomRadiusLineEdit->text().toFloat() > 0.0 ? tab1AtomRadiusLineEdit->text().toFloat() : 0.5);
}

/////////////////////////////////////////////////
// getTextureNumber
int CPOVRayDialog::getTextureNumber(void)
{
  if(tab1AtomTexture1RadioButton->isChecked()) return(1);
  if(tab1AtomTexture2RadioButton->isChecked()) return(2);
  if(tab1AtomTexture3RadioButton->isChecked()) return(3);
  if(tab1AtomTexture4RadioButton->isChecked()) return(4);
  return(1);
}

/////////////////////////////////////////////////
// getTexture
QString CPOVRayDialog::getTexture(void)
{
  QString texture = tab1AtomTextureLineEdit->text();
  if(texture.left(1) == "{") texture = texture.right(texture.length() - 1);
  if(texture.right(1) == "}") texture = texture.left(texture.length() - 1);
  return(texture);
}

/////////////////////////////////////////////////
// getInclude
QString CPOVRayDialog::getInclude(void)
{
  return(tab1AtomIncludeLineEdit->text());
}

/////////////////////////////////////////////////
// getBondRadius
float CPOVRayDialog::getBondRadius(void)
{
  return(tab2BondRadiusLineEdit->text().toFloat() > 0.0 ? tab2BondRadiusLineEdit->text().toFloat() : 0.1);
}

/////////////////////////////////////////////////
// getAlternateBondColor
int CPOVRayDialog::getAlternateBondColor(void)
{
  return(tab2BondColor1RadioButton->isChecked() ? 0 : 1);
}

/////////////////////////////////////////////////
// getBondColor
QColor CPOVRayDialog::getBondColor(void)
{
  return(tab2BondColorFrame->paletteBackgroundColor());
}

/////////////////////////////////////////////////
// getLight1Enabled
int CPOVRayDialog::getLight1Enabled(void)
{
  return(tab3Light1CheckBox->isChecked() ? 1 : 0);
}

/////////////////////////////////////////////////
// getLight2Enabled
int CPOVRayDialog::getLight2Enabled(void)
{
  return(tab3Light2CheckBox->isChecked() ? 1 : 0);
}

/////////////////////////////////////////////////
// getLight3Enabled
int CPOVRayDialog::getLight3Enabled(void)
{
  return(tab3Light3CheckBox->isChecked() ? 1 : 0);
}

/////////////////////////////////////////////////
// getShadowless
int CPOVRayDialog::getShadowless(void)
{
  return(tab3ShadowlessCheckBox->isChecked() ? 1 : 0);
}

/////////////////////////////////////////////////
// getLight1
CPOVCoordinate* CPOVRayDialog::getLight1(void)
{
  Light1->Reset();
  Light1->SetX((tab3Light1XLineEdit->text()).toFloat());
  Light1->SetY((tab3Light1YLineEdit->text()).toFloat());
  Light1->SetZ((tab3Light1ZLineEdit->text()).toFloat());
  return(Light1);
}

/////////////////////////////////////////////////
// getLight2
CPOVCoordinate* CPOVRayDialog::getLight2(void)
{
  Light2->Reset();
  Light2->SetX((tab3Light2XLineEdit->text()).toFloat());
  Light2->SetY((tab3Light2YLineEdit->text()).toFloat());
  Light2->SetZ((tab3Light2ZLineEdit->text()).toFloat());
  return(Light2);
}

/////////////////////////////////////////////////
// getLight3
CPOVCoordinate* CPOVRayDialog::getLight3(void)
{
  Light3->Reset();
  Light3->SetX((tab3Light3XLineEdit->text()).toFloat());
  Light3->SetY((tab3Light3YLineEdit->text()).toFloat());
  Light3->SetZ((tab3Light3ZLineEdit->text()).toFloat());
  return(Light3);
}

/////////////////////////////////////////////////
// getCameraRight
CPOVCoordinate* CPOVRayDialog::getCameraRight(void)
{
  CameraRight->Reset();
  CameraRight->SetX((tab4RightXLineEdit->text()).toFloat());
  CameraRight->SetY((tab4RightYLineEdit->text()).toFloat());
  CameraRight->SetZ((tab4RightZLineEdit->text()).toFloat());
  return(CameraRight);
}

/////////////////////////////////////////////////
// getCameraUp
CPOVCoordinate* CPOVRayDialog::getCameraUp(void)
{
  CameraUp->Reset();
  CameraUp->SetX((tab4UpXLineEdit->text()).toFloat());
  CameraUp->SetY((tab4UpYLineEdit->text()).toFloat());
  CameraUp->SetZ((tab4UpZLineEdit->text()).toFloat());
  return(CameraUp);
}

/////////////////////////////////////////////////
// getCameraDirection
CPOVCoordinate* CPOVRayDialog::getCameraDirection(void)
{
  CameraDirection->Reset();
  CameraDirection->SetX((tab4DirectionXLineEdit->text()).toFloat());
  CameraDirection->SetY((tab4DirectionYLineEdit->text()).toFloat());
  CameraDirection->SetZ((tab4DirectionZLineEdit->text()).toFloat());
  return(CameraDirection);
}

/////////////////////////////////////////////////
// getCameraLocation
CPOVCoordinate* CPOVRayDialog::getCameraLocation(void)
{
  CameraLocation->Reset();
  CameraLocation->SetX((tab4LocationXLineEdit->text()).toFloat());
  CameraLocation->SetY((tab4LocationYLineEdit->text()).toFloat());
  CameraLocation->SetZ((tab4LocationZLineEdit->text()).toFloat());
  return(CameraLocation);
}

/////////////////////////////////////////////////
// getCameraLookAt
CPOVCoordinate* CPOVRayDialog::getCameraLookAt(void)
{
  CameraLookAt->Reset();
  CameraLookAt->SetX((tab4LookAtXLineEdit->text()).toFloat());
  CameraLookAt->SetY((tab4LookAtYLineEdit->text()).toFloat());
  CameraLookAt->SetZ((tab4LookAtZLineEdit->text()).toFloat());
  return(CameraLookAt);
}


