/***************************************************************************
                          g94file.cpp  -  description
                             -------------------
    begin                : Thu Feb 3 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files
#include <g94file.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CG94Results
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// constructor
CG94Results::CG94Results(QObject *parent, const char *name)
  : QObject(parent, name)
{
  TypeOfCalculation = "";
  Title = "";
  G94Path = "";
  Charge = 0;
  Multiplicity = 0;
  RotConstA = 0.0;
  RotConstB = 0.0;
  RotConstC = 0.0;
  HF = 0.0;
  MP2 = 0.0;
  MP3 = 0.0;
  MP4D = 0.0;
  MP4DQ = 0.0;
  MP4SDQ = 0.0;
  MP4SDTQ = 0.0;
  MP5 = 0.0;
  PUHF = 0.0;
  PMP2_0 = 0.0;
  PMP3_0 = 0.0;
  CID = 0.0;
  CISD = 0.0;
  QCISD = 0.0;
  NumberOfAtoms = 0;
  NumberOfNormalModes = 0;
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CG94Results::~CG94Results()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CG94Results& CG94Results::operator=(CG94Results &G94Results)
{
  TypeOfCalculation = G94Results.TypeOfCalculation;
  Title = G94Results.Title;
  G94Path = G94Results.G94Path;
  Charge = G94Results.Charge;
  Multiplicity = G94Results.Multiplicity;
  MolMass = G94Results.MolMass;
  RotSymmNo = G94Results.RotSymmNo;
  RotConstA = G94Results.RotConstA;
  RotConstB = G94Results.RotConstB;
  RotConstC = G94Results.RotConstC;
  HF = G94Results.HF;
  MP2 = G94Results.MP2;
  MP3 = G94Results.MP3;
  MP4D = G94Results.MP4D;
  MP4DQ = G94Results.MP4DQ;
  MP4SDQ = G94Results.MP4SDQ;
  MP4SDTQ = G94Results.MP4SDTQ;
  MP5 = G94Results.MP5;
  PUHF = G94Results.PUHF;
  PMP2_0 = G94Results.PMP2_0;
  PMP3_0 = G94Results.PMP3_0;
  CID = G94Results.CID;
  CISD = G94Results.CISD;
  QCISD = G94Results.QCISD;
  NumberOfAtoms = G94Results.NumberOfAtoms;
  NumberOfNormalModes = G94Results.NumberOfNormalModes;

  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CG94Results::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CG94Atomcoordinate
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// constructor
CG94Atomcoordinate::CG94Atomcoordinate(QObject *parent, const char *name)
  : QObject(parent, name)
{
  Atomnumber = 0;
  x = (float)0.0;
  y = (float)0.0;
  z = (float)0.0;
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CG94Atomcoordinate::~CG94Atomcoordinate()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CG94Atomcoordinate& CG94Atomcoordinate::operator=(CG94Atomcoordinate &G94Atomcoordinate)
{
  // equalize members
  Atomnumber = G94Atomcoordinate.Atomnumber;
  x = G94Atomcoordinate.x;
  y = G94Atomcoordinate.y;
  z = G94Atomcoordinate.z;

  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CG94Atomcoordinate::Serialize()
{
  // not yet implemented
}


/////////////////////////////////////////////////////////////////////////////
// definition of the class CG94NormalModes
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// constructor
CG94NormalModes::CG94NormalModes(QObject *parent, const char *name)
  : QObject(parent, name)
{
  Frequency = (double)0.0;
  IRIntensity = (double)0.0;
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CG94NormalModes::~CG94NormalModes()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CG94NormalModes& CG94NormalModes::operator=(CG94NormalModes &G94NormalModes)
{
  // equalize members
  Frequency = G94NormalModes.Frequency;
  IRIntensity = G94NormalModes.IRIntensity;
  RamanIntensity = G94NormalModes.RamanIntensity;

  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CG94NormalModes::Serialize()
{
  // not yet implemented
}


/////////////////////////////////////////////////////////////////////////////
// definition of the class CG94File
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// constructor
CG94File::CG94File(QObject *parent, const char *name)
  : QObject(parent, name)
{
  // initialisations
  InstanceIsEmpty = TRUE;
  NumberOfSuccessfullCalculations = 0;

  // enabling auto-delete will delete objects that are removed
  G94ResultList.setAutoDelete(TRUE);
  G94AtomcoordinateList.setAutoDelete(TRUE);
  G94NormalModesList.setAutoDelete(TRUE);
  AtomMovementList.setAutoDelete(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CG94File::~CG94File()
{
  // AutoDelete is enabled!
  G94ResultList.clear();
  G94AtomcoordinateList.clear();
  G94NormalModesList.clear();
  AtomMovementList.clear();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CG94File& CG94File::operator=(CG94File &G94File)
{
  uint i = 0;
  // delete and copy lists
  // AutoDelete is enabled!
  G94ResultList.clear();
  G94AtomcoordinateList.clear();
  G94NormalModesList.clear();
  AtomMovementList.clear();

  // G94Results list
  CG94Results G94ResultsBuffer;
  if(G94File.G94ResultList.first())
  {
    for(i = 0; i <= G94File.G94ResultList.count() - 1; i++)
    {
      G94ResultsBuffer = *(CG94Results*)G94File.G94ResultList.current();
      G94Results = new CG94Results;
      if(!G94Results) break;
      *G94Results = G94ResultsBuffer;
      G94ResultList.append(G94Results);
      G94File.G94ResultList.next();
    }
  }

  // G94Atomcoordinate list
  CG94Atomcoordinate G94AtomcoordinateBuffer;
  if(G94File.G94AtomcoordinateList.first())
  {
    for(i = 0; i <= G94File.G94AtomcoordinateList.count() - 1; i++)
    {
      G94AtomcoordinateBuffer = *(CG94Atomcoordinate*)G94File.G94AtomcoordinateList.current();
      G94Atomcoordinate = new CG94Atomcoordinate;
      if(!G94Atomcoordinate) break;
      *G94Atomcoordinate = G94AtomcoordinateBuffer;
      G94AtomcoordinateList.append(G94Atomcoordinate);
      G94File.G94AtomcoordinateList.next();
    }
  }

  // G94NormalModes list
  CG94NormalModes G94NormalModesBuffer;
  if(G94File.G94NormalModesList.first())
  {
    for(i = 0; i <= G94File.G94NormalModesList.count() - 1; i++)
    {
      G94NormalModesBuffer = *(CG94NormalModes*)G94File.G94NormalModesList.current();
      G94NormalModes = new CG94NormalModes;
      if(!G94NormalModes) break;
      *G94NormalModes = G94NormalModesBuffer;
      G94NormalModesList.append(G94NormalModes);
      G94File.G94NormalModesList.next();
    }
  }

  // AtomMovement list
  CG94Atomcoordinate G94AtomMovementBuffer;
  if(G94File.AtomMovementList.first())
  {
    for(i = 0; i <= G94File.AtomMovementList.count() - 1; i++)
    {
      G94AtomMovementBuffer = *(CG94Atomcoordinate*)G94File.AtomMovementList.current();
      AtomMovement = new CG94Atomcoordinate;
      if(!AtomMovement) break;
      *AtomMovement = G94AtomMovementBuffer;
      AtomMovementList.append(AtomMovement);
      G94File.AtomMovementList.next();
    }
  }

  // copy members
  InstanceIsEmpty = G94File.InstanceIsEmpty;
  NumberOfSuccessfullCalculations = G94File.NumberOfSuccessfullCalculations;

  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CG94File::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// LoadFile
int CG94File::LoadFile(QString Path)
{
  // initialisations
  // AutoDelete is enabled!
  G94ResultList.clear();
  G94AtomcoordinateList.clear();
  G94NormalModesList.clear();
  AtomMovementList.clear();

  // general variables
  int i = 0;
  QFile G94File;
  int NumberOfAtoms = 0;
  NumberOfSuccessfullCalculations = 0;
  int NumberOfNormalModes = 0;
  int IsG94File = FALSE;
  int IsG94ForWindowsFile = FALSE;
  int TeilNr = 0;
  int FramelinesInStructureblock = 0;
  int LineNumber = 0;
  QString Line = "";
  int BlankPos = 0;
  double MolecularMass = 0.0;
  int RotSymmNumber = 0;

  // variables for reading frequencies
  int ReadFrequencySection = FALSE;
  int IsFrequencyOK = FALSE;
  int ReadAtomMovementCoordinates = FALSE;
  int MovementSectionLine = 0;
  int IsIRIntensityOK = FALSE;
  int IsRamanIntensityOK = FALSE;
  QString PartOfIRIntensityLine = "";
  QString PartOfRamanIntensityLine = "";
  QString PartOfFrequencyLine = "";
  QString PartOfMovementLine = "";
  int NumberOfModesPerSection = 0;
  double Freq[3];
  double IRInt[3];
  double RamanInt[3];

  // variables for results section
  int DoubleBSPos = 0;
  int CMPos = 0;
  QString ResultsSection = "";
  QString PartOfResultsSection = "";
  int ReadResultsSection = FALSE;

  // variables for structure section
  int OldNumberOfSuccessfullCalculations = NumberOfSuccessfullCalculations;
  QString PartOfCoordinateLine = "";
  QString PartOfRotConstLine = "";
  double RotConstantA = (double)0.0;
  double RotConstantB = (double)0.0;
  double RotConstantC = (double)0.0;
  int ReadStructureSection = FALSE;
  int ReadStructureSectionFirstTime = TRUE;

  // end of initialisation
  G94File.setName(Path);
  if(!G94File.open(IO_ReadOnly))
  {
    return FALSE;
  }
  else
  {
    while(!G94File.atEnd())
    {
      LineNumber++;
      G94File.readLine(Line, G94_FILE_MAX_LINE_LENGTH);

      // recognize gaussian file
      if((IsG94File == FALSE) && (LineNumber <= 30) && (Line.find("Entering Link 1 =") != -1))
      {
        IsG94File = TRUE;
      }
      if((IsG94File == FALSE) && (LineNumber >= 30))
      {
        return FALSE;
      }

      // spezial lines
      // molecular mass (not in #T-Output!)
      if(Line.find(" Molecular mass:") != -1)
      {
        MolecularMass = (Line.right(Line.length() - 16)).toFloat();
      }
      // symmetry number (not in #T-Output!)
      if(Line.find(" ROTATIONAL SYMMETRY NUMBER") != -1)
      {
        RotSymmNumber = (Line.right(Line.length() - 27)).toInt();
      }

      // frequency section
      if(ReadFrequencySection == TRUE)
      {
        if(Line.find(" Frequencies --") != -1)
        {
          IsFrequencyOK = TRUE;

          PartOfFrequencyLine = Line.right(Line.length() - 15);
          PartOfFrequencyLine = PartOfFrequencyLine.stripWhiteSpace();
          NumberOfModesPerSection = 1;
          Freq[NumberOfModesPerSection - 1] = PartOfFrequencyLine.toFloat();
          NumberOfNormalModes++;

          if((BlankPos = PartOfFrequencyLine.find(" ")) != -1)
          {
            NumberOfModesPerSection++;
            PartOfFrequencyLine = PartOfFrequencyLine.right(PartOfFrequencyLine.length() - BlankPos);
            PartOfFrequencyLine = PartOfFrequencyLine.stripWhiteSpace();
            Freq[NumberOfModesPerSection - 1] = PartOfFrequencyLine.toFloat();
            NumberOfNormalModes++;

            if((BlankPos = PartOfFrequencyLine.find(" ")) != -1)
            {
              NumberOfModesPerSection++;
              PartOfFrequencyLine = PartOfFrequencyLine.right(PartOfFrequencyLine.length() - BlankPos);
              PartOfFrequencyLine = PartOfFrequencyLine.stripWhiteSpace();
              Freq[NumberOfModesPerSection - 1] = PartOfFrequencyLine.toFloat();
              NumberOfNormalModes++;
            }
          }
        }

        // IR intensities
        if((Line.find(" IR Inten    --") != -1))
        {
          IsIRIntensityOK = TRUE;

          PartOfIRIntensityLine = Line.right(Line.length() - 15);
          PartOfIRIntensityLine = PartOfIRIntensityLine.stripWhiteSpace();
          IRInt[0] = PartOfIRIntensityLine.toFloat();

          if((NumberOfModesPerSection > 1) && ((BlankPos = PartOfIRIntensityLine.find(" ")) != -1))
          {
            PartOfIRIntensityLine = PartOfIRIntensityLine.right(PartOfIRIntensityLine.length() - BlankPos);
            PartOfIRIntensityLine = PartOfIRIntensityLine.stripWhiteSpace();
            IRInt[1] = PartOfIRIntensityLine.toFloat();

            if((NumberOfModesPerSection > 2) && ((BlankPos = PartOfIRIntensityLine.find(" ")) != -1))
            {
              PartOfIRIntensityLine = PartOfIRIntensityLine.right(PartOfIRIntensityLine.length() - BlankPos);
              PartOfIRIntensityLine = PartOfIRIntensityLine.stripWhiteSpace();
              IRInt[2] = PartOfIRIntensityLine.toFloat();
            }
          }
        }

        // Raman intensities
        if((Line.find(" Raman Activ --") != -1))
        {
          IsRamanIntensityOK = TRUE;

          PartOfRamanIntensityLine = Line.right(Line.length() - 15);
          PartOfRamanIntensityLine = PartOfRamanIntensityLine.stripWhiteSpace();

          RamanInt[0] = PartOfRamanIntensityLine.toFloat();

          if((NumberOfModesPerSection > 1) && ((BlankPos = PartOfRamanIntensityLine.find(" ")) != -1))
          {
            PartOfRamanIntensityLine = PartOfRamanIntensityLine.right(PartOfRamanIntensityLine.length() - BlankPos);
            PartOfRamanIntensityLine = PartOfRamanIntensityLine.stripWhiteSpace();
            RamanInt[1] = PartOfRamanIntensityLine.toFloat();

            if((NumberOfModesPerSection > 2) && ((BlankPos = PartOfRamanIntensityLine.find(" ")) != -1))
            {
              PartOfRamanIntensityLine = PartOfRamanIntensityLine.right(PartOfRamanIntensityLine.length() - BlankPos);
              PartOfRamanIntensityLine = PartOfRamanIntensityLine.stripWhiteSpace();
              RamanInt[2] = PartOfRamanIntensityLine.toFloat();
            }
          }
        }

        // atommovements
        if(ReadAtomMovementCoordinates == TRUE)
        {
          MovementSectionLine++;
          if((Line.left(4)).simplifyWhiteSpace() != "")
          {
            // read and sort atommovements
            i = 0;
            AtomMovement = new CG94Atomcoordinate;
            if(AtomMovement == NULL) return FALSE;

            // x1
            PartOfMovementLine = Line.right(Line.length() - 8);
            PartOfMovementLine = PartOfMovementLine.stripWhiteSpace();  
            AtomMovement->SetX(PartOfMovementLine.toFloat());

            // y1
            if((BlankPos = PartOfMovementLine.find(" ")) == -1) return FALSE;
            PartOfMovementLine = PartOfMovementLine.right(PartOfMovementLine.length() - BlankPos);
            PartOfMovementLine = PartOfMovementLine.stripWhiteSpace();
            AtomMovement->SetY(PartOfMovementLine.toFloat());

            // z1
            if((BlankPos = PartOfMovementLine.find(" ")) == -1) return FALSE;
            PartOfMovementLine = PartOfMovementLine.right(PartOfMovementLine.length() - BlankPos);
            PartOfMovementLine = PartOfMovementLine.stripWhiteSpace();
            AtomMovement->SetZ(PartOfMovementLine.toFloat());
              
            if(MovementSectionLine == 1)
            {
              AtomMovementList.append(AtomMovement);
            }
            else
            {
              if(NumberOfModesPerSection == 1)
              {
                AtomMovementList.append(AtomMovement);
              }

              if(NumberOfModesPerSection == 2)
              {
                for(i = 1; i < MovementSectionLine - 1; i++)
                {
                  AtomMovementList.prev();
                }
                AtomMovementList.insert(AtomMovementList.at(), AtomMovement);
              }

              if(NumberOfModesPerSection == 3)
              {
                for(i = 1; i < (2 * MovementSectionLine - 2); i++)
                {
                  AtomMovementList.prev();
                }
                AtomMovementList.insert(AtomMovementList.at(), AtomMovement);
              }
            }

            if(NumberOfModesPerSection > 1)
            {
              AtomMovement = new CG94Atomcoordinate;
              if(AtomMovement == NULL) return FALSE;

              // x2
              if((BlankPos = PartOfMovementLine.find(" ")) == -1) return FALSE;
              PartOfMovementLine = PartOfMovementLine.right(PartOfMovementLine.length() - BlankPos);
              PartOfMovementLine = PartOfMovementLine.stripWhiteSpace();  
              AtomMovement->SetX(PartOfMovementLine.toFloat());

              // y2
              if((BlankPos = PartOfMovementLine.find(" ")) == -1) return FALSE;
              PartOfMovementLine = PartOfMovementLine.right(PartOfMovementLine.length() - BlankPos);
              PartOfMovementLine = PartOfMovementLine.stripWhiteSpace();
              AtomMovement->SetY(PartOfMovementLine.toFloat());

              // z2
              if((BlankPos = PartOfMovementLine.find(" ")) == -1) return FALSE;
              PartOfMovementLine = PartOfMovementLine.right(PartOfMovementLine.length() - BlankPos);
              PartOfMovementLine = PartOfMovementLine.stripWhiteSpace();
              AtomMovement->SetZ(PartOfMovementLine.toFloat());
            
              if(MovementSectionLine == 1)
              {
                AtomMovementList.append(AtomMovement);
              }
              else
              {
                if((NumberOfModesPerSection == 1) || (NumberOfModesPerSection == 2))
                {
                  AtomMovementList.append(AtomMovement);
                }

                if(NumberOfModesPerSection == 3)
                {
                  for(i = 1; i < MovementSectionLine; i++)
                  {
                    AtomMovementList.next();
                  }
                  AtomMovementList.insert(AtomMovementList.at() + 1, AtomMovement);
                }
              }

              if(NumberOfModesPerSection > 2)
              {
                AtomMovement = new CG94Atomcoordinate;
                if(AtomMovement == NULL) return FALSE;

                // x3
                if((BlankPos = PartOfMovementLine.find(" ")) == -1) return FALSE;
                PartOfMovementLine = PartOfMovementLine.right(PartOfMovementLine.length() - BlankPos);
                PartOfMovementLine = PartOfMovementLine.stripWhiteSpace();  
                AtomMovement->SetX(PartOfMovementLine.toFloat());

                // y3
                if((BlankPos = PartOfMovementLine.find(" ")) == -1) return FALSE;
                PartOfMovementLine = PartOfMovementLine.right(PartOfMovementLine.length() - BlankPos);
                PartOfMovementLine = PartOfMovementLine.stripWhiteSpace();
                AtomMovement->SetY(PartOfMovementLine.toFloat());

                // z3
                if((BlankPos = PartOfMovementLine.find(" ")) == -1) return FALSE;
                PartOfMovementLine = PartOfMovementLine.right(PartOfMovementLine.length() - BlankPos);
                PartOfMovementLine = PartOfMovementLine.stripWhiteSpace();
                AtomMovement->SetZ(PartOfMovementLine.toFloat());

                AtomMovementList.append(AtomMovement);
              }
            }
          }
          else
          {
            ReadAtomMovementCoordinates = FALSE;
            MovementSectionLine = 0;
          }
        }

        // put data except atom movment into list
        if(Line.find(" Atom AN") != -1)
        {
          for(i = 0; i < NumberOfModesPerSection; i++)
          {
            G94NormalModes = new CG94NormalModes;
            if(G94NormalModes == NULL) return FALSE;
            if(IsFrequencyOK) G94NormalModes->Frequency = Freq[i];
            if(IsIRIntensityOK) G94NormalModes->IRIntensity = IRInt[i];
            if(IsRamanIntensityOK) G94NormalModes->RamanIntensity = RamanInt[i];

            G94NormalModesList.append(G94NormalModes);
          }

          IsFrequencyOK = FALSE;
          IsIRIntensityOK = FALSE;
          IsRamanIntensityOK = FALSE;

          // next line contains atom movements
          if(ReadAtomMovementCoordinates == TRUE) return FALSE;
          ReadAtomMovementCoordinates = TRUE;
        }
      }

      // recocnize the beginning of the frequency section
      if(Line.find("reduced masses (AMU)") != -1)
      {
        ReadFrequencySection = TRUE;
        NumberOfNormalModes = 0;
      }

      // recocnize the end of the frequency section
      if((Line == "") && (ReadFrequencySection == TRUE))
      {
        ReadFrequencySection = FALSE;
      }

      // structure section
      if(Line.find("Standard orientation:") != -1)
      {
        ReadStructureSection = TRUE;
        FramelinesInStructureblock = 0;
      }
      if((ReadStructureSection == TRUE) && (FramelinesInStructureblock == 2) && (Line.mid(2, 10) != "----------"))
      {
        if(ReadStructureSectionFirstTime == TRUE) NumberOfAtoms++;
        G94Atomcoordinate = new CG94Atomcoordinate;
        if(G94Atomcoordinate == NULL) return FALSE;

        // atomsymbol
        PartOfCoordinateLine = Line.right(Line.length() - 7);
        PartOfCoordinateLine = PartOfCoordinateLine.stripWhiteSpace();
        BlankPos = PartOfCoordinateLine.find(" ");
        if(BlankPos == -1) return FALSE;
        G94Atomcoordinate->Atomnumber = (PartOfCoordinateLine.left(BlankPos)).toInt();
        PartOfCoordinateLine = PartOfCoordinateLine.right(PartOfCoordinateLine.length() - BlankPos);
        PartOfCoordinateLine = PartOfCoordinateLine.stripWhiteSpace();

        // gaussian98 files contains an additional column "Atomic Type"
        BlankPos = PartOfCoordinateLine.find(" ");
        if(BlankPos == -1) return FALSE;
        if(BlankPos == 1)
        {
          PartOfCoordinateLine = Line.right(PartOfCoordinateLine.length() - 7);
          PartOfCoordinateLine = PartOfCoordinateLine.stripWhiteSpace();
        }

        // x-coordinate
        BlankPos = PartOfCoordinateLine.find(" ");
        if(BlankPos == -1) return FALSE;
        G94Atomcoordinate->SetX(PartOfCoordinateLine.toFloat());
        PartOfCoordinateLine = PartOfCoordinateLine.right(PartOfCoordinateLine.length() - BlankPos);
        PartOfCoordinateLine = PartOfCoordinateLine.stripWhiteSpace();

        // y-coordinate
        BlankPos = PartOfCoordinateLine.find(" ");
        if(BlankPos == -1) return FALSE;
        G94Atomcoordinate->SetY(PartOfCoordinateLine.toFloat());
        PartOfCoordinateLine = PartOfCoordinateLine.right(PartOfCoordinateLine.length() - BlankPos);
        PartOfCoordinateLine = PartOfCoordinateLine.stripWhiteSpace();

        // z-coordinate
        G94Atomcoordinate->SetZ(PartOfCoordinateLine.toFloat());

        // append list
        G94AtomcoordinateList.append(G94Atomcoordinate);
      }

      // recognize rotational constants, put data into atom coordinate list
      // and finish structure section
      if(FramelinesInStructureblock == 3)
      {
        ReadStructureSectionFirstTime = FALSE;
        FramelinesInStructureblock = 0;
        ReadStructureSection = FALSE;

        // rotational constants
        if((Line.find(" Rotational constants (GHZ):") != -1))
        {
          PartOfRotConstLine = Line.right(Line.length() - 28);
          PartOfRotConstLine = PartOfRotConstLine.stripWhiteSpace();

          RotConstantA = PartOfRotConstLine.toFloat();

          if((BlankPos = PartOfRotConstLine.find(" ")) != -1)
          {
            PartOfRotConstLine = PartOfRotConstLine.right(PartOfRotConstLine.length() - BlankPos);
            PartOfRotConstLine = PartOfRotConstLine.stripWhiteSpace();
            RotConstantB = PartOfRotConstLine.toFloat();

            if((BlankPos = PartOfRotConstLine.find(" ")) != -1)
            {
              PartOfRotConstLine = PartOfRotConstLine.right(PartOfRotConstLine.length() - BlankPos);
              PartOfRotConstLine = PartOfRotConstLine.stripWhiteSpace();
              RotConstantC = PartOfRotConstLine.toFloat();
            }
          }
        }

        if((OldNumberOfSuccessfullCalculations == NumberOfSuccessfullCalculations)
          && ((int)G94AtomcoordinateList.count() > NumberOfAtoms))
        {
          // structure belongs to the actually read calculation
          // delete previous structure
          for(int i = 1; i <= NumberOfAtoms; i++)
          {
            if(G94AtomcoordinateList.isEmpty()) return FALSE;
            G94AtomcoordinateList.removeFirst();
          }
        }

        OldNumberOfSuccessfullCalculations = NumberOfSuccessfullCalculations;
      }

      // recognize framelines of the structure section
      if((ReadStructureSection == TRUE) && (Line.mid(2, 10) == "----------"))
        FramelinesInStructureblock++;

      // results section
      // UNIX output format
      if(Line.left(5) == " 1\\1\\")
      {
        IsG94ForWindowsFile = FALSE;
        ReadResultsSection = TRUE;
        ReadStructureSectionFirstTime = TRUE;
      }
      // MS Windows output format
      if(Line.left(5) == " 1|1|")
      {
        IsG94ForWindowsFile = TRUE;
        ReadResultsSection = TRUE;
        ReadStructureSectionFirstTime = TRUE;
      }

      if((Line.simplifyWhiteSpace() == "") && (ReadResultsSection == TRUE))
      {
        // replace '|' by '\' if file was created by Gaussian for MS Windows
        if(IsG94ForWindowsFile)
          while(ResultsSection.find("|") != -1)
            ResultsSection.replace(ResultsSection.find("|"), 1, "\\");

        ReadResultsSection = FALSE;  
        TeilNr = 0;
        NumberOfSuccessfullCalculations++;

        G94Results = new CG94Results;
        if(G94Results == NULL) return FALSE;

        // results which are not from the results section
        G94Results->NumberOfAtoms = NumberOfAtoms;
        G94Results->NumberOfNormalModes = NumberOfNormalModes;
        G94Results->RotConstA = RotConstantA;
        G94Results->RotConstB = RotConstantB;
        G94Results->RotConstC = RotConstantC;
        G94Results->MolMass = MolecularMass;
        G94Results->RotSymmNo = RotSymmNumber;

        // get data from the results section
        if(ResultsSection.length() > 3)
        {
          do
          {
            TeilNr++;

            // cut until "\\"
            DoubleBSPos = ResultsSection.find("\\\\");
            if(DoubleBSPos == -1) return FALSE;

            PartOfResultsSection = ResultsSection.left(DoubleBSPos);
            ResultsSection = ResultsSection.right(ResultsSection.length() - DoubleBSPos - 2);

            switch (TeilNr)
            {
            case 1:
              {
                // Part 1 (general parameters):
                break;
              }

            case 2:
              {
                // Part 2 (type of calculation):
                G94Results->TypeOfCalculation = PartOfResultsSection;
                break;
              }

            case 3:
              {
                // Part 3 (titel):
                G94Results->Title = PartOfResultsSection;
                break;
              }

            case 4:
              {
                // Part 4 (charge, multiplicity):
                
                // charge
                CMPos = PartOfResultsSection.find(",");
                if (CMPos == -1) return FALSE;
                G94Results->Charge = (PartOfResultsSection.left(CMPos)).toInt();
                PartOfResultsSection = PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 1);

                // multiplicity
                CMPos = PartOfResultsSection.find("\\");
                if (CMPos == -1) return FALSE;
                G94Results->Multiplicity = (PartOfResultsSection.left(CMPos)).toInt();
                
                break;
              }
            default:
              {
                // path
                G94Results->G94Path = Path;

                // abs. energy
                if (((CMPos = PartOfResultsSection.find("HF=")) != -1) && (PartOfResultsSection.mid(CMPos - 2, 4) != "PUHF="))
                  G94Results->HF = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 3)).toFloat();

                if ((CMPos = PartOfResultsSection.find("MP2=")) != -1)
                  G94Results->MP2 = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 4)).toFloat();

                if ((CMPos = PartOfResultsSection.find("MP3=")) != -1)
                  G94Results->MP3 = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 4)).toFloat();

                if ((CMPos = PartOfResultsSection.find("MP4D=")) != -1)
                  G94Results->MP4D = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 5)).toFloat();
                  
                if ((CMPos = PartOfResultsSection.find("MP4DQ=")) != -1)
                  G94Results->MP4DQ = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 6)).toFloat();
                  
                if ((CMPos = PartOfResultsSection.find("MP4SDQ=")) != -1)
                  G94Results->MP4SDQ = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 7)).toFloat();
                  
                if ((CMPos = PartOfResultsSection.find("MP4SDTQ=")) != -1)
                  G94Results->MP4SDTQ = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 8)).toFloat();
                  
                if ((CMPos = PartOfResultsSection.find("MP5=")) != -1)
                  G94Results->MP5 = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 4)).toFloat();

                if ((CMPos = PartOfResultsSection.find("PUHF=")) != -1)
                  G94Results->PUHF = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 5)).toFloat();

                if ((CMPos = PartOfResultsSection.find("PMP2-0=")) != -1)
                  G94Results->PMP2_0 = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 7)).toFloat();

                if ((CMPos = PartOfResultsSection.find("PMP3-0=")) != -1)
                  G94Results->PMP3_0 = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 7)).toFloat();

                if ((CMPos = PartOfResultsSection.find("CID=")) != -1)
                  G94Results->CID = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 4)).toFloat();
                
                if (((CMPos = PartOfResultsSection.find("CISD=")) != -1) && (PartOfResultsSection.mid(CMPos - 1, 6) != "QCISD="))
                  G94Results->CISD = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 5)).toFloat();

                if ((CMPos = PartOfResultsSection.find("QCISD=")) != -1)
                  G94Results->QCISD = (PartOfResultsSection.right(PartOfResultsSection.length() - CMPos - 6)).toFloat();              

              }
            }

          }while(ResultsSection.length() > 2);

          G94ResultList.append(G94Results);

          // reset the following variables before reading next calculation  
          NumberOfAtoms = 0;
          NumberOfNormalModes = 0;
          MolecularMass = 0.0;
          RotSymmNumber = 0;
          ReadFrequencySection = FALSE;
          IsFrequencyOK = FALSE;
          IsIRIntensityOK = FALSE;
          IsRamanIntensityOK = FALSE;
          NumberOfModesPerSection = 0;
        }
        else
          return FALSE;
      }

      // format line and append to result section
      if(ReadResultsSection == TRUE)
      {
        Line = (Line.right(Line.length() - 1)).simplifyWhiteSpace();
        ResultsSection += Line;
      }
    }

    // results found
    InstanceIsEmpty = FALSE;
  }

  return IsG94File;
}

/////////////////////////////////////////////////////////////////////////////
// Get...

/////////////////////////////////////////////////////////////////////////////
// GetNumberOfCalculations
QString CG94File::GetPathName(void)
{
  if(InstanceIsEmpty) return NULL;
  CG94Results* G94ResultsBuffer;
  G94ResultsBuffer = (CG94Results*)G94ResultList.first();
  if (G94ResultsBuffer)
    return G94ResultsBuffer->G94Path;
  else
    return "";
}

/////////////////////////////////////////////////////////////////////////////
// GetNumberOfCalculations
int CG94File::GetNumberOfCalculations(void)
{
  return NumberOfSuccessfullCalculations;
}

/////////////////////////////////////////////////////////////////////////////
// GetNumberOfAtoms
int CG94File::GetNumberOfAtoms(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0;
  return G94ResultsBuffer->NumberOfAtoms;
}

/////////////////////////////////////////////////////////////////////////////
// GetTypeOfCalculation
QString CG94File::GetTypeOfCalculation(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return NULL;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return "";
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return "";
  return G94ResultsBuffer->TypeOfCalculation;
}

/////////////////////////////////////////////////////////////////////////////
// GetTitle
QString CG94File::GetTitle(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return NULL;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return "";
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return "";
  return G94ResultsBuffer->Title;
}

/////////////////////////////////////////////////////////////////////////////
// GetMultiplicity
int CG94File::GetMultiplicity(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0;
  CG94Results* G94ResultsBuffer = 0;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0;
  return G94ResultsBuffer->Multiplicity;
}

/////////////////////////////////////////////////////////////////////////////
// GetCharge
int CG94File::GetCharge(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0;
  return G94ResultsBuffer->Charge;
}

/////////////////////////////////////////////////////////////////////////////
// GetCoordinate
int CG94File::GetCoordinate(int CalNumber, int LineNumber, CG94Atomcoordinate& Coordinate)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations) || (LineNumber > GetNumberOfAtoms(CalNumber))) return FALSE;
  CG94Atomcoordinate* CoordinateBuffer = NULL;
  int i = 0;
  G94AtomcoordinateList.first();
  for(i = 1; i <= ((CalNumber - 1) * GetNumberOfAtoms(CalNumber) + LineNumber); i++)
  {
    if(!(CoordinateBuffer = (CG94Atomcoordinate*)G94AtomcoordinateList.current())) return FALSE;
    G94AtomcoordinateList.next();
  }
  Coordinate = *CoordinateBuffer;
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// GetNumberOfNormalModes
int CG94File::GetNumberOfNormalModes(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0;
  return G94ResultsBuffer->NumberOfNormalModes;
}

/////////////////////////////////////////////////////////////////////////////
// GetNormalMode
int CG94File::GetNormalMode(int CalNumber, int ModeNumber, CG94NormalModes& NormalMode)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations) || (ModeNumber > GetNumberOfNormalModes(CalNumber))) return FALSE;
  CG94NormalModes* NormalModeBuffer = NULL;
  int i = 0;
  G94NormalModesList.first();
  for(i = 0; i < CalNumber; i++)
  {
    for(int j = 1; (j <= GetNumberOfNormalModes(i + 1)); j++)
    {
      if(!(NormalModeBuffer = (CG94NormalModes*)G94NormalModesList.current())) return FALSE;
      G94NormalModesList.next();      
      if((j == ModeNumber) && (i == CalNumber - 1)) 
      {
        NormalMode = *NormalModeBuffer;
        return TRUE;
      }
    }
  }
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// GetRotConstA
double CG94File::GetRotConstA(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0;
  return G94ResultsBuffer->RotConstA;
}

/////////////////////////////////////////////////////////////////////////////
// GetRotConstB
double CG94File::GetRotConstB(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0;
  return G94ResultsBuffer->RotConstB;
}

/////////////////////////////////////////////////////////////////////////////
// GetRotConstC
double CG94File::GetRotConstC(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0;
  return G94ResultsBuffer->RotConstC;
}

/////////////////////////////////////////////////////////////////////////////
// GetRotationalSymmetryNumber
int CG94File::GetRotationalSymmetryNumber(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0;
  return G94ResultsBuffer->RotSymmNo;
}

/////////////////////////////////////////////////////////////////////////////
// GetMolecularMass
double CG94File::GetMolecularMass(int CalNumber)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0;
  return G94ResultsBuffer->MolMass;
}

/////////////////////////////////////////////////////////////////////////////
// GetEnergy
double CG94File::GetEnergy(int CalNumber, int Method)
{
  if((InstanceIsEmpty) || (CalNumber > NumberOfSuccessfullCalculations)) return 0.0;
  CG94Results* G94ResultsBuffer = NULL;
  int i = 0;
  double result = 0.0;
  G94ResultList.first();
  for(i = 1; i <= CalNumber; i++)
  {
    if(!(G94ResultsBuffer = (CG94Results*)G94ResultList.current())) return 0.0;
    G94ResultList.next();
  }
  if(G94ResultsBuffer == NULL) return 0.0;
  switch (Method)
  {
    case 0: {result = G94ResultsBuffer->HF; break;}
    case 1: {result = G94ResultsBuffer->MP2; break;}
    case 2: {result = G94ResultsBuffer->MP3; break;}
    case 3: {result = G94ResultsBuffer->MP4D; break;}
    case 4: {result = G94ResultsBuffer->MP4DQ; break;}
    case 5: {result = G94ResultsBuffer->MP4SDQ; break;}
    case 6: {result = G94ResultsBuffer->MP4SDTQ; break;}
    case 7: {result = G94ResultsBuffer->MP5; break;}
    case 8: {result = G94ResultsBuffer->PUHF; break;}
    case 9: {result =  G94ResultsBuffer->PMP2_0; break;}
    case 10: {result = G94ResultsBuffer->PMP3_0; break;}
    case 11: {result = G94ResultsBuffer->CID; break;}
    case 12: {result = G94ResultsBuffer->CISD; break;}
    case 13: {result = G94ResultsBuffer->QCISD; break;}
    default: result = G94ResultsBuffer->HF;
  }
  return result;
}

/////////////////////////////////////////////////////////////////////////////
// GetNormalModeMovement
int CG94File::GetNormalModeMovement(int CalNumber, int ModeNumber, int LineNumber, CG94Atomcoordinate& AtomMovement)
{
  CG94Atomcoordinate *CoordinateBuffer = NULL;

  AtomMovementList.first();
  for(int i = 0; i < CalNumber; i++)
  {
    for(int j = 1; (j <= (GetNumberOfNormalModes(i + 1) * GetNumberOfAtoms(i + 1))); j++)
    {
      if(!(CoordinateBuffer = (CG94Atomcoordinate*)AtomMovementList.current())) return FALSE;
      AtomMovementList.next();
      if((i == (CalNumber - 1)) && (j == (ModeNumber - 1) * GetNumberOfAtoms(i + 1) + LineNumber))
      {
        AtomMovement = *CoordinateBuffer;
        return TRUE;
      }
    }
  }
  return FALSE;
}
