/***************************************************************************
                          g94file.h  -  description
                             -------------------
    begin                : Thu Feb 3 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 
#ifndef _G94FILE
#define _G94FILE

/* include files for QT */
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>
#include <qfile.h>

/* application specific includes */
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <iostream>

#define HF_ENERGY 0
#define MP2_ENERGY 1
#define MP3_ENERGY 2
#define MP4D_ENERGY 3
#define MP4DQ_ENERGY 4
#define MP4SDQ_ENERGY 5
#define MP4SDTQ_ENERGY 6
#define MP5_ENERGY 7
#define PUHF_ENERGY 8
#define PMP2_0_ENERGY 9
#define PMP3_0_ENERGY 10
#define CID_ENERGY 11
#define CISD_ENERGY 12
#define QCISD_ENERGY 13

#define G94_FILE_MAX_LINE_LENGTH 256

enum CalType{HF, MP2, MP3, MP4D, MP4DQ, MP4SDQ, MP4SDTQ, MP5, PUHF, PMP2_0, PMP3_0, CID, CISD, QCISD};

/* class CG94Results */
class CG94Results : public QObject
{
  Q_OBJECT
  friend class CG94File;

public:
  /* constructor */
  CG94Results(QObject *parent=0, const char *name=0);
 
  /* destructor */
  ~CG94Results();
 
  /* operators */
  CG94Results& operator=(CG94Results &G94Results);
 
  /* methods */
  void Serialize();

protected:
  /* members */
  QString TypeOfCalculation;
  QString Title;
  QString G94Path;
  int Charge; 
  int Multiplicity; 
  double MolMass; 
  int RotSymmNo; 
  double RotConstA; 
  double RotConstB; 
  double RotConstC; 
  double HF; 
  double MP2; 
  double MP3; 
  double MP4D; 
  double MP4DQ; 
  double MP4SDQ; 
  double MP4SDTQ; 
  double MP5; 
  double PUHF; 
  double PMP2_0; 
  double PMP3_0; 
  double CID; 
  double CISD; 
  double QCISD; 
  int NumberOfAtoms; 
  int NumberOfNormalModes;
}; 


/* class CG94NormalModes */
class CG94NormalModes : public QObject
{
  Q_OBJECT
  friend class CG94File;
 
public: 
  /* constructor */
  CG94NormalModes(QObject *parent=0, const char *name=0);
 
  /* destructor */
  ~CG94NormalModes(); 
 
  /* operators */
  CG94NormalModes& operator=(CG94NormalModes &G94NormalModes); 

  /* methods */
  void Serialize();

  /* Get... */
  double GetFrequency(void) {return (Frequency);} 
  double GetIRIntensity(void) {return (IRIntensity);} 
  double GetRamanIntensity(void) {return (RamanIntensity);} 
 
protected: 
  /* members */
  double Frequency; 
  double IRIntensity; 
  double RamanIntensity;
};


/* class CG94Atomcoordinate */
class CG94Atomcoordinate : public QObject
{
  Q_OBJECT
  friend class CG94File;
 
public: 
  /* constructor */
  CG94Atomcoordinate(QObject *parent=0, const char *name=0);
 
  /* destructor */
  ~CG94Atomcoordinate(); 
 
  /* operators */
  CG94Atomcoordinate& operator=(CG94Atomcoordinate &G94Atomcoordinate); 
 
  /* methods */
  void Serialize();
 
  /* Get... */
  int GetAtomnumber(void) {return Atomnumber;} 
  float GetX(void) {return (x);} 
  float GetY(void) {return (y);} 
  float GetZ(void) {return (z);} 
 
  /* Set... */
  void SetAtomnumber(int OZ) {Atomnumber = OZ; return;} 
  void SetX(float newx) {x = newx; return;} 
  void SetY(float newy) {y = newy; return;} 
  void SetZ(float newz) {z = newz; return;} 

protected: 
  /* members */
  int Atomnumber; 
  float x; 
  float y; 
  float z;
}; 


/* class CG94File */
class CG94File : public QObject
{
  Q_OBJECT

public: 
  /* constructor */
  CG94File(QObject *parent=0, const char *name=0);
 
  /* destructor */
  ~CG94File(); 

  /* operators */
  CG94File& operator=(CG94File &G94File); 
 
  /* methods */
  void Serialize();
  int LoadFile(QString Path);

  /* Get ... */
  int GetNumberOfCalculations(void);
  int GetNumberOfAtoms(int CalNumber);
  int GetNumberOfNormalModes(int CalNumber);
  QString GetTypeOfCalculation(int CalNumber);
  QString GetTitle(int CalNumber);
  QString GetPathName(void);
  int GetMultiplicity(int CalNumber);
  int GetCharge(int CalNumber);
  double GetEnergy(int CalNumber, int Method = 0);
  double GetRotConstA(int CalNumber);
  double GetRotConstB(int CalNumber); 
  double GetRotConstC(int CalNumber); 
  double GetMolecularMass(int CalNumber);
  int GetRotationalSymmetryNumber(int CalNumber);
  int GetCoordinate(int CalNumber, int LineNumber, CG94Atomcoordinate& Coordinate);  
  int GetNormalMode(int CalNumber, int ModeNumber, CG94NormalModes& NormalMode);
  int GetNormalModeMovement(int CalNumber, int ModeNumber, int LineNumber, CG94Atomcoordinate& AtomMovement);

protected: 
  /* lists */
  QList <CG94Results> G94ResultList;
  QList <CG94Atomcoordinate> G94AtomcoordinateList;
  QList <CG94NormalModes> G94NormalModesList;
  QList <CG94Atomcoordinate> AtomMovementList;
 
  /* members */
  int InstanceIsEmpty;
  CG94Atomcoordinate* G94Atomcoordinate;
  CG94NormalModes* G94NormalModes; 
  CG94Results* G94Results;
  CG94Atomcoordinate* AtomMovement;
  int NumberOfSuccessfullCalculations;
};

#endif /* _G94FILE */
