/***************************************************************************
                          initfile.cpp  -  description
                             -------------------
    begin                : Sat Jan 22 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files
#include <initfile.h>

/////////////////////////////////////////////////////////////////////////
// class CSection

// Constructor/Destructor
CSection::CSection(const QString &sectionname, QObject *parent, const char *name)
  : QObject(parent, name)
{
  sectionName = sectionname;
  parameterList.setAutoDelete(TRUE);
}

CSection::~CSection()
{

}

int CSection::ContainsParameter(const QString &parameter)
{
  CParameter *tmpParameter = 0;
  for(tmpParameter = parameterList.first(); tmpParameter != 0; tmpParameter = parameterList.next())
    if(tmpParameter->GetParameterName() == parameter) return(parameterList.at());
  return(-1);
}

int CSection::AddParameter(const QString &parameter, const QString &parametervalue)
{
  int pos = -1;
  if((pos = ContainsParameter(parameter)) == -1)
  {
    parameterList.append(new CParameter(parameter));
    (parameterList.current())->SetParameterValue(parametervalue);
    return(parameterList.at());
  }
  else
  {
    (parameterList.at(pos))->SetParameterValue(parametervalue);
    return(pos);
  }
}

int CSection::DeleteParameter(const QString &parameter)
{
  int pos = -1;
  return((pos = ContainsParameter(parameter)) != -1 && parameterList.remove(pos) ? 1 : 0);
}


QString CSection::GetParameterValue(const QString &parameter)
{
  int pos = -1;
  if((pos = ContainsParameter(parameter)) != -1)
    return((parameterList.at(pos))->GetParameterValue());
  return("");
}

QString CSection::GetParameterValue(int pos)
{
  if(pos >= 0 && (int)parameterList.count() > pos)
    return(parameterList.at(pos)->GetParameterValue());
  return("");
}

QString CSection::GetParameterName(int pos)
{
  if(pos >= 0 && (int)parameterList.count() > pos)
    return(parameterList.at(pos)->GetParameterName());
  return("");
}

/////////////////////////////////////////////////////////////////////////
// class CParameter

// Constructor/Destructor
CParameter::CParameter(const QString &parametername, QObject *parent, const char *name)
  : QObject(parent, name)
{
  parameterName = parametername;
}

CParameter::~CParameter()
{

}

/////////////////////////////////////////////////////////////////////////
// class CStaticInitFile

QList<CSection> CStaticInitFile::sectionList;
int CStaticInitFile::isInitFileInUse = FALSE;
int CStaticInitFile::isOpen = FALSE;
QString CStaticInitFile::fileName = "";
QFile CStaticInitFile::initFile;
QString CStaticInitFile::result = "";
QString CStaticInitFile::sectionname = "";
QString CStaticInitFile::line = "";
QString CStaticInitFile::parametername = "";
QString CStaticInitFile::parametervalue = "";

// Constructor/Destructor
CStaticInitFile::CStaticInitFile(QObject *parent, const char *name)
  : QObject(parent, name)
{
}

CStaticInitFile::~CStaticInitFile()
{
}

// Methods
int CStaticInitFile::ContainsSection(const QString &section)
{
   CSection *tmpSection = 0;
   for(tmpSection = CStaticInitFile::sectionList.first(); tmpSection != 0; tmpSection = CStaticInitFile::sectionList.next())
     if(tmpSection->GetSectionName() == section) return(CStaticInitFile::sectionList.at());
   return(-1);
}

int CStaticInitFile::ContainsParameter(const QString &section, const QString &parameter)
{
   CSection *tmpSection = 0;
   for(tmpSection = CStaticInitFile::sectionList.first(); tmpSection != 0; tmpSection = CStaticInitFile::sectionList.next())
     if(tmpSection->GetSectionName() == section) break;
   if(tmpSection != 0)
     return((CStaticInitFile::sectionList.current())->ContainsParameter(parameter));
   return(-1);
}

int CStaticInitFile::AddSection(const QString &section)
{
   if(CStaticInitFile::ContainsSection(section) == -1)
   {
      CStaticInitFile::sectionList.append(new CSection(section));
      return(CStaticInitFile::sectionList.at());
   }
   return(-1);
}

int CStaticInitFile::DeleteSection(const QString &section)
{
  int pos = -1;
  return((pos = CStaticInitFile::ContainsSection(section)) != -1  && CStaticInitFile::sectionList.remove(pos) ? 1 : 0);
}

int CStaticInitFile::AddParameter(const QString &section, const QString &parameter, const QString &parametervalue)
{
  int pos = -1;
  CSection *tmpSection = 0;
  if(((pos = CStaticInitFile::ContainsSection(section)) != -1 || (pos = CStaticInitFile::AddSection(section)) != -1) &&
     (tmpSection = CStaticInitFile::sectionList.at(pos)) != 0)
    return(tmpSection->AddParameter(parameter, parametervalue));
  return(-1);
}

int CStaticInitFile::DeleteParameter(const QString &section, const QString &parameter)
{
  int pos = -1;
  CSection *tmpSection = 0;
  if((pos = CStaticInitFile::ContainsSection(section)) != -1 &&
     (tmpSection = CStaticInitFile::sectionList.at(pos)) != 0)
    return(tmpSection->DeleteParameter(parameter));
  return(-1);
}

QString CStaticInitFile::GetParameterValue(const QString &section, const QString &parameter)
{
  int pos = -1;
  CSection *tmpSection = 0;
  if((pos = CStaticInitFile::ContainsSection(section)) != -1 &&
     (tmpSection = CStaticInitFile::sectionList.at(pos)) != 0)
    return(tmpSection->GetParameterValue(parameter));
 return("");
}

QString CStaticInitFile::GetParameterValue(const QString &section, int position)
{
  int pos = -1;
  CSection *tmpSection = 0;
  if((pos = CStaticInitFile::ContainsSection(section)) != -1 &&
     (tmpSection = CStaticInitFile::sectionList.at(pos)) != 0)
    return(tmpSection->GetParameterValue(position));
 return("");
}

QString CStaticInitFile::GetParameterName(const QString &section, int position)
{
  int pos = -1;
  CSection *tmpSection = 0;
  if((pos = CStaticInitFile::ContainsSection(section)) != -1 &&
     (tmpSection = CStaticInitFile::sectionList.at(pos)) != 0)
    return(tmpSection->GetParameterName(position));
  return("");
}

QString CStaticInitFile::GetFileName(void)
{
  return(CStaticInitFile::fileName);
}

int CStaticInitFile::OpenInitFile(QString filename)
{
  int isErrorOccured = TRUE;
  if(!CStaticInitFile::isOpen && !CStaticInitFile::isInitFileInUse)
  {
    CStaticInitFile::fileName = filename;
    CStaticInitFile::isInitFileInUse = TRUE;
    CStaticInitFile::initFile.setName(CStaticInitFile::fileName);
    if(CStaticInitFile::initFile.open(IO_ReadWrite) && CStaticInitFile::initFile.exists())
    {
      isErrorOccured = FALSE;
      CStaticInitFile::isOpen = TRUE;
      CStaticInitFile::sectionList.setAutoDelete(TRUE);
      CStaticInitFile::sectionList.clear();
      // initialisation
      CStaticInitFile::sectionname = "";
      CStaticInitFile::line = "";
      CStaticInitFile::parametername = "";
      CStaticInitFile::parametervalue = "";
      // delete sections and reload initfile
      while (!CStaticInitFile::initFile.atEnd())
      {
        if (CStaticInitFile::initFile.readLine(CStaticInitFile::line, MAX_LINE_LENGTH) == -1)
        {
          isErrorOccured = TRUE;
          break;
        }
        // search for sections and add new ones
        if(CStaticInitFile::line.find("[") == 0 && CStaticInitFile::line.find("]") > 0)
        {
          CStaticInitFile::sectionname = CStaticInitFile::line.mid(1, CStaticInitFile::line.find("]") - 1);
          if(CStaticInitFile::AddSection(CStaticInitFile::sectionname) == -1)
          {
            isErrorOccured = TRUE;
            break;
          }
          continue;
        }
        if(CStaticInitFile::line.find("=") == -1)
          continue;
        // format found parameters
        CStaticInitFile::parametername = CStaticInitFile::line.left(CStaticInitFile::line.find("="));
        CStaticInitFile::parametervalue = CStaticInitFile::line.right(CStaticInitFile::line.length() - CStaticInitFile::line.find("=") - 1);
        CStaticInitFile::parametervalue.stripWhiteSpace();
        if(CStaticInitFile::parametervalue.right(2) == "\r\n")
          CStaticInitFile::parametervalue.truncate(CStaticInitFile::parametervalue.length() - 2);
        if(CStaticInitFile::parametervalue.right(1) == "\n")
          CStaticInitFile::parametervalue.truncate(CStaticInitFile::parametervalue.length() - 1);
        if(CStaticInitFile::parametervalue.right(1) == "\r")
          CStaticInitFile::parametervalue.truncate(CStaticInitFile::parametervalue.length() - 1);
        // add parameters
        if(CStaticInitFile::AddParameter(CStaticInitFile::sectionname, CStaticInitFile::parametername, CStaticInitFile::parametervalue) == -1)
        {
          isErrorOccured = TRUE;
          break;
        }
      }
    }
    CStaticInitFile::isInitFileInUse = FALSE;
  }
  return(!isErrorOccured);
}

int CStaticInitFile::CloseInitFile(void)
{
  int isErrorOccured = FALSE;
  QString newLine = "";
  CSection *tmpSection = 0;
  if(CStaticInitFile::isOpen && !CStaticInitFile::isInitFileInUse)
  {
    CStaticInitFile::isInitFileInUse = TRUE;
    // rewrite initfile
    CStaticInitFile::initFile.close();
    CStaticInitFile::initFile.remove(CStaticInitFile::fileName);
    CStaticInitFile::initFile.setName(CStaticInitFile::fileName);
    if(CStaticInitFile::initFile.open(IO_ReadWrite) && CStaticInitFile::initFile.exists())
    {
      for(tmpSection = CStaticInitFile::sectionList.first(); tmpSection != 0 && !isErrorOccured; tmpSection = CStaticInitFile::sectionList.next())
      {
        newLine = "[";
        newLine += tmpSection->GetSectionName();
        newLine += "]\r\n";
        if(initFile.writeBlock(newLine, newLine.length()) == -1)
        {
          isErrorOccured = TRUE;
          break;
        }
        for(int parameterItem = 0; parameterItem < tmpSection->GetNumberOfParameters() && !isErrorOccured; parameterItem++)
        {
          newLine = tmpSection->GetParameterName(parameterItem);
          newLine += "=";
          newLine += tmpSection->GetParameterValue(parameterItem);
          newLine += "\r\n";
          if(initFile.writeBlock(newLine, newLine.length()) == -1)
          {
            isErrorOccured = TRUE;
            break;
          }
        }
        newLine = "\r\n";
        if(initFile.writeBlock(newLine, newLine.length()) == -1)
        {
          isErrorOccured = TRUE;
          break;
        }
      }
      // close init file
      CStaticInitFile::initFile.close();
    }
    else
    {
      // opening init file fails
      isErrorOccured = TRUE;
    }
    CStaticInitFile::isInitFileInUse = FALSE;
  }
  return(!isErrorOccured);
}

QString CStaticInitFile::GetProfile(const QString &section, const QString &param, const QString &defaultvalue)
{
  // wait until file is not in use
  while(CStaticInitFile::isInitFileInUse){};
  // initialisations
  CStaticInitFile::isInitFileInUse = TRUE;
  if((CStaticInitFile::result = CStaticInitFile::GetParameterValue(section, param)) == "")
  {
    // section/parameter not found						
    CStaticInitFile::result = defaultvalue;
  }
  // get profile from section and parameter list
  CStaticInitFile::isInitFileInUse = FALSE;
  return(CStaticInitFile::result);
}

QString CStaticInitFile::GetProfile(const QString &section, int position, const QString &defaultvalue)
{
  // wait until file is not in use
  while(CStaticInitFile::isInitFileInUse){};
  // initialisations
  CStaticInitFile::isInitFileInUse = TRUE;
  if((CStaticInitFile::result = CStaticInitFile::GetParameterValue(section, position)) == "")
  {
    // section/parameter not found
    CStaticInitFile::result = defaultvalue;
  }
  // get profile from section and parameter list
  CStaticInitFile::isInitFileInUse = FALSE;
  return(CStaticInitFile::result);
}

int CStaticInitFile::WriteProfile(const QString &section, const QString &param, const QString &value)
{
  // wait until file is not in use
  while(CStaticInitFile::isInitFileInUse){};
  CStaticInitFile::isInitFileInUse = TRUE;
  int isErrorOccured = FALSE;
  // check if init file changed
  if(CStaticInitFile::GetParameterValue(section, param) != value && CStaticInitFile::AddParameter(section, param, value) == -1)
  {
    // setting new parameter fails
    isErrorOccured = TRUE;
  }
  CStaticInitFile::isInitFileInUse = FALSE;
  return(!isErrorOccured);
}

int CStaticInitFile::RemoveProfile(const QString &section, const QString &param)
{
  while(CStaticInitFile::isInitFileInUse){};
  CStaticInitFile::isInitFileInUse = TRUE;
  int isErrorOccured = FALSE;
  // delete parameter
  if(CStaticInitFile::DeleteParameter(section, param) == -1)
  {
    // deleting parameter fails
    isErrorOccured = TRUE;
  }
  CStaticInitFile::isInitFileInUse = FALSE;
  return(!isErrorOccured);
}

int CStaticInitFile::RemoveProfile(const QString &section)
{
  while(CStaticInitFile::isInitFileInUse){};
  CStaticInitFile::isInitFileInUse = TRUE;
  int isErrorOccured = FALSE;
  // delete section
  if(CStaticInitFile::DeleteSection(section) == -1)
  {
    // deleting parameter fails
    isErrorOccured = TRUE;
  }
  CStaticInitFile::isInitFileInUse = FALSE;
  return(!isErrorOccured);
}

int CStaticInitFile::GetNumberOfSections(void)
{
  return(CStaticInitFile::sectionList.count());
}

int CStaticInitFile::GetNumberOfParameters(const QString &section)
{
  int pos = -1;
  return((pos = CStaticInitFile::ContainsSection(section)) != -1  ? (CStaticInitFile::sectionList.at(pos))->GetNumberOfParameters() : 0);
}

/////////////////////////////////////////////////////////////////////////
// class CInitFile

// Constructor/Destructor
CInitFile::CInitFile(QObject *parent, const char *name)
  : QObject(parent, name)
{
}

CInitFile::~CInitFile()
{
}

// Methods:

// OpenInitFile
int CInitFile::OpenInitFile(const QString &filename)
{
  return(CStaticInitFile::OpenInitFile(filename));
}

// CloseInitFile
int CInitFile::CloseInitFile(void)
{
  return(CStaticInitFile::CloseInitFile());
}

// GetFileName
QString CInitFile::GetFileName(void)
{
  return(CStaticInitFile::GetFileName());
}

// GetProfile
QString CInitFile::GetProfile(const QString &section, const QString &param, const QString &defaultvalue)
{
  return(CStaticInitFile::GetProfile(section, param, defaultvalue));
}

// GetProfile
QString CInitFile::GetProfile(const QString &section, int position, const QString &defaultvalue)
{
  return(CStaticInitFile::GetProfile(section, position, defaultvalue));
}

// GetParameterName
QString CInitFile::GetParameterName(const QString &section, int position)
{
  return(CStaticInitFile::GetParameterName(section, position));
}

// WriteProfile
int CInitFile::WriteProfile(const QString &section, const QString &param, const QString &value)
{
  return(CStaticInitFile::WriteProfile(section, param, value));
}

// RemoveProfile
int CInitFile::RemoveProfile(const QString &section, const QString &param)
{
  return(CStaticInitFile::RemoveProfile(section, param));
}

// RemoveProfile
int CInitFile::RemoveProfile(const QString &section)
{
  return(CStaticInitFile::RemoveProfile(section));
}

// GetNumberOfSections
int CInitFile::GetNumberOfSections(void)
{
  return(CStaticInitFile::GetNumberOfSections());
}

// GetNumberOfParameters
int CInitFile::GetNumberOfParameters(const QString &section)
{
  return(CStaticInitFile::GetNumberOfParameters(section));
}

// throwErrorMsg
void CInitFile::throwErrorMsg(void)
{
  QMessageBox::critical( 0, "Error!", QString("An error occured while reading the initialisation file! Maybe the following file\n"
                                             + GetFileName()
                                             + "\ncontains invalid settings, is corrupt or you don't have read and write permissions!"
                                             + "\nA working file will be created, if you remove this file and start again. If you do so"
                                             + "\nall your individual settigs are lost!"));
}
