/***************************************************************************
              	            initfile.h  -  description
                             -------------------
    begin                : Sat Jan 22 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _INITFILE_CONFIG
#define _INITFILE_CONFIG

/* include files for QT */
#include <qobject.h>
#include <qstring.h>
#include <qfile.h>
#include <qdir.h>
#include <qlist.h>
#include <qmessagebox.h>

/* application specific includes */
#include <iostream>
#include <cstdio>
#include <cstdlib>

/* settings */
#define MAX_LINE_LENGTH 2048

/* class CParameter */
class CParameter : public QObject
{
  Q_OBJECT

public:
  CParameter(const QString &parametername, QObject *parent=0, const char *name=0);
  ~CParameter();
  QString GetParameterName(){return(parameterName);};
  QString GetParameterValue(){return(parameterValue);};
  void SetParameterName(const QString &parametername){parameterName = parametername;};
  void SetParameterValue(const QString &parametervalue){parameterValue = parametervalue;};

protected:
  QString parameterName;
  QString parameterValue;
};

/* class CSection */
class CSection : public QObject
{
  Q_OBJECT

public:
  CSection(const QString &sectionname, QObject *parent=0, const char *name=0);
  ~CSection();
  QString GetSectionName(){return(sectionName);};
  void SetSectionName(const QString &sectionname){sectionName = sectionname;};
  int ContainsParameter(const QString &parameter);
  int AddParameter(const QString &parameter, const QString &parametervalue = "");
  int DeleteParameter(const QString &parameter);
  QString GetParameterValue(const QString &parameter);
  QString GetParameterValue(int pos);
  QString GetParameterName(int pos);
  int GetNumberOfParameters(void){return(parameterList.count());};

protected:
  QList<CParameter> parameterList;
  QString sectionName;
};

/* class CStaticInitFile */
class CStaticInitFile : public QObject
{
  Q_OBJECT

public:
  CStaticInitFile(QObject *parent=0, const char *name=0);
  ~CStaticInitFile();
  static int OpenInitFile(QString filename);
  static int CloseInitFile(void);
  static QString GetFileName(void);
  static QString GetProfile(const QString &section, const QString &param, const QString &defaultvalue);
  static QString GetProfile(const QString &section, int position, const QString &defaultvalue);
  static QString GetParameterName(const QString &section, int position);
  static int WriteProfile(const QString &section, const QString &param, const QString &value);
  static int RemoveProfile(const QString &section, const QString &param);
  static int RemoveProfile(const QString &section);
  static int GetNumberOfSections(void);
  static int GetNumberOfParameters(const QString &section);
  
protected:
  static int ContainsSection(const QString &section);
  static int ContainsParameter(const QString &section, const QString &parameter);
  static int AddSection(const QString &section);
  static int AddParameter(const QString &section, const QString &parameter, const QString &parametervalue = "");
  static int DeleteSection(const QString &section);
  static int DeleteParameter(const QString &section, const QString &parameter);
  static QString GetParameterValue(const QString &section, const QString &parameter);
  static QString GetParameterValue(const QString &section, int position);

protected:
  static QList<CSection> sectionList;
  static int isOpen;
  static int isInitFileInUse;
  static QFile initFile;
  static QString fileName;
  static QString result;
  static QString sectionname;
  static QString line;
  static QString parametername;
  static QString parametervalue;
  CSection *section;
};

/* class CInitFile */
class CInitFile : public QObject
{
  Q_OBJECT

public:
  CInitFile(QObject *parent=0, const char *name=0);
  ~CInitFile();
  int OpenInitFile(const QString &file);
  int CloseInitFile(void);
  QString GetFileName(void);
  QString GetProfile(const QString &section, const QString &param, const QString &defaultvalue);
  QString GetProfile(const QString &section, int position, const QString &defaultvalue);
  QString GetParameterName(const QString &section, int position);
  int WriteProfile(const QString &section, const QString &param, const QString &value);
  int RemoveProfile(const QString &section, const QString &param);
  int RemoveProfile(const QString &section);
  int GetNumberOfSections(void);
  int GetNumberOfParameters(const QString &section);
  void throwErrorMsg(void);

protected:
  static CStaticInitFile staticInitFile;
};

#endif /* _INITFILE_CONFIG */
