/***************************************************************************
                          iplugin.cpp  -  description
                             -------------------
    begin                : Fre Jul 25 2003
    copyright            : (C) 2003 by Mario Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *  THIS INTERFACE PROGRAMFILE IS FREE SOFTWARE. YOU CAN REDISTRIBUTE IT   *
 *  AND/OR MODIFY IT. BECAUSE THIS FILE IS LICENSED FREE OF CHARGE, THERE  *
 *  IS NO WARRANTY FOR THIS PROGRAMFILE. EXCEPT WHEN OTHERWISE STATED IN   *
 *  WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM *
 *  "AS IS" WITHOUT WARRANTY OF ANY KIND. THE ENTIRE RISK AS TO THE        *
 *  QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM *
 *  PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,       *
 *  REPAIR OR CORRECTION.                                                  *
 *                                                                         *
 ***************************************************************************/

#include "iplugin.h"
using namespace kmpi;

/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPlugin
/////////////////////////////////////////////////////////////////////////////
CIPlugin::CIPlugin()
{
  elements = new(CIPluginElements);
  molecule = new(CIPluginMolecule);
}

CIPlugin::~CIPlugin()
{
  if(elements) delete elements;
  if(molecule) delete molecule;
}

/////////////////////////////////////////////////////////////////////////////
// CIPlugin events

/////////////////////////////////////////////////////////////////////////////
// CIPlugin set...
KMPI_RESULT CIPlugin::setMolecule(const CIPluginMolecule& mol)
{
  if(molecule)
  {
    *molecule = mol;
    return(KMPI_SUCCESS);
  }
  return(KMPI_ERROR);
}

KMPI_RESULT CIPlugin::setElements(const CIPluginElements& el)
{
  if(elements)
  {
    *elements = el;
    return(KMPI_SUCCESS);
  }
  return(KMPI_ERROR);
}

/////////////////////////////////////////////////////////////////////////////
// CIPlugin get...


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginFile
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CIPluginFile events

/////////////////////////////////////////////////////////////////////////////
// CIPluginFile set...

/////////////////////////////////////////////////////////////////////////////
// CIPluginFile get...


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginFileImport
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileImport events
KMPI_RESULT CIPluginFileImport::onOpenFile(const string&, string& errorMsg)
{
  errorMsg = (string)"";
  return(KMPI_SUCCESS);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileImport set...

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileImport get...

/////////////////////////////////////////////////////////////////////////////
// default import button XPM
const char* CIPluginFileImport::defaultImportXpm[] =
{
  "16 16 33 1",
  " 	c None",
  ".	c #AFAFCB",
  "+	c #A4A4B5",
  "@	c #DCDCDC",
  "#	c #A8A8D5",
  "$	c #BBBBE3",
  "%	c #555588",
  "&	c #8D8DD1",
  "*	c #7777A4",
  "=	c #C9C9CF",
  "-	c #6F6FAE",
  ";	c #AAAAFF",
  ">	c #4444FF",
  ",	c #C6C6FF",
  "'	c #595980",
  ")	c #9191D0",
  "!	c #9999FF",
  "~	c #646475",
  "{	c #6B6B8D",
  "]	c #7777CC",
  "^	c #4F4F93",
  "/	c #8484A6",
  "(	c #393971",
  "_	c #7777DD",
  ":	c #606077",
  "<	c #7D7DE3",
  "[	c #2D2DCC",
  "}	c #5555AA",
  "|	c #9191A8",
  "1	c #ABABAB",
  "2	c #444488",
  "3	c #000000",
  "4	c #FFFF00",
  "                ",
  "      .+        ",
  "    @#$%&*@     ",
  "   @=-;>,,'=    ",
  " &&&)%;>!!%~{{{ ",
  " ]]]^^;>!!^^]]] ",
  " +++/(;>__(/+++ ",
  "    @:<[}}|     ",
  "     12(++      ",
  "                ",
  "      3333      ",
  "      3443      ",
  "      3443      ",
  "     344443     ",
  "      3443      ",
  "       33       "
};


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginFileImportDialog
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileImportDialog events

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileImportDialog set...

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileImportDialog get...


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginFileExport
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileExport events
KMPI_RESULT CIPluginFileExport::onOpenFile(const string&, string& errorMsg)
{
  errorMsg = (string)"";
  return(KMPI_SUCCESS);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileExport set...

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileExport get...

/////////////////////////////////////////////////////////////////////////////
// default export button XPM
const char* CIPluginFileExport::defaultExportXpm[] =
{
  "16 16 33 1",
  " 	c None",
  ".	c #000000",
  "+	c #FFFF00",
  "@	c #AFAFCB",
  "#	c #A4A4B5",
  "$	c #DCDCDC",
  "%	c #A8A8D5",
  "&	c #BBBBE3",
  "*	c #555588",
  "=	c #8D8DD1",
  "-	c #7777A4",
  ";	c #C9C9CF",
  ">	c #6F6FAE",
  ",	c #AAAAFF",
  "'	c #4444FF",
  ")	c #C6C6FF",
  "!	c #595980",
  "~	c #9191D0",
  "{	c #9999FF",
  "]	c #646475",
  "^	c #6B6B8D",
  "/	c #7777CC",
  "(	c #4F4F93",
  "_	c #8484A6",
  ":	c #393971",
  "<	c #7777DD",
  "[	c #606077",
  "}	c #7D7DE3",
  "|	c #2D2DCC",
  "1	c #5555AA",
  "2	c #9191A8",
  "3	c #ABABAB",
  "4	c #444488",
  "       ..       ",
  "      .++.      ",
  "     .++++.     ",
  "      .++.      ",
  "      .++.      ",
  "      ....      ",
  "                ",
  "      @#        ",
  "    $%&*=-$     ",
  "   $;>,'))!;    ",
  " ===~*,'{{*]^^^ ",
  " ///((,'{{((/// ",
  " ###_:,'<<:_### ",
  "    $[}|112     ",
  "     34:##      ",
  "                "
};


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginFileExportDialog
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileExportDialog events

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileExportDialog set...

/////////////////////////////////////////////////////////////////////////////
// CIPluginFileExportDialog get...


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginColor
/////////////////////////////////////////////////////////////////////////////
CIPluginColor::CIPluginColor(unsigned int red, unsigned int green, unsigned int blue)
{
  colorRed = (red < 256 ? red : 255);
  colorGreen = (green < 256 ? green : 255);
  colorBlue = (blue < 256 ? blue : 255);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginColor operators...
CIPluginColor& CIPluginColor::operator=(const CIPluginColor& color)
{
  colorRed = color.colorRed;
  colorGreen = color.colorGreen;
  colorBlue = color.colorBlue;
  return(*this);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginColor set...
KMPI_RESULT CIPluginColor::setColor(unsigned int red, unsigned int green, unsigned int blue)
{
  colorRed = (red < 256 ? red : 255);
  colorGreen = (green < 256 ? green : 255);
  colorBlue = (blue < 256 ? blue : 255);
  return((colorRed == red && colorGreen == green && colorBlue == blue) ? KMPI_SUCCESS : KMPI_ERROR);
}

KMPI_RESULT CIPluginColor::setColorRed(unsigned int red)
{
  return(((colorRed = (red < 256 ? red : 255)) == red) ? KMPI_SUCCESS : KMPI_ERROR);
}

KMPI_RESULT CIPluginColor::setColorGreen(unsigned int green)
{
  return(((colorGreen = (green < 256 ? green : 255)) == green) ? KMPI_SUCCESS : KMPI_ERROR);
}

KMPI_RESULT CIPluginColor::setColorBlue(unsigned int blue)
{
  return(((colorBlue = (blue < 256 ? blue : 255)) == blue) ? KMPI_SUCCESS : KMPI_ERROR);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginColor get...


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginPosition
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CIPluginPosition operators...
CIPluginPosition& CIPluginPosition::operator=(const CIPluginPosition& position)
{
  positionX = position.positionX;
  positionY = position.positionY;
  positionZ = position.positionZ;
  return(*this);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginPosition set...
KMPI_RESULT CIPluginPosition::setPosition(float x, float y, float z)
{
  positionX = x;
  positionY = y;
  positionZ = z;
  return(KMPI_SUCCESS);
}

KMPI_RESULT CIPluginPosition::setPositionX(float x)
{
  positionX = x;
  return(KMPI_SUCCESS);
}

KMPI_RESULT CIPluginPosition::setPositionY(float y)
{
  positionY = y;
  return(KMPI_SUCCESS);
}

KMPI_RESULT CIPluginPosition::setPositionZ(float z)
{
  positionZ = z;
  return(KMPI_SUCCESS);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginPosition get...
float CIPluginPosition::getDistance(const CIPluginPosition& position1, const CIPluginPosition& position2)
{
  return((float)sqrt(pow((float)(position1.positionX - position2.positionX), 2.0f) +
                     pow((float)(position1.positionY - position2.positionY), 2.0f) +
                     pow((float)(position1.positionZ - position2.positionZ), 2.0f)));
}


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginElements
/////////////////////////////////////////////////////////////////////////////
const string CIPluginElements::elementSymbol[] =
{
  /* 1 */
  "H", "He",
  /* 2 */
  "Li", "Be", "B", "C", "N", "O", "F", "Ne",
  /* 3 */
  "Na", "Mg", "Al", "Si", "P", "S", "Cl", "Ar",
  /* 4 */
  "K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn", "Ga", "Ge", "As", "Se", "Br", "Kr",
  /* 5 */
  "Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd", "In", "Sn", "Sb", "Te", "I", "Xe",
  /* 6 */
  "Cs", "Ba",
  "La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", "Yb", "Lu",
  "Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg", "Tl", "Pb", "Bi", "Po", "At", "Rn",
  /* 7 */
  "Fr", "Ra",
  "Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm", "Bk", "Cf", "Es", "Fm", "Md", "No", "Lr",
  "Rf", "Ha", "Unh", "Uns", "Uno", "Une"
};

CIPluginElements::CIPluginElements()
{
  //element radius
  // 1
  elementRadius[0] = 0.320f;
  elementRadius[1] = 0.930f;
  // 2
  elementRadius[2] = 1.230f;
  elementRadius[3] = 0.900f;
  elementRadius[4] = 0.820f;
  elementRadius[5] = 0.770f;
  elementRadius[6] = 0.750f;
  elementRadius[7] = 0.730f;
  elementRadius[8] = 0.720f;
  elementRadius[9] = 0.710f;
  // 3
  elementRadius[10] = 1.540f;
  elementRadius[11] = 1.360f;
  elementRadius[12] = 1.180f;
  elementRadius[13] = 1.110f;
  elementRadius[14] = 1.060f;
  elementRadius[15] = 1.020f;
  elementRadius[16] = 0.990f;
  elementRadius[17] = 0.980f;
  // 4
  elementRadius[18] = 2.030f;
  elementRadius[19] = 1.740f;
  elementRadius[20] = 1.440f;
  elementRadius[21] = 1.320f;
  elementRadius[22] = 1.220f;
  elementRadius[23] = 1.180f;
  elementRadius[24] = 1.170f;
  elementRadius[25] = 1.170f;
  elementRadius[26] = 1.160f;
  elementRadius[27] = 1.150f;
  elementRadius[28] = 1.170f;
  elementRadius[29] = 1.250f;
  elementRadius[30] = 1.260f;
  elementRadius[31] = 1.220f;
  elementRadius[32] = 1.200f;
  elementRadius[33] = 1.160f;
  elementRadius[34] = 1.140f;
  elementRadius[35] = 1.120f;
  // 5
  elementRadius[36] = 2.160f;
  elementRadius[37] = 1.910f;
  elementRadius[38] = 1.620f;
  elementRadius[39] = 1.450f;
  elementRadius[40] = 1.340f;
  elementRadius[41] = 1.300f;
  elementRadius[42] = 1.270f;
  elementRadius[43] = 1.250f;
  elementRadius[44] = 1.250f;
  elementRadius[45] = 1.280f;
  elementRadius[46] = 1.340f;
  elementRadius[47] = 1.480f;
  elementRadius[48] = 1.440f;
  elementRadius[49] = 1.410f;
  elementRadius[50] = 1.400f;
  elementRadius[51] = 1.360f;
  elementRadius[52] = 1.330f;
  elementRadius[53] = 1.310f;
  // 6
  elementRadius[54] = 2.350f;
  elementRadius[55] = 1.980f;
  elementRadius[56] = 1.690f;
  elementRadius[57] = 1.650f;
  elementRadius[58] = 1.650f;
  elementRadius[59] = 1.640f;
  elementRadius[60] = 1.630f;
  elementRadius[61] = 1.620f;
  elementRadius[62] = 1.850f;
  elementRadius[63] = 1.610f;
  elementRadius[64] = 1.590f;
  elementRadius[65] = 1.590f;
  elementRadius[66] = 1.580f;
  elementRadius[67] = 1.570f;
  elementRadius[68] = 1.560f;
  elementRadius[69] = 1.740f;
  elementRadius[70] = 1.560f;
  elementRadius[71] = 1.440f;
  elementRadius[72] = 1.340f;
  elementRadius[73] = 1.300f;
  elementRadius[74] = 1.280f;
  elementRadius[75] = 1.260f;
  elementRadius[76] = 1.270f;
  elementRadius[77] = 1.300f;
  elementRadius[78] = 1.340f;
  elementRadius[79] = 1.490f;
  elementRadius[80] = 1.480f;
  elementRadius[81] = 1.470f;
  elementRadius[82] = 1.460f;
  elementRadius[83] = 1.460f;
  elementRadius[84] = 1.450f;
  elementRadius[85] = 1.440f;
  // 7
  elementRadius[86] = 2.510f;
  elementRadius[87] = 2.140f;
  elementRadius[88] = 1.750f;
  elementRadius[89] = 1.650f;
  elementRadius[90] = 1.530f;
  elementRadius[91] = 1.420f;
  elementRadius[92] = 1.410f;
  elementRadius[93] = 1.410f;
  elementRadius[94] = 1.610f;
  elementRadius[95] = 1.400f;
  elementRadius[96] = 1.400f;
  elementRadius[97] = 1.390f;
  elementRadius[98] = 1.390f;
  elementRadius[99] = 1.390f;
  elementRadius[100] = 1.380f;
  elementRadius[101] = 1.380f;
  elementRadius[102] = 1.380f;
  elementRadius[103] = 1.380f;
  elementRadius[104] = 1.380f;
  elementRadius[105] = 1.380f;
  elementRadius[106] = 1.380f;
  elementRadius[107] = 1.380f;
  elementRadius[108] = 1.380f;

  // element color
  // 1
  elementColor[0] = CIPluginColor(220, 220, 220);
  elementColor[1] = CIPluginColor(160, 160, 160);
  // 2
  elementColor[2] = CIPluginColor(160, 160, 160);
  elementColor[3] = CIPluginColor(160, 160, 160);
  elementColor[4] = CIPluginColor(160, 160, 160);
  elementColor[5] = CIPluginColor( 50,  50,  50);
  elementColor[6] = CIPluginColor(  0, 200,   0);
  elementColor[7] = CIPluginColor(  0,   0, 200);
  elementColor[8] = CIPluginColor(160, 160,   0);
  elementColor[9] = CIPluginColor(160, 160, 160);
  // 3
  elementColor[10] = CIPluginColor(160, 160, 160);
  elementColor[11] = CIPluginColor(160, 160, 160);
  elementColor[12] = CIPluginColor(160, 160, 160);
  elementColor[13] = CIPluginColor(160, 160, 160);
  elementColor[14] = CIPluginColor(200,   0,   0);
  elementColor[15] = CIPluginColor(220, 220,   0);
  elementColor[16] = CIPluginColor(  0, 170,   0);
  elementColor[17] = CIPluginColor(160, 160, 160);
  // 4
  elementColor[18] = CIPluginColor(160, 160, 160);
  elementColor[19] = CIPluginColor(160, 160, 160);
  elementColor[20] = CIPluginColor(160, 160, 160);
  elementColor[21] = CIPluginColor(160, 160, 160);
  elementColor[22] = CIPluginColor(160, 160, 160);
  elementColor[23] = CIPluginColor(160, 160, 160);
  elementColor[24] = CIPluginColor(160, 160, 160);
  elementColor[25] = CIPluginColor(160, 160, 160);
  elementColor[26] = CIPluginColor(160, 160, 160);
  elementColor[27] = CIPluginColor(160, 160, 160);
  elementColor[28] = CIPluginColor(160, 160, 160);
  elementColor[29] = CIPluginColor(160, 160, 160);
  elementColor[30] = CIPluginColor(160, 160, 160);
  elementColor[31] = CIPluginColor(160, 160, 160);
  elementColor[32] = CIPluginColor(160, 160, 160);
  elementColor[33] = CIPluginColor(170,   0,   0);
  elementColor[34] = CIPluginColor(100, 100,   0);
  elementColor[35] = CIPluginColor(160, 160, 160);
  // 5
  elementColor[36] = CIPluginColor(160, 160, 160);
  elementColor[37] = CIPluginColor(160, 160, 160);
  elementColor[38] = CIPluginColor(160, 160, 160);
  elementColor[39] = CIPluginColor(160, 160, 160);
  elementColor[40] = CIPluginColor(160, 160, 160);
  elementColor[41] = CIPluginColor(160, 160, 160);
  elementColor[42] = CIPluginColor(160, 160, 160);
  elementColor[43] = CIPluginColor(160, 160, 160);
  elementColor[44] = CIPluginColor(160, 160, 160);
  elementColor[45] = CIPluginColor(160, 160, 160);
  elementColor[46] = CIPluginColor(160, 160, 160);
  elementColor[47] = CIPluginColor(160, 160, 160);
  elementColor[48] = CIPluginColor(160, 160, 160);
  elementColor[49] = CIPluginColor(160, 160, 160);
  elementColor[50] = CIPluginColor(160, 160, 160);
  elementColor[51] = CIPluginColor(160, 160, 160);
  elementColor[52] = CIPluginColor( 90,  90, 100);
  elementColor[53] = CIPluginColor(160, 160, 160);
  // 6
  elementColor[54] = CIPluginColor(160, 160, 160);
  elementColor[55] = CIPluginColor(160, 160, 160);
  elementColor[56] = CIPluginColor(160, 160, 160);
  elementColor[57] = CIPluginColor(160, 160, 160);
  elementColor[58] = CIPluginColor(160, 160, 160);
  elementColor[59] = CIPluginColor(160, 160, 160);
  elementColor[60] = CIPluginColor(160, 160, 160);
  elementColor[61] = CIPluginColor(160, 160, 160);
  elementColor[62] = CIPluginColor(160, 160, 160);
  elementColor[63] = CIPluginColor(160, 160, 160);
  elementColor[64] = CIPluginColor(160, 160, 160);
  elementColor[65] = CIPluginColor(160, 160, 160);
  elementColor[66] = CIPluginColor(160, 160, 160);
  elementColor[67] = CIPluginColor(160, 160, 160);
  elementColor[68] = CIPluginColor(160, 160, 160);
  elementColor[69] = CIPluginColor(160, 160, 160);
  elementColor[70] = CIPluginColor(160, 160, 160);
  elementColor[71] = CIPluginColor(160, 160, 160);
  elementColor[72] = CIPluginColor(160, 160, 160);
  elementColor[73] = CIPluginColor(160, 160, 160);
  elementColor[74] = CIPluginColor(160, 160, 160);
  elementColor[75] = CIPluginColor(160, 160, 160);
  elementColor[76] = CIPluginColor(160, 160, 160);
  elementColor[77] = CIPluginColor(160, 160, 160);
  elementColor[78] = CIPluginColor(160, 160, 160);
  elementColor[79] = CIPluginColor(160, 160, 160);
  elementColor[80] = CIPluginColor(160, 160, 160);
  elementColor[81] = CIPluginColor(160, 160, 160);
  elementColor[82] = CIPluginColor(160, 160, 160);
  elementColor[83] = CIPluginColor(160, 160, 160);
  elementColor[84] = CIPluginColor(160, 160, 160);
  elementColor[85] = CIPluginColor(160, 160, 160);
  // 7
  elementColor[86] = CIPluginColor(160, 160, 160);
  elementColor[87] = CIPluginColor(160, 160, 160);
  elementColor[88] = CIPluginColor(160, 160, 160);
  elementColor[89] = CIPluginColor(160, 160, 160);
  elementColor[90] = CIPluginColor(160, 160, 160);
  elementColor[91] = CIPluginColor(160, 160, 160);
  elementColor[92] = CIPluginColor(160, 160, 160);
  elementColor[93] = CIPluginColor(160, 160, 160);
  elementColor[94] = CIPluginColor(160, 160, 160);
  elementColor[95] = CIPluginColor(160, 160, 160);
  elementColor[96] = CIPluginColor(160, 160, 160);
  elementColor[97] = CIPluginColor(160, 160, 160);
  elementColor[98] = CIPluginColor(160, 160, 160);
  elementColor[99] = CIPluginColor(160, 160, 160);
  elementColor[100] = CIPluginColor(160, 160, 160);
  elementColor[101] = CIPluginColor(160, 160, 160);
  elementColor[102] = CIPluginColor(160, 160, 160);
  elementColor[103] = CIPluginColor(160, 160, 160);
  elementColor[104] = CIPluginColor(160, 160, 160);
  elementColor[105] = CIPluginColor(160, 160, 160);
  elementColor[106] = CIPluginColor(160, 160, 160);
  elementColor[107] = CIPluginColor(160, 160, 160);
  elementColor[108] = CIPluginColor(160, 160, 160);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginElements operators...
CIPluginElements& CIPluginElements::operator=(const CIPluginElements& elements)
{
  int i = 0;
  for(i = 0; i < KMPI_MAX_ELEMENT_NUMBER; i++)
  {
    this->elementColor[i] = elements.elementColor[i];
    this->elementRadius[i] = elements.elementRadius[i];
  }
  return(*this);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginElements set...
KMPI_RESULT CIPluginElements::setRadius(unsigned int number, float radius)
{
  if(number > (unsigned int)0 && number <= (unsigned int)KMPI_MAX_ELEMENT_NUMBER && radius >= 0.0f)
  {
    elementRadius[number - 1] = radius;
    return(KMPI_SUCCESS);
  }
  return(KMPI_ERROR);
}

KMPI_RESULT CIPluginElements::setRadius(const string& symbol, float radius)
{
  return(setRadius(CIPluginElements::getNumber(symbol), radius));
}

KMPI_RESULT CIPluginElements::setColor(unsigned int number, const CIPluginColor& color)
{
  if(number > (unsigned int)0 && number <= (unsigned int)KMPI_MAX_ELEMENT_NUMBER)
  {
    elementColor[number - 1] = color;
    return(KMPI_SUCCESS);
  }
  return(KMPI_ERROR);
}

KMPI_RESULT CIPluginElements::setColor(const string& symbol, const CIPluginColor& color)
{
  return(setColor(CIPluginElements::getNumber(symbol), color));
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginElements get...
float CIPluginElements::getRadius(unsigned int number)
{
  if(number > (unsigned int)0 && number <= (unsigned int)KMPI_MAX_ELEMENT_NUMBER)
    return(elementRadius[number - 1]);
  return(0.0f);
}

float CIPluginElements::getRadius(const string& symbol)
{
  return(getRadius(CIPluginElements::getNumber(symbol)));
}

CIPluginColor CIPluginElements::getColor(unsigned int number)
{
  if(number > 0 && number <= (unsigned int)KMPI_MAX_ELEMENT_NUMBER)
    return(elementColor[number - 1]);
  return(CIPluginColor(0, 0, 0));
}

CIPluginColor CIPluginElements::getColor(const string& symbol)
{
  return(getColor(CIPluginElements::getNumber(symbol)));
}

string CIPluginElements::getSymbol(unsigned int number)
{
  if(number > (unsigned int)0 && number <= (unsigned int)KMPI_MAX_ELEMENT_NUMBER)
    return(CIPluginElements::elementSymbol[number - 1]);
  return((string)"");
}

unsigned int CIPluginElements::getNumber(const string& symbol)
{
  for(unsigned int i = 0; i < (unsigned int)KMPI_MAX_ELEMENT_NUMBER; i++)
    if(symbol == CIPluginElements::elementSymbol[i]) return(i + 1);
  return(0);
}


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginAtom
/////////////////////////////////////////////////////////////////////////////

CIPluginAtom::CIPluginAtom(void)
{
  atomPosition.setPosition(0.0f, 0.0f, 0.0f);
  atomNumber = 0;
  atomSymbol = string("");
  atomRadius = (float)KMPI_DEFAULT_ATOM_RADIUS;
  atomColor.setColor((unsigned int)KMPI_DEFAULT_ATOM_COLOR_RED, (unsigned int)KMPI_DEFAULT_ATOM_COLOR_GREEN, (unsigned int)KMPI_DEFAULT_ATOM_COLOR_BLUE);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginAtom operators...
CIPluginAtom& CIPluginAtom::operator=(const CIPluginAtom& atom)
{
  this->atomPosition = atom.atomPosition;
  this->atomNumber = atom.atomNumber;
  this->atomSymbol = atom.atomSymbol;
  this->atomRadius = atom.atomRadius;
  this->atomColor = atom.atomColor;
  return(*this);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginAtom set...
KMPI_RESULT CIPluginAtom::setByNumber(const CIPluginElements* pElements, unsigned int number)
{
  if(pElements && number > (unsigned int)0 && number <= (unsigned int)KMPI_MAX_ELEMENT_NUMBER)
  {
    atomNumber = number;
    atomSymbol = CIPluginElements::getSymbol(number);
    atomRadius = ((CIPluginElements*)pElements)->getRadius(number);
    atomColor = ((CIPluginElements*)pElements)->getColor(number);
    return(KMPI_SUCCESS);
  }
  return(KMPI_ERROR);
}

KMPI_RESULT CIPluginAtom::setBySymbol(const CIPluginElements* pElements, const string& symbol)
{
  return(setByNumber(pElements, CIPluginElements::getNumber(symbol)));
}

KMPI_RESULT CIPluginAtom::setRadius(float radius)
{
  if(radius >= 0.0f)
  {
    atomRadius = radius;
    return(KMPI_SUCCESS);
  }
  return(KMPI_ERROR);
}

KMPI_RESULT CIPluginAtom::setPosition(const CIPluginPosition& position)
{
  atomPosition = position;
  return(KMPI_SUCCESS);
}

KMPI_RESULT CIPluginAtom::setPosition(float x, float y, float z)
{
  atomPosition.setPosition(x, y, z);
  return(KMPI_SUCCESS);
}

KMPI_RESULT CIPluginAtom::setColor(const CIPluginColor& color)
{
  atomColor = color;
  return(KMPI_SUCCESS);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginAtom get...


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginBond
/////////////////////////////////////////////////////////////////////////////
CIPluginBond::CIPluginBond(void)
{
  memset((void*)&bondPartner, 0, sizeof(bondPartner));
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginBond operators...
CIPluginBond& CIPluginBond::operator=(const CIPluginBond& bond)
{
  memcpy((void*)this->bondPartner, (void*)&bond.bondPartner, sizeof(unsigned int[2]));
  return(*this);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginBond set...
KMPI_RESULT CIPluginBond::setBondPartner(unsigned int index1, unsigned int index2)
{
  if(index1 != index2 && index1 > 0 & index2 > 0)
  {
    bondPartner[0] = index1;
    bondPartner[1] = index2;
    return(KMPI_SUCCESS);
  }
  return(KMPI_ERROR);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginBond get...


/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginMolecule
/////////////////////////////////////////////////////////////////////////////

CIPluginMolecule::CIPluginMolecule(void)
{
  moleculeTitle = (string)"";
  atoms.resize(0);
  bonds.resize(0);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginMolecule operators...
CIPluginMolecule& CIPluginMolecule::operator=(const CIPluginMolecule& molecule)
{
  this->moleculeTitle = molecule.moleculeTitle;
  this->atoms = molecule.atoms;
  this->bonds = molecule.bonds;
  this->bondColor = molecule.bondColor;
  this->bondRadius = molecule.bondRadius;
  return(*this);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginMolecule add...
unsigned int CIPluginMolecule::addAtom(const CIPluginAtom& atom)
{
  try
  {
    atoms.push_back(atom);
    return((unsigned int)atoms.size());
  }
  catch(...)
  {
    return((unsigned int)0);
  }
}

unsigned int CIPluginMolecule::addAtom(const CIPluginElements* pElements, unsigned int number)
{
  CIPluginAtom a;  
  return(a.setByNumber(pElements, number) == KMPI_SUCCESS ? addAtom(a) : (unsigned int)0);
}

unsigned int CIPluginMolecule::addAtom(const CIPluginElements* pElements, const string& symbol)
{
  CIPluginAtom a;
  return(a.setBySymbol(pElements, symbol) == KMPI_SUCCESS ? addAtom(a) : (unsigned int)0);
}

unsigned int CIPluginMolecule::addBond(const CIPluginBond& bond)
{
  try
  {
    CIPluginBond b = bond;
    if(!CIPluginMolecule::isConnected(b.getBondPartner1(), b.getBondPartner2()))
    {
      bonds.push_back(bond);
      return((unsigned int)bonds.size());
    }
    return((unsigned int)0);
  }
  catch(...)
  {
    return((unsigned int)0);
  }
}

unsigned int CIPluginMolecule::addBond(unsigned int index1, unsigned int index2)
{
  CIPluginBond b;
  return(b.setBondPartner(index1, index2) == KMPI_SUCCESS ? addBond(b) : (unsigned int)0);
}
    
/////////////////////////////////////////////////////////////////////////////
// CIPluginMolecule delete...
KMPI_RESULT CIPluginMolecule::deleteAtom(unsigned int index, KMPI_BOOL deleteBonds)
{
  try
  {
    if(index <= atoms.size() && index > 0)
    {
      atoms.erase(atoms.begin() + (int)index - 1);
      if(deleteBonds)
      {
        for(bondIterator = bonds.begin(); bondIterator != bonds.end(); ++bondIterator)
        {
          CIPluginBond& bond = *bondIterator;
          if(bond.getBondPartner1() == index || bond.getBondPartner2() == index) bonds.erase(bondIterator);
        }
      }
      return(KMPI_SUCCESS);
    }
    return(KMPI_ERROR);
  }
  catch(...)
  {
    return(KMPI_ERROR);
  }
}

KMPI_RESULT CIPluginMolecule::deleteBond(unsigned int index1, unsigned int index2)
{
  try
  {
    if(index1 <= bonds.size() && index1 > 0 && index2 <= bonds.size() && index2 > 0)
    {
      for(bondIterator = bonds.begin(); bondIterator != bonds.end(); ++bondIterator)
      {
        CIPluginBond& bond = *bondIterator;
        if((bond.getBondPartner1() == index1 && bond.getBondPartner2() == index2) ||
           (bond.getBondPartner1() == index2 && bond.getBondPartner2() == index1))
        {
          bonds.erase(bondIterator);       
          return(KMPI_SUCCESS);
        }
      }
      return(KMPI_ERROR);      
    }
    return(KMPI_ERROR);
  }
  catch(...)
  {
    return(KMPI_ERROR);
  }
}

KMPI_RESULT CIPluginMolecule::deleteAll(KMPI_BOOL deleteAtoms, KMPI_BOOL deleteBonds)
{
  try
  {
    if(deleteAtoms) atoms.resize(0);
    if(deleteBonds) bonds.resize(0);
    return(KMPI_SUCCESS);
  }
  catch(...)
  {
    return(KMPI_ERROR);
  }
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginMolecule set...
KMPI_RESULT CIPluginMolecule::setAtomByNumber(unsigned int index, const CIPluginElements* pElements, unsigned int number)
{
  CIPluginAtom* pA = getAtom(index);
  return(pA ? pA->setByNumber(pElements, number) : KMPI_ERROR);
}

KMPI_RESULT CIPluginMolecule::setAtomBySymbol(unsigned int index, const CIPluginElements* pElements, const string& symbol)
{
  CIPluginAtom* pA = getAtom(index);
  return(pA ? pA->setBySymbol(pElements, symbol) : KMPI_ERROR);
}

KMPI_RESULT CIPluginMolecule::setAtomPosition(unsigned int index, const CIPluginPosition& position)
{
  CIPluginAtom* pA = getAtom(index);
  if(!pA) return(KMPI_ERROR);
  return(pA->setPosition(position));
}

KMPI_RESULT CIPluginMolecule::setAtomPosition(unsigned int index, float x, float y, float z)
{
  CIPluginAtom* pA = getAtom(index);
  if(!pA) return(KMPI_ERROR);
  return(pA->setPosition(x, y, z));
}

KMPI_RESULT CIPluginMolecule::setAtomColor(unsigned int index, const CIPluginColor& color)
{
  CIPluginAtom* pA = getAtom(index);
  if(!pA) return(KMPI_ERROR);
  return(pA->setColor(color));
}

KMPI_RESULT CIPluginMolecule::setAtomColor(unsigned int index, unsigned int red, unsigned int green, unsigned int blue)
{
  CIPluginAtom* pA = getAtom(index);
  return(pA ? pA->setColor(red, green, blue) : KMPI_ERROR);
}

KMPI_RESULT CIPluginMolecule::setAtomRadius(unsigned int index, float radius)
{
  CIPluginAtom* pA = getAtom(index);
  return(pA ? pA->setRadius(radius) : KMPI_ERROR);
}

KMPI_RESULT CIPluginMolecule::setBondColor(const CIPluginColor& color)
{
  bondColor = color;
  return(KMPI_SUCCESS);  
}

KMPI_RESULT CIPluginMolecule::setBondRadius(float radius)
{
  bondRadius = radius;
  return(KMPI_SUCCESS);
}

KMPI_RESULT CIPluginMolecule::setTitle(const string& title)
{
  moleculeTitle = title;
  return(KMPI_SUCCESS);
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginMolecule get...
CIPluginAtom* CIPluginMolecule::getAtom(unsigned int index)
{
  try
  {
    if(index <= atoms.size() && index > 0)
    {
      atomIterator = atoms.begin() + (int)index - 1;
      CIPluginAtom& atom = *atomIterator;
      return(&atom);
    }
    return((CIPluginAtom*)0);
  }
  catch(...)
  {
    return((CIPluginAtom*)0);
  }
}

unsigned int CIPluginMolecule::getAtomNumber(unsigned int index)
{
  CIPluginAtom* pA = getAtom(index);
  return(pA ? pA->getNumber() : (unsigned int)0);
}

string CIPluginMolecule::getAtomSymbol(unsigned int index)
{
  CIPluginAtom* pA = getAtom(index);
  return(pA ? pA->getSymbol() : (string)"");
}

CIPluginPosition CIPluginMolecule::getAtomPosition(unsigned int index)
{
  CIPluginAtom* pA = getAtom(index);
  CIPluginPosition p;
  return(pA ? pA->getPosition() : p);
}

CIPluginColor CIPluginMolecule::getAtomColor(unsigned int index)
{
  CIPluginAtom* pA = getAtom(index);
  CIPluginColor c;
  return(pA ? pA->getColor() : c);
}

float CIPluginMolecule::getAtomRadius(unsigned int index)
{
  CIPluginAtom* pA = getAtom(index);
  return(pA ? pA->getRadius() : 0.0f);
}

CIPluginBond* CIPluginMolecule::getBond(unsigned int index)
{
  try
  {
    if(index <= bonds.size() && index > 0)
    {
      bondIterator = bonds.begin() + (int)index - 1;
      CIPluginBond& bond = *bondIterator;
      return(&bond);
    }
    return((CIPluginBond*)0);
  }
  catch(...)
  {
    return((CIPluginBond*)0);
  }
}

/////////////////////////////////////////////////////////////////////////////
// CIPluginMolecule misc
KMPI_BOOL CIPluginMolecule::isConnected(unsigned int index1, unsigned int index2)
{
  try
  {
    if(index1 <= bonds.size() && index1 > 0 && index2 <= bonds.size() && index2)
    {
      for(bondIterator = bonds.begin(); bondIterator != bonds.end(); ++bondIterator)
      {
        CIPluginBond& bond = *bondIterator;
        if((bond.getBondPartner1() == index1 && bond.getBondPartner2() == index2) ||
           (bond.getBondPartner1() == index2 && bond.getBondPartner2() == index1))
          return(TRUE);
      }
      return(FALSE);
    }
    return(FALSE);
  }
  catch(...)
  {
    return(FALSE);
  }
}

KMPI_RESULT CIPluginMolecule::makeBonds(float bondFactor)
{
  try
  {
    if(deleteAll(FALSE, TRUE) == KMPI_ERROR) return(KMPI_ERROR);
    for(unsigned int i = 0; i < (unsigned int)atoms.size(); i++)
      for(unsigned int j = i + 1; j < (unsigned int)atoms.size(); j++)    
      {
        if((CIPluginPosition::getDistance(atoms[i].getPosition(), atoms[j].getPosition()) <= (atoms[i].getRadius() + atoms[j].getRadius()) * bondFactor) &&
           (addBond(i + 1, j + 1) == KMPI_ERROR)) return(KMPI_ERROR);
      }
    return(KMPI_SUCCESS);
  }
  catch(...)
  {
    return(KMPI_ERROR);
  }
}




