/***************************************************************************
                          iplugin.h  -  description
                             -------------------
    begin                : Fre Jul 25 2003
    copyright            : (C) 2003 by Mario Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *  THIS INTERFACE PROGRAMFILE IS FREE SOFTWARE. YOU CAN REDISTRIBUTE IT   *
 *  AND/OR MODIFY IT. BECAUSE THIS FILE IS LICENSED FREE OF CHARGE, THERE  *
 *  IS NO WARRANTY FOR THIS PROGRAMFILE. EXCEPT WHEN OTHERWISE STATED IN   *
 *  WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM *
 *  "AS IS" WITHOUT WARRANTY OF ANY KIND. THE ENTIRE RISK AS TO THE        *
 *  QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM *
 *  PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,       *
 *  REPAIR OR CORRECTION.                                                  *
 *                                                                         *
 ***************************************************************************/

#ifndef KMPI_H
#define KMPI_H

#include <cstdio>
#include <cstdlib>
#include <string>
#include <vector>
#include <cmath>

/* interface settings */
#define KMPI_INTERFACE_VERSION            "1.0.0"
#define KMPI_DEFAULT_IMPORT_TOOLTIP       "Import file via KMovisto plugin"
#define KMPI_DEFAULT_IMPORT_MENU          "Import file via KMovisto plugin"
#define KMPI_DEFAULT_IMPORT_WHATSTHIS     "Click this button to import via KMovisto plugin."
#define KMPI_DEFAULT_IMPORT_STATUSTEXT    "Importing file via KMovisto Plugin"
#define KMPI_DEFAULT_IMPORT_HIGHLIGHTTEXT "Importing file via KMovisto Plugin"

#define KMPI_DEFAULT_EXPORT_TOOLTIP       "Export file via KMovisto plugin"
#define KMPI_DEFAULT_EXPORT_MENU          "Export file via KMovisto plugin"
#define KMPI_DEFAULT_EXPORT_WHATSTHIS     "Click this button to export via KMovisto plugin."
#define KMPI_DEFAULT_EXPORT_STATUSTEXT    "Exporting file via KMovisto Plugin"
#define KMPI_DEFAULT_EXPORT_HIGHLIGHTTEXT "Exporting file via KMovisto Plugin"

/* default molecule settings */
#define KMPI_DEFAULT_ATOM_RADIUS          0.5
#define KMPI_DEFAULT_ATOM_COLOR_RED       127
#define KMPI_DEFAULT_ATOM_COLOR_GREEN     127
#define KMPI_DEFAULT_ATOM_COLOR_BLUE      127
#define KMPI_DEFAULT_BOND_RADIUS          0.1
#define KMPI_DEFAULT_BOND_COLOR_RED       127
#define KMPI_DEFAULT_BOND_COLOR_GREEN     127
#define KMPI_DEFAULT_BOND_COLOR_BLUE      127

using namespace std;

/* declare plugin class macro */
#define DECLARE_PLUGIN_CLASS(plugin) class plugin; extern "C" {plugin* createPlugin(void);} using namespace kmpi;

/* implement plugin class macro */
#define IMPLEMENT_PLUGIN_CLASS(plugin) plugin* createPlugin(void) {plugin* pluginObj = new plugin; return((plugin*) pluginObj);}

/* declaration of the namespace kmovistoplugin */
namespace kmpi
{
  /* enums */
  enum KMPI_INTERFACE_TYPE {KMPI_UNKNOWN_INTERFACE = 0,
                            KMPI_FILE_IMPORT = 1,
                            KMPI_FILE_IMPORT_DIALOG = 2,
                            KMPI_FILE_EXPORT = 3,
                            KMPI_FILE_EXPORT_DIALOG = 4};
  enum KMPI_RESULT {KMPI_SUCCESS = 0, KMPI_ERROR = 1};
  enum KMPI_BOOL {FALSE = 0, TRUE = 1};
  enum {KMPI_MAX_ELEMENT_NUMBER = 109};

  /* forward declarations */
  class CIPluginColor;
  class CIPluginPosition;
  class CIPluginElements;
  class CIPluginAtom;
  class CIPluginBond;
  class CIPluginMolecule;

  /* class CIPlugin */
  class CIPlugin
  {
  public:
    CIPlugin();
    virtual ~CIPlugin();

    /* plugin properties */
    virtual string getName(void) = 0;
    virtual string getDescription(void) = 0;
    virtual string getPluginVersion(void) = 0;
    virtual string getInterfaceVersion(void) const {return((string)KMPI_INTERFACE_VERSION);};
    virtual KMPI_INTERFACE_TYPE getInterfaceType(void) const = 0;
    virtual string getAuthor(void) = 0;
    virtual string getEmail(void) = 0;
    virtual string getHomepage(void) = 0;
    virtual string getLicense(void) = 0;

    /* events */
    virtual KMPI_RESULT onCreate(void) = 0;
    virtual KMPI_RESULT onDestroy(void) = 0;

    /* set... */
    KMPI_RESULT setMolecule(const CIPluginMolecule& mol);
    KMPI_RESULT setElements(const CIPluginElements& el);

    /* get... */
    CIPluginMolecule* getMolecule(void) {return(molecule);};
    CIPluginElements* getElements(void) {return(elements);};

  protected:
    /* protected members */
    CIPluginMolecule* molecule;
    CIPluginElements* elements;
  };

  /* class CIPluginFile */
  class CIPluginFile : public CIPlugin
  {
  public:
    CIPluginFile(KMPI_INTERFACE_TYPE interfacetype) : interfaceType((KMPI_INTERFACE_TYPE)interfacetype) {};
    virtual ~CIPluginFile() {};

    /* events */
    virtual KMPI_RESULT onCreate(void) = 0;
    virtual KMPI_RESULT onDestroy(void) = 0;
    virtual KMPI_RESULT onOpenFile(const string& file, string& errorMsg) = 0;

    /* set... */
    
    /* get... */
    virtual KMPI_INTERFACE_TYPE getInterfaceType(void) const = 0;
    virtual const char** getButtonXPM(void) = 0;
    virtual string getTooltipText(void) = 0;
    virtual string getMenuText(void) = 0;
    virtual string getWhatsThisText(void) = 0;
    virtual string getStatusText(void) = 0;
    virtual string getHighlightText(void) = 0;
    
  protected:
    /* protected members */
    const KMPI_INTERFACE_TYPE interfaceType;
  };

  /* class CIPluginFileImport */
  class CIPluginFileImport : public CIPluginFile
  {
  public:
    CIPluginFileImport(KMPI_INTERFACE_TYPE interfacetype = (KMPI_INTERFACE_TYPE)KMPI_FILE_IMPORT) : CIPluginFile(interfacetype) {};
    virtual ~CIPluginFileImport() {};

    /* events */
    virtual KMPI_RESULT onCreate(void) {return(KMPI_SUCCESS);};
    virtual KMPI_RESULT onDestroy(void) {return(KMPI_SUCCESS);};
    virtual KMPI_RESULT onOpenFile(const string& file, string& errorMsg);

    /* set... */

    /* get... */
    virtual KMPI_INTERFACE_TYPE getInterfaceType(void) const {return(interfaceType);};
    virtual const char** getButtonXPM(void) {return((const char**)CIPluginFileImport::defaultImportXpm);};
    virtual string getTooltipText(void) {return((string)KMPI_DEFAULT_IMPORT_TOOLTIP);};
    virtual string getMenuText(void) {return((string)KMPI_DEFAULT_IMPORT_MENU);};
    virtual string getWhatsThisText(void) {return((string)KMPI_DEFAULT_IMPORT_WHATSTHIS);};
    virtual string getStatusText(void) {return((string)KMPI_DEFAULT_IMPORT_STATUSTEXT);};
    virtual string getHighlightText(void) {return((string)KMPI_DEFAULT_IMPORT_HIGHLIGHTTEXT);};

  protected:
    /* protected members */
    static const char* defaultImportXpm[];
  };

  /* class CIPluginFileImportDialog */
  class CIPluginFileImportDialog : public CIPluginFileImport
  {
  public:
    CIPluginFileImportDialog(KMPI_INTERFACE_TYPE interfacetype = (KMPI_INTERFACE_TYPE)KMPI_FILE_IMPORT_DIALOG) : CIPluginFileImport(interfacetype) {};
    virtual ~CIPluginFileImportDialog() {};

    /* events */

    /* set... */

    /* get... */
  };
  
  /* class CIPluginFileExport */
  class CIPluginFileExport : public CIPluginFile
  {
  public:
    CIPluginFileExport(KMPI_INTERFACE_TYPE interfacetype = (KMPI_INTERFACE_TYPE)KMPI_FILE_EXPORT) : CIPluginFile(interfacetype) {};
    virtual ~CIPluginFileExport() {};

    /* events */
    virtual KMPI_RESULT onCreate(void) {return(KMPI_SUCCESS);};
    virtual KMPI_RESULT onDestroy(void) {return(KMPI_SUCCESS);};
    virtual KMPI_RESULT onOpenFile(const string& file, string& errorMsg);

    /* set... */

    /* get... */
    virtual KMPI_INTERFACE_TYPE getInterfaceType(void) const {return(interfaceType);};
    virtual const char** getButtonXPM(void) {return((const char**)CIPluginFileExport::defaultExportXpm);};
    virtual string getTooltipText(void) {return((string)KMPI_DEFAULT_EXPORT_TOOLTIP);};
    virtual string getMenuText(void) {return((string)KMPI_DEFAULT_EXPORT_MENU);};
    virtual string getWhatsThisText(void) {return((string)KMPI_DEFAULT_EXPORT_WHATSTHIS);};
    virtual string getStatusText(void) {return((string)KMPI_DEFAULT_EXPORT_STATUSTEXT);};
    virtual string getHighlightText(void) {return((string)KMPI_DEFAULT_EXPORT_HIGHLIGHTTEXT);};

  protected:
    /* protected members */
    static const char* defaultExportXpm[];
  };

  /* class CIPluginFileExportDialog */
  class CIPluginFileExportDialog : public CIPluginFileExport
  {
  public:
    CIPluginFileExportDialog(KMPI_INTERFACE_TYPE interfacetype = (KMPI_INTERFACE_TYPE)KMPI_FILE_EXPORT_DIALOG) : CIPluginFileExport(interfacetype) {};
    virtual ~CIPluginFileExportDialog() {};

    /* events */

    /* set... */

    /* get... */
  };

  /* class CIPluginColor */
  class  CIPluginColor
  {
  public:
    CIPluginColor(unsigned int red = 0, unsigned int green = 0, unsigned int blue = 0);
    CIPluginColor& operator=(const CIPluginColor& color);

    /* set... */
    KMPI_RESULT setColor(unsigned int red, unsigned int green, unsigned int blue);
    KMPI_RESULT setColorRed(unsigned int red);
    KMPI_RESULT setColorGreen(unsigned int green);
    KMPI_RESULT setColorBlue(unsigned int blue);

    /* get... */
    unsigned int getColorRed(void) {return(colorRed);};
    unsigned int getColorGreen(void) {return(colorGreen);};
    unsigned int getColorBlue(void) {return(colorBlue);};

  protected:
    unsigned int colorRed;
    unsigned int colorGreen;
    unsigned int colorBlue;
  };

  /* class CIPluginPosition */
  class  CIPluginPosition
  {
  public:
    CIPluginPosition(float x = 0.0f, float y = 0.0f, float z = 0.0f) : positionX(x), positionY(y), positionZ(z) {};
    CIPluginPosition& operator=(const CIPluginPosition& position);

    /* set... */
    KMPI_RESULT setPosition(float x, float y, float z);
    KMPI_RESULT setPositionX(float x);
    KMPI_RESULT setPositionY(float y);
    KMPI_RESULT setPositionZ(float z);

    /* get... */
    float getPositionX(void) {return(positionX);};
    float getPositionY(void) {return(positionY);};
    float getPositionZ(void) {return(positionZ);};
    static float getDistance(const CIPluginPosition& position1, const CIPluginPosition& position2);

  protected:
    float positionX;
    float positionY;
    float positionZ;
  };

  /* class CIPluginElements */
  class CIPluginElements
  {
  public:
    CIPluginElements();

    /* operators */
    CIPluginElements& operator=(const CIPluginElements& elements);

    /* set... */
    KMPI_RESULT setRadius(unsigned int number, float radius);
    KMPI_RESULT setRadius(const string& symbol, float radius);
    KMPI_RESULT setColor(unsigned int number, const CIPluginColor& color);
    KMPI_RESULT setColor(const string& symbol, const CIPluginColor& color);

    /* get... */
    float getRadius(unsigned int number);
    float getRadius(const string& symbol);
    CIPluginColor getColor(unsigned int number);
    CIPluginColor getColor(const string& symbol);
    static string getSymbol(unsigned int number);
    static unsigned int getNumber(const string& symbol);

  protected:
    static const string elementSymbol[];
    float elementRadius[KMPI_MAX_ELEMENT_NUMBER];
    CIPluginColor elementColor[KMPI_MAX_ELEMENT_NUMBER];
  };

  /* class CIPluginAtom */
  class CIPluginAtom
  {
  public:
    CIPluginAtom();

    /* operators */
    CIPluginAtom& operator=(const CIPluginAtom& atom);

    /* set... */
    KMPI_RESULT setByNumber(const CIPluginElements* pElements, unsigned int number);
    KMPI_RESULT setBySymbol(const CIPluginElements* pElements, const string& symbol);
    KMPI_RESULT setPosition(const CIPluginPosition& position);
    KMPI_RESULT setPosition(float x = 0.0f, float y = 0.0f, float z = 0.0f);
    KMPI_RESULT setColor(const CIPluginColor& color);
    KMPI_RESULT setColor(unsigned int red, unsigned int green, unsigned int blue) {return(atomColor.setColor(red, green, blue));};
    KMPI_RESULT setRadius(float radius);

    /* get... */
    unsigned int getNumber(void) {return(atomNumber);};
    string getSymbol(void) {return(atomSymbol);};
    CIPluginPosition getPosition(void) {return(atomPosition);};
    CIPluginColor getColor(void) {return(atomColor);};
    float getRadius(void) {return(atomRadius);};

  protected:
    unsigned int atomNumber;
    string atomSymbol;
    CIPluginPosition atomPosition;
    float atomRadius;
    CIPluginColor atomColor;
  };

  /* class CIPluginBond */
  class CIPluginBond
  {
  public:
    CIPluginBond(void);

    /* operators */
    CIPluginBond& operator=(const CIPluginBond& bond);

    /* set... */
    KMPI_RESULT setBondPartner(unsigned int index1, unsigned int index2);

    /* get... */
    unsigned int getBondPartner1(void) {return(bondPartner[0]);};
    unsigned int getBondPartner2(void) {return(bondPartner[1]);};

  protected:
    unsigned int bondPartner[2];
  };

  /* class CIPluginMolecule */
  class CIPluginMolecule
  {
  public:
    CIPluginMolecule(void);

    /* operators */
    CIPluginMolecule& operator=(const CIPluginMolecule& molecule);

    /* add... */
    unsigned int addAtom(const CIPluginAtom& atom);
    unsigned int addAtom(const CIPluginElements* pElements, unsigned int number);
    unsigned int addAtom(const CIPluginElements* pElements, const string& symbol);
    unsigned int addBond(const CIPluginBond& bond);
    unsigned int addBond(unsigned int index1, unsigned int index2);

    /* delete... */
    KMPI_RESULT deleteAtom(unsigned int index, KMPI_BOOL deleteBonds = TRUE);
    KMPI_RESULT deleteBond(unsigned int index1, unsigned int index2);
    KMPI_RESULT deleteAll(KMPI_BOOL deleteAtoms = TRUE, KMPI_BOOL deleteBonds = TRUE);

    /* set... */
    KMPI_RESULT setAtomByNumber(unsigned int index, const CIPluginElements* pElements, unsigned int number);
    KMPI_RESULT setAtomBySymbol(unsigned int index, const CIPluginElements* pElements, const string& symbol);
    KMPI_RESULT setAtomPosition(unsigned int index, const CIPluginPosition& position);
    KMPI_RESULT setAtomPosition(unsigned int index, float x = 0.0f, float y = 0.0f, float z = 0.0f);
    KMPI_RESULT setAtomColor(unsigned int index, const CIPluginColor& color);
    KMPI_RESULT setAtomColor(unsigned int index, unsigned int red, unsigned int green, unsigned int blue);
    KMPI_RESULT setAtomRadius(unsigned int index, float radius);
    KMPI_RESULT setBondColor(const CIPluginColor& color);
    KMPI_RESULT setBondColor(unsigned int red, unsigned int green, unsigned int blue) {return(bondColor.setColor(red, green, blue));};
    KMPI_RESULT setBondRadius(float radius);
    KMPI_RESULT setTitle(const string& title);

    /* get... */
    unsigned int getNumberOfAtoms(void) {return((unsigned int)atoms.size());}
    unsigned int getNumberOfBonds(void) {return((unsigned int)bonds.size());}
    CIPluginAtom* getAtom(unsigned int index);
    unsigned int getAtomNumber(unsigned int index);
    string getAtomSymbol(unsigned int index);
    CIPluginPosition getAtomPosition(unsigned int index);
    CIPluginColor getAtomColor(unsigned int index);
    float getAtomRadius(unsigned int index);
    CIPluginBond* getBond(unsigned int index);
    CIPluginColor getBondColor(void) {return(bondColor);};
    float getBondRadius(void) {return(bondRadius);};
    string getTitle(void) {return(moleculeTitle);};

    /* misc */
    KMPI_BOOL isConnected(unsigned int index1, unsigned int index2);
    KMPI_RESULT makeBonds(float bondFactor = 1.12f);

  protected:
    vector<CIPluginAtom> atoms;
    vector<CIPluginBond> bonds;
    vector<CIPluginAtom>::iterator atomIterator;
    vector<CIPluginBond>::iterator bondIterator;
    string moleculeTitle;
    float bondRadius;
    CIPluginColor bondColor;
  };
}

#endif

