/***************************************************************************
                          kmovisto.cpp  -  description
                             -------------------
    begin                : Sam Jan 22 16:27:34 CET 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmovisto.h"

/* pixmaps */
#include "filesave.xpm"
#include "fileopen.xpm"
#include "filenew.xpm"
#include "g94imp.xpm"
#include "xyzimp.xpm"
#include "povexp.xpm"
#include "vrmlexp.xpm"
#include "xyzexp.xpm"

/////////////////////////////////////////////////
// class CIDToolButton
CIDToolButton::CIDToolButton(int id,
                const QIconSet& iconSet,
                const QString& textLabel,
                QObject* receiver,
                const char* slot,
                QToolBar* parent,
                const char* name)
 : QToolButton(parent, name), idNumber(id)
{
  setIconSet(iconSet);
  setTextLabel(textLabel);
  connect(this, SIGNAL(clicked()), this, SLOT(_clicked()));
  connect(this, SIGNAL(_clickedID(int)), receiver, slot);
}

void CIDToolButton::_clicked(void)
{
  emit _clickedID(idNumber);
}

/////////////////////////////////////////////////
// class KMovisto
KMovisto::KMovisto()
{
  Caption = "KMovisto ";
  Caption += VERSION;
  setCaption(Caption);

  // initialisations
  numberOfItemsFileSubMenu1 = 0;
  numberOfItemsFileSubMenu2 = 0;

  // create view list object
  pViewList = new QPtrList<KMovistoView>();
  if(pViewList) pViewList->setAutoDelete(TRUE);

  // create pixmap lists
  pImportPixmapList = new QPtrList<QPixmap>();
  if(pImportPixmapList) pImportPixmapList->setAutoDelete(TRUE);
  pImportDialogPixmapList = new QPtrList<QPixmap>();
  if(pImportDialogPixmapList) pImportDialogPixmapList->setAutoDelete(TRUE);
  pExportPixmapList = new QPtrList<QPixmap>();
  if(pExportPixmapList) pExportPixmapList->setAutoDelete(TRUE);
  pExportDialogPixmapList = new QPtrList<QPixmap>();
  if(pExportDialogPixmapList) pExportDialogPixmapList->setAutoDelete(TRUE);

  // create plugin manager
  pluginManager = new CIPluginManager();

  // create tool bars
  fileToolbar = new QToolBar(this, "file operations");
  pluginToolbar = new QToolBar(this, "file plugin operations");
  
  // call inits to invoke all other construction parts
  initPluginManager();
  initDoc();
  initView();
  initMenuBar();
  initToolBar();
  initStatusBar();
  
  connect(doc, SIGNAL(documentChanged()), SLOT(markFilenameIfModified()));
}

KMovisto::~KMovisto()
{
  if(pViewList)
  {
    pViewList->clear();
    delete pViewList;
  }
  if(pImportPixmapList)
  {
    pImportPixmapList->clear();
    delete pImportPixmapList;
  }
  if(pImportDialogPixmapList)
  {
    pImportDialogPixmapList->clear();
    delete pImportDialogPixmapList;
  }
  if(pExportPixmapList)
  {
    pExportPixmapList->clear();
    delete pExportPixmapList;
  }
  if(pExportDialogPixmapList)
  {
    pExportDialogPixmapList->clear();
    delete pExportDialogPixmapList;
  }
  if(pluginManager) delete pluginManager;
}

void KMovisto::initPluginManager()
{
  // add plugins to plugin manager
  QString pluginProfile("");  
  QString library("");
  QString directory("");
  QString filter("");

  for(int i = 1; (library = initfile.GetParameterName("Plugin", i - 1)) != ""; i++)
  {
    if((int)ID_PLUGIN_MIN + i - 1 > (int)ID_PLUGIN_MAX &&
       QMessageBox::warning(0,
                            QString("Warning"),
                            QString("Maximum number of plugins reached. No more plugins loaded."),
                            QMessageBox::Ok,
                            QMessageBox::NoButton,
                            QMessageBox::NoButton))
      break;
    // get working directory and filter
    pluginProfile = initfile.GetProfile("Plugin", i - 1, "$HOME/*");
    directory = pluginProfile.left(pluginProfile.findRev("/", -1));
    filter = pluginProfile.right(pluginProfile.length() - pluginProfile.findRev("/", -1) - 1);
    // add library
    if(!pluginManager->addLibrary(library, ((int)ID_PLUGIN_MIN + i - 1), directory, filter) &&
       QMessageBox::critical(0,
                             QString("Error"),
                             QString("KMovisto plugin '" + library + "' is not loaded successfully.\nPlease check if the plugin library path is configured correctly.\n\nDo you want to continue loading plugins?"),
                             QString("No"),
                             QString("Yes"),
                             QString::null,
                             1) == 0)
      break;
  }
}

void KMovisto::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR
  
  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu

  fileMenu = new QPopupMenu();
  fileSubMenu1 = new QPopupMenu();
  fileSubMenu2 = new QPopupMenu();
  fileMenu->insertItem("&New", this, SLOT(slotFileNew()), CTRL+Key_N, ID_FILE_NEW);
  fileMenu->insertItem("&Open", this, SLOT(slotFileOpen()), CTRL+Key_O, ID_FILE_OPEN);
  fileMenu->insertItem("&Save", this, SLOT(slotFileSave()), CTRL+Key_S, ID_FILE_SAVE);
  fileMenu->insertItem("Save &as...", this, SLOT(slotFileSaveAs()), 0, ID_FILE_SAVE_AS);
  fileMenu->insertSeparator();
  fileSubMenu1->insertItem("Import &Gaussian file", this, SLOT(slotFileImportG94()), 0, ID_FILE_IMPORTG94);
  fileSubMenu1->insertItem("Import X&YZ file", this, SLOT(slotFileImportXYZ()), 0, ID_FILE_IMPORTXYZ);
  numberOfItemsFileSubMenu1 = fileSubMenu1->count();

  // add plugin file import menu
  if((pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT) +
      pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT_DIALOG)) > 0)
  fileSubMenu1->insertSeparator();
  // without file dialog
  initPluginMenuBar(fileSubMenu1, kmpi::KMPI_FILE_IMPORT, SLOT(slotFileImportPlugin(int)));
  // with file dialog
  initPluginMenuBar(fileSubMenu1, kmpi::KMPI_FILE_IMPORT_DIALOG, SLOT(slotFileImportDialogPlugin(int)));
  
  fileSubMenu2->insertItem("Export X&YZ file", this, SLOT(slotFileExportXYZ()), 0, ID_FILE_EXPORTXYZ);
  fileSubMenu2->insertItem("Export &POV-Ray file", this, SLOT(slotFileExportPOV()), 0, ID_FILE_EXPORTPOV);
  fileSubMenu2->insertItem("Export &VRML file", this, SLOT(slotFileExportVRML()), 0, ID_FILE_EXPORTVRML);
  numberOfItemsFileSubMenu2 = fileSubMenu2->count();

  // add plugin file export menu
  if((pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT) +
      pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT_DIALOG)) > 0)
  fileSubMenu2->insertSeparator();
  // without file dialog
  initPluginMenuBar(fileSubMenu2, kmpi::KMPI_FILE_EXPORT, SLOT(slotFileExportPlugin(int)));
  // with file dialog
  initPluginMenuBar(fileSubMenu2, kmpi::KMPI_FILE_EXPORT_DIALOG, SLOT(slotFileExportDialogPlugin(int)));
  
  fileMenu->insertItem("&Import", fileSubMenu1);
  fileMenu->insertItem("&Export", fileSubMenu2);
  fileMenu->insertSeparator();
  fileMenu->insertItem("&Print", this, SLOT(slotFilePrint()), CTRL+Key_P, ID_FILE_PRINT);
  fileMenu->insertSeparator();
  fileMenu->insertItem("E&xit", this, SLOT(slotFileQuit()), CTRL+Key_Q, ID_FILE_QUIT);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewMenu = new QPopupMenu();
  viewMenu->setCheckable(true);

  // add view menu items and select first item
  int index = 0;
  for(view = pViewList->first(); view != 0; view = pViewList->next())
  {
    viewMenu->insertItem(view->getMenuItem(), view->getViewID(), index++);
    viewMenu->setItemChecked(view->getViewID(), false);
  }
  view = pViewList->first();
  viewMenu->setItemChecked(view->getViewID(), true);

  viewMenu->insertSeparator();
  viewMenu->insertItem("Tool&bar", this, SLOT(slotViewToolBar()), 0, ID_VIEW_TOOLBAR);
  viewMenu->insertItem("&Statusbar", this, SLOT(slotViewStatusBar()), 0, ID_VIEW_STATUSBAR);
  viewMenu->setItemChecked(ID_VIEW_TOOLBAR, true);
  viewMenu->setItemChecked(ID_VIEW_STATUSBAR, true);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry moleculeMenu
  moleculeMenu=new QPopupMenu();
  moleculeMenu->insertItem("&Atoms", this, SLOT(slotMoleculeAtoms()), 0, ID_MOLECULE_ATOMS);
  moleculeMenu->insertItem("&Bonds", this, SLOT(slotMoleculeBonds()), 0, ID_MOLECULE_BONDS);
  moleculeMenu->insertItem("A&ppearance", this, SLOT(slotMoleculeAppearance()), 0, ID_MOLECULE_APPEARANCE);
  moleculeMenu->insertItem("&Title", this, SLOT(slotMoleculeTitle()), 0, ID_MOLECULE_TITLE);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry preferencesMenu
  preferencesMenu=new QPopupMenu();
  preferencesMenu->insertItem("&Directories", this, SLOT(slotPreferencesDirectories()), 0, ID_PREFERENCES_DIRECTORIES);
  preferencesMenu->insertItem("P&OV-Ray", this, SLOT(slotPreferencesPOVRay()), 0, ID_PREFERENCES_POVRAY);
  preferencesMenu->insertItem("&VRML", this, SLOT(slotPreferencesVRML()), 0, ID_PREFERENCES_VRML);
  preferencesMenu->insertItem("P&lugin manager", this, SLOT(slotPreferencesPlugins()), 0, ID_PREFERENCES_PLUGINS);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu=new QPopupMenu();
  helpMenu->insertItem("About...", this, SLOT(slotHelpAbout()), 0, ID_HELP_ABOUT);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  // set menuBar() the current menuBar 

  menuBar()->insertItem("&File", fileMenu);
  menuBar()->insertItem("&View", viewMenu);
  menuBar()->insertItem("&Molecule", moleculeMenu);
  menuBar()->insertItem("&Preferences",preferencesMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem("&Help", helpMenu);
  
  ///////////////////////////////////////////////////////////////////
  // CONNECT THE SUBMENU SLOTS WITH SIGNALS

  connect(fileMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(fileSubMenu1, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(fileSubMenu2, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(viewMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(moleculeMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(preferencesMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(helpMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(viewMenu, SIGNAL(activated(int)), SLOT(slotViewSelected(int)));
}

void KMovisto::initPluginMenuBar(QPopupMenu* menu, kmpi::KMPI_INTERFACE_TYPE type, const char* slot)
{
  CIPluginContainer* pluginContainer = 0;
  if(pluginManager)
  {
    for(pluginContainer = pluginManager->getPluginList(type)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(type)->next())
      menu->insertItem(QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getMenuText().c_str()),
                       this,
                       slot,
                       0,
                       pluginContainer->getId());
  }
}

void KMovisto::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  QPixmap openIcon, saveIcon, newIcon, g94impIcon, xyzimpIcon, povexpIcon, xyzexpIcon, vrmlexpIcon;
 
  newIcon = QPixmap(filenew);
  openIcon = QPixmap(fileopen);
  saveIcon = QPixmap(filesave);
  g94impIcon = QPixmap(g94imp);
  xyzimpIcon = QPixmap(xyzimp);
  xyzexpIcon = QPixmap(xyzexp);
  povexpIcon = QPixmap(povexp);
  vrmlexpIcon = QPixmap(vrmlexp);

  fileNew = new QToolButton(newIcon, "New File", 0, this, SLOT(slotFileNew()), fileToolbar);
  fileOpen = new QToolButton(openIcon, "Open File", 0, this, SLOT(slotFileOpen()), fileToolbar);
  fileSave = new QToolButton(saveIcon, "Save File", 0, this, SLOT(slotFileSave()), fileToolbar);
  fileToolbar->addSeparator();
  g94import = new QToolButton(g94impIcon, "Import Gaussian file", 0, this, SLOT(slotFileImportG94()), fileToolbar);
  xyzimport = new QToolButton(xyzimpIcon, "Import XYZ file", 0, this, SLOT(slotFileImportXYZ()), fileToolbar);
  xyzexport = new QToolButton(xyzexpIcon, "Export XYZ file", 0, this, SLOT(slotFileExportXYZ()), fileToolbar);
  povexport = new QToolButton(povexpIcon, "Export POV-Ray file", 0, this, SLOT(slotFileExportPOV()), fileToolbar);
  vrmlexport = new QToolButton(vrmlexpIcon, "Export VRML file", 0, this, SLOT(slotFileExportVRML()), fileToolbar);

  QWhatsThis::whatsThisButton(fileToolbar);
  QWhatsThis::add(fileNew,"Click this button to create a new file.\n\n"
                  "You can also select the New command from the File menu.");
  QWhatsThis::add(fileOpen,"Click this button to open a new file.\n\n"
                  "You can also select the Open command from the File menu.");
  QWhatsThis::add(fileSave,"Click this button to save the file you are "
                  "editing. You will be prompted for a file name.\n\n"
                  "You can also select the Save command from the File menu.");
  QWhatsThis::add(g94import,"Click this button to import a Gaussian file. "
                  "You will be prompted for a file name.\n\n"
                  "You can also select the Import submenu from the File menu.");
  QWhatsThis::add(xyzimport,"Click this button to import a XYZ file. "
                  "You will be prompted for a file name.\n\n"
                  "You can also select the Import submenu from the File menu.");
  QWhatsThis::add(povexport,"Click this button to export a POV-Ray file. "
                  "You will be prompted for a file name.\n\n"
                  "You can also select the Export submenu from the File menu.");
  QWhatsThis::add(xyzexport,"Click this button to export a XYZ file. "
                  "You will be prompted for a file name.\n\n"
                  "You can also select the Export submenu from the File menu.");
  QWhatsThis::add(vrmlexport,"Click this button to export a VRML file. "
                  "You will be prompted for a file name.\n\n"
                  "You can also select the Export submenu from the File menu.");

  // create additional toolbar for file plugins
  if((pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT) +
      pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT_DIALOG) +
      pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT) +
      pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT_DIALOG)) == 0)
  {
    pluginToolbar->hide();
  }
  else
  {
    // import without file dialog
    initPluginToolBar(pluginToolbar, kmpi::KMPI_FILE_IMPORT, SLOT(slotFileImportPlugin(int)));
    // import with file dialog
    initPluginToolBar(pluginToolbar, kmpi::KMPI_FILE_IMPORT_DIALOG, SLOT(slotFileImportDialogPlugin(int)));
    // export without file dialog
    initPluginToolBar(pluginToolbar, kmpi::KMPI_FILE_EXPORT, SLOT(slotFileExportPlugin(int)));
    // export with file dialog
    initPluginToolBar(pluginToolbar, kmpi::KMPI_FILE_EXPORT_DIALOG, SLOT(slotFileExportDialogPlugin(int)));
  }
    
  // create additional toolbars for each view if defined there
  KMovistoView *tmpview;
  for(tmpview = pViewList->first(); tmpview != 0; tmpview = pViewList->next())
  {
    tmpview->createToolbar(this);
    tmpview->hideToolbar();
  }
  tmpview = pViewList->first();
  tmpview->showToolbar();
}

void KMovisto::initPluginToolBar(QToolBar* toolbar, kmpi::KMPI_INTERFACE_TYPE type, const char* slot)
{
  // create additional toolbar for file plugins
  CIPluginContainer* pluginContainer = 0;
  QPixmap* pixmap = 0;
  CIDToolButton* toolButton = 0;
  QString whatsThisText = "";
  QPtrList<QPixmap>* pPixmapList = 0;
  switch(type)
  {
    case kmpi::KMPI_FILE_IMPORT: pPixmapList = pImportPixmapList; break;
    case kmpi::KMPI_FILE_IMPORT_DIALOG: pPixmapList = pImportDialogPixmapList; break;
    case kmpi::KMPI_FILE_EXPORT: pPixmapList = pExportPixmapList; break;
    case kmpi::KMPI_FILE_EXPORT_DIALOG: pPixmapList = pExportDialogPixmapList; break;
    default: pPixmapList = 0;
  }
  // add plugin toolbar button
  if(pPixmapList)
  {
    pPixmapList->clear();
    for(pluginContainer = pluginManager->getPluginList(type)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(type)->next())
    {
      pPixmapList->append(pixmap = (QPixmap*)new QPixmap(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getButtonXPM()));
      toolButton = new CIDToolButton(pluginContainer->getId(),
                                     *pixmap,
                                     QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getTooltipText().c_str()),
                                     this,
                                     slot,
                                     toolbar);
      whatsThisText = QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getWhatsThisText().c_str());
      if(whatsThisText.isEmpty()) whatsThisText = "No description for this Plugin available.";
      QWhatsThis::add(toolButton, whatsThisText);
    }
  }
}

void KMovisto::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::initDoc()
{
  doc = new KMovistoDoc();
  setCaption(Caption + " - " + doc->getMVTFileName());
  doc->newDoc();
  markFilenameIfModified();
}

void KMovisto::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // append views and set the main widget here

  pViewList->clear();
  // append views
  pViewList->append((KMovistoView*)new KMovistoViewGL(this, doc));
  pViewList->append((KMovistoView*)new KMovistoViewFlat(this, doc));
  pViewList->append((KMovistoView*)new KMovistoView3DAnaglyph(this, doc));
  pViewList->append((KMovistoView*)new KMovistoView3DPair(this, doc));
  pViewList->append((KMovistoView*)new KMovistoViewEdit(this, doc));
  uint viewID = ID_VIEW_MIN;
  for(view = pViewList->first(); view != 0 && viewID <= ID_VIEW_MAX; view = pViewList->next())
  {
    connect(view, SIGNAL(viewChanged()), this, SLOT(markFilenameIfModified()));
    connect(view, SIGNAL(statusHelpMsg(const QString&)), this, SLOT(slotStatusHelpMsg(const QString&)));
    connect(view, SIGNAL(statusHelpMsgPermanent(const QString&)), this, SLOT(slotStatusHelpMsgPermanent(const QString&)));
    connect(view, SIGNAL(statusHelpMsgDefault()), this, SLOT(slotStatusHelpMsgDefault()));
    view->setViewID(viewID++);
    view->hide();
    view->hideToolbar();
  }
  // select first view
  view = pViewList->first();
  view->show();
  view->showToolbar();
  setCentralWidget(view);
  resetView();
  markFilenameIfModified();
}

void KMovisto::resetView()
{
  ////////////////////////////////////////////////////////////////////
  // reset all views
  for(view = pViewList->first(); view != 0; view = pViewList->next()) view->resetView();
}

bool KMovisto::queryExit()
{
  int exit = 3;
  if(doc->getMolecule()->GetNumberOfAtoms())
    exit = QMessageBox::information(this,
                                    "Warning",
                                    "Your document contains unsaved changings.\nDo you really want to quit?",
                                    QMessageBox::Yes, QMessageBox::No);
  return (exit == 3);
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KMovisto::markFilenameIfModified()
{
  if(doc->isModified() && caption().right(2) != " *") setCaption(caption() + " *");
  if(!doc->isModified() && caption().right(2) == " *") setCaption(caption().left(caption().length() - 2));
}

void KMovisto::slotFileNew()
{
  if(doc->isModified() && doc->getMolecule()->GetNumberOfAtoms())
  {
    if(QMessageBox::information(this, "Warning",
                                      "Your document contains unsaved changings.\nDo you want to continue without saving?",
                                      QMessageBox::Yes, QMessageBox::No) == 3)

    {
      statusBar()->message("Creating new file...");
      doc->newDoc(1);
      resetView();
    }
  }
  else
  {
    statusBar()->message("Creating new file...");
    doc->newDoc(1);
    resetView();
  }
  setCaption(Caption + " - " + doc->getMVTFileName());
  markFilenameIfModified();
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileOpen()
{
  // check if the document contains unsaved data
  if(doc->isModified() && doc->getMolecule()->GetNumberOfAtoms() &&
     QMessageBox::information(this, "Warning",
                                    "Your document contains unsaved changings.\nDo you want to continue without saving?",
                                    QMessageBox::Yes, QMessageBox::No) != 3)
  {
    return;
  }
  // continue...
  statusBar()->message("Opening file...");
  QString currentDir = doc->getMVTFileName();
  QString fileName = QFileDialog::getOpenFileName(doc->getMVTFileName(),0,this);
  QDir::setCurrent(currentDir);
  if (!fileName.isEmpty())
  {
    resetView();
    if(doc->load(fileName))
    {
      doc->setMVTFileName(fileName);
      setCaption(Caption + " - " + doc->getMVTFileName());
      QString message="Loaded document: " + fileName;
      statusBar()->message(message, 2000);
    }
    else
    {
      QMessageBox::critical( 0, "Error!", QString("File not successfully opened!\nMaybe the selected file does not contain molecule data\nor is not a valid MVT file."));
      doc->newDoc(1);
      resetView();
    }
    markFilenameIfModified();
  }
  else
  {
    statusBar()->message("Opening aborted", 2000);
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileSave()
{
  statusBar()->message("Saving file...");
  doc->save();
  setCaption(Caption + " - " + doc->getMVTFileName());
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileSaveAs()
{
  statusBar()->message("Saving file under new filename...");
  QFile MVTFile;
  int writeFile = 1;
  QString currentDir = doc->getMVTFileName();
  QString fn = QFileDialog::getSaveFileName(currentDir, "*.mvt", this);
  QDir::setCurrent(currentDir);
  if (!fn.isEmpty())
  {
    // append filter if nessesary
    if((fn.right(4)).upper() != ".MVT" && fn.find(".") == -1) fn += ".mvt";
    // check, if selected file exists
    MVTFile.setName(fn);
    if (MVTFile.exists())
    {
       // mvt file exists
       if(QMessageBox::warning(0,
                               QString("Warning"),
                               QString("This MVT file already exists. Do you want\nto overwrite it?"),
                               QString("Yes"),
                               QString("No"),
                               QString::null,
                               1) == 1) writeFile = 0;
    }
    if(writeFile)
    {
       // The user selected a file
       if(!doc->saveAs(fn))
       {
         // xyz file not successfully saved
         QMessageBox::critical( 0, "Error!", QString("MVT file not successfully saved!"));
       }
      else
      {
        QString message="saved file: " + fn;
        doc->setMVTFileName(fn);
        setCaption(Caption + " - " + doc->getMVTFileName());
        statusBar()->message(message, 2000);
      }
    }
  }
  else
  {
    statusBar()->message("Saving aborted", 2000);
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileImportG94()
{
  statusBar()->message("Importing Gaussian file...");
  int CalNumber = 1;
  int g94FileAccepted = TRUE;
  QString fileName;
  QString workDir = doc->getWorkDir(G94IMP);
  CG94File g94file;
  QFileDialog fileDialog(0, 0, TRUE);
  // check if workDir is an environment variable
  if((workDir.left(1)) == "$" && (getenv(workDir.right(workDir.length() - 1)) != ""))
    workDir = getenv(workDir.right(workDir.length() - 1));
  // open gaussian file for input
  fileName = fileDialog.getOpenFileName(workDir, doc->getFilter(G94IMP), this);
  if(!fileName.isEmpty())
  {
    // The user selected a valid existing file
    if(g94file.LoadFile(fileName))
    {
      // Gaussian file successfully loaded
      if(g94file.GetNumberOfCalculations() > 1)
      {
        // Show dialog if there are more than 1 calculation
        CImpG94Dialog importG94Dialog;
        importG94Dialog.SetG94File(g94file);
        if(importG94Dialog.exec() == QDialog::Accepted)
          CalNumber = importG94Dialog.GetSelection();
        else
          g94FileAccepted = FALSE;
      }
      if(g94FileAccepted)
      {
        // Import data from file
        resetView();
        if(doc->importG94File(CalNumber, g94file))
        {
          QString message="Imported file: " + fileName;
          statusBar()->message(message, 2000);
        }
        else
        {
          // Gaussian file not successfully imported
          QMessageBox::critical( 0, "Error!", QString("Gaussian file not successfully imported!"));
        }
      }
    }
    else
    {
      // This is not a valid Gaussian file
      QMessageBox::critical( 0, "Error!", QString("This is not a valid Gaussian file!"));
    }
  }
  else
  {
    statusBar()->message("Importing aborted", 2000);
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileImportXYZ()
{
  statusBar()->message("Importing XYZ file...");
  QString fileName;
  QString workDir = doc->getWorkDir(XYZIMP);
  CXYZFile xyzfile;
  QFileDialog fileDialog(0, 0, TRUE);
  // check if workDir is an environment variable
  if((workDir.left(1)) == "$" && (getenv(workDir.right(workDir.length() - 1)) != ""))
    workDir = getenv(workDir.right(workDir.length() - 1));
  // open xyz file for input
  fileName = fileDialog.getOpenFileName(workDir, doc->getFilter(XYZIMP), this);
  if(!fileName.isEmpty())
  {
    // The user selected a valid existing file
    if(xyzfile.LoadFile(fileName))
    {
      // Import data from file
      resetView();
      if(doc->importXYZFile(xyzfile))
      {
        QString message="Imported file: " + fileName;
        statusBar()->message(message, 2000);
      }
      else
      {
        // xyz file not successfully imported
        QMessageBox::critical( 0, "Error!", QString("XYZ file not successfully imported!"));
      }
    }
    else
    {
      // This is not a valid XYZ file
      QMessageBox::critical( 0, "Error!", QString("This is not a valid XYZ file!"));
    }
  }
  else
  {
    statusBar()->message("Importing aborted", 2000);
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileImportDialogPlugin(int id)
{
  kmpi::CIPluginFile* pluginFile = (kmpi::CIPluginFile*)pluginManager->getPlugin(id);
  CIPluginContainer* pluginContainer = pluginManager->getPluginContainer(id);
  if(pluginFile && pluginContainer)
  {
    statusBar()->message(QString(pluginFile->getStatusText().c_str()));
    QString fileName;
    QString workDir = pluginContainer->getWorkingDirectory();
    QFileDialog fileDialog(0, 0, TRUE);
    // check if workDir is an environment variable
    if((workDir.left(1)) == "$" && (getenv(workDir.right(workDir.length() - 1)) != ""))
      workDir = getenv(workDir.right(workDir.length() - 1));  
    fileName = fileDialog.getOpenFileName(workDir, pluginContainer->getWorkingFilter(), this);
    if(!fileName.isEmpty())
    {      
      // get current molecule and pse settings
      kmpi::CIPluginMolecule molecule;
      kmpi::CIPluginElements elements;
      doc->getPeriodicSystem()->GetElements(elements);
      doc->getMolecule()->GetMolecule(molecule, elements);
      molecule.deleteAll();
      string file((const char*)fileName);
      string errorMsg("");      
      // plugin initialisation
      pluginFile->setElements(elements);
      pluginFile->setMolecule(molecule);
      // open file                                                                              
      if(pluginFile->onOpenFile(file, errorMsg) == kmpi::KMPI_SUCCESS)
      {
        resetView();
        // write document data
        elements = *(pluginFile->getElements());
        molecule = *(pluginFile->getMolecule());
        if(doc->importPluginFile(molecule, elements))
        {
          QString message="Imported file: " + fileName;
          statusBar()->message(message, 2000);
        }
        else
        {
          // file not successfully imported
          QMessageBox::critical( 0, "Error!", QString("File not successfully imported!"));
        }        
      }
      else
      {
        // file not successfully imported
        QMessageBox::critical( 0, "Error!", "Message from plugin:\n" + QString(errorMsg.c_str()));
      }
    }
    else
    {
      statusBar()->message("Importing aborted", 2000);
    }
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileImportPlugin(int id)
{
  kmpi::CIPluginFile* pluginFile = (kmpi::CIPluginFile*)pluginManager->getPlugin(id);
  if(pluginFile)
  {
    statusBar()->message(QString(pluginFile->getStatusText().c_str()));
    // get current molecule and pse settings
    kmpi::CIPluginMolecule molecule;
    kmpi::CIPluginElements elements;
    doc->getPeriodicSystem()->GetElements(elements);
    doc->getMolecule()->GetMolecule(molecule, elements);
    molecule.deleteAll();
    string file("");
    string errorMsg("");
    // plugin initialisation
    pluginFile->setElements(elements);
    pluginFile->setMolecule(molecule);
    // open file
    if(pluginFile->onOpenFile(file, errorMsg) == kmpi::KMPI_SUCCESS)
    {
      resetView();
      elements = *(pluginFile->getElements());
      molecule = *(pluginFile->getMolecule());
      if(doc->importPluginFile(molecule, elements))
      {
        QString message="File imported";
        statusBar()->message(message, 2000);
      }
      else
      {
        // file not successfully imported
        QMessageBox::critical( 0, "Error!", QString("File not successfully imported!"));
      }      
    }
    else
    {
      // file not successfully imported
      QMessageBox::critical( 0, "Error!", "Message from plugin:\n" + QString(errorMsg.c_str()));
    }
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileExportPOV()
{
  statusBar()->message("Exporting POV-Ray file...");
  QString fileName;
  QString workDir = doc->getWorkDir(POVEXP);
  CPOVFile povfile;
  QFile tmp_povfile;
  QString filter = doc->getFilter(POVEXP);
  int exportFile = 1;
  QFileDialog fileDialog(0, 0, TRUE);
  // check if number of Atoms > 0
  if(doc->getMolecule()->GetNumberOfAtoms() > 0)
  {
    // check if workDir is an environment variable
    if((workDir.left(1)) == "$" && (getenv(workDir.right(workDir.length() - 1)) != ""))
      workDir = getenv(workDir.right(workDir.length() - 1));
     // open pov file for output
     fileName = fileDialog.getSaveFileName(workDir, filter, this);
    if(!fileName.isEmpty())
    {
      // append export filter if nessesary
      if(filter.left(1) == "*") filter =  filter.right(filter.length() - 1);
      if(filter.length() > 0 && filter.left(1) != ".") filter = "." + filter;
      if(fileName.right(filter.length()) != filter) fileName += filter;
      // check, if selected file exists
      tmp_povfile.setName(fileName);
      if (tmp_povfile.exists())
      {
        // pov file exists
        if(QMessageBox::warning(0,
                                QString("Warning"),
                                QString("This POV-Ray file already exists. Do you want\nto overwrite it?"),
                                QString("Yes"),
                                QString("No"),
                                QString::null,
                                1) == 1) exportFile = 0;
      }
      if(exportFile)
      {
        // Export data from file
        if(doc->exportPOVFile(povfile))
        {
          // The user selected a file
          if(!povfile.SaveFile(fileName))
          {
            // pov file not successfully exported
            QMessageBox::critical( 0, "Error!", QString("POV-Ray file not successfully exported!"));
          }
          QString message="Exported file: " + fileName;
          statusBar()->message(message, 2000);
        }
        else
        {
          // pov file not successfully exported
          QMessageBox::critical( 0, "Error!", QString("POV-Ray file not successfully exported!"));
        }
      }
    }
    else
    {
      statusBar()->message("Exporting aborted", 2000);
    }
  }
  else
  {
    // Document does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("Document does not contain atoms!"));
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileExportVRML()
{
  statusBar()->message("Exporting VRML file...");
  QString fileName;
  QString workDir = doc->getWorkDir(VRMLEXP);
  CVRMLFile vrmlfile;
  QFile tmp_vrmlfile;
  QString filter = doc->getFilter(VRMLEXP);
  int exportFile = 1;
  QFileDialog fileDialog(0, 0, TRUE);
  // check if number of Atoms > 0
  if(doc->getMolecule()->GetNumberOfAtoms() > 0)
  {
    // check if workDir is an environment variable
    if((workDir.left(1)) == "$" && (getenv(workDir.right(workDir.length() - 1)) != ""))
      workDir = getenv(workDir.right(workDir.length() - 1));
     // open VRML file for output
     fileName = fileDialog.getSaveFileName(workDir, filter, this);
    if(!fileName.isEmpty())
    {
      // append export filter if nessesary
      if(filter.left(1) == "*") filter =  filter.right(filter.length() - 1);
      if(filter.length() > 0 && filter.left(1) != ".") filter = "." + filter;
      if(fileName.right(filter.length()) != filter) fileName += filter;
      // check, if selected file exists
      tmp_vrmlfile.setName(fileName);
      if (tmp_vrmlfile.exists())
      {
        // vrml file exists
        if(QMessageBox::warning( 0,
                                QString("Warning"),
                                QString("This VRML file already exists. Do you want\nto overwrite it?"),
                                QString("Yes"),
                                QString("No"),
                                QString::null,
                                1) == 1) exportFile = 0;
      }
      if(exportFile)
      {
        // Export data from file
        if(doc->exportVRMLFile(vrmlfile))
        {
          // The user selected a file
          if(!vrmlfile.SaveFile(fileName))
          {
            // VRML file not successfully exported
            QMessageBox::critical( 0, "Error!", QString("VRML file not successfully exported!"));
          }
          QString message="Exported file: " + fileName;
          statusBar()->message(message, 2000);
        }
        else
        {
          // VRML file not successfully exported
          QMessageBox::critical( 0, "Error!", QString("VRML file not successfully exported!"));
        }
      }
    }
    else
    {
      statusBar()->message("Exporting aborted", 2000);
    }
  }
  else
  {
    // Document does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("Document does not contain atoms!"));
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileExportXYZ()
{
  statusBar()->message("Exporting XYZ file...");
  QString fileName;
  QString workDir = doc->getWorkDir(XYZEXP);
  CXYZFile xyzfile;
  QFile tmp_xyzfile;
  QString filter = doc->getFilter(XYZEXP);
  int exportFile = 1;
  QFileDialog fileDialog(0, 0, TRUE);
  // check if number of Atoms > 0
  if(doc->getMolecule()->GetNumberOfAtoms() > 0)
  {
    // check if workDir is an environment variable
    if((workDir.left(1)) == "$" && (getenv(workDir.right(workDir.length() - 1)) != ""))
      workDir = getenv(workDir.right(workDir.length() - 1));
     // open xyz file for output
     fileName = fileDialog.getSaveFileName(workDir, filter, this);
    if(!fileName.isEmpty())
    {
      // append export filter if nessesary
      if(filter.left(1) == "*") filter =  filter.right(filter.length() - 1);
      if(filter.length() > 0 && filter.left(1) != ".") filter = "." + filter;
      if(fileName.right(filter.length()) != filter) fileName += filter;    
      // check, if selected file exists
      tmp_xyzfile.setName(fileName);
      if (tmp_xyzfile.exists())
      {
        // xyz file exists
        if(QMessageBox::warning( 0,
                                QString("Warning"),
                                QString("This XYZ file already exists. Do you want\nto overwrite it?"),
                                QString("Yes"),
                                QString("No"),
                                QString::null,
                                1) == 1) exportFile = 0;
      }
      if(exportFile)
      {
        // Export data from file
        if(doc->exportXYZFile(xyzfile))
        {
          // The user selected a file
          if(!xyzfile.SaveFile(fileName))
          {
            // xyz file not successfully exported
            QMessageBox::critical( 0, "Error!", QString("XYZ file not successfully exported!"));
          }
          QString message="Exported file: " + fileName;
          statusBar()->message(message, 2000);
        }
        else
        {
          // xyz file not successfully exported
          QMessageBox::critical( 0, "Error!", QString("XYZ file not successfully exported!"));
        }
      }
    }
    else
    {
      statusBar()->message("Exporting aborted", 2000);
    }
  }
  else
  {
    // Document does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("Document does not contain atoms!"));
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileExportDialogPlugin(int id)
{
  kmpi::CIPluginFile* pluginFile = (kmpi::CIPluginFile*)pluginManager->getPlugin(id);
  CIPluginContainer* pluginContainer = pluginManager->getPluginContainer(id);
  if(pluginFile && pluginContainer)
  {
    statusBar()->message(QString(pluginFile->getStatusText().c_str()));
    QString fileName;
    QString workDir = pluginContainer->getWorkingDirectory();
    QString filter = pluginContainer->getWorkingFilter();
    QFile tmp_pluginfile;
    int exportFile = 1;
    QFileDialog fileDialog(0, 0, TRUE);
    // check if number of Atoms > 0
    if(doc->getMolecule()->GetNumberOfAtoms() > 0)
    {
      // check if workDir is an environment variable
      if((workDir.left(1)) == "$" && (getenv(workDir.right(workDir.length() - 1)) != ""))
        workDir = getenv(workDir.right(workDir.length() - 1));
      // open file for output
      fileName = fileDialog.getSaveFileName(workDir, filter, this);
      if(!fileName.isEmpty())
      {
        // append export filter if nessesary
        if(filter.left(1) == "*") filter = filter.right(filter.length() - 1);
        if(filter.length() > 0 && filter.left(1) != ".") filter = "." + filter;
        if(fileName.right(filter.length()) != filter) fileName += filter;
        // check, if selected file exists
        tmp_pluginfile.setName(fileName);
        if (tmp_pluginfile.exists())
        {
          // file exists
          if(QMessageBox::warning(0,
                                  QString("Warning"),
                                  QString("This file already exists. Do you want\nto overwrite it?"),
                                  QString("Yes"),
                                  QString("No"),
                                  QString::null,
                                  1) == 1) exportFile = 0;
        }
        if(exportFile)
        {
          // get current molecule and pse settings
          kmpi::CIPluginMolecule molecule;
          kmpi::CIPluginElements elements;
          doc->getPeriodicSystem()->GetElements(elements);
          doc->getMolecule()->GetMolecule(molecule, elements);
          string file((const char*)fileName);
          string errorMsg("");
          // plugin initialisation
          pluginFile->setElements(elements);
          pluginFile->setMolecule(molecule);
          if(doc->exportPluginFile(molecule, elements))
          {
            if(pluginFile->onOpenFile(file, errorMsg) == kmpi::KMPI_ERROR)
            {
              // file not successfully imported
              QMessageBox::critical( 0, "Error!", "Message from plugin:\n" + QString(errorMsg.c_str()));
            }
            else
            {
              QString message="Exported file: " + fileName;
              statusBar()->message(message, 2000);
            }
          }
          else
          {
            // file not successfully exported
            QMessageBox::critical( 0, "Error!", QString("File not successfully exported!"));
          }            
        }
      }
      else
      {
        statusBar()->message("Exporting aborted", 2000);
      }
    }
    else
    {
      // Document does not contain atoms
      QMessageBox::critical( 0, "Error!", QString("Document does not contain atoms!"));
    }
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileExportPlugin(int id)
{
  kmpi::CIPluginFile* pluginFile = (kmpi::CIPluginFile*)pluginManager->getPlugin(id);
  if(pluginFile)
  {
    statusBar()->message(QString(pluginFile->getStatusText().c_str()));
    int exportFile = 1;
    // check if number of Atoms > 0
    if(doc->getMolecule()->GetNumberOfAtoms() > 0)
    {
      if(exportFile)
      {
        // get current molecule and pse settings
        kmpi::CIPluginMolecule molecule;
        kmpi::CIPluginElements elements;
        doc->getPeriodicSystem()->GetElements(elements);
        doc->getMolecule()->GetMolecule(molecule, elements);
        string file("");
        string errorMsg("");
        // plugin initialisation
        pluginFile->setElements(elements);
        pluginFile->setMolecule(molecule);
        if(doc->exportPluginFile(molecule, elements))
        {
          if(pluginFile->onOpenFile(file, errorMsg) == kmpi::KMPI_ERROR)
          {
            // file not successfully imported
            QMessageBox::critical( 0, "Error!", "Message from plugin:\n" + QString(errorMsg.c_str()));
          }
          else
          {
            QString message="File exported";
            statusBar()->message(message, 2000);
          }
        }
        else
        {
          // file not successfully exported
          QMessageBox::critical( 0, "Error!", QString("File not successfully exported!"));
        }
      }
    }
    else
    {
      // Document does not contain atoms
      QMessageBox::critical( 0, "Error!", QString("Document does not contain atoms!"));
    }
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotViewSelected(int id)
{
  KMovistoView *tmpview;
  for(view = pViewList->first(); view != 0; view = pViewList->next())
    if(id == (int)view->getViewID())
    {
      statusBar()->message("Change view mode");
      // hide all views
      for(tmpview = pViewList->first(); tmpview != 0; tmpview = pViewList->next())
      {
        tmpview->hide();
        tmpview->hideToolbar();
        viewMenu->setItemChecked(tmpview->getViewID(), false);
      }
      // show selected view
      view->show();
      view->showToolbar();
      viewMenu->setItemChecked(id, true);
      setCentralWidget(view);
      statusBar()->message(IDS_STATUS_DEFAULT);
      break;
    }
}

void KMovisto::slotFilePrint()
{
  statusBar()->message("Printing...");
  QPrinter printer;
  if (printer.setup(this))
  {
    QPaintDeviceMetrics pdm(&printer);
    QRect rectClient(0, 0, pdm.width(), pdm.height());
    if(!((KMovistoView*)centralWidget())->print(printer, rectClient))
    {
      // error while printing
      QMessageBox::critical( 0, "Error!", QString("An Error occured while printing!\nMaybe not enough memory."));
    }
  };

  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotFileQuit()
{ 
  statusBar()->message("Exiting application...");
  ///////////////////////////////////////////////////////////////////
  // exits the Application
  if(doc->isModified())
  {
    if(queryExit()) qApp->quit();
  }
  else
    qApp->quit();

  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotViewToolBar()
{
  statusBar()->message("Toggle toolbar...");
  ///////////////////////////////////////////////////////////////////
  // turn file oolbar on or off

  if (fileToolbar->isVisible())
  {
    // file toolbar
    fileToolbar->hide();
    // view toolbar
    for(KMovistoView *tmpview = pViewList->first(); tmpview != 0; tmpview = pViewList->next()) tmpview->hideToolbar();
    // plugin toolbar
    pluginToolbar->hide();
    // item
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, false);
  } 
  else
  {
    // file toolbar
    fileToolbar->show();
    // view toolbar
    ((KMovistoView*)centralWidget())->showToolbar();
    // plugin toolbar
    pluginToolbar->show();
    // item
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, true);
  };
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotViewStatusBar()
{
  statusBar()->message("Toggle statusbar...");
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  
  if (statusBar()->isVisible())
  {
    statusBar()->hide();
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, false);
  }
  else
  {
    statusBar()->show();
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, true);
  }
  
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotMoleculeAtoms()
{
  statusBar()->message("Changing atom parameters");
  // Show dialog
  CAtomsDialog atomsDialog;
  atomsDialog.setPeriodicSystem(doc->getPeriodicSystem());
  atomsDialog.setMolecule(doc->getMolecule());
  if(atomsDialog.exec() == QDialog::Accepted)
  {
    // update document data
    doc->setPeriodicSystem(atomsDialog.getPeriodicSystem());
    doc->setMolecule(atomsDialog.getMolecule());
    if(atomsDialog.getApplyStatus())
    {
      // write new settings to init file
      if(!doc->writeAtomProperties())
      {
        initfile.throwErrorMsg();
      }
    }
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotMoleculeBonds()
{  
  statusBar()->message("Changing bond parameters");
  // Show dialog
  CBondsDialog bondsDialog;
  bondsDialog.setMolecule(doc->getMolecule());
  if(bondsDialog.exec() == QDialog::Accepted)
  {
    // update document data
    doc->setMolecule(bondsDialog.getMolecule());
    if(bondsDialog.getApplyStatus())
    {
      // write new settings to init file
      if(!doc->writeBondProperties())
      {
        initfile.throwErrorMsg();
      }
    }  
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotMoleculeAppearance()
{
  statusBar()->message("Changing molecule appearance");
  // Show dialog
  CAppearanceDialog appearanceDialog;
  appearanceDialog.setMolecule(doc->getMolecule());
  appearanceDialog.setBGColorPrint(doc->getBGColorPrint());
  if(appearanceDialog.exec() == QDialog::Accepted)
  {
    // update document data
    doc->setMolecule(appearanceDialog.getMolecule());
    doc->setBGColorPrint(appearanceDialog.getBGColorPrint());
    if(appearanceDialog.getApplyStatus())
    {
      // write new settings to init file
      if(!doc->writeAppearanceProperties())
      {
        initfile.throwErrorMsg();
      }
    }  
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotMoleculeTitle()
{
  statusBar()->message("Changing title");
  // Show dialog
  CTitleDialog titleDialog;
  titleDialog.setMolecule(doc->getMolecule());
  if(titleDialog.exec() == QDialog::Accepted)
  {
    // update document data
    doc->setMolecule(titleDialog.getMolecule());
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotPreferencesDirectories()
{
  statusBar()->message("Changing working directories");
  // Show dialog
  CDirectoriesDialog directoriesDialog;
  directoriesDialog.setDirectory(doc->getWorkDir(G94IMP), G94IMP);
  directoriesDialog.setDirectory(doc->getWorkDir(XYZIMP), XYZIMP);
  directoriesDialog.setDirectory(doc->getWorkDir(XYZEXP), XYZEXP);
  directoriesDialog.setDirectory(doc->getWorkDir(POVEXP), POVEXP);
  directoriesDialog.setDirectory(doc->getWorkDir(VRMLEXP), VRMLEXP);
  directoriesDialog.setFilter(doc->getFilter(G94IMP), G94IMP);
  directoriesDialog.setFilter(doc->getFilter(XYZIMP), XYZIMP);
  directoriesDialog.setFilter(doc->getFilter(XYZEXP), XYZEXP);
  directoriesDialog.setFilter(doc->getFilter(POVEXP), POVEXP);
  directoriesDialog.setFilter(doc->getFilter(VRMLEXP), VRMLEXP);
  directoriesDialog.setPluginProperties(pluginManager);
  if(directoriesDialog.exec() == QDialog::Accepted)
  {
    // update document data
    doc->setWorkDir(directoriesDialog.getDirectory(G94IMP), G94IMP);
    doc->setWorkDir(directoriesDialog.getDirectory(XYZIMP), XYZIMP);
    doc->setWorkDir(directoriesDialog.getDirectory(XYZEXP), XYZEXP);
    doc->setWorkDir(directoriesDialog.getDirectory(POVEXP), POVEXP);
    doc->setWorkDir(directoriesDialog.getDirectory(VRMLEXP), VRMLEXP);
    doc->setFilter(directoriesDialog.getFilter(G94IMP), G94IMP);
    doc->setFilter(directoriesDialog.getFilter(XYZIMP), XYZIMP);
    doc->setFilter(directoriesDialog.getFilter(XYZEXP), XYZEXP);
    doc->setFilter(directoriesDialog.getFilter(POVEXP), POVEXP);
    doc->setFilter(directoriesDialog.getFilter(VRMLEXP), VRMLEXP);
    directoriesDialog.getPluginProperties(pluginManager);
    if(directoriesDialog.getApplyStatus())
    {
      // write new settings to init file
      if(doc->writeDirectoryProperties())
      {
        // write plugin settings
        bool error = FALSE;
        QString pluginProfile = "";
        CIPluginContainer* pluginContainer = 0;
        initfile.RemoveProfile("Plugin");
        // write current plugin settings
        for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->first();
            !error && pluginContainer;
            pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->next())
        {
          pluginProfile = pluginContainer->getWorkingDirectory() + "/" + pluginContainer->getWorkingFilter();
          error = (error ? TRUE : error = !(bool)initfile.WriteProfile("Plugin", pluginContainer->getLibrary(), pluginProfile));         
        }
        for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->first();
            !error && pluginContainer;
            pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->next())
        {
          pluginProfile = pluginContainer->getWorkingDirectory() + "/" + pluginContainer->getWorkingFilter();
          error = (error ? TRUE : error = !(bool)initfile.WriteProfile("Plugin", pluginContainer->getLibrary(), pluginProfile));
        }
        for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->first();
            !error && pluginContainer;
            pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->next())
        {
          pluginProfile = pluginContainer->getWorkingDirectory() + "/" + pluginContainer->getWorkingFilter();
          error = (error ? TRUE : error = !(bool)initfile.WriteProfile("Plugin", pluginContainer->getLibrary(), pluginProfile));
        }
        for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->first();
            !error && pluginContainer;
            pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->next())
        {
          pluginProfile = pluginContainer->getWorkingDirectory() + "/" + pluginContainer->getWorkingFilter();
          error = (error ? TRUE : error = !(bool)initfile.WriteProfile("Plugin", pluginContainer->getLibrary(), pluginProfile));
        }
        if(error)
        {
          initfile.throwErrorMsg();
        }             
      }
      else
      {
        initfile.throwErrorMsg();
      }
    }  
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotPreferencesPOVRay()
{
  statusBar()->message("Changing POV-Ray parameters");
  // Show dialog
  CPOVRayDialog povrayDialog;
  povrayDialog.setAlternateAtomRadius(doc->getPOVAlternateAtomRadius());
  povrayDialog.setAlternateAtomColor(doc->getPOVAlternateAtomColor());
  povrayDialog.setAtomColor(doc->getPOVAtomColor());
  povrayDialog.setAtomRadius(doc->getPOVAtomRadius());
  povrayDialog.setTextureNumber(doc->getPOVTextureNumber());
  povrayDialog.setTexture(doc->getPOVTexture());
  povrayDialog.setInclude(doc->getPOVInclude());
  povrayDialog.setBondRadius(doc->getPOVBondRadius());
  povrayDialog.setAlternateBondColor(doc->getPOVAlternateBondColor());
  povrayDialog.setBondColor(doc->getPOVBondColor());
  povrayDialog.setLight1Enabled(doc->getPOVLight1Enabled());
  povrayDialog.setLight2Enabled(doc->getPOVLight2Enabled());
  povrayDialog.setLight3Enabled(doc->getPOVLight3Enabled());
  povrayDialog.setShadowless(doc->getPOVShadowless());
  povrayDialog.setLight1(doc->getPOVLight1());
  povrayDialog.setLight2(doc->getPOVLight2());
  povrayDialog.setLight3(doc->getPOVLight3());
  povrayDialog.setCameraRight(doc->getPOVCameraRight());
  povrayDialog.setCameraUp(doc->getPOVCameraUp());
  povrayDialog.setCameraDirection(doc->getPOVCameraDirection());
  povrayDialog.setCameraLocation(doc->getPOVCameraLocation());
  povrayDialog.setCameraLookAt(doc->getPOVCameraLookAt());
  if(povrayDialog.exec() == QDialog::Accepted)
  {
    // update document data
    doc->setPOVAlternateAtomRadius(povrayDialog.getAlternateAtomRadius());
    doc->setPOVAlternateAtomColor(povrayDialog.getAlternateAtomColor());
    doc->setPOVAtomColor(povrayDialog.getAtomColor());
    doc->setPOVAtomRadius(povrayDialog.getAtomRadius());
    doc->setPOVTextureNumber(povrayDialog.getTextureNumber());
    doc->setPOVTexture(povrayDialog.getTexture());
    doc->setPOVInclude(povrayDialog.getInclude());
    doc->setPOVBondRadius(povrayDialog.getBondRadius());
    doc->setPOVAlternateBondColor(povrayDialog.getAlternateBondColor());
    doc->setPOVBondColor(povrayDialog.getBondColor());
    doc->setPOVLight1Enabled(povrayDialog.getLight1Enabled());
    doc->setPOVLight2Enabled(povrayDialog.getLight2Enabled());
    doc->setPOVLight3Enabled(povrayDialog.getLight3Enabled());
    doc->setPOVShadowless(povrayDialog.getShadowless());
    doc->setPOVLight1(povrayDialog.getLight1());
    doc->setPOVLight2(povrayDialog.getLight2());
    doc->setPOVLight3(povrayDialog.getLight3());
    doc->setPOVCameraRight(povrayDialog.getCameraRight());
    doc->setPOVCameraUp(povrayDialog.getCameraUp());
    doc->setPOVCameraDirection(povrayDialog.getCameraDirection());
    doc->setPOVCameraLocation(povrayDialog.getCameraLocation());
    doc->setPOVCameraLookAt(povrayDialog.getCameraLookAt());
    if(povrayDialog.getApplyStatus())
    {
      // write new settings to init file
      if(!doc->writePOVRayProperties())
      {
        initfile.throwErrorMsg();
      }
    }  
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotPreferencesVRML()
{
  statusBar()->message("Changing VRML parameters");
  // Show dialog
  CVRMLDialog vrmlDialog;
  vrmlDialog.setAlternateAtomRadius(doc->getVRMLAlternateAtomRadius());
  vrmlDialog.setAlternateAtomColor(doc->getVRMLAlternateAtomColor());
  vrmlDialog.setAtomColor(doc->getVRMLAtomColor());
  vrmlDialog.setAtomRadius(doc->getVRMLAtomRadius());
  vrmlDialog.setEnableTextureMapping(doc->getVRMLEnableTextureMapping());
  vrmlDialog.setImageTextureURL(doc->getVRMLImageTextureURL());
  vrmlDialog.setEnableDiffuseColor(doc->getVRMLEnableDiffuseColor());
  vrmlDialog.setEnableEmissiveColor(doc->getVRMLEnableEmissiveColor());
  vrmlDialog.setEnableSpecularColor(doc->getVRMLEnableSpecularColor());
  vrmlDialog.setDiffuseColorFactor(doc->getVRMLDiffuseColorFactor());
  vrmlDialog.setEmissiveColorFactor(doc->getVRMLEmissiveColorFactor());
  vrmlDialog.setSpecularColorFactor(doc->getVRMLSpecularColorFactor());
  vrmlDialog.setShininess(doc->getVRMLShininess());
  vrmlDialog.setBondRadius(doc->getVRMLBondRadius());
  vrmlDialog.setAlternateBondColor(doc->getVRMLAlternateBondColor());
  vrmlDialog.setBondColor(doc->getVRMLBondColor());
  if(vrmlDialog.exec() == QDialog::Accepted)
  {
    // update document data
    doc->setVRMLAlternateAtomRadius(vrmlDialog.getAlternateAtomRadius());
    doc->setVRMLAlternateAtomColor(vrmlDialog.getAlternateAtomColor());
    doc->setVRMLAtomColor(vrmlDialog.getAtomColor());
    doc->setVRMLAtomRadius(vrmlDialog.getAtomRadius());
    doc->setVRMLEnableTextureMapping(vrmlDialog.getEnableTextureMapping());
    doc->setVRMLImageTextureURL(vrmlDialog.getImageTextureURL());
    doc->setVRMLEnableDiffuseColor(vrmlDialog.getEnableDiffuseColor());
    doc->setVRMLEnableEmissiveColor(vrmlDialog.getEnableEmissiveColor());
    doc->setVRMLEnableSpecularColor(vrmlDialog.getEnableSpecularColor());
    doc->setVRMLDiffuseColorFactor(vrmlDialog.getDiffuseColorFactor());
    doc->setVRMLEmissiveColorFactor(vrmlDialog.getEmissiveColorFactor());
    doc->setVRMLSpecularColorFactor(vrmlDialog.getSpecularColorFactor());
    doc->setVRMLShininess(vrmlDialog.getShininess());
    doc->setVRMLBondRadius(vrmlDialog.getBondRadius());
    doc->setVRMLAlternateBondColor(vrmlDialog.getAlternateBondColor());
    doc->setVRMLBondColor(vrmlDialog.getBondColor());
    if(vrmlDialog.getApplyStatus())
    {
      // write new settings to init file
      if(!doc->writeVRMLProperties())
      {
        initfile.throwErrorMsg();
      }
    }
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotPreferencesPlugins()
{
  statusBar()->message("Plugin configuration");
  int i = 0;
  // Show dialog
  CPluginDialog pluginDialog;
  pluginDialog.setPluginManager(pluginManager);
  if(pluginDialog.exec() == QDialog::Accepted && pluginDialog.isModified())
  {
    // update plugin manager
    pluginDialog.getPluginManager(pluginManager);
    
    // update import menu
    for(i = fileSubMenu1->count() - 1; i > numberOfItemsFileSubMenu1 - 1; i--) fileSubMenu1->removeItemAt(i);
    if((pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT) +
        pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT_DIALOG)) > 0)
    fileSubMenu1->insertSeparator();
    initPluginMenuBar(fileSubMenu1, kmpi::KMPI_FILE_IMPORT, SLOT(slotFileImportPlugin(int)));
    initPluginMenuBar(fileSubMenu1, kmpi::KMPI_FILE_IMPORT_DIALOG, SLOT(slotFileImportDialogPlugin(int)));
    // update export menu
    for(i = fileSubMenu2->count() - 1; i > numberOfItemsFileSubMenu2 - 1; i--) fileSubMenu2->removeItemAt(i);
    if((pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT) +
        pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT_DIALOG)) > 0)
    fileSubMenu2->insertSeparator();
    initPluginMenuBar(fileSubMenu2, kmpi::KMPI_FILE_EXPORT, SLOT(slotFileExportPlugin(int)));
    initPluginMenuBar(fileSubMenu2, kmpi::KMPI_FILE_EXPORT_DIALOG, SLOT(slotFileExportDialogPlugin(int)));

    // update toolbar
    pluginToolbar->clear();
    if((pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT) +
        pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT_DIALOG) +
        pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT) +
        pluginManager->getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT_DIALOG)) == 0)
    {
      pluginToolbar->hide();
    }
    else
    {
      initPluginToolBar(pluginToolbar, kmpi::KMPI_FILE_IMPORT, SLOT(slotFileImportPlugin(int)));
      initPluginToolBar(pluginToolbar, kmpi::KMPI_FILE_IMPORT_DIALOG, SLOT(slotFileImportDialogPlugin(int)));
      initPluginToolBar(pluginToolbar, kmpi::KMPI_FILE_EXPORT, SLOT(slotFileExportPlugin(int)));
      initPluginToolBar(pluginToolbar, kmpi::KMPI_FILE_EXPORT_DIALOG, SLOT(slotFileExportDialogPlugin(int)));
      pluginToolbar->show();
    }
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::slotHelpAbout()
{
  QMessageBox::about(this,"About...",
                     IDS_APP_ABOUT );
}

void KMovisto::slotStatusHelpMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 3000);
}

void KMovisto::slotStatusHelpMsgPermanent(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar permanent (text)
  statusBar()->message(text);
}

void KMovisto::slotStatusHelpMsgDefault(void)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar to default ()
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void KMovisto::statusCallback(int id_)
{
  switch (id_)
  {
    case ID_FILE_NEW:
         slotStatusHelpMsg("Creates a new document");
         break;

    case ID_FILE_OPEN:
         slotStatusHelpMsg("Opens an existing document");
         break;

    case ID_FILE_SAVE:
         slotStatusHelpMsg("Saves the actual document");
         break;

    case ID_FILE_SAVE_AS:
         slotStatusHelpMsg("Saves the actual document as...");
         break;

    case ID_FILE_IMPORTG94:
         slotStatusHelpMsg("Imports a Gaussian file");
         break;

    case ID_FILE_IMPORTXYZ:
         slotStatusHelpMsg("Imports a XYZ file");
         break;

    case ID_FILE_EXPORTXYZ:
         slotStatusHelpMsg("Exports a XYZ file");
         break;

    case ID_FILE_EXPORTPOV:
         slotStatusHelpMsg("Exports a POV-Ray file");
         break;

    case ID_FILE_EXPORTVRML:
         slotStatusHelpMsg("Exports a VRML file");
         break;

    case ID_FILE_PRINT:
         slotStatusHelpMsg("Prints out the actual document");
         break;

    case ID_FILE_QUIT:
         slotStatusHelpMsg("Quits the application");
         break;

    case ID_VIEW_TOOLBAR:
         slotStatusHelpMsg("Enables/disables the toolbar");
         break;

    case ID_VIEW_STATUSBAR:
         slotStatusHelpMsg("Enables/disables the statusbar");
         break;

    case ID_MOLECULE_ATOMS:
         slotStatusHelpMsg("Changes the atom parameters");
         break;

    case ID_MOLECULE_BONDS:
         slotStatusHelpMsg("Changes the bond parameters");
         break;

    case ID_MOLECULE_APPEARANCE:
         slotStatusHelpMsg("Changes the molecule appearance");
         break;

    case ID_MOLECULE_TITLE:
         slotStatusHelpMsg("Changes the title");
         break;

    case ID_PREFERENCES_DIRECTORIES:
         slotStatusHelpMsg("Changes the working directories");
         break;

    case ID_PREFERENCES_POVRAY:
         slotStatusHelpMsg("Changes the POV-Ray parameters");
         break;

    case ID_PREFERENCES_VRML:
         slotStatusHelpMsg("Changes the VRML parameters");
         break;

    case ID_PREFERENCES_PLUGINS:
         slotStatusHelpMsg("Changes the plugin settings");
         break;

    case ID_HELP_ABOUT:
         slotStatusHelpMsg("Shows an aboutbox");
         break;
         
    default:
         if(id_ >= (int)ID_VIEW_MIN && id_ <= (int)ID_VIEW_MAX)
         {
           slotStatusHelpMsg("Select preferred view");
         }         
         if(id_ >= (int)ID_PLUGIN_MIN && id_ <= (int)ID_PLUGIN_MAX)
         {
           // show plugin text
           kmpi::CIPlugin* plugin = pluginManager->getPlugin(id_);
           if(plugin) slotStatusHelpMsg(QString(((kmpi::CIPluginFile*)plugin)->getHighlightText().c_str()));
         }
  }
}


