/***************************************************************************
                          kmovisto.h  -  description
                             -------------------
    begin                : Sam Jan 22 16:27:34 CET 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMOVISTO_H
#define KMOVISTO_H

/* include files for QT */
#include <qstring.h>
#include <qpopupmenu.h>
#include <qmainwindow.h>
#include <qaccel.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qpixmap.h>
#include <qtoolbutton.h>
#include <qapp.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qptrlist.h>

/* application specific includes */
#include <cstdio>
#include <cstdlib>
#include "kmovistoview.h"
#include "kmovistodoc.h"
#include "resource.h"
#include "dialog_g94imp.h"
#include "dialog_atoms.h"
#include "dialog_bonds.h"
#include "dialog_appearance.h"
#include "dialog_title.h"
#include "dialog_directories.h"
#include "dialog_povray.h"
#include "dialog_vrml.h"
#include "dialog_plugin.h"
#include "g94file.h"
#include "xyzfile.h"
#include "povfile.h"
#include "vrmlfile.h"
#include "initfile.h"
#include "ipluginmanager.h"

/* class CIDToolButton */
class CIDToolButton : public QToolButton
{
  Q_OBJECT

public:
  CIDToolButton(int id,
                const QIconSet& iconSet,
                const QString& textLabel,
                QObject* receiver,
                const char* slot,
                QToolBar* parent,
                const char* name = 0);

signals:
  void _clickedID(int id);

private slots:
  void _clicked(void);

private:
  const int idNumber;  
}; 

/* class KMovisto */
class KMovisto : public QMainWindow
{
  Q_OBJECT
  
public:
  KMovisto();
  ~KMovisto();
  void initPluginManager();
  void initMenuBar();
  void initPluginMenuBar(QPopupMenu* menu, kmpi::KMPI_INTERFACE_TYPE type, const char* slot);
  void initToolBar();
  void initPluginToolBar(QToolBar* toolbar, kmpi::KMPI_INTERFACE_TYPE type, const char* slot);
  void initStatusBar();
  void initDoc();
  void initView();
  void resetView();
  bool queryExit();

public slots:
  void markFilenameIfModified();
  void statusCallback(int id_);
  void slotFileNew();
  void slotFileOpen();
  void slotFileSave();
  void slotFileSaveAs();
  void slotFileImportG94();
  void slotFileImportXYZ();
  void slotFileImportPlugin(int id);
  void slotFileImportDialogPlugin(int id);
  void slotFileExportPOV();
  void slotFileExportVRML();
  void slotFileExportXYZ();
  void slotFileExportPlugin(int id);
  void slotFileExportDialogPlugin(int id);
  void slotFilePrint();
  void slotFileQuit();
  void slotViewSelected(int id);
  void slotViewToolBar();
  void slotViewStatusBar();
  void slotMoleculeAtoms();
  void slotMoleculeBonds();
  void slotMoleculeAppearance();
  void slotMoleculeTitle();
  void slotPreferencesDirectories();
  void slotPreferencesPOVRay();
  void slotPreferencesVRML();
  void slotPreferencesPlugins();
  void slotHelpAbout();
  void slotStatusHelpMsg(const QString &text);
  void slotStatusHelpMsgPermanent(const QString &text);
  void slotStatusHelpMsgDefault();

private:
  QString Caption;
  KMovistoView *view;
  QPtrList<KMovistoView> *pViewList;
  QPtrList<QPixmap> *pImportPixmapList;
  QPtrList<QPixmap> *pImportDialogPixmapList;
  QPtrList<QPixmap> *pExportPixmapList;
  QPtrList<QPixmap> *pExportDialogPixmapList;
  int numberOfItemsFileSubMenu1;
  int numberOfItemsFileSubMenu2;
  KMovistoDoc *doc;
  /* plugin manager */
  CIPluginManager* pluginManager;
  /* file_menu contains all items of the menubar entry "File" */
  QPopupMenu *fileMenu;
  QPopupMenu *fileSubMenu1;
  QPopupMenu *fileSubMenu2;
  /* edit_menu contains all items of the menubar entry "Edit" */
  QPopupMenu *viewMenu;
  /* molecule_menu contains all items of the menubar entry "Molecule" */
  QPopupMenu *moleculeMenu;
  /* preferences_menu contains all items of the menubar entry "Preferences" */
  QPopupMenu *preferencesMenu;
  /* view_menu contains all items of the menubar entry "Help" */
  QPopupMenu *helpMenu;
  /* the file toolbar */
  QToolBar *fileToolbar;
  /* the plugin toolbar */
  QToolBar *pluginToolbar;
  /* the initialisation file */
  CInitFile initfile;
  /* toolbar buttons */
  QToolButton *fileNew;
  QToolButton *fileOpen;
  QToolButton *fileSave;
  QToolButton *g94import;
  QToolButton *xyzimport;
  QToolButton *xyzexport;
  QToolButton *povexport;
  QToolButton *vrmlexport;
};
#endif 
