/***************************************************************************
                          kmovistodoc.cpp  -  description
                             -------------------
    begin                : Sam Jan 22 16:27:34 CET 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmovistodoc.h"

KMovistoDoc::KMovistoDoc()
{
  periodicsystem = new CKPeriodicSystem(this, "doCKPeriodicSystem");
  molecule = new CKGLMolecule(this, "docKGLMolecule");
  MVTFile = new CMVTFile(this, "docMVTfile");
  Light1POV = new CPOVCoordinate(this, "docLight1POV");
  Light2POV = new CPOVCoordinate(this, "docLight2POV");
  Light3POV = new CPOVCoordinate(this, "docLight3POV");
  CameraRightPOV = new CPOVCoordinate(this, "docCameraRightPOV");
  CameraUpPOV = new  CPOVCoordinate(this, "docCameraUpPOV");
  CameraDirectionPOV = new CPOVCoordinate(this, "docCameraDirectionPOV");
  CameraLocationPOV = new  CPOVCoordinate(this, "docCameraLocationPOV");
  CameraLookAtPOV = new  CPOVCoordinate(this, "docCameraLookAtPOV");
  modified = FALSE;
  setParametersToDefault();
  MVTFile->Reset(1);
}

KMovistoDoc::~KMovistoDoc()
{
}

void KMovistoDoc::newDoc(int resetFileName)
{
  int current3DMode = molecule->Get3DMode();
  modified = TRUE;
  MVTFile->Reset(resetFileName);
  // reset and delete existing molecule
  molecule->ResetCoordinateSystem();
  molecule->SetZoomFactor(1.0);
  if(!molecule->InitMolecule())
  {
    // Delete old molecule was not successfull
    QMessageBox::critical( 0, "Error!", QString("Initialising molecule data was not successfull!"));
  }
  molecule->SetTitle("Untitled");
  // read (and if necesary write default) parameters from initialisation file
  if(!readInitFile())
  {
    // reading the initialisation file fails
    initfile.throwErrorMsg();
  }
  molecule->Set3DMode(current3DMode);
  // document changed...
  emit documentChanged();
}

bool KMovistoDoc::save()
{
  return(saveAs(MVTFile->GetFileName()));
}

bool KMovistoDoc::saveAs(const QString &absfilename)
{
  modified = FALSE;
  int i = 0;
  CMVTAtom MVTAtom;
  CMVTBond MVTBond;
  CMVTCoordinate MVTCoordinate;
  MVTFile->Reset(0);
  // section: atoms
  for(i = 1; i <= molecule->GetNumberOfAtoms(); i++)
  {
    MVTAtom.SetLabel(molecule->GetLabel(i - 1));
    MVTAtom.SetColor(molecule->GetColor(i - 1));
    MVTAtom.SetRadius(molecule->GetRadius(i - 1));
    MVTAtom.SetOrdinalNumber(molecule->GetOrdinalNumber(i - 1));
    MVTAtom.SetX((molecule->GetAtomCoordinate(i - 1, 0, 0)).x);
    MVTAtom.SetY((molecule->GetAtomCoordinate(i - 1, 0, 0)).y);
    MVTAtom.SetZ((molecule->GetAtomCoordinate(i - 1, 0, 0)).z);
    if(!MVTFile->SetAtom(&MVTAtom)) return(FALSE);
  }
  // section: bonds  
  for(i = 1; i <= molecule->GetNumberOfBonds(); i++)
  {
    MVTBond.SetLabel(i);
    MVTBond.SetAtomLabel(1, molecule->GetLabel((molecule->GetBond(i - 1)).x));
    MVTBond.SetAtomLabel(2, molecule->GetLabel((molecule->GetBond(i - 1)).y));
    if(!MVTFile->SetBond(&MVTBond)) return(FALSE);
  }
  // section: title
  MVTFile->SetTitle(molecule->GetTitle());
  // section: matrix
  MVTCoordinate.SetX((molecule->GetCoordinateSystem(X_AXIS)).x);
  MVTCoordinate.SetY((molecule->GetCoordinateSystem(X_AXIS)).y);
  MVTCoordinate.SetZ((molecule->GetCoordinateSystem(X_AXIS)).z);
  if(!MVTFile->SetCoordinateSystem(X_AXIS, &MVTCoordinate)) return(FALSE);
  MVTCoordinate.SetX((molecule->GetCoordinateSystem(Y_AXIS)).x);
  MVTCoordinate.SetY((molecule->GetCoordinateSystem(Y_AXIS)).y);
  MVTCoordinate.SetZ((molecule->GetCoordinateSystem(Y_AXIS)).z);
  if(!MVTFile->SetCoordinateSystem(Y_AXIS, &MVTCoordinate)) return(FALSE);
  MVTCoordinate.SetX((molecule->GetCoordinateSystem(Z_AXIS)).x);
  MVTCoordinate.SetY((molecule->GetCoordinateSystem(Z_AXIS)).y);
  MVTCoordinate.SetZ((molecule->GetCoordinateSystem(Z_AXIS)).z);
  if(!MVTFile->SetCoordinateSystem(Z_AXIS, &MVTCoordinate)) return(FALSE);
  // section: translation
  MVTCoordinate.SetX((molecule->GetTranslation()).x);
  MVTCoordinate.SetY((molecule->GetTranslation()).y);
  MVTCoordinate.SetZ((molecule->GetTranslation()).z);
  MVTFile->SetTranslation(&MVTCoordinate);
  // section: settings
  MVTFile->SetBondRadius(molecule->GetBondRadius());  
  MVTFile->SetBondFudge(molecule->GetBondFudge());
  MVTFile->SetBondColor(molecule->GetBondColor());
  MVTFile->SetRadiusReduction(molecule->GetRadiusFactor());
  MVTFile->SetPerspective(molecule->GetPerspectiveFactor());
  MVTFile->SetLabel(molecule->GetLabelType());
  // save all
  return(MVTFile->SaveFile(absfilename));
}

bool KMovistoDoc::load(const QString &absfilename)
{
  newDoc();
  modified = FALSE;
  int i = 0;
  ELEMENT element;
  CMVTAtom MVTAtom;
  CMVTBond MVTBond;
  CMVTCoordinate MVTCoordinate;
  if(MVTFile->LoadFile(absfilename))
  {
    // section: atoms
    molecule->SetZoomFactor(1.0);
    if(molecule->DeleteMolecule())
    {
      for(i = 1; i <= MVTFile->GetNumberOfAtoms(); i++)
      {
        element.x = (MVTFile->GetAtom(i))->GetX();
        element.y = (MVTFile->GetAtom(i))->GetY();
        element.z = (MVTFile->GetAtom(i))->GetZ();
        element.OrdinalNumber = (MVTFile->GetAtom(i))->GetOrdinalNumber();
        element.Label = (MVTFile->GetAtom(i))->GetLabel();
        element.Radius = (MVTFile->GetAtom(i))->GetRadius();
        element.Color = (MVTFile->GetAtom(i))->GetColor();
        element.Pattern = periodicsystem->GetPattern(element.OrdinalNumber);
        if(!molecule->AddAtom(element, 0))
        {
          newDoc();
          return(FALSE);
        }
      }
      // section: bonds
      for(i = 1; i <= MVTFile->GetNumberOfBonds(); i++)
      {
        if(!molecule->AddBond((MVTFile->GetBond(i))->GetAtomLabel(1) - 1, (MVTFile->GetBond(i))->GetAtomLabel(2) - 1))
        {
          newDoc();
          return(FALSE);
        }
      }
      // section: title
      if(!molecule->SetTitle(MVTFile->GetTitle()))
      {
        newDoc();
        return(FALSE);
      }
      // section: matrix
      if(!molecule->SetMatrix((MVTFile->GetCoordinateSystem(X_AXIS))->GetX(),
                              (MVTFile->GetCoordinateSystem(X_AXIS))->GetY(),
                              (MVTFile->GetCoordinateSystem(X_AXIS))->GetZ(),
                              (MVTFile->GetCoordinateSystem(Y_AXIS))->GetX(),
                              (MVTFile->GetCoordinateSystem(Y_AXIS))->GetY(),
                              (MVTFile->GetCoordinateSystem(Y_AXIS))->GetZ(),
                              (MVTFile->GetCoordinateSystem(Z_AXIS))->GetX(),
                              (MVTFile->GetCoordinateSystem(Z_AXIS))->GetY(),
                              (MVTFile->GetCoordinateSystem(Z_AXIS))->GetZ(),
                              1))
      {
         if(QMessageBox::warning(0,
                                 QString("Warning"),
                                 QString("This file contains an invalid rotational matrix.\nDo you want to reset and try to continue?"),
                                 QString("No"),
                                 QString("Yes"),
                                 QString::null,
                                 1) == 1)
        {
          // rotational matrix contains errors and will be reset
          molecule->ResetCoordinateSystem();
          MVTCoordinate.SetX((molecule->GetCoordinateSystem(X_AXIS)).x);
          MVTCoordinate.SetY((molecule->GetCoordinateSystem(X_AXIS)).y);
          MVTCoordinate.SetZ((molecule->GetCoordinateSystem(X_AXIS)).z);
          if(!MVTFile->SetCoordinateSystem(X_AXIS, &MVTCoordinate))
          {
            newDoc();
            return(FALSE);
          }
          MVTCoordinate.SetX((molecule->GetCoordinateSystem(Y_AXIS)).x);
          MVTCoordinate.SetY((molecule->GetCoordinateSystem(Y_AXIS)).y);
          MVTCoordinate.SetZ((molecule->GetCoordinateSystem(Y_AXIS)).z);
          if(!MVTFile->SetCoordinateSystem(Y_AXIS, &MVTCoordinate))
          {
            newDoc();
            return(FALSE);
          }
          MVTCoordinate.SetX((molecule->GetCoordinateSystem(Z_AXIS)).x);
          MVTCoordinate.SetY((molecule->GetCoordinateSystem(Z_AXIS)).y);
          MVTCoordinate.SetZ((molecule->GetCoordinateSystem(Z_AXIS)).z);
          if(!MVTFile->SetCoordinateSystem(Z_AXIS, &MVTCoordinate))
          {
            newDoc();
            return(FALSE);
          }
        }
        else
        {
          // user aborts
          newDoc();
          return(FALSE);
        }
      }
      // section: translation
      if(!molecule->SetTranslation((MVTFile->GetTranslation())->GetX(),
                                   (MVTFile->GetTranslation())->GetY(),
                                   (MVTFile->GetTranslation())->GetZ()))
      {
         if(QMessageBox::warning(0,
                                 QString("Warning"),
                                 QString("This file contains an invalid translation values.\nDo you want to reset and try to continue?"),
                                 QString("No"),
                                 QString("Yes"),
                                 QString::null,
                                 1) == 1)
        {
          // translation values invalid and will be reset
          if(!molecule->SetTranslation(0.0, 0.0, 0.0))
          {
            newDoc();
            return(FALSE);
          }
        }
        else
        {
          // user aborts
          newDoc();
          return(FALSE);
        }
      }
      // section: settings
      if(MVTFile->GetBondRadius() > 1.0 || MVTFile->GetBondRadius() < 0.0 ||
         MVTFile->GetBondFudge() > 10.0 || MVTFile->GetBondFudge() < 0.0 ||
         ((MVTFile->GetBondColor()).rgb() & 16777215) > 16777215 ||
         MVTFile->GetRadiusReduction() > 1.0 || MVTFile->GetRadiusReduction() < 0.0 ||
         MVTFile->GetPerspective() > 10.0 || MVTFile->GetPerspective() < 0.0 ||
         MVTFile->GetLabel() > 3 || MVTFile->GetLabel() < 0)
        {
          // read values are not valid
           if(QMessageBox::warning(0,
                                   QString("Warning"),
                                   QString("This file contains invalid settings.\nDo you want to reset and try to continue?"),
                                   QString("No"),
                                   QString("Yes"),
                                   QString::null,
                                   1) == 1)
          {
            // settings contain errors and will be reset
            if(!molecule->InitMolecule())
            {
              // reset not successfull
              newDoc();
              return(FALSE);
            }
          }
          else
          {
            // user aborts
            newDoc();
            return(FALSE);
          }
        }
        else
        {
          // read values are valid
          if(!(molecule->SetBondRadius(MVTFile->GetBondRadius()) &&
               molecule->SetBondFudge(MVTFile->GetBondFudge(), 0) &&
               molecule->SetBondColor(MVTFile->GetBondColor()) &&
               molecule->SetRadiusFactor(MVTFile->GetRadiusReduction()) &&
               molecule->SetPerspectiveFactor(MVTFile->GetPerspective()) &&
               molecule->SetLabelType(MVTFile->GetLabel())))
          {
            // error
            newDoc();
            return(FALSE);
          }
        }
        // reading all sections was successfull
    }
    else
    {
      // Delete old molecule was not successfull
      QMessageBox::critical( 0, "Error!", QString("Delete old molecule was not successfull!"));
      newDoc();
      return(FALSE);
    }
    emit documentChanged();
    return(TRUE);
  }
  else
  {
    // loading file was not successfull
    newDoc();
    return(FALSE);
  }
}

/////////////////////////////////////////////////
// isModified
bool KMovistoDoc::isModified() const
{
  return modified;
}

/////////////////////////////////////////////////
// importG94File
int KMovistoDoc::importG94File(int CalNumber, CG94File &G94File)
{
  int i = 1;
  modified = TRUE;
  CG94Atomcoordinate G94Atomcoordinate;
  ELEMENT element;
  if(G94File.GetNumberOfAtoms(CalNumber) > 0)
  {
    // set data
    if(molecule->DeleteMolecule())
    {
      molecule->ResetMovements();
      molecule->SetZoomFactor(1.0);
      molecule->SetTitle(G94File.GetTitle(CalNumber));
      for(i = 1; i <= G94File.GetNumberOfAtoms(CalNumber); i++)
      {
        G94File.GetCoordinate(CalNumber, i, G94Atomcoordinate);
        element.x = G94Atomcoordinate.GetX();
        element.y = G94Atomcoordinate.GetY();
        element.z = G94Atomcoordinate.GetZ();
        element.OrdinalNumber = G94Atomcoordinate.GetAtomnumber();
        element.Label = i;
        element.Radius = periodicsystem->GetRadius(element.OrdinalNumber);
        element.Color = periodicsystem->GetColor(element.OrdinalNumber);
        element.Pattern = periodicsystem->GetPattern(element.OrdinalNumber);
        if(!molecule->AddAtom(element)) return(FALSE);
      }
    }
    else
    {
      // Delete old molecule was not successfull
      QMessageBox::critical( 0, "Error!", QString("Delete old molecule was not successfull!"));
      return(FALSE);
    }
  }
  else
  {
    // Gaussian file does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("Gaussian file does not contain atoms!"));
    return(FALSE);
  }
  // import successfull
  emit documentChanged();
  return(TRUE);
}

/////////////////////////////////////////////////
// importXYZFile
int KMovistoDoc::importXYZFile(CXYZFile &XYZFile)
{
  int i = 1;
  modified = TRUE;
  CXYZCoordinate XYZCoordinate;
  ELEMENT element;
  if(XYZFile.GetNumberOfAtoms() > 0)
  {
    // set data
    if(molecule->DeleteMolecule())
    {
      molecule->ResetMovements();
      molecule->SetZoomFactor(1.0);
      molecule->SetTitle(XYZFile.GetTitle());
      for(i = 1; i <= XYZFile.GetNumberOfAtoms(); i++)
      {
        XYZFile.GetCoordinate(i, XYZCoordinate);
        element.x = XYZCoordinate.GetX();
        element.y = XYZCoordinate.GetY();
        element.z = XYZCoordinate.GetZ();
        element.OrdinalNumber = periodicsystem->GetOrdinalNumber(XYZCoordinate.GetAtomsymbol());
        element.Label = i;
        element.Radius = periodicsystem->GetRadius(element.OrdinalNumber);
        element.Color = periodicsystem->GetColor(element.OrdinalNumber);
        element.Pattern = periodicsystem->GetPattern(element.OrdinalNumber);
        molecule->AddAtom(element);
      }
    }
    else
    {
      // Delete old molecule was not successfull
      QMessageBox::critical( 0, "Error!", QString("Removing previous molecule was not successfull!"));
      return(FALSE);
    }
  }
  else
  {
    // XYZ file does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("XYZ file does not contain atoms!"));
    return(FALSE);
  }
  // import successfull
  emit documentChanged();
  return(TRUE);
}



/////////////////////////////////////////////////
// imortPluginFile
int KMovistoDoc::importPluginFile(kmpi::CIPluginMolecule &pluginMolecule, kmpi::CIPluginElements &pluginElements)
{
  modified = TRUE;
  if(pluginMolecule.getNumberOfAtoms() > 0)
  {
    if(!molecule->DeleteMolecule() || !periodicsystem->SetElements(pluginElements) || !molecule->SetMolecule(pluginMolecule))
    {
      // invalid molecule data
      QMessageBox::critical( 0, "Error!", QString("Molecule data not successfully imported!"));
      return(FALSE);
    }
  }
  else
  {
    // XYZ file does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("File does not contain atoms!"));
    return(FALSE);
  }
  // import successfull
  emit documentChanged();
  return(TRUE);
}

/////////////////////////////////////////////////
// exportXYZFile
int KMovistoDoc::exportXYZFile(CXYZFile &XYZFile)
{
  int i = 1;
  CXYZCoordinate XYZCoordinate;
  POINT3D point3d;
  if(molecule->GetNumberOfAtoms() > 0)
  {
    // set data
    XYZFile.SetTitle(molecule->GetTitle());
    for(i = 0; i < molecule->GetNumberOfAtoms(); i++)
    {
      XYZCoordinate.SetAtomsymbol(periodicsystem->GetSymbol(molecule->GetOrdinalNumber(i)));
      point3d = molecule->GetAtomCoordinate(i, 1, 1);
      XYZCoordinate.SetX(point3d.x);
      XYZCoordinate.SetY(point3d.y);
      XYZCoordinate.SetZ(point3d.z);
      XYZFile.SetCoordinate(XYZCoordinate);
    }
  }
  else
  {
    // Document does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("Document does not contain atoms!"));
    return(FALSE);
  }
  // export successfull
  return(TRUE);
}

/////////////////////////////////////////////////
// exportPOVFile
int KMovistoDoc::exportPOVFile(CPOVFile &POVFile)
{
  int i = 1;
  CPOVAtom POVAtom;
  CPOVBond POVBond;
  CPOVCoordinate POVCoordinate;
  POINT3D point3d;
  INTPOINT2D intpoint2d;
  if(molecule->GetNumberOfAtoms() > 0)
  {
    // set general parameter
    POVFile.SetTitle(molecule->GetTitle());
    POVFile.SetHeader("*** This file was generated by KMovisto ***");
    POVFile.SetFoot("*** End of generated file ***");
    POVFile.SetLight1Enabled(Light1EnabledPOV);
    POVFile.SetLight2Enabled(Light2EnabledPOV);
    POVFile.SetLight3Enabled(Light3EnabledPOV);
    POVFile.SetLight1(Light1POV);
    POVFile.SetLight2(Light2POV);
    POVFile.SetLight3(Light3POV);
    POVFile.SetShadowless(ShadowlessPOV);
    POVFile.SetCameraRight(CameraRightPOV);
    POVFile.SetCameraUp(CameraUpPOV);
    POVFile.SetCameraDirection(CameraDirectionPOV);
    POVFile.SetCameraLocation(CameraLocationPOV);
    POVFile.SetCameraLookA(CameraLookAtPOV);
    switch(TextureNumberPOV)
    {
      case 1:
      {
        POVFile.SetIncludeFile("");
        break;
      }
      case 2:
      {
        POVFile.SetIncludeFile("");
        break;
      }
      case 3:
      {
        POVFile.SetIncludeFile("");
        break;
      }
      case 4:
      {
        POVFile.SetIncludeFile(IncludePOV);
        break;
      }
      default:
      {
        POVFile.SetIncludeFile("");
        break;
      }
    }
    // set atom parameter
    for(i = 0; i < molecule->GetNumberOfAtoms(); i++)
    {
      POVAtom.SetLabel(i + 1);
      switch(TextureNumberPOV)
      {
        case 1:
        {
          POVAtom.SetTexture("Phong_Shiny");
          break;
        }
        case 2:
        {
          POVAtom.SetTexture("Dull");
          break;
        }
        case 3:
        {
          POVAtom.SetTexture("Metal");
          break;
        }
        case 4:
        {
          POVAtom.SetTexture(TexturePOV);
          break;
        }
        default:
        {
          POVAtom.SetTexture("Phong_Shiny");
          break;
        }
      }
      if(AlternateAtomColorPOV)
        POVAtom.SetColor(AtomColorPOV);
      else
        POVAtom.SetColor(periodicsystem->GetColor(molecule->GetOrdinalNumber(i)));
      if(AlternateAtomRadiusPOV)
        POVAtom.SetRadius(molecule->GetRadiusFactor() * AtomRadiusPOV);
      else
        POVAtom.SetRadius(molecule->GetRadiusFactor() * periodicsystem->GetRadius(molecule->GetOrdinalNumber(i)));
      point3d = molecule->GetAtomCoordinate(i, 1, 1);
      POVAtom.SetX(point3d.x);
      POVAtom.SetY(point3d.y);
      POVAtom.SetZ(point3d.z);
      POVFile.SetAtom(&POVAtom);
    }
    // set bond parameter
    for(i = 0; i < molecule->GetNumberOfBonds(); i++)
    {
      POVBond.SetLabel(i + 1);
      switch(TextureNumberPOV)
      {
        case 1:
        {
          POVBond.SetTexture("Phong_Shiny");
          break;
        }
        case 2:
        {
          POVBond.SetTexture("Dull");
          break;
        }
        case 3:
        {
          POVBond.SetTexture("Metal");
          break;
        }
        case 4:
        {
          POVBond.SetTexture(TexturePOV);
          break;
        }
        default:
        {
          POVBond.SetTexture("Phong_Shiny");
          break;
        }
      }
      if(AlternateBondColorPOV)
        POVBond.SetColor(BondColorPOV);
      else
        POVBond.SetColor(molecule->GetBondColor());
      POVBond.SetRadius(BondRadiusPOV);
      intpoint2d = molecule->GetBond(i);
      point3d = molecule->GetAtomCoordinate(intpoint2d.x, 1, 1);
      POVBond.SetX(1, point3d.x);
      POVBond.SetY(1, point3d.y);
      POVBond.SetZ(1, point3d.z);
      point3d = molecule->GetAtomCoordinate(intpoint2d.y, 1, 1);
      POVBond.SetX(2, point3d.x);
      POVBond.SetY(2, point3d.y);
      POVBond.SetZ(2, point3d.z);
      POVFile.SetBond(&POVBond);
    }
  }
  else
  {
    // Document does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("Document does not contain atoms!"));
    return(FALSE);
  }
  // export successfull
  return(TRUE);
}

/////////////////////////////////////////////////
// exportVRMLFile
int KMovistoDoc::exportVRMLFile(CVRMLFile &VRMLFile)
{
  int i = 1;
  CVRMLAtom VRMLAtom;
  CVRMLBond VRMLBond;
  POINT3D point3d;
  INTPOINT2D intpoint2d;
  if(molecule->GetNumberOfAtoms() > 0)
  {
    // set general parameter
    VRMLFile.SetTitle(molecule->GetTitle());
    VRMLFile.SetHeader("*** This file was generated by KMovisto ***");
    VRMLFile.SetFoot("*** End of generated file ***");
    VRMLFile.SetEnableTextureMapping(EnableTextureMappingVRML);
    VRMLFile.SetImageTextureURL(ImageTextureURLVRML);
    VRMLFile.SetEnableDiffuseColor(EnableDiffuseColorVRML);
    VRMLFile.SetEnableEmissiveColor(EnableEmissiveColorVRML);
    VRMLFile.SetEnableSpecularColor(EnableSpecularColorVRML);
    VRMLFile.SetDiffuseColorFactor(DiffuseColorFactorVRML);
    VRMLFile.SetEmissiveColorFactor(EmissiveColorFactorVRML);
    VRMLFile.SetSpecularColorFactor(SpecularColorFactorVRML);
    VRMLFile.SetShininess(ShininessVRML);
    // set atom parameter
    for(i = 0; i < molecule->GetNumberOfAtoms(); i++)
    {
      VRMLAtom.SetLabel(i + 1);
      if(AlternateAtomColorVRML)
        VRMLAtom.SetColor(AtomColorVRML);
      else
        VRMLAtom.SetColor(periodicsystem->GetColor(molecule->GetOrdinalNumber(i)));
      if(AlternateAtomRadiusVRML)
        VRMLAtom.SetRadius(molecule->GetRadiusFactor() * AtomRadiusVRML);
      else
        VRMLAtom.SetRadius(molecule->GetRadiusFactor() * periodicsystem->GetRadius(molecule->GetOrdinalNumber(i)));
      point3d = molecule->GetAtomCoordinate(i, 1, 1);
      VRMLAtom.SetX(point3d.x);
      VRMLAtom.SetY(point3d.y);
      VRMLAtom.SetZ(point3d.z);
      VRMLFile.SetAtom(&VRMLAtom);
    }
    // set bond parameter
    for(i = 0; i < molecule->GetNumberOfBonds(); i++)
    {
      VRMLBond.SetLabel(i + 1);

      if(AlternateBondColorVRML)
        VRMLBond.SetColor(BondColorVRML);
      else
        VRMLBond.SetColor(molecule->GetBondColor());
      VRMLBond.SetRadius(BondRadiusVRML);
      intpoint2d = molecule->GetBond(i);
      point3d = molecule->GetAtomCoordinate(intpoint2d.x, 1, 1);
      VRMLBond.SetX(1, point3d.x);
      VRMLBond.SetY(1, point3d.y);
      VRMLBond.SetZ(1, point3d.z);
      point3d = molecule->GetAtomCoordinate(intpoint2d.y, 1, 1);
      VRMLBond.SetX(2, point3d.x);
      VRMLBond.SetY(2, point3d.y);
      VRMLBond.SetZ(2, point3d.z);
      VRMLFile.SetBond(&VRMLBond);
    }
  }
  else
  {
    // Document does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("Document does not contain atoms!"));
    return(FALSE);
  }
  // export successfull
  return(TRUE);
}

/////////////////////////////////////////////////
// exportPluginFile
int KMovistoDoc::exportPluginFile(kmpi::CIPluginMolecule &pluginMolecule, kmpi::CIPluginElements &pluginElements)
{
  if(molecule->GetNumberOfAtoms() > 0)
  {
    // set data
    if(!molecule->GetMolecule(pluginMolecule, pluginElements))
    {
      // invalid molecule data
      QMessageBox::critical( 0, "Error!", QString("Molecule data not successfully exported!"));
      return(FALSE);
    }
  }
  else
  {
    // Document does not contain atoms
    QMessageBox::critical( 0, "Error!", QString("Document does not contain atoms!"));
    return(FALSE);
  }
  // export successfull
  return(TRUE);
}

/////////////////////////////////////////////////
// getWorkDir
QString KMovistoDoc::getWorkDir(int filetype)
{
  QString tmp_dir = "";
  switch(filetype)
  {
    case G94IMP:
    {
      return(G94ImpWorkDir);
      break;
    }
    case XYZIMP:
    {
      return(XYZImpWorkDir);
      break;
    }
    case POVEXP:
    {
      return(POVExpWorkDir);
      break;
    }
    case XYZEXP:
    {
      return(XYZExpWorkDir);
      break;
    }
    case VRMLEXP:
    {
      return(VRMLExpWorkDir);
      break;
    }
    default:
    {
      return("/");
    }
  }
}

/////////////////////////////////////////////////
// getFilter
QString KMovistoDoc::getFilter(int filetype)
{
  switch(filetype)
  {
    case G94IMP:
    {
      return(G94ImpFilter);
      break;
    }
    case XYZIMP:
    {
      return(XYZImpFilter);
      break;
    }
    case POVEXP:
    {
      return(POVExpFilter);
      break;
    }
    case XYZEXP:
    {
      return(XYZExpFilter);
      break;
    }
    case VRMLEXP:
    {
      return(VRMLExpFilter);
      break;
    }
    default:
    {
      return("*");
    }
  }
}

/////////////////////////////////////////////////
// setWorkDir
void KMovistoDoc::setWorkDir(QString newWorkDir, int filetype)
{
  switch(filetype)
  {
    case G94IMP:
    {
      G94ImpWorkDir = newWorkDir;
      break;
    }
    case XYZIMP:
    {
      XYZImpWorkDir = newWorkDir;
      break;
    }
    case POVEXP:
    {
      POVExpWorkDir = newWorkDir;
      break;
    }
    case XYZEXP:
    {
      XYZExpWorkDir = newWorkDir;
      break;
    }
    case VRMLEXP:
    {
      VRMLExpWorkDir = newWorkDir;
      break;
    }
    default:
    {
      G94ImpWorkDir = "/";
      XYZImpWorkDir = "/";
      POVExpWorkDir = "/";
      XYZExpWorkDir = "/";
      VRMLExpWorkDir = "/";
    }
  }
}

/////////////////////////////////////////////////
// setFilter
void KMovistoDoc::setFilter(QString newFilter, int filetype)
{
  switch(filetype)
  {
    case G94IMP:
    {
      G94ImpFilter = newFilter;
      break;
    }
    case XYZIMP:
    {
      XYZImpFilter = newFilter;
      break;
    }
    case POVEXP:
    {
      POVExpFilter = newFilter;
      break;
    }
    case XYZEXP:
    {
      XYZExpFilter = newFilter;
      break;
    }
    case VRMLEXP:
    {
      VRMLExpFilter = newFilter;
      break;
    }
    default:
    {
      G94ImpFilter = "*";
      XYZImpFilter = "*";
      POVExpFilter = "*";
      XYZExpFilter = "*";
      VRMLExpFilter = "*";
    }
  }
}

/////////////////////////////////////////////////////////////////////////////
// setPeriodicSystem
void KMovistoDoc::setPeriodicSystem(CKPeriodicSystem *ps)
{
  modified = TRUE;
  *periodicsystem = *ps;
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setMolecule
void KMovistoDoc::setMolecule(CKGLMolecule *mol)
{
  modified = TRUE;
  *molecule = *mol;
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setRadiusReduction
void KMovistoDoc::setRadiusReduction(float RadiusReduction)
{
  modified = TRUE;
  molecule->SetRadiusFactor(RadiusReduction);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setPerspective
void KMovistoDoc::setPerspective(float Perspective)
{
  modified = TRUE;
  molecule->SetPerspectiveFactor(Perspective);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setLabel
void KMovistoDoc::setLabel(int Label)
{
  modified = TRUE;
  molecule->SetLabelType(Label);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setAtomSlices
void KMovistoDoc::setAtomSlices(int AtomSlices)
{
  modified = TRUE;
  molecule->SetAtomSlices(AtomSlices);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setAtomStacks
void KMovistoDoc::setAtomStacks(int AtomStacks)
{
  modified = TRUE;
  molecule->SetAtomStacks(AtomStacks);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setBondSlices
void KMovistoDoc::setBondSlices(int BondSlices)
{
  modified = TRUE;
  molecule->SetBondSlices(BondSlices);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setBondRadius
void KMovistoDoc::setBondRadius(float BondRadius)
{
  modified = TRUE;
  molecule->SetBondRadius(BondRadius);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setBondColor
void KMovistoDoc::setBondColor(QColor BondColor)
{
  modified = TRUE;
  molecule->SetBondColor(BondColor);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setBondFudge
void KMovistoDoc::setBondFudge(float BondFudge)
{
  modified = TRUE;
  molecule->SetBondFudge(BondFudge);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setTitle
void KMovistoDoc::setTitle(QString Title)
{
  modified = TRUE;
  molecule->SetTitle(Title);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setBGColorPrint
void KMovistoDoc::setBGColorPrint(QColor colorPrint)
{
  bgColorPrint = colorPrint;
}

/////////////////////////////////////////////////////////////////////////////
// setBGColorGL
void KMovistoDoc::setBGColorGL(QColor color)
{
  modified = TRUE;
  molecule->SetBackgroundColorGL(color);
  emit documentChanged();
}

/////////////////////////////////////////////////////////////////////////////
// setBGColor
void KMovistoDoc::setBGColor(QColor color)
{
  modified = TRUE;
  molecule->SetBackgroundColor(color);
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVAlternateAtomRadius
void KMovistoDoc::setPOVAlternateAtomRadius(int AlternateAtomRadius)
{
  modified = TRUE;
  AlternateAtomRadiusPOV = AlternateAtomRadius;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVAlternateAtomColor
void KMovistoDoc::setPOVAlternateAtomColor(int AlternateAtomColor)
{
  modified = TRUE;
  AlternateAtomColorPOV = AlternateAtomColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVAtomColor
void KMovistoDoc::setPOVAtomColor(QColor AtomColor)
{
  modified = TRUE;
  AtomColorPOV = AtomColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVAtomRadius
void KMovistoDoc::setPOVAtomRadius(float AtomRadius)
{
  modified = TRUE;
  AtomRadiusPOV = AtomRadius;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVTextureNumber
void KMovistoDoc::setPOVTextureNumber(int TextureNumber)
{
  modified = TRUE;
  TextureNumberPOV = TextureNumber;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVTexture
void KMovistoDoc::setPOVTexture(QString Texture)
{
  modified = TRUE;
  TexturePOV = Texture;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVInclude
void KMovistoDoc::setPOVInclude(QString Include)
{
  modified = TRUE;
  IncludePOV = Include;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVBondRadius
void KMovistoDoc::setPOVBondRadius(float BondRadius)
{
  modified = TRUE;
  BondRadiusPOV = BondRadius;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVAlternateBondColor
void KMovistoDoc::setPOVAlternateBondColor(int AlternateBondColor)
{
  modified = TRUE;
  AlternateBondColorPOV = AlternateBondColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVBondColor
void KMovistoDoc::setPOVBondColor(QColor BondColor)
{
  modified = TRUE;
  BondColorPOV = BondColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVLight1Enabled
void KMovistoDoc::setPOVLight1Enabled(int Light1Enabled)
{
  modified = TRUE;
  Light1EnabledPOV = Light1Enabled;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVLight2Enabled
void KMovistoDoc::setPOVLight2Enabled(int Light2Enabled)
{
  modified = TRUE;
  Light2EnabledPOV = Light2Enabled;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVLight3Enabled
void KMovistoDoc::setPOVLight3Enabled(int Light3Enabled)
{
  modified = TRUE;
  Light3EnabledPOV = Light3Enabled;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVShadowless
void KMovistoDoc::setPOVShadowless(int Shadowless)
{
  modified = TRUE;
  ShadowlessPOV = Shadowless;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVLight1
void KMovistoDoc::setPOVLight1(CPOVCoordinate* Light1)
{
  modified = TRUE;
  *Light1POV = *Light1;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVLight2
void KMovistoDoc::setPOVLight2(CPOVCoordinate* Light2)
{
  modified = TRUE;
  *Light2POV = *Light2;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVLight3
void KMovistoDoc::setPOVLight3(CPOVCoordinate* Light3)
{
  modified = TRUE;
  *Light3POV = *Light3;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVCameraRight
void KMovistoDoc::setPOVCameraRight(CPOVCoordinate* CameraRight)
{
  modified = TRUE;
  *CameraRightPOV = *CameraRight;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVCameraUp
void KMovistoDoc::setPOVCameraUp(CPOVCoordinate* CameraUp)
{
  modified = TRUE;
  *CameraUpPOV = *CameraUp;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVCameraDirection
void KMovistoDoc::setPOVCameraDirection(CPOVCoordinate* CameraDirection)
{
  modified = TRUE;
  *CameraDirectionPOV = *CameraDirection;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVCameraLocation
void KMovistoDoc::setPOVCameraLocation(CPOVCoordinate* CameraLocation)
{
  modified = TRUE;
  *CameraLocationPOV = *CameraLocation;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setPOVCameraLookAt
void KMovistoDoc::setPOVCameraLookAt(CPOVCoordinate* CameraLookAt)
{
  modified = TRUE;
  *CameraLookAtPOV = *CameraLookAt;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLAlternateAtomRadius
void KMovistoDoc::setVRMLAlternateAtomRadius(int AlternateAtomRadius)
{
  modified = TRUE;
  AlternateAtomRadiusVRML = AlternateAtomRadius;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLAlternateAtomColor
void KMovistoDoc::setVRMLAlternateAtomColor(int AlternateAtomColor)
{
  modified = TRUE;
  AlternateAtomColorVRML = AlternateAtomColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLAtomColor
void KMovistoDoc::setVRMLAtomColor(QColor AtomColor)
{
  modified = TRUE;
  AtomColorVRML = AtomColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLAtomRadius
void KMovistoDoc::setVRMLAtomRadius(float AtomRadius)
{
  modified = TRUE;
  AtomRadiusVRML = AtomRadius;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLBondRadius
void KMovistoDoc::setVRMLBondRadius(float BondRadius)
{
  modified = TRUE;
  BondRadiusVRML = BondRadius;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLAlternateBondColor
void KMovistoDoc::setVRMLAlternateBondColor(int AlternateBondColor)
{
  modified = TRUE;
  AlternateBondColorVRML = AlternateBondColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLBondColor
void KMovistoDoc::setVRMLBondColor(QColor BondColor)
{
  modified = TRUE;
  BondColorVRML = BondColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLEnableTextureMapping
void KMovistoDoc::setVRMLEnableTextureMapping(int EnableTextureMapping)
{
  modified = TRUE;
  EnableTextureMappingVRML = EnableTextureMapping;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLImageTextureURL
void KMovistoDoc::setVRMLImageTextureURL(QString ImageTextureURL)
{
  modified = TRUE;
  ImageTextureURLVRML = ImageTextureURL;
  emit documentChanged();
}


/////////////////////////////////////////////////
// setVRMLEnableDiffuseColor
void KMovistoDoc::setVRMLEnableDiffuseColor(int EnableDiffuseColor)
{
  modified = TRUE;
  EnableDiffuseColorVRML = EnableDiffuseColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLEnableEmissiveColor
void KMovistoDoc::setVRMLEnableEmissiveColor(int EnableEmissiveColor)
{
  modified = TRUE;
  EnableEmissiveColorVRML = EnableEmissiveColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLEnableSpecularColor
void KMovistoDoc::setVRMLEnableSpecularColor(int EnableSpecularColor)
{
  modified = TRUE;
  EnableSpecularColorVRML = EnableSpecularColor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLDiffuseColorFactor
void KMovistoDoc::setVRMLDiffuseColorFactor(float DiffuseColorFactor)
{
  modified = TRUE;
  DiffuseColorFactorVRML = DiffuseColorFactor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLEmissiveColorFactor
void KMovistoDoc::setVRMLEmissiveColorFactor(float EmissiveColorFactor)
{
  modified = TRUE;
  EmissiveColorFactorVRML = EmissiveColorFactor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLSpecularColorFactor
void KMovistoDoc::setVRMLSpecularColorFactor(float SpecularColorFactor)
{
  modified = TRUE;
  SpecularColorFactorVRML = SpecularColorFactor;
  emit documentChanged();
}

/////////////////////////////////////////////////
// setVRMLShininess
void KMovistoDoc::setVRMLShininess(float Shininess)
{
  modified = TRUE;
  ShininessVRML = Shininess;
  emit documentChanged();
}

/////////////////////////////////////////////////
// writeAtomProperties
int KMovistoDoc::writeAtomProperties(void)
{
  return(periodicsystem->SaveAll() &&
         molecule->SaveAtomSlicesGL() &&
         molecule->SaveAtomStacksGL());
}

/////////////////////////////////////////////////
// writeBondProperties
int KMovistoDoc::writeBondProperties(void)
{
  return(molecule->SaveBondRadiusGL() &&
         molecule->SaveBondFudge() &&
         molecule->SaveBondColor() &&
         molecule->SaveBondSlicesGL());
}

/////////////////////////////////////////////////
// writeAppearanceProperties
int KMovistoDoc::writeAppearanceProperties(void)
{
  return(molecule->SaveRadiusFactor() &&
         molecule->SavePerspectiveFactor() &&
         molecule->SaveLabelType() &&
         molecule->SaveBackgroundColor() &&
         molecule->SaveBackgroundColorGL() &&
         writePrintBGColorProperties());
}

/////////////////////////////////////////////////
// writePrintBGColorProperties
int KMovistoDoc::writePrintBGColorProperties(void)
{
  QString str_bgColorPrint = "";
  str_bgColorPrint.setNum(bgColorPrint.rgb() & 16777215);
  return(initfile.WriteProfile("Parameter", "BGColorPrint", str_bgColorPrint));
}

/////////////////////////////////////////////////
// writeDirectoryProperties
int KMovistoDoc::writeDirectoryProperties(void)
{
  return(initfile.WriteProfile("Directory", "ImportG94Dir", G94ImpWorkDir)
      && initfile.WriteProfile("Directory", "ImportXYZDir", XYZImpWorkDir)
      && initfile.WriteProfile("Directory", "ExportPOVDir", POVExpWorkDir)
      && initfile.WriteProfile("Directory", "ExportXYZDir", XYZExpWorkDir)
      && initfile.WriteProfile("Directory", "ExportVRMLDir", VRMLExpWorkDir)
      && initfile.WriteProfile("Filter", "ImportG94Filter", G94ImpFilter)
      && initfile.WriteProfile("Filter", "ImportXYZFilter", XYZImpFilter)
      && initfile.WriteProfile("Filter", "ExportPOVFilter", POVExpFilter)
      && initfile.WriteProfile("Filter", "ExportXYZFilter", XYZExpFilter)
      && initfile.WriteProfile("Filter", "ExportVRMLFilter", VRMLExpFilter));
}

/////////////////////////////////////////////////
// writePOVRayProperties
int KMovistoDoc::writePOVRayProperties(void)
{
  QString str_TextureNumberPOV;
  QString str_AlternateAtomColorPOV;
  QString str_AtomColorPOV;
  QString str_AlternateAtomRadiusPOV;
  QString str_AtomRadiusPOV;
  QString str_AlternateBondColorPOV;
  QString str_BondColorPOV;
  QString str_BondRadiusPOV;
  QString str_Light1EnabledPOV;
  QString str_Light2EnabledPOV;
  QString str_Light3EnabledPOV;
  QString str_ShadowlessPOV;
  QString str_Light1XPOV;
  QString str_Light1YPOV;
  QString str_Light1ZPOV;
  QString str_Light2XPOV;
  QString str_Light2YPOV;
  QString str_Light2ZPOV;
  QString str_Light3XPOV;
  QString str_Light3YPOV;
  QString str_Light3ZPOV;
  QString str_CameraRightXPOV;
  QString str_CameraRightYPOV;
  QString str_CameraRightZPOV;
  QString str_CameraUpXPOV;
  QString str_CameraUpYPOV;
  QString str_CameraUpZPOV;
  QString str_CameraDirectionXPOV;
  QString str_CameraDirectionYPOV;
  QString str_CameraDirectionZPOV;
  QString str_CameraLocationXPOV;
  QString str_CameraLocationYPOV;
  QString str_CameraLocationZPOV;
  QString str_CameraLookAtXPOV;
  QString str_CameraLookAtYPOV;
  QString str_CameraLookAtZPOV;  

  str_TextureNumberPOV.setNum(TextureNumberPOV);
  str_AlternateAtomColorPOV = (AlternateAtomColorPOV ? str_AlternateAtomColorPOV = "TRUE" : str_AlternateAtomColorPOV = "FALSE");
  str_AtomColorPOV.setNum(AtomColorPOV.rgb() & 16777215);
  str_AlternateAtomRadiusPOV = (AlternateAtomRadiusPOV ? str_AlternateAtomRadiusPOV = "TRUE" : str_AlternateAtomRadiusPOV = "FALSE");
  str_AtomRadiusPOV.setNum(AtomRadiusPOV);
  str_AlternateBondColorPOV = (AlternateBondColorPOV ? str_AlternateBondColorPOV = "TRUE" : str_AlternateBondColorPOV = "FALSE");
  str_BondColorPOV.setNum(BondColorPOV.rgb() & 16777215);
  str_BondRadiusPOV.setNum(BondRadiusPOV);
  str_Light1EnabledPOV = (Light1EnabledPOV ? str_Light1EnabledPOV = "TRUE" : str_Light1EnabledPOV = "FALSE");
  str_Light2EnabledPOV = (Light2EnabledPOV ? str_Light2EnabledPOV = "TRUE" : str_Light2EnabledPOV = "FALSE");
  str_Light3EnabledPOV = (Light3EnabledPOV ? str_Light3EnabledPOV = "TRUE" : str_Light3EnabledPOV = "FALSE");
  str_ShadowlessPOV = (ShadowlessPOV ? str_ShadowlessPOV = "TRUE" : str_ShadowlessPOV = "FALSE");
  str_Light1XPOV.setNum(Light1POV->GetX());
  str_Light1YPOV.setNum(Light1POV->GetY());
  str_Light1ZPOV.setNum(Light1POV->GetZ());
  str_Light2XPOV.setNum(Light2POV->GetX());
  str_Light2YPOV.setNum(Light2POV->GetY());
  str_Light2ZPOV.setNum(Light2POV->GetZ());
  str_Light3XPOV.setNum(Light3POV->GetX());
  str_Light3YPOV.setNum(Light3POV->GetY());
  str_Light3ZPOV.setNum(Light3POV->GetZ());
  str_CameraRightXPOV.setNum(CameraRightPOV->GetX());
  str_CameraRightYPOV.setNum(CameraRightPOV->GetY());
  str_CameraRightZPOV.setNum(CameraRightPOV->GetZ());
  str_CameraUpXPOV.setNum(CameraUpPOV->GetX());
  str_CameraUpYPOV.setNum(CameraUpPOV->GetY());
  str_CameraUpZPOV.setNum(CameraUpPOV->GetZ());
  str_CameraDirectionXPOV.setNum(CameraDirectionPOV->GetX());
  str_CameraDirectionYPOV.setNum(CameraDirectionPOV->GetY());
  str_CameraDirectionZPOV.setNum(CameraDirectionPOV->GetZ());
  str_CameraLocationXPOV.setNum(CameraLocationPOV->GetX());
  str_CameraLocationYPOV.setNum(CameraLocationPOV->GetY());
  str_CameraLocationZPOV.setNum(CameraLocationPOV->GetZ());
  str_CameraLookAtXPOV.setNum(CameraLookAtPOV->GetX());
  str_CameraLookAtYPOV.setNum(CameraLookAtPOV->GetY());
  str_CameraLookAtZPOV.setNum(CameraLookAtPOV->GetZ());  

  return(initfile.WriteProfile("POV-Ray", "TextureNumber", str_TextureNumberPOV)
      && initfile.WriteProfile("POV-Ray", "Texture", TexturePOV)
      && initfile.WriteProfile("POV-Ray", "Include", IncludePOV)
      && initfile.WriteProfile("POV-Ray", "AlternateAtomColor", str_AlternateAtomColorPOV)
      && initfile.WriteProfile("POV-Ray", "AtomColor", str_AtomColorPOV)
      && initfile.WriteProfile("POV-Ray", "AlternateAtomRadius", str_AlternateAtomRadiusPOV)
      && initfile.WriteProfile("POV-Ray", "AtomRadius", str_AtomRadiusPOV)
      && initfile.WriteProfile("POV-Ray", "AlternateBondColor", str_AlternateBondColorPOV)
      && initfile.WriteProfile("POV-Ray", "BondColor", str_BondColorPOV)
      && initfile.WriteProfile("POV-Ray", "BondRadius", str_BondRadiusPOV)
      && initfile.WriteProfile("POV-Ray", "Light3Enabled", str_Light3EnabledPOV)
      && initfile.WriteProfile("POV-Ray", "Light2Enabled", str_Light2EnabledPOV)
      && initfile.WriteProfile("POV-Ray", "Light1Enabled", str_Light1EnabledPOV)
      && initfile.WriteProfile("POV-Ray", "Shadowless", str_ShadowlessPOV)
      && initfile.WriteProfile("POV-Ray", "ZLight3", str_Light3ZPOV)
      && initfile.WriteProfile("POV-Ray", "YLight3", str_Light3YPOV)
      && initfile.WriteProfile("POV-Ray", "XLight3", str_Light3XPOV)
      && initfile.WriteProfile("POV-Ray", "ZLight2", str_Light2ZPOV)
      && initfile.WriteProfile("POV-Ray", "YLight2", str_Light2YPOV)
      && initfile.WriteProfile("POV-Ray", "XLight2", str_Light2XPOV)
      && initfile.WriteProfile("POV-Ray", "ZLight1", str_Light1ZPOV)
      && initfile.WriteProfile("POV-Ray", "YLight1", str_Light1YPOV)
      && initfile.WriteProfile("POV-Ray", "XLight1", str_Light1XPOV)
      && initfile.WriteProfile("POV-Ray", "ZCameraLookAt", str_CameraLookAtZPOV)
      && initfile.WriteProfile("POV-Ray", "YCameraLookAt", str_CameraLookAtYPOV)
      && initfile.WriteProfile("POV-Ray", "XCameraLookAt", str_CameraLookAtXPOV)
      && initfile.WriteProfile("POV-Ray", "ZCameraLocation", str_CameraLocationZPOV)
      && initfile.WriteProfile("POV-Ray", "YCameraLocation", str_CameraLocationYPOV)
      && initfile.WriteProfile("POV-Ray", "XCameraLocation", str_CameraLocationXPOV)
      && initfile.WriteProfile("POV-Ray", "ZCameraDirection", str_CameraDirectionZPOV)
      && initfile.WriteProfile("POV-Ray", "YCameraDirection", str_CameraDirectionYPOV)
      && initfile.WriteProfile("POV-Ray", "XCameraDirection", str_CameraDirectionXPOV)
      && initfile.WriteProfile("POV-Ray", "ZCameraUp", str_CameraUpZPOV)
      && initfile.WriteProfile("POV-Ray", "YCameraUp", str_CameraUpYPOV)
      && initfile.WriteProfile("POV-Ray", "XCameraUp", str_CameraUpXPOV)
      && initfile.WriteProfile("POV-Ray", "ZCameraRight", str_CameraRightZPOV)
      && initfile.WriteProfile("POV-Ray", "YCameraRight", str_CameraRightYPOV)
      && initfile.WriteProfile("POV-Ray", "XCameraRight", str_CameraRightXPOV));
}    

/////////////////////////////////////////////////
// writeVRMLProperties
int KMovistoDoc::writeVRMLProperties(void)
{
  QString str_AlternateAtomColorVRML;
  QString str_AtomColorVRML;
  QString str_AlternateAtomRadiusVRML;
  QString str_AtomRadiusVRML;
  QString str_AlternateBondColorVRML;
  QString str_BondColorVRML;
  QString str_BondRadiusVRML;
  QString str_EnableTextureMappingVRML;
  QString str_EnableDiffuseColorVRML;
  QString str_EnableEmissiveColorVRML;
  QString str_EnableSpecularColorVRML;
  QString str_DiffuseColorFactorVRML;
  QString str_EmissiveColorFactorVRML;
  QString str_SpecularColorFactorVRML;
  QString str_ShininessVRML;

  str_AlternateAtomColorVRML = (AlternateAtomColorVRML ? str_AlternateAtomColorVRML = "TRUE" : str_AlternateAtomColorVRML = "FALSE");
  str_AtomColorVRML.setNum(AtomColorVRML.rgb() & 16777215);
  str_AlternateAtomRadiusVRML = (AlternateAtomRadiusVRML ? str_AlternateAtomRadiusVRML = "TRUE" : str_AlternateAtomRadiusVRML = "FALSE");
  str_AtomRadiusVRML.setNum(AtomRadiusVRML);
  str_AlternateBondColorVRML = (AlternateBondColorVRML ? str_AlternateBondColorVRML = "TRUE" : str_AlternateBondColorVRML = "FALSE");
  str_BondColorVRML.setNum(BondColorVRML.rgb() & 16777215);
  str_BondRadiusVRML.setNum(BondRadiusVRML);
  str_EnableTextureMappingVRML = (EnableTextureMappingVRML ? str_EnableTextureMappingVRML = "TRUE" : str_EnableTextureMappingVRML = "FALSE");
  str_EnableDiffuseColorVRML = (EnableDiffuseColorVRML ? str_EnableDiffuseColorVRML = "TRUE" : str_EnableDiffuseColorVRML = "FALSE");
  str_EnableEmissiveColorVRML = (EnableEmissiveColorVRML ? str_EnableEmissiveColorVRML = "TRUE" : str_EnableEmissiveColorVRML = "FALSE");
  str_EnableSpecularColorVRML = (EnableSpecularColorVRML ? str_EnableSpecularColorVRML = "TRUE" : str_EnableSpecularColorVRML = "FALSE");
  str_DiffuseColorFactorVRML.setNum(DiffuseColorFactorVRML);
  str_EmissiveColorFactorVRML.setNum(EmissiveColorFactorVRML);
  str_SpecularColorFactorVRML.setNum(SpecularColorFactorVRML);
  str_ShininessVRML.setNum(ShininessVRML);

  return(initfile.WriteProfile("VRML", "AlternateAtomColor", str_AlternateAtomColorVRML)
      && initfile.WriteProfile("VRML", "AtomColor", str_AtomColorVRML)
      && initfile.WriteProfile("VRML", "AlternateAtomRadius", str_AlternateAtomRadiusVRML)
      && initfile.WriteProfile("VRML", "AtomRadius", str_AtomRadiusVRML)
      && initfile.WriteProfile("VRML", "AlternateBondColor", str_AlternateBondColorVRML)
      && initfile.WriteProfile("VRML", "BondColor", str_BondColorVRML)
      && initfile.WriteProfile("VRML", "BondRadius", str_BondRadiusVRML)
      && initfile.WriteProfile("VRML", "ImageTextureURL", ImageTextureURLVRML)
      && initfile.WriteProfile("VRML", "EnableTextureMapping", str_EnableTextureMappingVRML)
      && initfile.WriteProfile("VRML", "EnableDiffuseColor", str_EnableDiffuseColorVRML)
      && initfile.WriteProfile("VRML", "EnableEmissiveColor", str_EnableEmissiveColorVRML)
      && initfile.WriteProfile("VRML", "EnableSpecularColor", str_EnableSpecularColorVRML)
      && initfile.WriteProfile("VRML", "DiffuseColorFactor", str_DiffuseColorFactorVRML)
      && initfile.WriteProfile("VRML", "EmissiveColorFactor", str_EmissiveColorFactorVRML)
      && initfile.WriteProfile("VRML", "SpecularColorFactor", str_SpecularColorFactorVRML)
      && initfile.WriteProfile("VRML", "Shininess", str_ShininessVRML));
}

/////////////////////////////////////////////////
//setParametersToDefault
void KMovistoDoc::setParametersToDefault(void)
{
  // directories
  G94ImpWorkDir = "$HOME";
  XYZImpWorkDir = "$HOME";
  POVExpWorkDir = "$HOME";
  XYZExpWorkDir = "$HOME";
  VRMLExpWorkDir = "$HOME";
  // filters
  G94ImpFilter = "*";
  XYZImpFilter = "*";
  POVExpFilter = "*";
  XYZExpFilter = "*";
  VRMLExpFilter = "*";
  // general parameters
  molecule->SetBondFudge(1.12);
  molecule->SetBondColor(2105376);
  molecule->SetRadiusFactor(0.5);
  molecule->SetPerspectiveFactor(2.5);
  molecule->SetLabelType(0);
  molecule->SetBackgroundColor(QColor(255, 255, 255));
  molecule->SetBackgroundColorGL(QColor(0, 0, 0));
  molecule->SetAtomSlices(18);
  molecule->SetAtomStacks(9);
  molecule->SetBondSlices(9);
  molecule->SetBondRadius(0.1);        
  bgColorPrint = QColor(255, 255, 255);

  // POV-Ray parameters
  TextureNumberPOV = 1;
  TexturePOV = "";
  IncludePOV = "";
  AlternateAtomColorPOV = FALSE;
  AtomColorPOV = 8421504;
  AlternateAtomRadiusPOV = FALSE;
  AtomRadiusPOV = 1.0;
  AlternateBondColorPOV = TRUE;
  BondColorPOV = 13421772;
  BondRadiusPOV = 0.1;
  Light1EnabledPOV = TRUE;
  Light2EnabledPOV = TRUE;
  Light3EnabledPOV = FALSE;
  ShadowlessPOV = FALSE;
  Light1POV->SetX(-20.0);
  Light1POV->SetY(35.0);
  Light1POV->SetZ(35.0);
  Light2POV->SetX(0.0);
  Light2POV->SetY(0.0);
  Light2POV->SetZ(25.0);
  Light3POV->SetX(0.0);
  Light3POV->SetY(0.0);
  Light3POV->SetZ(0.0);
  CameraRightPOV->SetX(-1.33333);
  CameraRightPOV->SetY(0.0);
  CameraRightPOV->SetZ(0.0);
  CameraUpPOV->SetX(0.0);
  CameraUpPOV->SetY(1.0);
  CameraUpPOV->SetZ(0.0);
  CameraDirectionPOV->SetX(0.0);
  CameraDirectionPOV->SetY(0.0);
  CameraDirectionPOV->SetZ(1.0);
  CameraLocationPOV->SetX(0.0);
  CameraLocationPOV->SetY(0.0);
  CameraLocationPOV->SetZ(5.0);
  CameraLookAtPOV->SetX(0.0);
  CameraLookAtPOV->SetY(0.0);
  CameraLookAtPOV->SetZ(0.0);
  // VRML parameters
  AlternateAtomColorVRML = FALSE;
  AtomColorVRML = 8421504;
  AlternateAtomRadiusVRML = FALSE;
  AtomRadiusVRML = 1.0;
  AlternateBondColorVRML = TRUE;
  BondColorVRML = 13421772;
  BondRadiusVRML = 0.1;
  ImageTextureURLVRML = "";
  EnableTextureMappingVRML = FALSE;
  EnableDiffuseColorVRML = TRUE;
  EnableEmissiveColorVRML = FALSE;
  EnableSpecularColorVRML = FALSE;
  DiffuseColorFactorVRML = 1.0;
  EmissiveColorFactorVRML = 0.0;
  SpecularColorFactorVRML = 0.0;
  ShininessVRML = 1.0;
}

/////////////////////////////////////////////////
// readInitFile
int KMovistoDoc::readInitFile()
{
  // 1.) Molecule & Appearance properties
  // 2.) file properties
  // 3.) POV-Ray properties
  // 4.) VRML properties
  return(readMoleculeAppearanceProperties() &&
         readDirectoryProperties() &&
         readPOVRayProperties() &&
         readVRMLProperties());
}

/////////////////////////////////////////////////
// readMoleculeAppearanceProperties
int KMovistoDoc::readMoleculeAppearanceProperties()
{
  QString str_bgColorPrint = "";

  // background colors for printing
  if((str_bgColorPrint = initfile.GetProfile("Parameter", "BGColorPrint", "")) == "")
  {
    str_bgColorPrint = "16777215";
    if(!initfile.WriteProfile("Parameter", "BGColorPrint", str_bgColorPrint)) return(FALSE);
  }
  bgColorPrint = (QColor)str_bgColorPrint.toInt();

  return(periodicsystem->RestoreDataFromINIFile() &&
         molecule->RestoreDataFromINIFile());
}

/////////////////////////////////////////////////
// readDirectoryProperties
int KMovistoDoc::readDirectoryProperties()
{
  // working directories
  if((G94ImpWorkDir = initfile.GetProfile("Directory", "ImportG94Dir", "")) == "")
  {
    G94ImpWorkDir = "$HOME";    
    if(!initfile.WriteProfile("Directory", "ImportG94Dir", G94ImpWorkDir)) return(FALSE);
  }
  if((XYZImpWorkDir = initfile.GetProfile("Directory", "ImportXYZDir", "")) == "")
  {
    XYZImpWorkDir = "$HOME";
    if(!initfile.WriteProfile("Directory", "ImportXYZDir", XYZImpWorkDir)) return(FALSE);
  }
  if((POVExpWorkDir = initfile.GetProfile("Directory", "ExportPOVDir", "")) == "")
  {
    POVExpWorkDir = "$HOME";
    if(!initfile.WriteProfile("Directory", "ExportPOVDir", POVExpWorkDir)) return(FALSE);
  }
  if((XYZExpWorkDir = initfile.GetProfile("Directory", "ExportXYZDir", "")) == "")
  {
    XYZExpWorkDir = "$HOME";
    if(!initfile.WriteProfile("Directory", "ExportXYZDir", XYZExpWorkDir)) return(FALSE);
  }
  if((VRMLExpWorkDir = initfile.GetProfile("Directory", "ExportVRMLDir", "")) == "")
  {
    VRMLExpWorkDir = "$HOME";
    if(!initfile.WriteProfile("Directory", "ExportVRMLDir", VRMLExpWorkDir)) return(FALSE);
  }
  // filters
  if((G94ImpFilter = initfile.GetProfile("Filter", "ImportG94Filter", "")) == "")
  {
    G94ImpFilter = "*";    
    if(!initfile.WriteProfile("Filter", "ImportG94Filter", G94ImpFilter)) return(FALSE);
  }
  if((XYZImpFilter = initfile.GetProfile("Filter", "ImportXYZFilter", "")) == "")
  {
    XYZImpFilter = "*.xyz";
    if(!initfile.WriteProfile("Filter", "ImportXYZFilter", XYZImpFilter)) return(FALSE);
  }
  if((POVExpFilter = initfile.GetProfile("Filter", "ExportPOVFilter", "")) == "")
  {
    POVExpFilter = "*.pov";
    if(!initfile.WriteProfile("Filter", "ExportPOVFilter", POVExpFilter)) return(FALSE);
  }
  if((XYZExpFilter = initfile.GetProfile("Filter", "ExportXYZFilter", "")) == "")
  {
    XYZExpFilter = "*.xyz";
    if(!initfile.WriteProfile("Filter", "ExportXYZFilter", XYZExpFilter)) return(FALSE);
  }
  if((VRMLExpFilter = initfile.GetProfile("Filter", "ExportVRMLFilter", "")) == "")
  {
    VRMLExpFilter = "*.wrl";
    if(!initfile.WriteProfile("Filter", "ExportVRMLFilter", VRMLExpFilter)) return(FALSE);
  }
  // initialisation succsesfull
  return(TRUE);
}

/////////////////////////////////////////////////
// readPOVRayProperties
int KMovistoDoc::readPOVRayProperties()
{
  // string pendants (required for none string values)
  // of POV-Ray parameters
  QString str_TextureNumberPOV = "";
  QString str_AlternateAtomColorPOV = "";
  QString str_AtomColorPOV = "";
  QString str_AlternateAtomRadiusPOV = "";
  QString str_AtomRadiusPOV = "";
  QString str_AlternateBondColorPOV = "";
  QString str_BondColorPOV = "";
  QString str_BondRadiusPOV = "";
  QString str_Light1EnabledPOV = "";
  QString str_Light2EnabledPOV = "";
  QString str_Light3EnabledPOV = "";
  QString str_ShadowlessPOV;
  QString str_Light1XPOV = "";
  QString str_Light1YPOV = "";
  QString str_Light1ZPOV = "";
  QString str_Light2XPOV = "";
  QString str_Light2YPOV = "";
  QString str_Light2ZPOV = "";
  QString str_Light3XPOV = "";
  QString str_Light3YPOV = "";
  QString str_Light3ZPOV = "";
  QString str_CameraRightXPOV = "";
  QString str_CameraRightYPOV = "";
  QString str_CameraRightZPOV = "";
  QString str_CameraUpXPOV = "";
  QString str_CameraUpYPOV = "";
  QString str_CameraUpZPOV = "";
  QString str_CameraDirectionXPOV = "";
  QString str_CameraDirectionYPOV = "";
  QString str_CameraDirectionZPOV = "";
  QString str_CameraLocationXPOV = "";
  QString str_CameraLocationYPOV = "";
  QString str_CameraLocationZPOV = "";
  QString str_CameraLookAtXPOV = "";
  QString str_CameraLookAtYPOV = "";
  QString str_CameraLookAtZPOV = "";
  // POV-Ray parameters
  if((str_TextureNumberPOV = initfile.GetProfile("POV-Ray", "TextureNumber", "")) == "")
  {
    str_TextureNumberPOV = "1";
    if(!initfile.WriteProfile("POV-Ray", "TextureNumber", str_TextureNumberPOV)) return(FALSE);
  }
  TextureNumberPOV = str_TextureNumberPOV.toInt();
  if((TexturePOV = initfile.GetProfile("POV-Ray", "Texture", "")) == "")
  {
    TexturePOV = "";
    if(!initfile.WriteProfile("POV-Ray", "Texture", TexturePOV)) return(FALSE);
  }
  if((IncludePOV = initfile.GetProfile("POV-Ray", "Include", "")) == "")
  {
    IncludePOV = "";
    if(!initfile.WriteProfile("POV-Ray", "Include", IncludePOV)) return(FALSE);
  }
  if((str_AlternateAtomColorPOV = initfile.GetProfile("POV-Ray", "AlternateAtomColor", "")) == "")
  {
    str_AlternateAtomColorPOV = "FALSE";
    if(!initfile.WriteProfile("POV-Ray", "AlternateAtomColor", str_AlternateAtomColorPOV)) return(FALSE);
  }
  if(str_AlternateAtomColorPOV.upper() == "TRUE")
    AlternateAtomColorPOV = 1;
  if(str_AlternateAtomColorPOV.upper() == "FALSE")
    AlternateAtomColorPOV = 0;
  if((str_AlternateAtomColorPOV.upper() != "TRUE") && (str_AlternateAtomColorPOV.upper() != "FALSE"))
    return(FALSE);
  if((str_AtomColorPOV = initfile.GetProfile("POV-Ray", "AtomColor", "")) == "")
  {
    str_AtomColorPOV = "8421504";
    if(!initfile.WriteProfile("POV-Ray", "AtomColor", str_AtomColorPOV)) return(FALSE);
  }
  AtomColorPOV = (QColor)str_AtomColorPOV.toInt();
  if((str_AlternateAtomRadiusPOV = initfile.GetProfile("POV-Ray", "AlternateAtomRadius", "")) == "")
  {
    str_AlternateAtomRadiusPOV = "FALSE";
    if(!initfile.WriteProfile("POV-Ray", "AlternateAtomRadius", str_AlternateAtomRadiusPOV)) return(FALSE);
  }
  if(str_AlternateAtomRadiusPOV.upper() == "TRUE")
    AlternateAtomRadiusPOV = 1;
  if(str_AlternateAtomRadiusPOV.upper() == "FALSE")
    AlternateAtomRadiusPOV = 0;
  if((str_AlternateAtomRadiusPOV.upper() != "TRUE") && (str_AlternateAtomRadiusPOV.upper() != "FALSE"))
    return(FALSE);
  if((str_AtomRadiusPOV = initfile.GetProfile("POV-Ray", "AtomRadius", "")) == "")
  {
    str_AtomRadiusPOV = "1.0";
    if(!initfile.WriteProfile("POV-Ray", "AtomRadius", str_AtomRadiusPOV)) return(FALSE);
  }
  AtomRadiusPOV = str_AtomRadiusPOV.toFloat();
  if((str_AlternateBondColorPOV = initfile.GetProfile("POV-Ray", "AlternateBondColor", "")) == "")
  {
    str_AlternateBondColorPOV = "TRUE";
    if(!initfile.WriteProfile("POV-Ray", "AlternateBondColor", str_AlternateBondColorPOV)) return(FALSE);
  }
  if(str_AlternateBondColorPOV.upper() == "TRUE")
    AlternateBondColorPOV = 1;
  if(str_AlternateBondColorPOV.upper() == "FALSE")
    AlternateBondColorPOV = 0;
  if((str_AlternateBondColorPOV.upper() != "TRUE") && (str_AlternateBondColorPOV.upper() != "FALSE"))
    return(FALSE);
  if((str_BondColorPOV = initfile.GetProfile("POV-Ray", "BondColor", "")) == "")
  {
    str_BondColorPOV = "13421772";    
    if(!initfile.WriteProfile("POV-Ray", "BondColor", str_BondColorPOV)) return(FALSE);
  }
  BondColorPOV = (QColor)str_BondColorPOV.toInt();
  if((str_BondRadiusPOV = initfile.GetProfile("POV-Ray", "BondRadius", "")) == "")
  {
    str_BondRadiusPOV = "0.1";    
    if(!initfile.WriteProfile("POV-Ray", "BondRadius", str_BondRadiusPOV)) return(FALSE);
  }
  BondRadiusPOV = str_BondRadiusPOV.toFloat();
  if((str_Light3EnabledPOV = initfile.GetProfile("POV-Ray", "Light3Enabled", "")) == "")
  {
    str_Light3EnabledPOV = "FALSE";
    if(!initfile.WriteProfile("POV-Ray", "Light3Enabled", str_Light3EnabledPOV)) return(FALSE);
  }
  if(str_Light3EnabledPOV.upper() == "TRUE")
    Light3EnabledPOV = 1;
  if(str_Light3EnabledPOV.upper() == "FALSE")
    Light3EnabledPOV = 0;
  if((str_Light3EnabledPOV.upper() != "TRUE") && (str_Light3EnabledPOV.upper() != "FALSE"))
    return(FALSE);
  if((str_Light2EnabledPOV = initfile.GetProfile("POV-Ray", "Light2Enabled", "")) == "")
  {
    str_Light2EnabledPOV = "TRUE";
    if(!initfile.WriteProfile("POV-Ray", "Light2Enabled", str_Light2EnabledPOV)) return(FALSE);
  }
  if(str_Light2EnabledPOV.upper() == "TRUE")
    Light2EnabledPOV = 1;
  if(str_Light2EnabledPOV.upper() == "FALSE")
    Light2EnabledPOV = 0;
  if((str_Light2EnabledPOV.upper() != "TRUE") && (str_Light2EnabledPOV.upper() != "FALSE"))
    return(FALSE);
  if((str_Light1EnabledPOV = initfile.GetProfile("POV-Ray", "Light1Enabled", "")) == "")
  {
    str_Light1EnabledPOV = "TRUE";
    if(!initfile.WriteProfile("POV-Ray", "Light1Enabled", str_Light1EnabledPOV)) return(FALSE);
  }
  if(str_Light1EnabledPOV.upper() == "TRUE")
    Light1EnabledPOV = 1;
  if(str_Light1EnabledPOV.upper() == "FALSE")
    Light1EnabledPOV = 0;
  if((str_Light1EnabledPOV.upper() != "TRUE") && (str_Light1EnabledPOV.upper() != "FALSE"))
    return(FALSE);
  if((str_ShadowlessPOV = initfile.GetProfile("POV-Ray", "Shadowless", "")) == "")
  {
    str_ShadowlessPOV = "FALSE";
    if(!initfile.WriteProfile("POV-Ray", "Shadowless", str_ShadowlessPOV)) return(FALSE);
  }
  if(str_ShadowlessPOV.upper() == "TRUE")
    ShadowlessPOV = 1;
  if(str_ShadowlessPOV.upper() == "FALSE")
    ShadowlessPOV = 0;
  if((str_ShadowlessPOV.upper() != "TRUE") && (str_ShadowlessPOV.upper() != "FALSE"))
    return(FALSE);
  if((Light1EnabledPOV == 0 || Light2EnabledPOV == 0) && Light3EnabledPOV == 1) Light3EnabledPOV = 0;
  if(Light1EnabledPOV == 0 && Light2EnabledPOV == 1) Light2EnabledPOV = 0;
  if((str_Light3ZPOV = initfile.GetProfile("POV-Ray", "ZLight3", "")) == "")
  {
    str_Light3ZPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "ZLight3", str_Light3ZPOV)) return(FALSE);
  }
  Light3POV->SetZ(str_Light3ZPOV.toFloat());
  if((str_Light3YPOV = initfile.GetProfile("POV-Ray", "YLight3", "")) == "")
  {
    str_Light3YPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "YLight3", str_Light3YPOV)) return(FALSE);
  }
  Light3POV->SetY(str_Light3YPOV.toFloat());
  if((str_Light3XPOV = initfile.GetProfile("POV-Ray", "XLight3", "")) == "")
  {
    str_Light3XPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "XLight3", str_Light3XPOV)) return(FALSE);
  }
  Light3POV->SetX(str_Light3XPOV.toFloat());
  if((str_Light2ZPOV = initfile.GetProfile("POV-Ray", "ZLight2", "")) == "")
  {
    str_Light2ZPOV = "25.0";    
    if(!initfile.WriteProfile("POV-Ray", "ZLight2", str_Light2ZPOV)) return(FALSE);
  }
  Light2POV->SetZ(str_Light2ZPOV.toFloat());
  if((str_Light2YPOV = initfile.GetProfile("POV-Ray", "YLight2", "")) == "")
  {
    str_Light2YPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "YLight2", str_Light2YPOV)) return(FALSE);
  }
  Light2POV->SetY(str_Light2YPOV.toFloat());
  if((str_Light2XPOV = initfile.GetProfile("POV-Ray", "XLight2", "")) == "")
  {
    str_Light2XPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "XLight2", str_Light2XPOV)) return(FALSE);
  }
  Light2POV->SetX(str_Light2XPOV.toFloat());
  if((str_Light1ZPOV = initfile.GetProfile("POV-Ray", "ZLight1", "")) == "")
  {
    str_Light1ZPOV = "20.0";    
    if(!initfile.WriteProfile("POV-Ray", "ZLight1", str_Light1ZPOV)) return(FALSE);
  }
  Light1POV->SetZ(str_Light1ZPOV.toFloat());
  if((str_Light1YPOV = initfile.GetProfile("POV-Ray", "YLight1", "")) == "")
  {
    str_Light1YPOV = "35.0";    
    if(!initfile.WriteProfile("POV-Ray", "YLight1", str_Light1YPOV)) return(FALSE);
  }
  Light1POV->SetY(str_Light1YPOV.toFloat());
  if((str_Light1XPOV = initfile.GetProfile("POV-Ray", "XLight1", "")) == "")
  {
    str_Light1XPOV = "35.0";    
    if(!initfile.WriteProfile("POV-Ray", "XLight1", str_Light1XPOV)) return(FALSE);
  }
  Light1POV->SetX(str_Light1XPOV.toFloat());
  if((str_CameraLookAtZPOV = initfile.GetProfile("POV-Ray", "ZCameraLookAt", "")) == "")
  {
    str_CameraLookAtZPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "ZCameraLookAt", str_CameraLookAtZPOV)) return(FALSE);
  }
  CameraLookAtPOV->SetZ(str_CameraLookAtZPOV.toFloat());
  if((str_CameraLookAtYPOV = initfile.GetProfile("POV-Ray", "YCameraLookAt", "")) == "")
  {
    str_CameraLookAtYPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "YCameraLookAt", str_CameraLookAtYPOV)) return(FALSE);
  }
  CameraLookAtPOV->SetY(str_CameraLookAtYPOV.toFloat());
  if((str_CameraLookAtXPOV = initfile.GetProfile("POV-Ray", "XCameraLookAt", "")) == "")
  {
    str_CameraLookAtXPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "XCameraLookAt", str_CameraLookAtXPOV)) return(FALSE);
  }
  CameraLookAtPOV->SetX(str_CameraLookAtXPOV.toFloat());
  if((str_CameraLocationZPOV = initfile.GetProfile("POV-Ray", "ZCameraLocation", "")) == "")
  {
    str_CameraLocationZPOV = "5.0";    
    if(!initfile.WriteProfile("POV-Ray", "ZCameraLocation", str_CameraLocationZPOV)) return(FALSE);
  }
  CameraLocationPOV->SetZ(str_CameraLocationZPOV.toFloat());
  if((str_CameraLocationYPOV = initfile.GetProfile("POV-Ray", "YCameraLocation", "")) == "")
  {
    str_CameraLocationYPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "YCameraLocation", str_CameraLocationYPOV)) return(FALSE);
  }
  CameraLocationPOV->SetY(str_CameraLocationYPOV.toFloat());
  if((str_CameraLocationXPOV = initfile.GetProfile("POV-Ray", "XCameraLocation", "")) == "")
  {
    str_CameraLocationXPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "XCameraLocation", str_CameraLocationXPOV)) return(FALSE);
  }
  CameraLocationPOV->SetX(str_CameraLocationXPOV.toFloat());
  if((str_CameraDirectionZPOV = initfile.GetProfile("POV-Ray", "ZCameraDirection", "")) == "")
  {
    str_CameraDirectionZPOV = "1.0";    
    if(!initfile.WriteProfile("POV-Ray", "ZCameraDirection", str_CameraDirectionZPOV)) return(FALSE);
  }
  CameraDirectionPOV->SetZ(str_CameraDirectionZPOV.toFloat());
  if((str_CameraDirectionYPOV = initfile.GetProfile("POV-Ray", "YCameraDirection", "")) == "")
  {
    str_CameraDirectionYPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "YCameraDirection", str_CameraDirectionYPOV)) return(FALSE);
  }
  CameraDirectionPOV->SetY(str_CameraDirectionYPOV.toFloat());
  if((str_CameraDirectionXPOV = initfile.GetProfile("POV-Ray", "XCameraDirection", "")) == "")
  {
    str_CameraDirectionXPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "XCameraDirection", str_CameraDirectionXPOV)) return(FALSE);
  }
  CameraDirectionPOV->SetX(str_CameraDirectionXPOV.toFloat());
  if((str_CameraUpZPOV = initfile.GetProfile("POV-Ray", "ZCameraUp", "")) == "")
  {
    str_CameraUpZPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "ZCameraUp", str_CameraUpZPOV)) return(FALSE);
  }
  CameraUpPOV->SetZ(str_CameraUpZPOV.toFloat());
  if((str_CameraUpYPOV = initfile.GetProfile("POV-Ray", "YCameraUp", "")) == "")
  {
    str_CameraUpYPOV = "1.0";    
    if(!initfile.WriteProfile("POV-Ray", "YCameraUp", str_CameraUpYPOV)) return(FALSE);
  }
  CameraUpPOV->SetY(str_CameraUpYPOV.toFloat());
  if((str_CameraUpXPOV = initfile.GetProfile("POV-Ray", "XCameraUp", "")) == "")
  {
    str_CameraUpXPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "XCameraUp", str_CameraUpXPOV)) return(FALSE);
  }
  CameraUpPOV->SetX(str_CameraUpXPOV.toFloat());
  if((str_CameraRightZPOV = initfile.GetProfile("POV-Ray", "ZCameraRight", "")) == "")
  {
    str_CameraRightZPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "ZCameraRight", str_CameraRightZPOV)) return(FALSE);
  }
  CameraRightPOV->SetZ(str_CameraRightZPOV.toFloat());
  if((str_CameraRightYPOV = initfile.GetProfile("POV-Ray", "YCameraRight", "")) == "")
  {
    str_CameraRightYPOV = "0.0";    
    if(!initfile.WriteProfile("POV-Ray", "YCameraRight", str_CameraRightYPOV)) return(FALSE);
  }
  CameraRightPOV->SetY(str_CameraRightYPOV.toFloat());
  if((str_CameraRightXPOV = initfile.GetProfile("POV-Ray", "XCameraRight", "")) == "")
  {
    str_CameraRightXPOV = "-1.33333";    
    if(!initfile.WriteProfile("POV-Ray", "XCameraRight", str_CameraRightXPOV)) return(FALSE);
  }
  CameraRightPOV->SetX(str_CameraRightXPOV.toFloat());
  // initialisation succsesfull
  return(TRUE);  
}

/////////////////////////////////////////////////
// readVRMLProperties
int KMovistoDoc::readVRMLProperties()
{
  // string pendants (required for none string values)
  // of VRML parameters
  QString str_AlternateAtomColorVRML = "";
  QString str_AtomColorVRML = "";
  QString str_AlternateAtomRadiusVRML = "";
  QString str_AtomRadiusVRML = "";
  QString str_AlternateBondColorVRML = "";
  QString str_BondColorVRML = "";
  QString str_BondRadiusVRML = "";
  QString str_EnableTextureMappingVRML = "";
  QString str_EnableDiffuseColorVRML = "";
  QString str_EnableEmissiveColorVRML = "";
  QString str_EnableSpecularColorVRML = "";
  QString str_DiffuseColorFactorVRML = "";
  QString str_EmissiveColorFactorVRML = "";
  QString str_SpecularColorFactorVRML = "";
  QString str_ShininessVRML = "";
  // VRML parameters
  if((ImageTextureURLVRML = initfile.GetProfile("VRML", "ImageTextureURL", "")) == "")
  {
    ImageTextureURLVRML = "";
    if(!initfile.WriteProfile("VRML", "ImageTextureURL", ImageTextureURLVRML)) return(FALSE);
  }
  if((str_AlternateAtomColorVRML = initfile.GetProfile("VRML", "AlternateAtomColor", "")) == "")
  {
    str_AlternateAtomColorVRML = "FALSE";
    if(!initfile.WriteProfile("VRML", "AlternateAtomColor", str_AlternateAtomColorVRML)) return(FALSE);
  }
  if(str_AlternateAtomColorVRML.upper() == "TRUE")
    AlternateAtomColorVRML = 1;
  if(str_AlternateAtomColorVRML.upper() == "FALSE")
    AlternateAtomColorVRML = 0;
  if((str_AlternateAtomColorVRML.upper() != "TRUE") && (str_AlternateAtomColorVRML.upper() != "FALSE"))
    return(FALSE);
  if((str_AtomColorVRML = initfile.GetProfile("VRML", "AtomColor", "")) == "")
  {
    str_AtomColorVRML = "8421504";
    if(!initfile.WriteProfile("VRML", "AtomColor", str_AtomColorVRML)) return(FALSE);
  }
  AtomColorVRML = (QColor)str_AtomColorVRML.toInt();
  if((str_AlternateAtomRadiusVRML = initfile.GetProfile("VRML", "AlternateAtomRadius", "")) == "")
  {
    str_AlternateAtomRadiusVRML = "FALSE";
    if(!initfile.WriteProfile("VRML", "AlternateAtomRadius", str_AlternateAtomRadiusVRML)) return(FALSE);
  }
  if(str_AlternateAtomRadiusVRML.upper() == "TRUE")
    AlternateAtomRadiusVRML = 1;
  if(str_AlternateAtomRadiusVRML.upper() == "FALSE")
    AlternateAtomRadiusVRML = 0;
  if((str_AlternateAtomRadiusVRML.upper() != "TRUE") && (str_AlternateAtomRadiusVRML.upper() != "FALSE"))
    return(FALSE);
  if((str_AtomRadiusVRML = initfile.GetProfile("VRML", "AtomRadius", "")) == "")
  {
    str_AtomRadiusVRML = "1.0";
    if(!initfile.WriteProfile("VRML", "AtomRadius", str_AtomRadiusVRML)) return(FALSE);
  }
  AtomRadiusVRML = str_AtomRadiusVRML.toFloat();
  if((str_AlternateBondColorVRML = initfile.GetProfile("VRML", "AlternateBondColor", "")) == "")
  {
    str_AlternateBondColorVRML = "TRUE";
    if(!initfile.WriteProfile("VRML", "AlternateBondColor", str_AlternateBondColorVRML)) return(FALSE);
  }
  if(str_AlternateBondColorVRML.upper() == "TRUE")
    AlternateBondColorVRML = 1;
  if(str_AlternateBondColorVRML.upper() == "FALSE")
    AlternateBondColorVRML = 0;
  if((str_AlternateBondColorVRML.upper() != "TRUE") && (str_AlternateBondColorVRML.upper() != "FALSE"))
    return(FALSE);
  if((str_BondColorVRML = initfile.GetProfile("VRML", "BondColor", "")) == "")
  {
    str_BondColorVRML = "13421772";
    if(!initfile.WriteProfile("VRML", "BondColor", str_BondColorVRML)) return(FALSE);
  }
  BondColorVRML = (QColor)str_BondColorVRML.toInt();
  if((str_BondRadiusVRML = initfile.GetProfile("VRML", "BondRadius", "")) == "")
  {
    str_BondRadiusVRML = "0.1";
    if(!initfile.WriteProfile("VRML", "BondRadius", str_BondRadiusVRML)) return(FALSE);
  }
  BondRadiusVRML = str_BondRadiusVRML.toFloat();
  if((str_EnableTextureMappingVRML = initfile.GetProfile("VRML", "EnableTextureMapping", "")) == "")
  {
    str_EnableTextureMappingVRML = "FALSE";
    if(!initfile.WriteProfile("VRML", "EnableTextureMapping", str_EnableTextureMappingVRML)) return(FALSE);
  }
  if(str_EnableTextureMappingVRML.upper() == "TRUE")
    EnableTextureMappingVRML = 1;
  if(str_EnableTextureMappingVRML.upper() == "FALSE")
    EnableTextureMappingVRML = 0;
  if((str_EnableTextureMappingVRML.upper() != "TRUE") && (str_EnableTextureMappingVRML.upper() != "FALSE"))
    return(FALSE);
  if((str_EnableDiffuseColorVRML = initfile.GetProfile("VRML", "EnableDiffuseColor", "")) == "")
  {
    str_EnableDiffuseColorVRML = "TRUE";
    if(!initfile.WriteProfile("VRML", "EnableDiffuseColor", str_EnableDiffuseColorVRML)) return(FALSE);
  }
  if(str_EnableDiffuseColorVRML.upper() == "TRUE")
    EnableDiffuseColorVRML = 1;
  if(str_EnableDiffuseColorVRML.upper() == "FALSE")
    EnableDiffuseColorVRML = 0;
  if((str_EnableDiffuseColorVRML.upper() != "TRUE") && (str_EnableDiffuseColorVRML.upper() != "FALSE"))
    return(FALSE);
  if((str_EnableEmissiveColorVRML = initfile.GetProfile("VRML", "EnableEmissiveColor", "")) == "")
  {
    str_EnableEmissiveColorVRML = "FALSE";
    if(!initfile.WriteProfile("VRML", "EnableEmissiveColor", str_EnableEmissiveColorVRML)) return(FALSE);
  }
  if(str_EnableEmissiveColorVRML.upper() == "TRUE")
    EnableEmissiveColorVRML = 1;
  if(str_EnableEmissiveColorVRML.upper() == "FALSE")
    EnableEmissiveColorVRML = 0;
  if((str_EnableEmissiveColorVRML.upper() != "TRUE") && (str_EnableEmissiveColorVRML.upper() != "FALSE"))
    return(FALSE);
  if((str_EnableSpecularColorVRML = initfile.GetProfile("VRML", "EnableSpecularColor", "")) == "")
  {
    str_EnableSpecularColorVRML = "FALSE";
    if(!initfile.WriteProfile("VRML", "EnableSpecularColor", str_EnableSpecularColorVRML)) return(FALSE);
  }
  if(str_EnableSpecularColorVRML.upper() == "TRUE")
    EnableSpecularColorVRML = 1;
  if(str_EnableSpecularColorVRML.upper() == "FALSE")
    EnableSpecularColorVRML = 0;
  if((str_EnableSpecularColorVRML.upper() != "TRUE") && (str_EnableSpecularColorVRML.upper() != "FALSE"))
    return(FALSE);
  if((str_DiffuseColorFactorVRML = initfile.GetProfile("VRML", "DiffuseColorFactor", "")) == "")
  {
    str_DiffuseColorFactorVRML = "1.0";
    if(!initfile.WriteProfile("VRML", "DiffuseColorFactor", str_DiffuseColorFactorVRML)) return(FALSE);
  }
  DiffuseColorFactorVRML = str_DiffuseColorFactorVRML.toFloat();
  if((str_EmissiveColorFactorVRML = initfile.GetProfile("VRML", "EmissiveColorFactor", "")) == "")
  {
    str_EmissiveColorFactorVRML = "0.0";
    if(!initfile.WriteProfile("VRML", "EmissiveColorFactor", str_EmissiveColorFactorVRML)) return(FALSE);
  }
  EmissiveColorFactorVRML = str_EmissiveColorFactorVRML.toFloat();
  if((str_SpecularColorFactorVRML = initfile.GetProfile("VRML", "SpecularColorFactor", "")) == "")
  {
    str_SpecularColorFactorVRML = "0.0";
    if(!initfile.WriteProfile("VRML", "SpecularColorFactor", str_SpecularColorFactorVRML)) return(FALSE);
  }
  SpecularColorFactorVRML = str_SpecularColorFactorVRML.toFloat();
  if((str_ShininessVRML = initfile.GetProfile("VRML", "Shininess", "")) == "")
  {
    str_ShininessVRML = "1.0";
    if(!initfile.WriteProfile("VRML", "Shininess", str_ShininessVRML)) return(FALSE);
  }
  ShininessVRML = str_ShininessVRML.toFloat();
  // initialisation succsesfull
  return(TRUE);
}

