/***************************************************************************
                          kmovistoview.cpp  -  description
                            -------------------
    begin                : Sam Jan 22 16:27:34 CET 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmovistoview.h"

/* pixmaps */
#include "editchange.xpm"
#include "editadd.xpm"
#include "editdelete.xpm"
#include "editmove.xpm"
#include "editconnect.xpm"
#include "editdisconnect.xpm"
#include "anaglyphredgreen.xpm"
#include "anaglyphredcyan.xpm"
#include "anaglyphredblue.xpm"

/////////////////////////////////////////////////////////////////////////////
// definition of the class CDisplayFrame
/////////////////////////////////////////////////////////////////////////////

CDisplayFrame::CDisplayFrame(QWidget *parent, const char *name, WFlags f)
  : QFrame(parent, name, f)
{
}

void CDisplayFrame::mousePressEvent(QMouseEvent *event)
{
  emit mousePressed(event);
}

void CDisplayFrame::mouseMoveEvent(QMouseEvent *event)
{
  emit mouseMoved(event);
}

void CDisplayFrame::mouseReleaseEvent(QMouseEvent *event)
{
  emit mouseReleased(event);
}

void CDisplayFrame::paintEvent(QPaintEvent *paintevent)
{
  emit signalPaintEvent(paintevent);
  QFrame::paintEvent(paintevent);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CDisplayFrameGL
/////////////////////////////////////////////////////////////////////////////

CDisplayFrameGL::CDisplayFrameGL(QWidget *parent, const char *name, CKGLMolecule *pmol)
  : QGLWidget(pmol->GetGLFormat(), parent, name)
{
  pmolecule = pmol;
}

CDisplayFrameGL::~CDisplayFrameGL()
{
  pmolecule->destroyGL(this);
}

void CDisplayFrameGL::mousePressEvent(QMouseEvent *event)
{
  emit mousePressed(event);
}

void CDisplayFrameGL::mouseMoveEvent(QMouseEvent *event)
{
  emit mouseMoved(event);
}

void CDisplayFrameGL::mouseReleaseEvent(QMouseEvent *event)
{
  emit mouseReleased(event);
}

void CDisplayFrameGL::paintEvent(QPaintEvent *paintevent)
{
  emit signalPaintEvent(paintevent);
  QGLWidget::paintEvent(paintevent);
}

void CDisplayFrameGL::initializeGL()
{
  pmolecule->initializeGL();
}

void CDisplayFrameGL::paintGL()
{
  pmolecule->paintGL(this);
}

void CDisplayFrameGL::resizeGL(int width, int height)
{
  pmolecule->resizeGL(width, height);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CControlPanel
/////////////////////////////////////////////////////////////////////////////

CControlPanel::CControlPanel(QWidget *parent, const char *name, WFlags f)
  : QWidget(parent, name, f)
{
  resize(350, 130);
  setMinimumSize(QSize(350, 130));
  setCaption("ControlPanel");

  AtomBox = new QGroupBox(this, "AtomBox");
  AtomBox->setGeometry(QRect(10, 10, 340, 95));
  AtomBox->setTitle("Atom");

  AtomSymbolLabel = new QLabel(AtomBox, "AtomSymbolLabel");
  AtomSymbolLabel->setGeometry(QRect(10, 20, 110, 20));
  AtomSymbolLabel->setText("Symbol:");

  ElementComboBox = new QComboBox(FALSE, AtomBox, "ElementComboBox");
  ElementComboBox->setGeometry(QRect(130, 20, 50, 20));

  AutoConnectCheckBox = new QCheckBox(AtomBox, "AutoConnectCheckBox");
  AutoConnectCheckBox->setGeometry(QRect(10, 70, 120, 21));
  AutoConnectCheckBox->setText("A&uto connect");
  AutoConnectCheckBox->setChecked(TRUE);

  OrdinalNumberLabel = new QLabel(AtomBox, "OrdinalNumberLabel");
  OrdinalNumberLabel->setGeometry(QRect(10, 45, 110, 20));
  OrdinalNumberLabel->setText("Ordinal Number:");

  OrdinalNumber = new QLabel(AtomBox, "OrdinalNumber");
  OrdinalNumber->setGeometry(QRect(130, 45, 50, 21));
  OrdinalNumber->setText("1");
  OrdinalNumber->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));

  XPosLabel = new QLabel(AtomBox, "XPosLabel");
  XPosLabel->setGeometry(QRect(210, 20, 50, 20));
  XPosLabel->setText("X:");

  YPosLabel = new QLabel(AtomBox, "YPosLabel");
  YPosLabel->setGeometry(QRect(210, 45, 50, 20));
  YPosLabel->setText("Y:");

  ZPosLabel = new QLabel(AtomBox, "ZPosLabel");
  ZPosLabel->setGeometry(QRect(210, 70, 50, 20));
  ZPosLabel->setText("Z:");

  ZPos = new QLabel(AtomBox, "ZPos");
  ZPos->setGeometry(QRect(250, 70, 70, 20));
  ZPos->setText("0");
  ZPos->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));

  YPos = new QLabel(AtomBox, "YPos");
  YPos->setGeometry(QRect(250, 45, 70, 20));
  YPos->setText("0");
  YPos->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));

  XPos = new QLabel(AtomBox, "XPos");
  XPos->setGeometry(QRect(250, 20, 70, 20));
  XPos->setText("0");
  XPos->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));

  // initialisations
  for(int i = 1; i < 110; i++)
    ElementComboBox->insertItem(ps.GetSymbol(i));

  // connections
  connect(ElementComboBox, SIGNAL(highlighted(int)), SLOT(comboBoxHighlighted(int)));

  reset();
}

void CControlPanel::setGeometry(int x, int y, int w, int h)
{
  QWidget::setGeometry(x, y, w, h);
  AtomBox->setGeometry(QRect(10, 10, 350, 100));
}

void CControlPanel::setElement(int ordinalNumber)
{
  if(ElementComboBox && OrdinalNumber)
  {
    ElementComboBox->setCurrentItem(ordinalNumber - 1);
    OrdinalNumber->setNum(ordinalNumber);
  }
}

void CControlPanel::setX(float newX)
{
  if(XPos) XPos->setNum(newX);
}

void CControlPanel::setY(float newY)
{
  if(YPos) YPos->setNum(newY);
}

void CControlPanel::setZ(float newZ)
{
  if(ZPos) ZPos->setNum(newZ);
}

void CControlPanel::setAutoconnect(bool checked)
{
  AutoConnectCheckBox->setChecked(checked);
}

void CControlPanel::reset()
{
  setElement(1);
  setX(0);
  setY(0);
  setZ(0);
  setAutoconnect(TRUE);
}

// slots
void CControlPanel::comboBoxHighlighted(int index)
{
  emit ordinalNumberChanged(index);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CEditToolbar
/////////////////////////////////////////////////////////////////////////////

CEditToolbar::CEditToolbar(QMainWindow *parent, const char *name)
  : QToolBar(parent, name)
{
  changeModeIcon = QPixmap(editchange);
  moveModeIcon = QPixmap(editmove);
  addModeIcon = QPixmap(editadd);
  deleteModeIcon = QPixmap(editdelete);
  connectModeIcon = QPixmap(editconnect);
  disconnectModeIcon = QPixmap(editdisconnect);

  editModeChange = new QToolButton(changeModeIcon, "Change", 0, this, SLOT(slotChangeMode()), this);
  editModeMove = new QToolButton(moveModeIcon, "Move", 0, this, SLOT(slotMoveMode()), this);
  editModeAdd = new QToolButton(addModeIcon, "Add", 0, this, SLOT(slotAddMode()), this);
  editModeDelete = new QToolButton(deleteModeIcon, "Delete", 0, this, SLOT(slotDeleteMode()), this);
  editModeConnect = new QToolButton(connectModeIcon, "Connect", 0, this, SLOT(slotConnectMode()), this);
  editModeDisconnect = new QToolButton(disconnectModeIcon, "Disconnect", 0, this, SLOT(slotDisconnectMode()), this);

  editModeChange->setToggleButton(true);
  editModeMove->setToggleButton(true);
  editModeAdd->setToggleButton(true);
  editModeDelete->setToggleButton(true);
  editModeConnect->setToggleButton(true);
  editModeDisconnect->setToggleButton(true);

  QWhatsThis::add(editModeChange,"Click this button to change elements.");
  QWhatsThis::add(editModeAdd,"Click this button to add atoms.");
  QWhatsThis::add(editModeConnect,"Click this button to connect atoms.");
  QWhatsThis::add(editModeMove,"Click this button to move atoms.");
  QWhatsThis::add(editModeDelete,"Click this button to delete atoms.");
  QWhatsThis::add(editModeDisconnect,"Click this button to disconnect atoms.");
}

void CEditToolbar::setEditMode(int Mode)
{
  if(Mode != CHANGE && editModeChange->state() == QButton::On)
    editModeChange->toggle();
  if(Mode != ADD && editModeAdd->state() == QButton::On)
    editModeAdd->toggle();
  if(Mode != CONNECT && editModeConnect->state() == QButton::On)
    editModeConnect->toggle();
  if(Mode != MOVE && editModeMove->state() == QButton::On)
    editModeMove->toggle();
  if(Mode != DELETE && editModeDelete->state() == QButton::On)
    editModeDelete->toggle();
  if(Mode != DISCONNECT && editModeDisconnect->state() == QButton::On)
    editModeDisconnect->toggle();
}

void CEditToolbar::slotChangeMode()
{
  slotToggleEditModeButtons(CHANGE);
}

void CEditToolbar::slotAddMode()
{
  slotToggleEditModeButtons(ADD);
}

void CEditToolbar::slotConnectMode()
{
  slotToggleEditModeButtons(CONNECT);
}

void CEditToolbar::slotMoveMode()
{
  slotToggleEditModeButtons(MOVE);
}

void CEditToolbar::slotDeleteMode()
{
  slotToggleEditModeButtons(DELETE);
}

void CEditToolbar::slotDisconnectMode()
{
  slotToggleEditModeButtons(DISCONNECT);
}

void CEditToolbar::slotToggleEditModeButtons(int Mode)
{
  setEditMode(Mode);

  if(editModeChange->state() == QButton::Off &&
     editModeAdd->state() == QButton::Off &&
     editModeConnect->state() == QButton::Off &&
     editModeMove->state() == QButton::Off &&
     editModeDelete->state() == QButton::Off &&
     editModeDisconnect->state() == QButton::Off)
    Mode = DISABLED;

  emit editModeChanged(Mode);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CAnaglyphToolbar
/////////////////////////////////////////////////////////////////////////////

CAnaglyphToolbar::CAnaglyphToolbar(QMainWindow *parent, const char *name)
  : QToolBar(parent, name)
{
  anaglyphRedGreenModeIcon = QPixmap(anaglyphRedGreen);
  anaglyphRedCyanModeIcon = QPixmap(anaglyphRedCyan);
  anaglyphRedBlueModeIcon = QPixmap(anaglyphRedBlue);

  anaglyphModeRedGreen = new QToolButton(anaglyphRedGreenModeIcon, "Red/Green", 0, this, SLOT(slotRedGreenMode()), this);
  anaglyphModeRedCyan = new QToolButton(anaglyphRedCyanModeIcon, "Red/Cyan", 0, this, SLOT(slotRedCyanMode()), this);
  anaglyphModeRedBlue = new QToolButton(anaglyphRedBlueModeIcon, "Red/Blue", 0, this, SLOT(slotRedBlueMode()), this);

  anaglyphModeRedGreen->setToggleButton(true);
  anaglyphModeRedCyan->setToggleButton(true);
  anaglyphModeRedBlue->setToggleButton(true);

  QWhatsThis::add(anaglyphModeRedGreen,"Click this button to change to red/green mode.");
  QWhatsThis::add(anaglyphModeRedCyan,"Click this button to change to red/cyan mode.");
  QWhatsThis::add(anaglyphModeRedBlue,"Click this button to change to red/blue mode.");
}

void CAnaglyphToolbar::setAnaglyphMode(int Mode)
{
  if(anaglyphModeRedGreen->state() == QButton::On)
    anaglyphModeRedGreen->toggle();
  if(anaglyphModeRedCyan->state() == QButton::On)
    anaglyphModeRedCyan->toggle();
  if(anaglyphModeRedBlue->state() == QButton::On)
    anaglyphModeRedBlue->toggle();
  if(Mode == RED_GREEN && anaglyphModeRedGreen->state() == QButton::Off)
    anaglyphModeRedGreen->toggle();
  if(Mode == RED_CYAN && anaglyphModeRedCyan->state() == QButton::Off)
    anaglyphModeRedCyan->toggle();
  if(Mode == RED_BLUE && anaglyphModeRedBlue->state() == QButton::Off)
    anaglyphModeRedBlue->toggle();
}

void CAnaglyphToolbar::slotRedGreenMode()
{
  slotToggleAnaglyphModeButtons(RED_GREEN);
}

void CAnaglyphToolbar::slotRedCyanMode()
{
  slotToggleAnaglyphModeButtons(RED_CYAN);
}

void CAnaglyphToolbar::slotRedBlueMode()
{
  slotToggleAnaglyphModeButtons(RED_BLUE);
}

void CAnaglyphToolbar::slotToggleAnaglyphModeButtons(int Mode)
{
  setAnaglyphMode(Mode);

  if(anaglyphModeRedGreen->state() == QButton::Off &&
     anaglyphModeRedCyan->state() == QButton::Off &&
     anaglyphModeRedBlue->state() == QButton::Off)
    Mode = DISABLED;

  emit anaglyphModeChanged(Mode);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class KMovistoView
/////////////////////////////////////////////////////////////////////////////

KMovistoView::KMovistoView(QWidget *parent, KMovistoDoc *doc)
  : QWidget(parent)
{
  // initialisations
  pdoc = doc;
  resetView();
  ptoolbar = 0;
  viewID = 0;
  menuItem = "&default view";

  // group box and label
  titleGroupBox = new QGroupBox(this, "titleGroupBox");
  titleGroupBox->setFrameStyle(35);
  titleGroupBox->setLineWidth(1);
  titleGroupBox->setMidLineWidth(0);
  titleGroupBox->setFocusPolicy(QWidget::NoFocus);
  titleGroupBox->setBackgroundMode(QWidget::PaletteBackground);
  titleGroupBox->QFrame::setMargin(0);
  titleGroupBox->setAlignment(AlignLeft);
  titleGroupBox->setTitle("");

  titleLabel = new QLabel(titleGroupBox, "titleLabel");
  titleLabel->setFrameStyle(0);
  titleLabel->setLineWidth(1);
  titleLabel->setMidLineWidth(0);
  titleLabel->setText(pdoc->getMolecule()->GetTitle());
  titleLabel->setFocusPolicy(QWidget::NoFocus);
  titleLabel->setBackgroundMode(QWidget::PaletteBackground);
  titleLabel->QFrame::setMargin(0);
  titleLabel->setMargin(0);
  titleLabel->setAlignment(AlignLeft|AlignVCenter|ExpandTabs);

  // connect doc with the view
  connect(doc, SIGNAL(documentChanged()), this, SLOT(slotDocumentChanged()));
}

KMovistoView::~KMovistoView()
{
}

void KMovistoView::resetView(void)
{
  mouseOldX = 0;
  mouseOldY = 0;
  rightButtonPressed = FALSE;
  midButtonPressed = FALSE;
  leftButtonPressed = FALSE;
  controlKeyPressed = FALSE;
  shiftKeyPressed = FALSE;
}

void KMovistoView::createToolbar(QMainWindow*)
{
  ptoolbar = 0;
}

void KMovistoView::showToolbar(void)
{
  if(ptoolbar) ptoolbar->show();
}

void KMovistoView::hideToolbar(void)
{
  if(ptoolbar) ptoolbar->hide();
}

void KMovistoView::setViewID(uint id)
{
  viewID = id;
}

QString KMovistoView::getMenuItem(void)
{
  return(menuItem);
}

uint KMovistoView::getViewID(void)
{
  return(viewID);
}

int KMovistoView::rotateMolecule(int mouseX, int mouseY, int axis)
{
  if(leftButtonPressed)
  {
    if(controlKeyPressed && !shiftKeyPressed)
    {
      // control key pressed -> rotation z-axis
      switch(axis)
      {
        case FRAME_X:
          // molecule view is from x direction: rotate x axis
          pdoc->getMolecule()->Rotate(X_AXIS, -0.02 * float(mouseY - mouseOldY));
          break;
        case FRAME_Y:
          // molecule view is from y direction: rotate y axis
          pdoc->getMolecule()->Rotate(Y_AXIS, -0.02 * float(mouseY - mouseOldY));
          break;
        case FRAME_Z:
          // molecule view is from z direction: rotate z axis
          pdoc->getMolecule()->Rotate(Z_AXIS, 0.02 * float(mouseY - mouseOldY));
          break;
      }
      return(TRUE);
    }
    if(!controlKeyPressed && !shiftKeyPressed)
    {
      // control key and shift key not pressed -> rotation x-, y-axis
      switch(axis)
      {
        case FRAME_X:
          // molecule view is from x direction: rotate y/z axis
          pdoc->getMolecule()->Rotate(Y_AXIS, 0.02 * float(mouseX - mouseOldX));
          pdoc->getMolecule()->Rotate(Z_AXIS, 0.02 * float(mouseY - mouseOldY));
          break;
        case FRAME_Y:
          // molecule view is from y direction: rotate z/x axis
          pdoc->getMolecule()->Rotate(Z_AXIS, 0.02 * float(mouseX - mouseOldX));
          pdoc->getMolecule()->Rotate(X_AXIS, 0.02 * float(mouseY - mouseOldY));
          break;
        case FRAME_Z:
          // molecule view is from z direction: rotate x/y axis
          pdoc->getMolecule()->Rotate(Y_AXIS, 0.02 * float(mouseX - mouseOldX));
          pdoc->getMolecule()->Rotate(X_AXIS, 0.02 * float(mouseY - mouseOldY));
          break;
      }
      return(TRUE);
    }
  }
  return(FALSE);
}

int KMovistoView::translateMolecule(int mouseX, int mouseY, int axis)
{
  if(leftButtonPressed && shiftKeyPressed && !controlKeyPressed)
  {
    // shift key pressed -> translation
    pdoc->getMolecule()->TranslateAbsolute((mouseX  - mouseOldX), (mouseOldY - mouseY), axis);
    return(TRUE);
  }
  return(FALSE);
}

int KMovistoView::zoomMolecule(int mouseY)
{
  if(rightButtonPressed)
  {
    pdoc->getMolecule()->SetZoomFactor(pdoc->getMolecule()->GetZoomFactor() + 0.01 * float(mouseY - mouseOldY));
    if(pdoc->getMolecule()->GetZoomFactor() < 0.0) pdoc->getMolecule()->SetZoomFactor(0.0);
    if(pdoc->getMolecule()->GetZoomFactor() > 10.0) pdoc->getMolecule()->SetZoomFactor(10.0);
    return(TRUE);
  }
  return(FALSE);
}

int KMovistoView::centerMolecule(void)
{
  if(midButtonPressed)
  {
    // reset
    pdoc->getMolecule()->ResetMovements();
    return(TRUE);
  }
  return(FALSE);
}

QFrame* KMovistoView::createMainFrame(const char *name)
{
  QFrame* mainFrame;
  mainFrame = new QFrame(this, name);
  mainFrame->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  mainFrame->setFocusPolicy(QWidget::NoFocus);
  mainFrame->setBackgroundMode(QWidget::PaletteBackground);
  mainFrame->QFrame::setMargin(0);
  mainFrame->setBackgroundColor(QColor(255, 255, 255));
  return(mainFrame);
}

CDisplayFrame* KMovistoView::createDisplayFrame(QWidget *parent, const char *name)
{
  CDisplayFrame* frame;
  frame = new CDisplayFrame(parent, name);
  frame->setFrameStyle(QFrame::NoFrame);
  frame->setFocusPolicy(QWidget::NoFocus);
  frame->setBackgroundMode(QWidget::PaletteBackground);
  frame->QFrame::setMargin(0);
  frame->setBackgroundColor(QColor(255, 255, 255));
  return(frame);
}

void KMovistoView::connectDisplayFrame(CDisplayFrame* frame)
{
  connect(frame, SIGNAL(mousePressed(QMouseEvent *)), this, SLOT(slotMousePressed(QMouseEvent *)));
  connect(frame, SIGNAL(mouseMoved(QMouseEvent *)), this, SLOT(slotMouseMoved(QMouseEvent *)));
  connect(frame, SIGNAL(mouseReleased(QMouseEvent *)), this, SLOT(slotMouseReleased(QMouseEvent *)));
  connect(frame, SIGNAL(signalPaintEvent(QPaintEvent *)), this, SLOT(slotPaintEvent(QPaintEvent *)));
}

int  KMovistoView::print(QPrinter &printer, const QRect &rectClient)
{
  int success = 0;
  CKGLMolecule* printmolecule = 0;
  printmolecule = new CKGLMolecule(this, "temporary molecule");
  if(printmolecule)
  {
    *printmolecule = *(pdoc->getMolecule());
    if(printmolecule->SetView(VIEW_FRONT) &&
       printmolecule->SetBackgroundColor(pdoc->getBGColorPrint()) &&
       printmolecule->DrawStructure((QPaintDevice*)&printer, rectClient)) success = TRUE;
    delete printmolecule;
  }
  return(success);
}

// slots
void KMovistoView::slotDocumentChanged()
{
  titleLabel->setText(pdoc->getMolecule()->GetTitle());
  repaint();
}

void KMovistoView::slotMousePressed(QMouseEvent *MouseEvent)
{
  mouseOldX = MouseEvent->x();
  mouseOldY = MouseEvent->y();
  controlKeyPressed = (MouseEvent->state() == (ControlButton & ~ShiftButton));
  shiftKeyPressed = (MouseEvent->state() == (ShiftButton & ~ControlButton));
  switch (MouseEvent->button())
  {
    case LeftButton:
    {
      leftButtonPressed = TRUE;
      midButtonPressed = FALSE;
      rightButtonPressed = FALSE;
      break;
    }
    case MidButton:
    {
      leftButtonPressed = FALSE;
      midButtonPressed = TRUE;
      rightButtonPressed = FALSE;
      break;
    }
    case RightButton:
    {
      leftButtonPressed = FALSE;
      midButtonPressed = FALSE;
      rightButtonPressed = TRUE;
      break;
    }
    default:
    {
      rightButtonPressed = FALSE;
      midButtonPressed = FALSE;
      leftButtonPressed = FALSE;
    }
  }
  // emit viewChanged if doc is not modified
  if(!pdoc->isModified())
  {
    pdoc->setModified();
    emit viewChanged();
  }
}

void KMovistoView::slotMouseMoved(QMouseEvent *MouseEvent)
{
  // store actual mouse coordinates
  mouseOldX = MouseEvent->x();
  mouseOldY = MouseEvent->y();
  // emit viewChanged if doc is not modified
  if(!pdoc->isModified())
  {
    pdoc->setModified();
    emit viewChanged();
  }
}

void KMovistoView::slotMouseReleased(QMouseEvent*)
{
  rightButtonPressed = FALSE;
  midButtonPressed = FALSE;
  leftButtonPressed = FALSE;
  controlKeyPressed = FALSE;
  shiftKeyPressed = FALSE;
}

void KMovistoView::slotPaintEvent(QPaintEvent*)
{
  repaint();
}

// events
void KMovistoView::resizeEvent(QResizeEvent*)
{
  titleGroupBox->setGeometry(1, 2, width() - 2, 25);
  titleLabel->setGeometry(5, 5, titleGroupBox->width() - 10, titleGroupBox->height() - 10);
}

void KMovistoView::paintEvent(QPaintEvent*)
{

}

/////////////////////////////////////////////////////////////////////////////
// definition of the class KMovistoViewGL
/////////////////////////////////////////////////////////////////////////////

KMovistoViewGL::KMovistoViewGL(QWidget *parent, KMovistoDoc *doc)
  : KMovistoView(parent, doc)
{
  // initialisations
  menuItem = "&openGL mode";

  // create frames
  mainFrameZ = createMainFrame("mainFrameZ");
  glframeZ = createDisplayFrameGL(mainFrameZ, "glframeZ", pdoc->getMolecule());
  connectDisplayFrameGL(glframeZ);
}

CDisplayFrameGL* KMovistoViewGL::createDisplayFrameGL(QWidget *parent, const char *name, CKGLMolecule *pmol)
{
  CDisplayFrameGL* glframe;
  glframe = new CDisplayFrameGL(parent, name, pmol);
  return(glframe);
}

void KMovistoViewGL::connectDisplayFrameGL(CDisplayFrameGL* glframe)
{
  connect(glframe, SIGNAL(mousePressed(QMouseEvent *)), this, SLOT(slotMousePressed(QMouseEvent *)));
  connect(glframe, SIGNAL(mouseMoved(QMouseEvent *)), this, SLOT(slotMouseMoved(QMouseEvent *)));
  connect(glframe, SIGNAL(mouseReleased(QMouseEvent *)), this, SLOT(slotMouseReleased(QMouseEvent *)));
  connect(glframe, SIGNAL(signalPaintEvent(QPaintEvent *)), this, SLOT(slotPaintEvent(QPaintEvent *)));
}

int KMovistoViewGL::print(QPrinter &printer, const QRect &rectClient)
{
  int success = 0;
  QPixmap pixmap;
  QPainter painter;
  CKGLMolecule* printmolecule = 0;
  CDisplayFrameGL* printwidget = 0;
       
  printmolecule = new CKGLMolecule(this, "temporary molecule");
  if(printmolecule)
  {
    *printmolecule = *(pdoc->getMolecule());
    printmolecule->SetView(VIEW_FRONT);
    printmolecule->SetBackgroundColorGL(pdoc->getBGColorPrint());
    printwidget = new CDisplayFrameGL(this, "temporary glwidget", printmolecule);
    if(printwidget)
    {
      printwidget->setFixedSize(rectClient.width(), rectClient.height());
      pixmap = printwidget->renderPixmap(rectClient.width(), rectClient.height());
      
      if (!pixmap.isNull())
      {
        // print the pixmap
        success = 1;
        painter.begin((QPaintDevice*)&printer);
        painter.drawPixmap(0, 0, pixmap);
        painter.end();
      }
            
      delete printwidget;
    }
    delete printmolecule;
  }
  return(success); 
}

// slots
void KMovistoViewGL::slotDocumentChanged()
{
  KMovistoView::slotDocumentChanged();
  glframeZ->updateGL();
}

void KMovistoViewGL::slotMouseMoved(QMouseEvent *MouseEvent)
{
  // rotate, translate, zoom molecule
  if(rotateMolecule(MouseEvent->x(), MouseEvent->y()) ||
     translateMolecule(MouseEvent->x(), MouseEvent->y()) ||
     zoomMolecule(MouseEvent->y()))
  {
    repaint();
  }
  KMovistoView::slotMouseMoved(MouseEvent);
}

void KMovistoViewGL::slotMousePressed(QMouseEvent *MouseEvent)
{
  KMovistoView::slotMousePressed(MouseEvent);
  // reset molecule movements
  if(centerMolecule())
  {
    repaint();
  }
}

void KMovistoViewGL::resizeEvent(QResizeEvent *resizeevent)
{
  KMovistoView::resizeEvent(resizeevent);
  int w = width();
  int h = height();
  mainFrameZ->setGeometry(1, 29, w - 2, h - 30);
  glframeZ->setGeometry(mainFrameZ->contentsRect());
}

void KMovistoViewGL::paintEvent(QPaintEvent*)
{
  pdoc->getMolecule()->SetView(VIEW_FRONT);
  glframeZ->updateGL();
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class KMovistoView3DAnaglyph
/////////////////////////////////////////////////////////////////////////////

KMovistoView3DAnaglyph::KMovistoView3DAnaglyph(QWidget *parent, KMovistoDoc *doc)
  : KMovistoViewGL(parent, doc)
{
  // initialisations
  menuItem = "3D &anaglyph";
  anaglyphMode = RED_GREEN;
}

void KMovistoView3DAnaglyph::createToolbar(QMainWindow* mainwindow)
{
  ptoolbar = (QToolBar*)new CAnaglyphToolbar(mainwindow, "anaglyph mode settings");
  // connect anaglyph mode changings
  connect(ptoolbar, SIGNAL(anaglyphModeChanged(int)), this, SLOT(slotAnaglyphModeChanged(int)));
}

int KMovistoView3DAnaglyph::print(QPrinter &printer, const QRect &rectClient)
{
  int success = 0;
  QPixmap pixmap;
  QPixmap pixmapRight;
  QPixmap pixmapLeft;
  QPainter painter;
  CKGLMolecule* printmolecule = 0;
  CDisplayFrameGL* printwidget = 0;

  printmolecule = new CKGLMolecule(this, "temporary molecule");
  if(printmolecule)
  {
    *printmolecule = *(pdoc->getMolecule());
    printmolecule->SetView(VIEW_FRONT);
    printwidget = new CDisplayFrameGL(this, "temporary glwidget", printmolecule);
    if(printwidget)
    {
      printwidget->setFixedSize(rectClient.width(), rectClient.height());
      printmolecule->SetBackgroundColorGL(pdoc->getBGColorPrint());
      printmolecule->Set3DMode(MODE3D_RED_ONLY);
      // render right pixmap
      pixmapRight = printwidget->renderPixmap(rectClient.width(), rectClient.height());
      // render left pixmap
      if(anaglyphMode == RED_GREEN)
        printmolecule->Set3DMode(MODE3D_GREEN_ONLY);
      if(anaglyphMode == RED_CYAN)
        printmolecule->Set3DMode(MODE3D_CYAN_ONLY);
      if(anaglyphMode == RED_BLUE)
        printmolecule->Set3DMode(MODE3D_BLUE_ONLY);
      pixmapLeft = printwidget->renderPixmap(rectClient.width(), rectClient.height());
      // create destination pixmap
      pixmap.resize(rectClient.width(), rectClient.height());
      
      // continue if all pixmaps are valid
      if (!pixmap.isNull() && !pixmapRight.isNull() && !pixmapLeft.isNull())
      {
        bitBlt(&pixmap, 0, 0, &pixmapLeft, 0, 0, rectClient.width(), rectClient.height(), Qt::CopyROP);
        bitBlt(&pixmap, 0, 0, &pixmapRight, 0, 0, rectClient.width(), rectClient.height(), (printmolecule->GetBackgroundColorGL() == QColor(0, 0, 0) ? Qt::OrROP : Qt::AndROP));

        // print the pixmap
        success = 1;
        painter.begin((QPaintDevice*)&printer);
        painter.drawPixmap(0, 0, pixmap);
        painter.end();
      }

      delete printwidget;
    }
    delete printmolecule;
  }
  return(success);
}

// events
void KMovistoView3DAnaglyph::showEvent(QShowEvent*)
{
  anaglyphMode = RED_GREEN;
  ((CAnaglyphToolbar*)ptoolbar)->setAnaglyphMode(anaglyphMode);
  pdoc->getMolecule()->Set3DMode(MODE3D_RED_GREEN);
}

void KMovistoView3DAnaglyph::hideEvent(QHideEvent*)
{
  pdoc->getMolecule()->Set3DMode(MODE3D_DISABLED);
  anaglyphMode = DISABLED;
  ((CAnaglyphToolbar*)ptoolbar)->setAnaglyphMode(anaglyphMode);
}

void KMovistoView3DAnaglyph::slotAnaglyphModeChanged(int Mode)
{
  anaglyphMode = Mode;
  switch(anaglyphMode)
  {
    case RED_GREEN:
      emit statusHelpMsgPermanent("Red/Green mode.");
      pdoc->getMolecule()->Set3DMode(MODE3D_RED_GREEN);
      repaint();
      break;
    case RED_CYAN:
      emit statusHelpMsgPermanent("Red/Cyan mode.");
      pdoc->getMolecule()->Set3DMode(MODE3D_RED_CYAN);
      repaint();
      break;
    case RED_BLUE:
      emit statusHelpMsgPermanent("Red/Blue mode.");
      pdoc->getMolecule()->Set3DMode(MODE3D_RED_BLUE);
      repaint();
      break;
    default:
      emit statusHelpMsgDefault();
      break;
  }
  repaint();
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class KMovistoView3DPair
/////////////////////////////////////////////////////////////////////////////

KMovistoView3DPair::KMovistoView3DPair(QWidget *parent, KMovistoDoc *doc)
  : KMovistoViewGL(parent, doc)
{
  // initialisations
  menuItem = "3D stereo &pair";
}

// events
void KMovistoView3DPair::showEvent(QShowEvent*)
{
  pdoc->getMolecule()->Set3DMode(MODE3D_COLORED_IMAGES);
}

void KMovistoView3DPair::hideEvent(QHideEvent*)
{
  pdoc->getMolecule()->Set3DMode(MODE3D_DISABLED);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class KMovistoViewFlat
/////////////////////////////////////////////////////////////////////////////

KMovistoViewFlat::KMovistoViewFlat(QWidget *parent, KMovistoDoc *doc)
  : KMovistoView(parent, doc)
{
  // initialisations
  menuItem = "&flat mode";

  // create frames
  mainFrameZ = createMainFrame("mainFrameZ");
  frameZ = createDisplayFrame(mainFrameZ, "frameZ");
  connectDisplayFrame(frameZ);
}

// slots
void KMovistoViewFlat::slotMouseMoved(QMouseEvent *MouseEvent)
{
  // rotate, translate, zoom molecule
  if(rotateMolecule(MouseEvent->x(), MouseEvent->y()) ||
     translateMolecule(MouseEvent->x(), MouseEvent->y()) ||
     zoomMolecule(MouseEvent->y()))
  {
    repaint();
  }
  KMovistoView::slotMouseMoved(MouseEvent);
}

void KMovistoViewFlat::slotMousePressed(QMouseEvent *MouseEvent)
{
  KMovistoView::slotMousePressed(MouseEvent);
  // reset molecule movements
  if(centerMolecule())
  {
    repaint();
  }
}

void KMovistoViewFlat::resizeEvent(QResizeEvent *resizeevent)
{
  KMovistoView::resizeEvent(resizeevent);
  int w = width();
  int h = height();
  mainFrameZ->setGeometry(1, 29, w - 2, h - 30);
  frameZ->setGeometry(mainFrameZ->contentsRect());
}

void KMovistoViewFlat::paintEvent(QPaintEvent*)
{
  pdoc->getMolecule()->SetView(VIEW_FRONT);
  pdoc->getMolecule()->DrawStructure(frameZ, frameZ->rect());
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class KMovistoViewEdit
/////////////////////////////////////////////////////////////////////////////

KMovistoViewEdit::KMovistoViewEdit(QWidget *parent, KMovistoDoc *doc)
  : KMovistoView(parent, doc)
{
  // initialisations
  menuItem = "&edit mode";
  mouseEventFrame = NO_FRAME;
  FirstCapturedAtom = -1;
  LastCapturedAtom = -1;
  editMode = DISABLED;

  // create frames
  mainFrameX = createMainFrame("mainFrameX");
  mainFrameY = createMainFrame("mainFrameY");
  mainFrameZ = createMainFrame("mainFrameZ");
  frameX = createDisplayFrame(mainFrameX, "frameX");
  frameY = createDisplayFrame(mainFrameY, "frameY");
  frameZ = createDisplayFrame(mainFrameZ, "frameZ");
  connectDisplayFrame(frameX);
  connectDisplayFrame(frameY);
  connectDisplayFrame(frameZ);

  // labels
  labelFrameX = new QLabel(this, "labelFrameX");
  labelFrameX->setFrameStyle(0);
  labelFrameX->setLineWidth(1);
  labelFrameX->setMidLineWidth(0);
  labelFrameX->setText("right (x axis)");
  labelFrameX->setFocusPolicy(QWidget::NoFocus);
  labelFrameX->setBackgroundMode(QWidget::PaletteBackground);
  labelFrameX->QFrame::setMargin(0);
  labelFrameX->setMargin(0);
  labelFrameX->setAlignment(AlignLeft|AlignVCenter|ExpandTabs);

  labelFrameY = new QLabel(this, "labelFrameY");
  labelFrameY->setFrameStyle(0);
  labelFrameY->setLineWidth(1);
  labelFrameY->setMidLineWidth(0);
  labelFrameY->setText("top (y axis)");
  labelFrameY->setFocusPolicy(QWidget::NoFocus);
  labelFrameY->setBackgroundMode(QWidget::PaletteBackground);
  labelFrameY->QFrame::setMargin(0);
  labelFrameY->setMargin(0);
  labelFrameY->setAlignment(AlignLeft|AlignVCenter|ExpandTabs);

  labelFrameZ = new QLabel(this, "labelFrameZ");
  labelFrameZ->setFrameStyle(0);
  labelFrameZ->setLineWidth(1);
  labelFrameZ->setMidLineWidth(0);
  labelFrameZ->setText("front (z axis)");
  labelFrameZ->setFocusPolicy(QWidget::NoFocus);
  labelFrameZ->setBackgroundMode(QWidget::PaletteBackground);
  labelFrameZ->QFrame::setMargin(0);
  labelFrameZ->setMargin(0);
  labelFrameZ->setAlignment(AlignLeft|AlignVCenter|ExpandTabs);

  // create control panel
  controlPanel = new CControlPanel(this, "ControlPanel");
  controlPanel->setFocusPolicy(QWidget::NoFocus);
  controlPanel->setBackgroundMode(QWidget::PaletteBackground);

  // connect ordinal number changings
  connect(controlPanel, SIGNAL(ordinalNumberChanged(int)), this, SLOT(slotOrdinalNumberChanged(int)));
}

void KMovistoViewEdit::createToolbar(QMainWindow* mainwindow)
{
  ptoolbar = (QToolBar*)new CEditToolbar(mainwindow, "edit mode operations");
  // connect edit mode changings
  connect(ptoolbar, SIGNAL(editModeChanged(int)), this, SLOT(slotEditModeChanged(int)));
}

// slots
void KMovistoViewEdit::slotOrdinalNumberChanged(int index)
{
  CKPeriodicSystem periodicSystem;
  switch(editMode)
  {
    case CHANGE:
      if(index >= 0 && LastCapturedAtom != -1)
      {
        pdoc->getMolecule()->SetOrdinalNumber(LastCapturedAtom, index + 1);
        pdoc->getMolecule()->SetRadius(LastCapturedAtom, periodicSystem.GetRadius(index + 1));
        pdoc->getMolecule()->SetColor(LastCapturedAtom, periodicSystem.GetColor(index + 1));
        repaint();
      }
      break;
    case ADD:
    case MOVE:
    case DELETE:
    case CONNECT:
    case DISCONNECT:
    case DISABLED:
      break;
  }
}

void KMovistoViewEdit::slotEditModeChanged(int Mode)
{
  editMode = Mode;
  switch(editMode)
  {
    case ADD:
      if(controlPanel->getOrdinalNumber() < 1)
        controlPanel->setElement(1);
      emit statusHelpMsgPermanent("Select element and press left button.");
    case CHANGE:
    case MOVE:
    case DELETE:
    case CONNECT:
    case DISCONNECT:
      frameX->setCursor(pointingHandCursor);
      frameY->setCursor(pointingHandCursor);
      frameZ->setCursor(pointingHandCursor);
      if(editMode != ADD) emit statusHelpMsgPermanent("Select atom(s).");
      break;
    case DISABLED:
    default:
      frameX->setCursor(arrowCursor);
      frameY->setCursor(arrowCursor);
      frameZ->setCursor(arrowCursor);
      emit statusHelpMsgDefault();
      break;
  }
  pdoc->getMolecule()->SetPattern(Qt::SolidPattern);
  controlPanel->reset();
  FirstCapturedAtom = -1;
  LastCapturedAtom = -1;
  repaint();
}

void KMovistoViewEdit::slotMousePressed(QMouseEvent *MouseEvent)
{
  KMovistoView::slotMousePressed(MouseEvent);
  int i = 0;
  int CapturedAtom = -1;
  int View = Z_AXIS;
  ELEMENT Element;
  CKPeriodicSystem periodicSystem;
  mouseEventFrame = NO_FRAME;
  if((frameX->frameRect()).contains(frameX->mapFromGlobal(MouseEvent->globalPos())))
  {
    mouseEventFrame = FRAME_X;
    View = X_AXIS;
  }
  if(mouseEventFrame == NO_FRAME && (frameY->frameRect()).contains(frameY->mapFromGlobal(MouseEvent->globalPos())))
  {
    mouseEventFrame = FRAME_Y;
    View = Y_AXIS;
  }
  if(mouseEventFrame == NO_FRAME && (frameZ->frameRect()).contains(frameZ->mapFromGlobal(MouseEvent->globalPos())))
  {
    mouseEventFrame = FRAME_Z;
    View = Z_AXIS;
  }
  if(leftButtonPressed)
  {
    // toggle selection
    if((CapturedAtom = pdoc->getMolecule()->Capture(MouseEvent->x(), MouseEvent->y(), View)) != -1)
    {
      if(pdoc->getMolecule()->GetPattern(CapturedAtom) == Qt::SolidPattern)
      {
        FirstCapturedAtom = LastCapturedAtom;
        LastCapturedAtom = CapturedAtom;
      }
      else
      {
        if(CapturedAtom == LastCapturedAtom)
          LastCapturedAtom = FirstCapturedAtom;
        FirstCapturedAtom = -1;
      }
      // repaint pattern
      pdoc->getMolecule()->SetPattern(Qt::SolidPattern);
    }

    // left button pressed
    switch(editMode)
    {
      case CHANGE:
      case MOVE:
      case DELETE:
        if(CapturedAtom != -1 && LastCapturedAtom == CapturedAtom)
        {
          pdoc->getMolecule()->SetPattern(LastCapturedAtom, Qt::DiagCrossPattern);
          if(editMode == DELETE)
            emit statusHelpMsgPermanent("Press right button.");
        }
        break;
      case CONNECT:
      case DISCONNECT:
        if(CapturedAtom != -1 && FirstCapturedAtom != -1)
          pdoc->getMolecule()->SetPattern(FirstCapturedAtom, Qt::DiagCrossPattern);
        if(CapturedAtom != -1 && LastCapturedAtom != -1)
          pdoc->getMolecule()->SetPattern(LastCapturedAtom, Qt::DiagCrossPattern);
        if(FirstCapturedAtom != -1 && LastCapturedAtom != -1)
          emit statusHelpMsgPermanent("Press right button.");
        break;
      case ADD:
        Element.x = 0.0;
        Element.y = 0.0;
        Element.z = 0.0;
        Element.OrdinalNumber = controlPanel->getOrdinalNumber();
        Element.Label = pdoc->getMolecule()->GetNumberOfAtoms() + 1;
        Element.Radius = periodicSystem.GetRadius(Element.OrdinalNumber);
        Element.Color = periodicSystem.GetColor(Element.OrdinalNumber);
        Element.Pattern = Qt::DiagCrossPattern;
        LastCapturedAtom = Element.Label - 1;
        pdoc->getMolecule()->SetPattern(Qt::SolidPattern);
        pdoc->getMolecule()->AddAtom(Element, controlPanel->getAutoConnectMode());
        pdoc->getMolecule()->SetAbsoluteAtomCoordinate(LastCapturedAtom, MouseEvent->x(), MouseEvent->y(), controlPanel->getAutoConnectMode(), mouseEventFrame);
				repaint();
        break;
      case DISABLED:
        break;
    }
    if(editMode !=DISABLED && CapturedAtom != -1)
    {
      controlPanel->setElement(pdoc->getMolecule()->GetOrdinalNumber(LastCapturedAtom));
      controlPanel->setX((pdoc->getMolecule()->GetAtomCoordinate(LastCapturedAtom, 1, 1)).x);
      controlPanel->setY((pdoc->getMolecule()->GetAtomCoordinate(LastCapturedAtom, 1, 1)).y);
      controlPanel->setZ((pdoc->getMolecule()->GetAtomCoordinate(LastCapturedAtom, 1, 1)).z);
    }
    else
    {
      controlPanel->setElement(1);
      controlPanel->setX(0.0);
      controlPanel->setY(0.0);
      controlPanel->setZ(0.0);
    }
    repaint();
  }
  else if(rightButtonPressed)
  {
    // right button pressed
    switch(editMode)
    {
      case CHANGE:
        break;
      case MOVE:
        break;
      case DELETE:
        if(!pdoc->getMolecule()->DeleteAtom(LastCapturedAtom))
          QMessageBox::critical(0, "Error!", QString("No Atom selected!"));
        else
        {
          for(i = 0; i < pdoc->getMolecule()->GetNumberOfAtoms(); i++)
            pdoc->getMolecule()->SetLabel(i, i + 1);
          repaint();
        }
        emit statusHelpMsgPermanent("Select atom(s).");
        break;
      case ADD:
        break;
      case CONNECT:
        if(FirstCapturedAtom == -1 || LastCapturedAtom == -1)
        {
          QMessageBox::critical(0, "Error!", QString("Select two atoms to connect!"));
          break;
        }
        if(!pdoc->getMolecule()->AddBond(FirstCapturedAtom, LastCapturedAtom))
          QMessageBox::critical(0, "Error!", QString("Atoms are already connected!"));
        FirstCapturedAtom = -1;
        LastCapturedAtom = -1;
        pdoc->getMolecule()->SetPattern(Qt::SolidPattern);
        repaint();
        emit statusHelpMsgPermanent("Select atom(s).");
        break;
      case DISCONNECT:
        if(FirstCapturedAtom == -1 || LastCapturedAtom == -1)
        {
          QMessageBox::critical(0, "Error!", QString("Select two atoms to disconnect!"));
          break;
        }
        if(!pdoc->getMolecule()->DeleteBond(FirstCapturedAtom, LastCapturedAtom))
          QMessageBox::critical(0, "Error!", QString("Atoms are not connected!"));
        FirstCapturedAtom = -1;
        LastCapturedAtom = -1;
        pdoc->getMolecule()->SetPattern(Qt::SolidPattern);
        repaint();
        emit statusHelpMsgPermanent("Select atom(s).");
        break;
      case DISABLED:
        break;
    }
  }
  else if(midButtonPressed)
  {
    // middle button pressed
    if(centerMolecule())
      repaint();
  }
}

void KMovistoViewEdit::slotMouseMoved(QMouseEvent *MouseEvent)
{
  switch(editMode)
  {
    case CHANGE:
      break;
    case MOVE:
      if(pdoc->getMolecule()->GetPattern(LastCapturedAtom) != Qt::SolidPattern)
      {
        pdoc->getMolecule()->SetAbsoluteAtomCoordinate(LastCapturedAtom, MouseEvent->x(), MouseEvent->y(), controlPanel->getAutoConnectMode(), mouseEventFrame);
        controlPanel->setElement(pdoc->getMolecule()->GetOrdinalNumber(LastCapturedAtom));
        controlPanel->setX((pdoc->getMolecule()->GetAtomCoordinate(LastCapturedAtom, 1, 1)).x);
        controlPanel->setY((pdoc->getMolecule()->GetAtomCoordinate(LastCapturedAtom, 1, 1)).y);
        controlPanel->setZ((pdoc->getMolecule()->GetAtomCoordinate(LastCapturedAtom, 1, 1)).z);
        repaint();
      }
      break;
    case DELETE:
      break;
    case ADD:
      break;
    case CONNECT:
      break;
    case DISCONNECT:
      break;
    case DISABLED:
      // rotate, translate, zoom molecule
      if(rotateMolecule(MouseEvent->x(), MouseEvent->y(), mouseEventFrame) ||
         translateMolecule(MouseEvent->x(), MouseEvent->y(), mouseEventFrame) ||
         zoomMolecule(MouseEvent->y()))
      {
        repaint();
      }
      break;
  }
  KMovistoView::slotMouseMoved(MouseEvent);
}

void KMovistoViewEdit::slotMouseReleased(QMouseEvent *MouseEvent)
{
  switch(editMode)
  {
    case CHANGE:
      break;
    case MOVE:
      pdoc->getMolecule()->SetPattern(LastCapturedAtom, Qt::SolidPattern);
      FirstCapturedAtom = -1;
      LastCapturedAtom = -1;
      repaint();
      break;
    case DELETE:
      break;
    case ADD:
      break;
    case CONNECT:
      break;
    case DISCONNECT:
      break;
    case DISABLED:
      break;
  }

  KMovistoView::slotMouseReleased(MouseEvent);
  mouseEventFrame = NO_FRAME;
}

// events
void KMovistoViewEdit::hideEvent(QHideEvent*)
{
  pdoc->getMolecule()->SetPattern(Qt::SolidPattern);
  editMode = DISABLED;
  ((CEditToolbar*)ptoolbar)->setEditMode(editMode);
}

void KMovistoViewEdit::resizeEvent(QResizeEvent *resizeevent)
{
  KMovistoView::resizeEvent(resizeevent);
  int w = width();
  int h = height();
  controlPanel->setGeometry(2, h - 120, w, 130);
  mainFrameX->setGeometry(2, 50, w / 3 - 4, h - 170);
  mainFrameY->setGeometry(w / 3 + 2 , 50, w / 3 - 4, h - 170);
  mainFrameZ->setGeometry(2 * w / 3 + 3, 50, w / 3 - 4, h - 170);
  frameX->setGeometry(mainFrameX->contentsRect());
  frameY->setGeometry(mainFrameY->contentsRect());
  frameZ->setGeometry(mainFrameZ->contentsRect());
  labelFrameX->setGeometry(2, 29, frameX->width() - 10, 20);
  labelFrameY->setGeometry(w / 3 + 2, 29, frameY->width() - 10, 20);
  labelFrameZ->setGeometry(2 * w / 3 + 3, 29, frameZ->width() - 10, 20);
}

void KMovistoViewEdit::paintEvent(QPaintEvent*)
{
  pdoc->getMolecule()->SetView(VIEW_RIGHT);
  pdoc->getMolecule()->DrawStructure(frameX, frameX->rect());
  pdoc->getMolecule()->SetView(VIEW_TOP);
  pdoc->getMolecule()->DrawStructure(frameY, frameY->rect());
  pdoc->getMolecule()->SetView(VIEW_FRONT);
  pdoc->getMolecule()->DrawStructure(frameZ, frameZ->rect());
}
