/***************************************************************************
                          kmovistoview.h  -  description
                             -------------------
    begin                : Sam Jan 22 16:27:34 CET 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMOVISTOVIEW_H
#define KMOVISTOVIEW_H

/* include files for QT */
#include <qwidget.h>
#include <qframe.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qwhatsthis.h>
#include <qpainter.h>

/* application specific includes */
#include "kmovistodoc.h"
#include "kglmolecule.h"

/* constants */
#define NO_FRAME 0
#define FRAME_X 1
#define FRAME_Y 2
#define FRAME_Z 3

#define DISABLED 0

#define CHANGE 1
#define MOVE 2
#define ADD 3
#define DELETE 4
#define CONNECT 5
#define DISCONNECT 6

#define RED_GREEN 1
#define RED_CYAN 2
#define RED_BLUE 3

/* class CDisplayFrame */
class CDisplayFrame : public QFrame
{
  Q_OBJECT

public:
  CDisplayFrame(QWidget *parent = 0, const char *name = 0, WFlags f = 0);

protected:
  void mousePressEvent(QMouseEvent *event);
  void mouseMoveEvent(QMouseEvent *event);
  void mouseReleaseEvent(QMouseEvent *event);
  void paintEvent(QPaintEvent *paintevent);

signals:
  void mousePressed(QMouseEvent *event);
  void mouseMoved(QMouseEvent *event);
  void mouseReleased(QMouseEvent *event);
  void signalPaintEvent(QPaintEvent *paintevent);
};

/* class CDisplayFrameGL */
class CDisplayFrameGL : public QGLWidget
{
  Q_OBJECT

public:
  CDisplayFrameGL(QWidget *parent = 0, const char *name = 0, CKGLMolecule *pmol = 0);
  ~CDisplayFrameGL();

protected:
  void mousePressEvent(QMouseEvent *event);
  void mouseMoveEvent(QMouseEvent *event);
  void mouseReleaseEvent(QMouseEvent *event);
  void paintEvent(QPaintEvent *paintevent);
  virtual void initializeGL();
  virtual void paintGL();
  virtual void resizeGL(int width, int height);

protected:
  CKGLMolecule *pmolecule;

signals:
  void mousePressed(QMouseEvent *event);
  void mouseMoved(QMouseEvent *event);
  void mouseReleased(QMouseEvent *event);
  void signalPaintEvent(QPaintEvent *paintevent);
};

/* class CControlPanel */
class CControlPanel : public QWidget
{
    Q_OBJECT

public:
  CControlPanel(QWidget* parent = 0, const char* name = 0, WFlags f = 0);
  void reset(void);
  /* get... */
  bool getAutoConnectMode(void) {return(AutoConnectCheckBox->isChecked());};
  int getOrdinalNumber(void) {return(ElementComboBox->currentItem() + 1);};
  /* set... */
  void setElement(int ordinalNumber);
  void setX(float newX);
  void setY(float newY);
  void setZ(float newZ);
  void setAutoconnect(bool checked);
  virtual void setGeometry(int x, int y, int w, int h);

signals:
  void ordinalNumberChanged(int index);

private slots:
  void comboBoxHighlighted(int index);

protected:
  QGroupBox* AtomBox;
  QCheckBox* AutoConnectCheckBox;
  QLabel* AtomSymbolLabel;
  QLabel* OrdinalNumber;
  QLabel* OrdinalNumberLabel;
  QLabel* XPosLabel;
  QLabel* YPosLabel;
  QLabel* ZPosLabel;
  QLabel* XPos;
  QLabel* YPos;
  QLabel* ZPos;
  QComboBox* ElementComboBox;
  CKPeriodicSystem ps;
};

/* class CEditToolbar */
class CEditToolbar : public QToolBar
{
  Q_OBJECT

public:
  CEditToolbar(QMainWindow *parent = 0, const char *name = 0);
  void setEditMode(int Mode);

signals:
  void editModeChanged(int Mode);

protected slots:
  void slotChangeMode();
  void slotAddMode();
  void slotConnectMode();
  void slotMoveMode();
  void slotDeleteMode();
  void slotDisconnectMode();
  void slotToggleEditModeButtons(int Mode);

protected:
  QPixmap changeModeIcon;
  QPixmap addModeIcon;
  QPixmap connectModeIcon;
  QPixmap moveModeIcon;
  QPixmap deleteModeIcon;
  QPixmap disconnectModeIcon;
  QToolButton *editModeChange;
  QToolButton *editModeAdd;
  QToolButton *editModeConnect;
  QToolButton *editModeMove;
  QToolButton *editModeDelete;
  QToolButton *editModeDisconnect;
};

/* class CAnaglyphToolbar */
class CAnaglyphToolbar : public QToolBar
{
  Q_OBJECT

public:
  CAnaglyphToolbar(QMainWindow *parent = 0, const char *name = 0);
  void setAnaglyphMode(int Mode);

signals:
  void anaglyphModeChanged(int Mode);

protected slots:
  void slotRedGreenMode();
  void slotRedCyanMode();
  void slotRedBlueMode();
  void slotToggleAnaglyphModeButtons(int Mode);

protected:
  QPixmap anaglyphRedGreenModeIcon;
  QPixmap anaglyphRedCyanModeIcon;
  QPixmap anaglyphRedBlueModeIcon;
  QToolButton *anaglyphModeRedGreen;
  QToolButton *anaglyphModeRedCyan;
  QToolButton *anaglyphModeRedBlue;
};

/* class KMovistoView */
class KMovistoView : public QWidget
{
  Q_OBJECT

public:
  KMovistoView(QWidget *parent = 0, KMovistoDoc* doc = 0);
  ~KMovistoView();
  void resetView(void);
  virtual void createToolbar(QMainWindow*);
  virtual int print(QPrinter &printer, const QRect &rectClient);
  void showToolbar(void);
  void hideToolbar(void);
  void setViewID(uint id);
  uint getViewID(void);
  QString getMenuItem(void);

protected:
  QFrame* createMainFrame(const char *name = 0);
  CDisplayFrame* createDisplayFrame(QWidget *parent = 0, const char *name = 0);
  void connectDisplayFrame(CDisplayFrame* frame);
  void paintEvent(QPaintEvent*);
  void resizeEvent(QResizeEvent*);
  int rotateMolecule(int mouseX, int mouseY, int axis = FRAME_Z);
  int translateMolecule(int mouseX, int mouseY, int axis = FRAME_Z);
  int centerMolecule(void);
  int zoomMolecule(int mouseY);

protected slots:
  void slotDocumentChanged();
  void slotMousePressed(QMouseEvent *MouseEvent);
  void slotMouseMoved(QMouseEvent *MouseEvent);
  void slotMouseReleased(QMouseEvent*);
  void slotPaintEvent(QPaintEvent*);

signals:
  void viewChanged();
  void statusHelpMsg(const QString&);
  void statusHelpMsgPermanent(const QString&);
  void statusHelpMsgDefault();

protected:
  KMovistoDoc *pdoc;
  QToolBar *ptoolbar;
  QGroupBox *titleGroupBox;
  QLabel *titleLabel;
  int rightButtonPressed;
  int midButtonPressed;
  int leftButtonPressed;
  int controlKeyPressed;
  int shiftKeyPressed;
  int mouseOldX;
  int mouseOldY;
  uint viewID;
  QString menuItem;
};

/* class KMovistoViewGL */

class KMovistoViewGL : public KMovistoView
{
  Q_OBJECT

public:
  KMovistoViewGL(QWidget *parent = 0, KMovistoDoc *doc = 0);
  CDisplayFrameGL* createDisplayFrameGL(QWidget *parent = 0, const char *name = 0, CKGLMolecule *pmol = 0);
  void connectDisplayFrameGL(CDisplayFrameGL* glframe);
  virtual int print(QPrinter &printer, const QRect &rectClient);

protected:
  void paintEvent(QPaintEvent*);
  void resizeEvent(QResizeEvent *resizeevent);

protected slots:
  void slotDocumentChanged();
  void slotMouseMoved(QMouseEvent *MouseEvent);
  void slotMousePressed(QMouseEvent *MouseEvent);

protected:
  QFrame *mainFrameZ;
  CDisplayFrameGL *glframeZ;
};

/* class KMovistoViewFlat */

class KMovistoViewFlat : public KMovistoView
{
  Q_OBJECT

public:
  KMovistoViewFlat(QWidget *parent = 0, KMovistoDoc *doc = 0);

protected:
  void paintEvent(QPaintEvent*);
  void resizeEvent(QResizeEvent *resizeevent);

protected slots:
  void slotMouseMoved(QMouseEvent *MouseEvent);
  void slotMousePressed(QMouseEvent *MouseEvent);

protected:
  QFrame *mainFrameZ;
  CDisplayFrame *frameZ;
};

/* class KMovistoView3DAnaglyph */

class KMovistoView3DAnaglyph : public KMovistoViewGL
{
  Q_OBJECT

public:
  KMovistoView3DAnaglyph(QWidget *parent = 0, KMovistoDoc *doc = 0);
  virtual void createToolbar(QMainWindow* mainwindow);
  virtual int print(QPrinter &printer, const QRect &rectClient);

protected:
  void showEvent(QShowEvent*);
  void hideEvent(QHideEvent*);

public slots:
  void slotAnaglyphModeChanged(int Mode);

protected:
  QFrame *mainFrameZ;
  CDisplayFrame *frameZ;
  int anaglyphMode;
};

/* class KMovistoView3DPair */

class KMovistoView3DPair : public KMovistoViewGL
{
  Q_OBJECT

public:
  KMovistoView3DPair(QWidget *parent = 0, KMovistoDoc *doc = 0);

protected:
  void showEvent(QShowEvent*);
  void hideEvent(QHideEvent*);

protected:
  QFrame *mainFrameZ;
  CDisplayFrame *frameZ;
};

/* class KMovistoViewEdit */

class KMovistoViewEdit : public KMovistoView
{
  Q_OBJECT

public:
  KMovistoViewEdit(QWidget *parent = 0, KMovistoDoc *doc = 0);
  virtual void createToolbar(QMainWindow* mainwindow);

protected:
  void hideEvent(QHideEvent*);
  void paintEvent(QPaintEvent*);
  void resizeEvent(QResizeEvent *resizeevent);

public slots:
  void slotEditModeChanged(int Mode);

protected slots:
  void slotMousePressed(QMouseEvent *MouseEvent);
  void slotMouseMoved(QMouseEvent *MouseEvent);
  void slotMouseReleased(QMouseEvent *MouseEvent);
  void slotOrdinalNumberChanged(int index);

protected:
  CControlPanel *controlPanel;
  int mouseEventFrame;
  QFrame *mainFrameX;
  QFrame *mainFrameY;
  QFrame *mainFrameZ;
  CDisplayFrame *frameX;
  CDisplayFrame *frameY;
  CDisplayFrame *frameZ;
  QLabel *labelFrameX;
  QLabel *labelFrameY;
  QLabel *labelFrameZ;
  int FirstCapturedAtom;
  int LastCapturedAtom;
  int editMode;
};

#endif
