/***************************************************************************
                          mvtfile.cpp  -  description
                             -------------------
    begin                : Mon Apr 17 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files
#include <mvtfile.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTCoordinate
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTCoordinate::CMVTCoordinate(QObject *parent, const char *name)
  : QObject(parent, name)
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTCoordinate::~CMVTCoordinate()
{

}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CMVTCoordinate::Reset(void)
{
  x = (float)0.0;
  y = (float)0.0;
  z = (float)0.0;
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTCoordinate& CMVTCoordinate::operator=(CMVTCoordinate &MVTCoordinate)
{
  // equalize members
  x = MVTCoordinate.x;
  y = MVTCoordinate.y;
  z = MVTCoordinate.z;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CMVTCoordinate::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTAtom
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTAtom::CMVTAtom(QObject *parent, const char *name)
  : QObject(parent, name)
{
  label = 0;
  color.setRgb(0, 0, 0);
  radius = (float)0.0;
  ordinalNumber = 0;
  atomcoordinate.Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTAtom::~CMVTAtom()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTAtom& CMVTAtom::operator=(CMVTAtom &MVTAtom)
{
  // equalize members
  label = MVTAtom.label;
  color = MVTAtom.color;
  radius = MVTAtom.radius;
  ordinalNumber = MVTAtom.ordinalNumber;
  atomcoordinate = MVTAtom.atomcoordinate;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CMVTAtom::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTBond
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTBond::CMVTBond(QObject *parent, const char *name)
  : QObject(parent, name)
{
  label = 0;
  atomLabel[1] = 0;
  atomLabel[2] = 0;
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTBond::~CMVTBond()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTBond& CMVTBond::operator=(CMVTBond &MVTBond)
{
  // equalize members
  label = MVTBond.label;
  atomLabel[1] = MVTBond.atomLabel[1];
  atomLabel[2] = MVTBond.atomLabel[2];
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CMVTBond::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// GetAtomLabel
int CMVTBond::GetAtomLabel(int atom)
{
  if(atom != 1 && atom != 2) return(-1);
  return(atomLabel[atom]);
}

/////////////////////////////////////////////////////////////////////////////
// SetAtomLabel
int CMVTBond::SetAtomLabel(int atom, int newlabel)
{
  if(atom != 1 && atom != 2) return FALSE;
  atomLabel[atom] = newlabel;
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTSection
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTSection::CMVTSection(QObject *parent,
                         const char *name,
                         QString newSectionName,
                         int newNumberOfColumns,
                         int newSectionType)
  : QObject(parent, name)
{
  SectionName = newSectionName;
  NumberOfColumns = newNumberOfColumns;
  SectionType = newSectionType;
  if(SectionType > 2 || SectionType < 1) SectionType = OPTIONAL;
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
CMVTSection::~CMVTSection()
{
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTSection& CMVTSection::operator=(CMVTSection &MVTSection)
{
  // equalize members
  SectionName = MVTSection.SectionName;
  NumberOfColumns = MVTSection.NumberOfColumns;
  SectionType = MVTSection.SectionType;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// ReadSection
int CMVTSection::ReadSection(QFile &MVTFile)
{
  int LineNumber = 0;
  int ColumnNumber = 0;
  int sectionStartLine = 0;
  int readingSection = 0;
  QString Line = "";
  QString unformatedLine = "";
  if(!MVTFile.open(IO_ReadOnly))
  {
    return FALSE;
  }
  else
  {
    while(!MVTFile.atEnd())
    {
      // Reading was not successfull
      if(MVTFile.readLine(Line, MVT_FILE_MAX_LINE_LENGTH) == -1)
      {
        MVTFile.close();
        return(FALSE);
      }
      unformatedLine = Line;
      // format string
      Line = Line.simplifyWhiteSpace();
      Line = Line.upper();
      // search for sections
      if(Line.left(2) != "//")
      {
        LineNumber++;
        if(Line.find(SectionType == MAIN ? "#MAIN SECTION:" : "#OPTIONAL SECTION:") != -1 && Line.find(SectionName.upper()) != -1)
        {
          sectionStartLine = LineNumber;
        }
        if(Line.find("#BEGIN") != -1 && LineNumber == sectionStartLine + 1)
        {
          readingSection = 1;
        }
        if(Line.find("#END") != -1 && readingSection == 1)
        {
          readingSection = 0;
          MVTFile.close();
          return(NumberOfColumns == ColumnNumber);
        }
        if(Line.find("#BEGIN") == -1 && readingSection == 1)
        {
          // reading section
          ColumnNumber = 0;
          if(NumberOfColumns == 1)
          {
            // work with unformated line if only one column
            ColumnNumber++;
            if(!ReadData(unformatedLine, LineNumber - sectionStartLine - 1, ColumnNumber)) return(FALSE);
          }
          else
          {
            // work with formated line if more than one column
            while(Line.find(" ") != -1 && NumberOfColumns > 1)
            {
              ColumnNumber++;
              if(!ReadData(Line.left(Line.find(" ")), LineNumber - sectionStartLine - 1, ColumnNumber)) return(FALSE);
              Line = Line.right(Line.length() - Line.find(" ") - 1);
            }
            if(!Line.isEmpty())
            {
              ColumnNumber++;
              if(!ReadData(Line, LineNumber - sectionStartLine - 1, ColumnNumber)) return(FALSE);
            }
          }
        }           
      }
    }
  }
  MVTFile.close();

  return((SectionType == MAIN && NumberOfColumns == ColumnNumber && sectionStartLine != 0) ||
         (SectionType == OPTIONAL && (sectionStartLine == 0 || (sectionStartLine != 0 && NumberOfColumns == ColumnNumber))));
}

/////////////////////////////////////////////////////////////////////////////
// WriteSection
int CMVTSection::WriteSection(QFile &MVTFile)
{
  int LineNumber = 0;
  int ColumnNumber = 0;
  QString Line = "";
  QString tmp_Line = " ";
  if(!MVTFile.open(IO_ReadWrite))
  {
    return(FALSE);
  }
  else
  {
    // go to the end of file
    while(!MVTFile.atEnd())
    {
      // reading was not successfull
      if(MVTFile.readLine(Line, MVT_FILE_MAX_LINE_LENGTH) == -1)
      {
        MVTFile.close();
        return(FALSE);
      }
      // format string
      Line = Line.simplifyWhiteSpace();
      Line = Line.upper();
      // section exists
      if(Line.find(SectionType == MAIN ? "#MAIN SECTION:" : "#OPTIONAL SECTION:") != -1  && Line.find(SectionName.upper()) != -1)
      {
        MVTFile.close();
        return(FALSE);
      }
    }
    // Write new section
    Line = (SectionType == MAIN ? "#MAIN SECTION: " : "#OPTIONAL SECTION: ") + SectionName.upper() + "\r\n";
    if(MVTFile.writeBlock(Line, Line.length()) == -1)
    {
      MVTFile.close();
      return(FALSE);
    }
    Line = "#BEGIN\r\n";
    if(MVTFile.writeBlock(Line, Line.length()) == -1)
    {
      MVTFile.close();
      return(FALSE);
    }
    while(!tmp_Line.isEmpty())
    {
      Line = "";
      LineNumber++;
      for(ColumnNumber = 1; ColumnNumber <= NumberOfColumns; ColumnNumber++)
      {
        if(GetNumberOfLines() == 0 && LineNumber == 1)
          tmp_Line = (ColumnNumber == 1 ? "//NO DATA AVAILABLE" : " ");
        else
          tmp_Line = WriteData(LineNumber, ColumnNumber);
        Line += tmp_Line;
        if(ColumnNumber != NumberOfColumns)
          Line += "  ";
        else
          Line += "\r\n";
      }
      if(tmp_Line.isEmpty() || MVTFile.writeBlock(Line, Line.length()) == -1)
      {
        MVTFile.close();
        return(FALSE);
      }          
      tmp_Line = WriteData(LineNumber + 1, 1);      
    }
    Line = "#END\r\n";
    if(MVTFile.writeBlock(Line, Line.length()) == -1)
    {
      MVTFile.close();
      return(FALSE);
    }    
  }
  MVTFile.close();
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTSectionAtoms
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTSectionAtoms::CMVTSectionAtoms(QObject *parent,
                                   const char *name,
                                   QString newSectionName,
                                   int columns,
                                   int SectionType)
  : CMVTSection(parent, name, newSectionName, columns, SectionType)
{
  // enabling auto-delete will delete objects that are removed
  AtomList.setAutoDelete(TRUE);
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTSectionAtoms::~CMVTSectionAtoms()
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTSectionAtoms& CMVTSectionAtoms::operator=(CMVTSectionAtoms &MVTSectionAtoms)
{
  uint i = 0;
  Reset();
  // equalize members
  SectionName = MVTSectionAtoms.SectionName;
  NumberOfColumns = MVTSectionAtoms.NumberOfColumns;
  SectionType = MVTSectionAtoms.SectionType;
  // AtomList
  AtomList.clear();
  if(MVTSectionAtoms.AtomList.first())
  {
    for(i = 0; i <= MVTSectionAtoms.AtomList.count() - 1; i++)
    {
      Atom = new CMVTAtom;
      if(!Atom)
      {
        Reset();
        return *this;
      }
      Atom = (CMVTAtom*)MVTSectionAtoms.AtomList.current();
      AtomList.append(Atom);
      MVTSectionAtoms.AtomList.next();
    }
  }
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// ReadData
int CMVTSectionAtoms::ReadData(QString data, int, int column)
{
  QColor tmpAtomColor;
  // create new Atom
  if(column == 1)
  {
    if(!(Atom = new CMVTAtom))
    {
       Reset();
      return (FALSE);
    }
  }
  // collect Data
  switch(column)
  {
    case(1):
    {
      Atom->SetLabel(data.toInt());
      break;
    }
    case(2):
    {
      Atom->SetOrdinalNumber(data.toInt());
      break;
    }
    case(3):
    {
      Atom->SetX(data.toFloat());
      break;
    }
    case(4):
    {
      Atom->SetY(data.toFloat());
      break;
    }
    case(5):
    {
      Atom->SetZ(data.toFloat());
      break;
    }
    case(6):
    {
      tmpAtomColor.setRgb(data.toInt(), (Atom->GetColor()).green(), (Atom->GetColor()).blue());
      Atom->SetColor(tmpAtomColor);
      break;
    }
    case(7):
    {
      tmpAtomColor.setRgb((Atom->GetColor()).red(), data.toInt(), (Atom->GetColor()).blue());
      Atom->SetColor(tmpAtomColor);
      break;
    }
    case(8):
    {
      tmpAtomColor.setRgb((Atom->GetColor()).red(), (Atom->GetColor()).green(), data.toInt());
      Atom->SetColor(tmpAtomColor);
      break;
    }
    case(9):
    {
      Atom->SetRadius(data.toFloat());
      break;
    }
    default:
    {
      return(FALSE);
      break;
    }    
  }
  if(NumberOfColumns == column) AtomList.append(Atom);
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// WriteData
QString CMVTSectionAtoms::WriteData(int line, int column)
{
  Atom = GetAtom(line);
  QString returnValue = "";
  if(Atom == NULL) return("");
  switch(column)
  {
    case(1):
    {
      returnValue.setNum(Atom->GetLabel());
      while(returnValue.length() < 3) returnValue += " ";
      break;
    }
    case(2):
    {
      returnValue.setNum(Atom->GetOrdinalNumber());
      while(returnValue.length() < 3) returnValue += " ";      
      break;
    }
    case(3):
    {
      returnValue.setNum(Atom->GetX(), 'f');
      if(returnValue.left(1) != "-") returnValue = " " + returnValue;
      break;
    }
    case(4):
    {
      returnValue.setNum(Atom->GetY(), 'f');
      if(returnValue.left(1) != "-") returnValue = " " + returnValue;
      break;
    }
    case(5):
    {
      returnValue.setNum(Atom->GetZ(), 'f');
      if(returnValue.left(1) != "-") returnValue = " " + returnValue;
      break;
    }
    case(6):
    {
      returnValue.setNum((Atom->GetColor()).red());
      while(returnValue.length() < 3) returnValue = " " + returnValue;
      break;
    }
    case(7):
    {
      returnValue.setNum((Atom->GetColor()).green());
      while(returnValue.length() < 3) returnValue = " " + returnValue;
      break;
    }
    case(8):
    {
      returnValue.setNum((Atom->GetColor()).blue());
      while(returnValue.length() < 3) returnValue = " " + returnValue;  
      break;
    }
    case(9):
    {
      returnValue.setNum(Atom->GetRadius(), 'f');
      break;
    }
    default:
    {
      returnValue = "";
      break;
    }    
  }
  return(returnValue);
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CMVTSectionAtoms::Reset(void)
{
  // Reset list  
  AtomList.clear();
}

/////////////////////////////////////////////////////////////////////////////
// GetAtom
CMVTAtom* CMVTSectionAtoms::GetAtom(uint LineNumber)
{
  if(LineNumber > AtomList.count() || LineNumber < 1) return NULL;
  uint i = 0;
  AtomList.first();
  for(i = 1; i <= LineNumber; i++)
  {
    if(!(Atom = (CMVTAtom*)AtomList.current())) return NULL;
    AtomList.next();
  }
  return(Atom);
}

/////////////////////////////////////////////////////////////////////////////
// SetAtom
int CMVTSectionAtoms::SetAtom(CMVTAtom* newAtom)
{
  if(!(Atom = new CMVTAtom))
  {
    Reset();
    return (FALSE);
  }
  *Atom = *newAtom;
  AtomList.append(Atom);
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTSectionBonds
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTSectionBonds::CMVTSectionBonds(QObject *parent,
                                   const char *name,
                                   QString newSectionName,
                                   int columns,
                                   int SectionType)
  : CMVTSection(parent, name, newSectionName, columns, SectionType)
{
  // enabling auto-delete will delete objects that are removed
  BondList.setAutoDelete(TRUE);
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTSectionBonds::~CMVTSectionBonds()
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTSectionBonds& CMVTSectionBonds::operator=(CMVTSectionBonds &MVTSectionBonds)
{
  uint i = 0;
  Reset();
  // equalize members
  SectionName = MVTSectionBonds.SectionName;
  NumberOfColumns = MVTSectionBonds.NumberOfColumns;
  SectionType = MVTSectionBonds.SectionType;
  // AtomList
  BondList.clear();
  if(MVTSectionBonds.BondList.first())
  {
    for(i = 0; i <= MVTSectionBonds.BondList.count() - 1; i++)
    {
      Bond = new CMVTBond;
      if(!Bond)
      {
        Reset();
        return *this;
      }
      Bond = (CMVTBond*)MVTSectionBonds.BondList.current();
      BondList.append(Bond);
      MVTSectionBonds.BondList.next();
    }
  }
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// ReadData
int CMVTSectionBonds::ReadData(QString data, int, int column)
{
  // create new Bond
  if(column == 1)
  {
    if(!(Bond = new CMVTBond))
    {
       Reset();
      return (FALSE);
    }
  }
  // collect Data
  switch(column)
  {
    case(1):
    {
      Bond->SetLabel(data.toInt());
      break;
    }
    case(2):
    {
      Bond->SetAtomLabel(1, (data.left(data.find(","))).toInt());
      Bond->SetAtomLabel(2, (data.right(data.length() - data.find(",") - 1)).toInt());
      break;
    }
    default:
    {
      return(FALSE);
      break;
    }    
  }
  if(NumberOfColumns == column) BondList.append(Bond);
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// WriteData
QString CMVTSectionBonds::WriteData(int line, int column)
{
  Bond = GetBond(line);
  QString returnValue = "";
  QString tmpReturnValue = "";  
  if(Bond == NULL) return("");
  if(BondList.count() == 0) return(" ");
  switch(column)
  {
    case(1):
    {
      returnValue.setNum(Bond->GetLabel());
      while(returnValue.length() < 3) returnValue += " ";
      break;
    }
    case(2):
    {
      returnValue.setNum(Bond->GetAtomLabel(1));
      tmpReturnValue.setNum(Bond->GetAtomLabel(2));
      returnValue += "," + tmpReturnValue;
      break;
    }
    default: {returnValue = ""; break;}    
  }
  return(returnValue);
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CMVTSectionBonds::Reset(void)
{
  // Reset list  
  BondList.clear();
}

/////////////////////////////////////////////////////////////////////////////
// GetBond
CMVTBond* CMVTSectionBonds::GetBond(uint LineNumber)
{
  if(LineNumber > BondList.count() || LineNumber < 1) return NULL;
  uint i = 0;
  BondList.first();
  for(i = 1; i <= LineNumber; i++)
  {
    if(!(Bond = (CMVTBond*)BondList.current())) return NULL;
    BondList.next();
  }
  return(Bond);
}

/////////////////////////////////////////////////////////////////////////////
// SetBond
int CMVTSectionBonds::SetBond(CMVTBond* newBond)
{
  if(!(Bond = new CMVTBond))
  {
    Reset();
    return (FALSE);
  }
  *Bond = *newBond;
  BondList.append(Bond);
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTSectionMatrix
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTSectionMatrix::CMVTSectionMatrix(QObject *parent,
                                     const char *name,
                                     QString newSectionName,
                                     int columns,
                                     int SectionType)
  : CMVTSection(parent, name, newSectionName, columns, SectionType)
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTSectionMatrix::~CMVTSectionMatrix()
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTSectionMatrix& CMVTSectionMatrix::operator=(CMVTSectionMatrix &MVTSectionMatrix)
{
  Reset();
  // equalize members
  SectionName = MVTSectionMatrix.SectionName;
  NumberOfColumns = MVTSectionMatrix.NumberOfColumns;
  SectionType = MVTSectionMatrix.SectionType;
  // Matrix
  MVTCoordinate[1] = MVTSectionMatrix.MVTCoordinate[1];
  MVTCoordinate[2] = MVTSectionMatrix.MVTCoordinate[2];
  MVTCoordinate[3] = MVTSectionMatrix.MVTCoordinate[3];
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// ReadData
int CMVTSectionMatrix::ReadData(QString data, int line, int column)
{
  if(column < 1 || column > 3) return(FALSE);
  switch(line)
  {
    case(1):
    {
      MVTCoordinate[column].SetX(data.toFloat());
      break;
    }
    case(2):
    {
      MVTCoordinate[column].SetY(data.toFloat());
      break;
    }
    case(3):
    {
      MVTCoordinate[column].SetZ(data.toFloat());
      break;
    }
    default: {return(FALSE); break;}    
  }
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// WriteData
QString CMVTSectionMatrix::WriteData(int line, int column)
{
  QString returnValue = "";    
  if(column < 1 || column > 3) return("");
  switch(line)
  {
    case(1):
    {
      returnValue.setNum(MVTCoordinate[column].GetX(), 'f');
      if(returnValue.left(1) != "-") returnValue = " " + returnValue;
      break;
    }
    case(2):
    {
      returnValue.setNum(MVTCoordinate[column].GetY(), 'f');
      if(returnValue.left(1) != "-") returnValue = " " + returnValue;
      break;
    }
    case(3):
    {
      returnValue.setNum(MVTCoordinate[column].GetZ(), 'f');
      if(returnValue.left(1) != "-") returnValue = " " + returnValue;
      break;
    }
    default: {returnValue = ""; break;}    
  }
  return(returnValue);
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CMVTSectionMatrix::Reset(void)
{
  MVTCoordinate[1].SetX(1.0);
  MVTCoordinate[1].SetY(0.0);
  MVTCoordinate[1].SetZ(0.0);
  MVTCoordinate[2].SetX(0.0);
  MVTCoordinate[2].SetY(1.0);
  MVTCoordinate[2].SetZ(0.0);
  MVTCoordinate[3].SetX(0.0);
  MVTCoordinate[3].SetY(0.0);
  MVTCoordinate[3].SetZ(1.0);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTSectionTranslation
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTSectionTranslation::CMVTSectionTranslation(QObject *parent,
                                               const char *name,
                                               QString newSectionName,
                                               int columns,
                                               int SectionType)
  : CMVTSection(parent, name, newSectionName, columns, SectionType)
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTSectionTranslation::~CMVTSectionTranslation()
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTSectionTranslation& CMVTSectionTranslation::operator=(CMVTSectionTranslation &MVTSectionTranslation)
{
  Reset();
  // equalize members
  SectionName = MVTSectionTranslation.SectionName;
  NumberOfColumns = MVTSectionTranslation.NumberOfColumns;
  SectionType = MVTSectionTranslation.SectionType;
  // Translation
  Translation = MVTSectionTranslation.Translation;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// ReadData
int CMVTSectionTranslation::ReadData(QString data, int line, int column)
{
  if(column != 1) return(FALSE);
  switch(line)
  {
    case(1):
    {
      Translation.SetX(data.toFloat());
      break;
    }
    case(2):
    {
      Translation.SetY(data.toFloat());
      break;
    }
    case(3):
    {
      Translation.SetZ(data.toFloat());
      break;
    }
    default: {return(FALSE); break;}
  }
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// WriteData
QString CMVTSectionTranslation::WriteData(int line, int column)
{
  QString returnValue = "";
  if(column != 1) return("");
  switch(line)
  {
    case(1):
    {
      returnValue.setNum(Translation.GetX(), 'f');
      if(returnValue.left(1) != "-") returnValue = " " + returnValue;
      break;
    }
    case(2):
    {
      returnValue.setNum(Translation.GetY(), 'f');
      if(returnValue.left(1) != "-") returnValue = " " + returnValue;
      break;
    }
    case(3):
    {
      returnValue.setNum(Translation.GetZ(), 'f');
      if(returnValue.left(1) != "-") returnValue = " " + returnValue;
      break;
    }
    default: {returnValue = ""; break;}
  }
  return(returnValue);
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CMVTSectionTranslation::Reset(void)
{
  Translation.Reset();
}

/////////////////////////////////////////////////////////////////////////////
// GetCoordinateSystem
CMVTCoordinate* CMVTSectionMatrix::GetCoordinateSystem(int axis)
{
  if(axis < 1 || axis > 3) return(NULL);
  return(&MVTCoordinate[axis]);
}

/////////////////////////////////////////////////////////////////////////////
// SetCoordinateSystem
int CMVTSectionMatrix::SetCoordinateSystem(int axis, CMVTCoordinate* newMVTCoordinate)
{
  if(axis < 1 || axis > 3) return(FALSE);
  MVTCoordinate[axis] = *newMVTCoordinate;
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTSectionTitle
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTSectionTitle::CMVTSectionTitle(QObject *parent,
                                   const char *name,
                                   QString newSectionName,
                                   int columns,
                                   int SectionType)
  : CMVTSection(parent, name, newSectionName, columns, SectionType)
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTSectionTitle::~CMVTSectionTitle()
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTSectionTitle& CMVTSectionTitle::operator=(CMVTSectionTitle &MVTSectionTitle)
{
  Reset();
  // equalize members
  SectionName = MVTSectionTitle.SectionName;
  NumberOfColumns = MVTSectionTitle.NumberOfColumns;
  SectionType = MVTSectionTitle.SectionType;
  // Title
  Title = MVTSectionTitle.Title;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// ReadData
int CMVTSectionTitle::ReadData(QString data, int line, int column)
{
  if(column != 1 || line != 1) return(FALSE);
  Title = data.stripWhiteSpace();
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// WriteData
QString CMVTSectionTitle::WriteData(int line, int column)
{
  QString returnValue = "";    
  if(column != 1 || line != 1) return("");
  return(Title.length() == 0 ? QString(" ") : Title);
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CMVTSectionTitle::Reset(void)
{
  Title = "";
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTSectionSettings
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTSectionSettings::CMVTSectionSettings(QObject *parent,
                                         const char *name,
                                         QString newSectionName,
                                         int columns,
                                         int SectionType)
  : CMVTSection(parent, name, newSectionName, columns, SectionType)
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTSectionSettings::~CMVTSectionSettings()
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTSectionSettings& CMVTSectionSettings::operator=(CMVTSectionSettings &MVTSectionSettings)
{
  Reset();
  // equalize members
  SectionName = MVTSectionSettings.SectionName;
  NumberOfColumns = MVTSectionSettings.NumberOfColumns;
  SectionType = MVTSectionSettings.SectionType;
  Parameter = MVTSectionSettings.Parameter;
  // Settings
  Label = MVTSectionSettings.Label;
  Perspective = MVTSectionSettings.Perspective;
  RadiusReduction = MVTSectionSettings.RadiusReduction;
  BondColor = MVTSectionSettings.BondColor;
  BondFudge = MVTSectionSettings.BondFudge;
  BondRadius = MVTSectionSettings.BondRadius;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// ReadData
int CMVTSectionSettings::ReadData(QString data, int, int column)
{
  switch(column)
  {
    case(1):
    {
      if(data.upper() == "LABEL=" ||
         data.upper() == "PERSPECTIVE=" ||
         data.upper() == "RADIUSREDUCTION=" ||
         data.upper() == "BONDCOLOR=" ||
         data.upper() == "BONDFUDGE=" ||
         data.upper() == "BONDRADIUS=")
      {
        Parameter = data.upper();
      }
      else
      {
        Parameter = "";
        return(FALSE);
      }
      break;
    }
    case(2):
    {
      if(Parameter.isEmpty()) return(FALSE);
      if(Parameter == "LABEL=" && !data.isEmpty())
      {
        if((data != "NOLABEL") && (data != "ATOMSYMBOL") && (data != "ORDINALNUMBER") && (data != "ATOMNUMBER")) return(FALSE);        
        if(data == "NOLABEL") Label = 0;
        if(data == "ATOMSYMBOL") Label = 1;
        if(data == "ORDINALNUMBER") Label = 2;
        if(data == "ATOMNUMBER") Label = 3;
      }
      if(Parameter == "PERSPECTIVE=" && !data.isEmpty()) Perspective = data.toFloat();
      if(Parameter == "RADIUSREDUCTION=" && !data.isEmpty()) RadiusReduction = data.toFloat();
      if(Parameter == "BONDCOLOR=" && !data.isEmpty()) BondColor = (QColor)data.toInt();
      if(Parameter == "BONDFUDGE=" && !data.isEmpty()) BondFudge = data.toFloat();
      if(Parameter == "BONDRADIUS=" && !data.isEmpty()) BondRadius = data.toFloat();
      Parameter = "";
      break;
    }
    default:
    {
      return(FALSE);
      break;
    }    
  }
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// WriteData
QString CMVTSectionSettings::WriteData(int line, int column)
{
  QString returnValue = "";    
  if(column < 1 || column > 2) return("");
  if(column == 1)
  {
    switch(line)
    {
      case(1): {returnValue = "Label="; break;}
      case(2): {returnValue = "Perspective="; break;}
      case(3): {returnValue = "RadiusReduction="; break;}
      case(4): {returnValue = "BondColor="; break;}
      case(5): {returnValue = "BondFudge="; break;}
      case(6): {returnValue = "BondRadius="; break;}
      default: {returnValue = ""; break;}    
    }
  }
  else
  {
    switch(line)
    {
      case(1):
      {
        switch(Label)
        {
          case(0): {returnValue = "NOLABEL"; break;}
          case(1): {returnValue = "ATOMSYMBOL"; break;}
          case(2): {returnValue = "ORDINALNUMBER"; break;}
          case(3): {returnValue = "ATOMNUMBER"; break;}
          default: {returnValue = ""; break;}
        }  
        break;
      }
      case(2): {returnValue.setNum(Perspective); break;}
      case(3): {returnValue.setNum(RadiusReduction); break;}
      case(4): {returnValue.setNum(BondColor.rgb() & 16777215); break;}
      case(5): {returnValue.setNum(BondFudge); break;}
      case(6): {returnValue.setNum(BondRadius); break;}
      default: {returnValue = ""; break;}    
    }
  }
  return(returnValue);
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CMVTSectionSettings::Reset(void)
{
  Parameter = "";
  // data
  Label = 0;
  Perspective = 2.5;
  RadiusReduction = 0.5;
  BondColor = 2105376;
  BondFudge = 1.12;
  BondRadius = 0.1;
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CMVTFile
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CMVTFile::CMVTFile(QObject *parent, const char *name)
  : QObject(parent, name)
{
  // Reset members;
  Reset(1);
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CMVTFile::~CMVTFile()
{
  Reset(1);
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CMVTFile& CMVTFile::operator=(CMVTFile &MVTFile)
{
  // clear
  Reset(1);
  // equalize members
  SectionAtoms = MVTFile.SectionAtoms;
  SectionBonds = MVTFile.SectionBonds;
  SectionMatrix = MVTFile.SectionMatrix;
  SectionTranslation = MVTFile.SectionTranslation;
  SectionTitle = MVTFile.SectionTitle;
  SectionSettings = MVTFile.SectionSettings;
  // members
  FileName = MVTFile.FileName;  
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CMVTFile::Reset(int resetFileName)
{
  // Reset members;
  SectionAtoms.Reset();
  SectionBonds.Reset();
  SectionMatrix.Reset();
  SectionTranslation.Reset();
  SectionTitle.Reset();
  SectionSettings.Reset();
  if(resetFileName) FileName = QDir::currentDirPath() + "/" + "untitled.mvt";
}

/////////////////////////////////////////////////////////////////////////////
// SaveFile
int CMVTFile::SaveFile(QString filename)
{
  FileName = filename;
  QFile MVTFile;
  MVTFile.setName(FileName);
  if(MVTFile.exists() && !MVTFile.remove()) return(FALSE);
  if(FileName.isEmpty()) return(FALSE);
  if(!SectionAtoms.WriteSection(MVTFile)) return(FALSE);
  if(!SectionBonds.WriteSection(MVTFile)) return(FALSE);
  if(!SectionTitle.WriteSection(MVTFile)) return(FALSE);
  if(!SectionMatrix.WriteSection(MVTFile)) return(FALSE);
  if(!SectionTranslation.WriteSection(MVTFile)) return(FALSE);
  if(!SectionSettings.WriteSection(MVTFile)) return(FALSE);
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// LoadFile
int CMVTFile::LoadFile(QString filename)
{
  FileName = filename;
  QFile MVTFile;
  MVTFile.setName(FileName);

  if(!SectionAtoms.ReadSection(MVTFile)) return(FALSE);
  if(!SectionBonds.ReadSection(MVTFile)) return(FALSE);
  if(!SectionTitle.ReadSection(MVTFile)) return(FALSE);
  if(!SectionMatrix.ReadSection(MVTFile)) return(FALSE);
  if(!SectionTranslation.ReadSection(MVTFile)) return(FALSE);
  if(!SectionSettings.ReadSection(MVTFile)) return(FALSE);
  return(TRUE);
}
