/***************************************************************************
                          mvtfile.h  -  description
                             -------------------
    begin                : Mon Apr 17 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _MVTFILE
#define _MVTFILE

#define X_AXIS 1
#define Y_AXIS 2
#define Z_AXIS 3
#define MAIN 1
#define OPTIONAL 2

/* include files for QT */
#include <qobject.h>
#include <qstring.h>
#include <qfile.h>
#include <qcolor.h>
#include <qdir.h>
#include <qlist.h>

/* application specific includes */
#include <iostream>

#define MVT_FILE_MAX_LINE_LENGTH 256

/* class CMVTCoordinate */
class CMVTCoordinate : public QObject
{
  Q_OBJECT
  friend class CMVTFile;

public:
  /* Constructor */
  CMVTCoordinate(QObject *parent=0, const char *name=0);

  /* Destructor */
  ~CMVTCoordinate();

  /* Operators */
  CMVTCoordinate& operator=(CMVTCoordinate &MVTCoordinate);

  /* Methods */
  void Reset(void);
  void Serialize();

  /* Get... */
  float GetX(void) {return (x);}
  float GetY(void) {return (y);}
  float GetZ(void) {return (z);}

  /* Set... */
  void SetX(float newx) {x = newx; return;}
  void SetY(float newy) {y = newy; return;}
  void SetZ(float newz) {z = newz; return;}

protected:
  /* Members */
  float x;
  float y;
  float z;
};

/* class CMVTAtom */
class CMVTAtom : public QObject
{
  Q_OBJECT

public:
  /* Constructor */
  CMVTAtom(QObject *parent=0, const char *name=0);

  /* Destructor */
  ~CMVTAtom();

  /* Operators */
  CMVTAtom& operator=(CMVTAtom &CMVTAtom);

  /* Methods */
  void Serialize();

  /* Get... */
  int GetLabel(void) {return (label);}
  QColor GetColor(void) {return (color);}
  float GetRadius(void) {return (radius);}
  int GetOrdinalNumber(void) {return (ordinalNumber);}
  float GetX(void) {return (atomcoordinate.GetX());}
  float GetY(void) {return (atomcoordinate.GetY());};
  float GetZ(void) {return (atomcoordinate.GetZ());}

  /* Set... */
  void SetLabel(int newlabel) {label = newlabel; return;}
  void SetColor(QColor newcolor) {color = newcolor; return;}
  void SetRadius(float newradius) {radius = newradius; return;}
  void SetOrdinalNumber(int newOrdinalNumber) {ordinalNumber = newOrdinalNumber; return;}
  void SetX(float newx) {atomcoordinate.SetX(newx);}
  void SetY(float newy) {atomcoordinate.SetY(newy);}
  void SetZ(float newz) {atomcoordinate.SetZ(newz);}

protected:
  /* Members */
  int label;
  QColor color;
  float radius;
  int ordinalNumber;
  CMVTCoordinate atomcoordinate;
};

/* class CMVTBond */
class CMVTBond : public QObject
{
  Q_OBJECT

public:
  /* Constructor */
  CMVTBond(QObject *parent=0, const char *name=0);

  /* Destructor */
  ~CMVTBond();

  /* Operators */
  CMVTBond& operator=(CMVTBond &MVTBond);

  /* Methods */
  void Serialize();

  /* Get... */
  int GetLabel(void) {return (label);}
  int GetAtomLabel(int atom);

  /* Set... */
  void SetLabel(int newlabel) {label = newlabel; return;}
  int SetAtomLabel(int atom, int newlabel);

protected:
  /* Members */
  int label;
  int atomLabel[3];
};

/* class CMVTSection */
class CMVTSection : public QObject
{
  Q_OBJECT
  friend class CMVTFile;

public:
  /* Constructor */
  CMVTSection(QObject *parent = 0,
              const char *name = 0,
              QString newSectionName = "",
              int newNumberOfColumns = 0,
              int SectionType = OPTIONAL);
  /* Destructor */
  ~CMVTSection();

  /* Operators */
  CMVTSection& operator=(CMVTSection &MVTSection);

  /* Methods */
  int ReadSection(QFile &MVTFile);
  int WriteSection(QFile &MVTFile);
  
  /* Set ... */
  void SetSectionType(int newSectionType) {SectionType = newSectionType;};
  void SetSectionName(QString newSectionName) {SectionName = newSectionName;};
  void SetNumberOfColumns(int newNumberOfColumns) {NumberOfColumns = newNumberOfColumns;};

  /* Get ... */
  int GetSectionType(void) {return(SectionType);};
  QString GetSectionName(void) {return(SectionName);};
  virtual int GetNumberOfColumns(void) {return(NumberOfColumns);};
  virtual int GetNumberOfLines(void) = 0;

protected:
  /* Methods */  
  virtual int ReadData(QString data, int line, int column) = 0;
  virtual QString WriteData(int line, int column) = 0;

  /* Members */
  QString SectionName;
  int NumberOfColumns;
  int SectionType;
};

/* class CMVTSectionAtoms */
class CMVTSectionAtoms : public CMVTSection
{
  Q_OBJECT

public:
  /* Constructor */
  CMVTSectionAtoms(QObject *parent=0,
                   const char *name=0,
                   QString newSectionName = "ATOMS",
                   int columns = 9,
                   int SectionType = MAIN);

  /* Destructor */
  ~CMVTSectionAtoms();

  /* Operators */
  CMVTSectionAtoms& operator=(CMVTSectionAtoms &MVTSectionAtoms);

  /* Methods */
  void Reset(void);

  /* Get... */
  CMVTAtom* GetAtom(uint LineNumber);
  virtual int GetNumberOfLines(void) {return AtomList.count();};

  /* Set... */
  int SetAtom(CMVTAtom* newAtom);

protected:
  /* Methods */
  virtual int ReadData(QString data, int, int column);
  virtual QString WriteData(int line, int column);

  /* Members */
  QList <CMVTAtom> AtomList;
  CMVTAtom* Atom;
};

/* class CMVTSectionBonds */
class CMVTSectionBonds : public CMVTSection
{
  Q_OBJECT

public:
  /* Constructor */
  CMVTSectionBonds(QObject *parent=0,
                   const char *name=0,
                   QString newSectionName = "BONDS",
                   int columns = 2,
                   int SectionType = MAIN);

  /* Destructor */
  ~CMVTSectionBonds();

  /* Operators */
  CMVTSectionBonds& operator=(CMVTSectionBonds &MVTSectionBonds);

  /* Methods */
  void Reset(void);

  /* Get... */
  CMVTBond* GetBond(uint LineNumber);
  virtual int GetNumberOfLines(void) {return BondList.count();};

  /* Set... */
  int SetBond(CMVTBond* newBond);

protected:
  /* Methods */
  virtual int ReadData(QString data, int, int column);
  virtual QString WriteData(int line, int column);

  /* Members */
  QList <CMVTBond> BondList;
  CMVTBond* Bond;
};

/* class CMVTSectionMatrix */
class CMVTSectionMatrix : public CMVTSection
{
  Q_OBJECT

public:
  /* Constructor */
  CMVTSectionMatrix(QObject *parent=0,
                   const char *name=0,
                   QString newSectionName = "MATRIX",
                   int columns = 3,
                   int SectionType = OPTIONAL);

  /* Destructor */
  ~CMVTSectionMatrix();

  /* Operators */
  CMVTSectionMatrix& operator=(CMVTSectionMatrix &MVTSectionMatrix);

  /* Methods */
  void Reset(void);

  /* Get... */
  CMVTCoordinate* GetCoordinateSystem(int axis);
  virtual int GetNumberOfLines(void) {return 3;};

  /* Set... */
  int SetCoordinateSystem(int axis, CMVTCoordinate* newMVTCoordinate);

protected:
  /* Methods */
  virtual int ReadData(QString data, int line, int column);
  virtual QString WriteData(int line, int column);

  /* Members */
  CMVTCoordinate MVTCoordinate[4];
};

/* class CMVTSectionTranslation */
class CMVTSectionTranslation : public CMVTSection
{
  Q_OBJECT

public:
  /* Constructor */
  CMVTSectionTranslation(QObject *parent=0,
                         const char *name=0,
                         QString newSectionName = "TRANSLATION",
                         int columns = 1,
                         int SectionType = OPTIONAL);

  /* Destructor */
  ~CMVTSectionTranslation();

  /* Operators */
  CMVTSectionTranslation& operator=(CMVTSectionTranslation &MVTSectionTranslation);

  /* Methods */
  void Reset(void);

  /* Get... */
  CMVTCoordinate* GetTranslation(void) {return(&Translation);};
  virtual int GetNumberOfLines(void) {return 3;};

  /* Set... */
  void SetTranslation(CMVTCoordinate* newTranslation) {Translation = *newTranslation;};

protected:
  /* Methods */
  virtual int ReadData(QString data, int line, int column);
  virtual QString WriteData(int line, int column);

  /* Members */
  CMVTCoordinate Translation;
};

/* class CMVTSectionTitle */
class CMVTSectionTitle : public CMVTSection
{
  Q_OBJECT

public:
  /* Constructor */
  CMVTSectionTitle(QObject *parent=0,
                   const char *name=0,
                   QString newSectionName = "TITLE",
                   int columns = 1,
                   int SectionType = OPTIONAL);

  /* Destructor */
  ~CMVTSectionTitle();

  /* Operators */
  CMVTSectionTitle& operator=(CMVTSectionTitle &MVTSectionTitle);

  /* Methods */
  void Reset(void);

  /* Get... */
  QString GetTitle(void) {return(Title);};
  virtual int GetNumberOfLines(void) {return 1;};

  /* Set... */
  void SetTitle(QString newTitle) {Title = newTitle;};

protected:
  /* Methods */
  virtual int ReadData(QString data, int line, int column);
  virtual QString WriteData(int line, int column);

  /* Members */
  QString Title;
};

/* class CMVTSectionSettings */
class CMVTSectionSettings : public CMVTSection
{
  Q_OBJECT

public:
  /* Constructor */
  CMVTSectionSettings(QObject *parent=0,
                      const char *name=0,
                      QString newSectionName = "SETTINGS",
                      int columns = 2,
                      int SectionType = OPTIONAL);

  /* Destructor */
  ~CMVTSectionSettings();

  /* Operators */
  CMVTSectionSettings& operator=(CMVTSectionSettings &MVTSectionSettings);

  /* Methods */
  void Reset(void);

  /* Get... */
  int GetLabel(void) {return Label;};  
  float GetPerspective(void) {return Perspective;};
  float GetRadiusReduction(void) {return RadiusReduction;};    
  QColor GetBondColor(void) {return BondColor;};  
  float GetBondFudge(void) {return BondFudge;};
  float GetBondRadius(void) {return BondRadius;};  
  virtual int GetNumberOfLines(void) {return 5;};

  /* Set... */
  void SetLabel(int newLabel) {Label = newLabel;};  
  void SetPerspective(float newPerspective) {Perspective = newPerspective;};
  void SetRadiusReduction(float newRadiusReduction) {RadiusReduction = newRadiusReduction;};    
  void SetBondColor(QColor newBondColor) {BondColor = newBondColor;};  
  void SetBondFudge(float newBondFudge) {BondFudge = newBondFudge;};  
  void SetBondRadius(float newBondRadius) {BondRadius = newBondRadius;};

protected:
  /* Methods */
  virtual int ReadData(QString data, int, int column);
  virtual QString WriteData(int line, int column);

  /* Members */
  QString Parameter;
  /* data */
  int Label;
  float Perspective;
  float RadiusReduction;
  QColor BondColor;
  float BondFudge;
  float BondRadius;
};

/* class CMVTFile */
class CMVTFile : public QObject
{
  Q_OBJECT

public:
  /* constructor */
  CMVTFile(QObject *parent=0, const char *name=0);

  /* destructor */
  ~CMVTFile();

  /* Operators */
  CMVTFile& operator=(CMVTFile &MVTFile);

  /* methods */
  void Reset(int resetFileName = 0);
  int SaveFile(QString filename);
  int LoadFile(QString filename);

  /* Get... */
  QString GetFileName(void) {return FileName;};
  int GetNumberOfAtoms(void) {return SectionAtoms.GetNumberOfLines();};  
  int GetNumberOfBonds(void) {return SectionBonds.GetNumberOfLines();};
  CMVTAtom* GetAtom(uint LineNumber) {return SectionAtoms.GetAtom(LineNumber);};
  CMVTBond* GetBond(uint LineNumber) {return SectionBonds.GetBond(LineNumber);};  
  QString GetTitle(void) {return SectionTitle.GetTitle();};
  CMVTCoordinate* GetCoordinateSystem(int axis) {return SectionMatrix.GetCoordinateSystem(axis);};
  CMVTCoordinate* GetTranslation(void) {return SectionTranslation.GetTranslation();};
  int GetLabel(void) {return SectionSettings.GetLabel();};
  float GetPerspective(void) {return SectionSettings.GetPerspective();};
  float GetRadiusReduction(void) {return SectionSettings.GetRadiusReduction();};    
  QColor GetBondColor(void) {return SectionSettings.GetBondColor();};  
  float GetBondFudge(void) {return SectionSettings.GetBondFudge();};  
  float GetBondRadius(void) {return SectionSettings.GetBondRadius();};

  /* Set ... */
  void SetFileName(QString newFileName) {FileName = newFileName;};
  int SetAtom(CMVTAtom* NewAtom) {return(SectionAtoms.SetAtom(NewAtom));};
  int SetBond(CMVTBond* NewBond) {return(SectionBonds.SetBond(NewBond));};
  void SetTitle(QString newTitle) {return(SectionTitle.SetTitle(newTitle));};
  int SetCoordinateSystem(int axis, CMVTCoordinate* newMVTCoordinate) {return SectionMatrix.SetCoordinateSystem(axis, newMVTCoordinate);};
  void SetTranslation(CMVTCoordinate* newTranslation) {SectionTranslation.SetTranslation(newTranslation);};
  void SetLabel(int newLabel) {SectionSettings.SetLabel(newLabel);};
  void SetPerspective(float newPerspective) {SectionSettings.SetPerspective(newPerspective);};
  void SetRadiusReduction(float newRadiusReduction) {SectionSettings.SetRadiusReduction(newRadiusReduction);};    
  void SetBondColor(QColor newBondColor) {SectionSettings.SetBondColor(newBondColor);};  
  void SetBondFudge(float newBondFudge) {SectionSettings.SetBondFudge(newBondFudge);};  
  void SetBondRadius(float newBondRadius) {SectionSettings.SetBondRadius(newBondRadius);};

protected:
  /* protected members */
  CMVTSectionAtoms SectionAtoms;
  CMVTSectionBonds SectionBonds;
  CMVTSectionTitle SectionTitle;
  CMVTSectionMatrix SectionMatrix;
  CMVTSectionTranslation SectionTranslation;
  CMVTSectionSettings SectionSettings;
  QString FileName;
};
#endif /* _MVTFILE */
