/***************************************************************************
                          povfile.cpp  -  description
                             -------------------
    begin                : Sun Feb 6 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files
#include <povfile.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CPOVCoordinate
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CPOVCoordinate::CPOVCoordinate(QObject *parent, const char *name)
  : QObject(parent, name)
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CPOVCoordinate::~CPOVCoordinate()
{

}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CPOVCoordinate::Reset(void)
{
  x = (float)0.0;
  y = (float)0.0;
  z = (float)0.0;
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CPOVCoordinate& CPOVCoordinate::operator=(CPOVCoordinate &POVCoordinate)
{
  // equalize members
  x = POVCoordinate.x;
  y = POVCoordinate.y;
  z = POVCoordinate.z;

  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CPOVCoordinate::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CPOVAtom
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CPOVAtom::CPOVAtom(QObject *parent, const char *name)
  : QObject(parent, name)
{
  label = 0;
  color.setRgb(0, 0, 0);
  radius = (float)0.0;
  texture = "";
  atomcoordinate.Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CPOVAtom::~CPOVAtom()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CPOVAtom& CPOVAtom::operator=(CPOVAtom &POVAtom)
{
  // equalize members
  label = POVAtom.label;
  color = POVAtom.color;
  radius = POVAtom.radius;
  texture = POVAtom.texture;
  atomcoordinate = POVAtom.atomcoordinate;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CPOVAtom::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CPOVBond
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CPOVBond::CPOVBond(QObject *parent, const char *name)
  : QObject(parent, name)
{
  label = 0;
  color.setRgb(0, 0, 0);
  radius = (float)0.0;
  texture = "";
  bondcoordinate[1].Reset();
  bondcoordinate[2].Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CPOVBond::~CPOVBond()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CPOVBond& CPOVBond::operator=(CPOVBond &POVBond)
{
  // equalize members
  label = POVBond.label;
  color = POVBond.color;
  radius = POVBond.radius;
  texture = POVBond.texture;
  bondcoordinate[1] = POVBond.bondcoordinate[1];
  bondcoordinate[2] = POVBond.bondcoordinate[2];

  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CPOVBond::Serialize()
{
  // not yet implemented
}


/////////////////////////////////////////////////////////////////////////////
// GetX
float CPOVBond::GetX(int atom)
{
  if(atom != 1 && atom != 2) return 0.0;
  return bondcoordinate[atom].GetX();
}

/////////////////////////////////////////////////////////////////////////////
// GetY
float CPOVBond::GetY(int atom)
{
  if(atom != 1 && atom != 2) return 0.0;
  return bondcoordinate[atom].GetY();
}

/////////////////////////////////////////////////////////////////////////////
// GetZ
float CPOVBond::GetZ(int atom)
{
  if(atom != 1 && atom != 2) return 0.0;
  return bondcoordinate[atom].GetZ();
}

/////////////////////////////////////////////////////////////////////////////
// GetBond
void CPOVBond::GetBond(CPOVCoordinate& Coordinate1, CPOVCoordinate& Coordinate2)
{
  Coordinate1 = bondcoordinate[1];
  Coordinate2 = bondcoordinate[2];
}

/////////////////////////////////////////////////////////////////////////////
// SetX
int CPOVBond::SetX(int atom, float newx)
{
  if(atom != 1 && atom != 2) return FALSE;
  bondcoordinate[atom].SetX(newx);
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// SetY
int CPOVBond::SetY(int atom, float newy)
{
  if(atom != 1 && atom != 2) return FALSE;
  bondcoordinate[atom].SetY(newy);
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// SetZ
int CPOVBond::SetZ(int atom, float newz)
{
  if(atom != 1 && atom != 2) return FALSE;
  bondcoordinate[atom].SetZ(newz);
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// SetBond
void CPOVBond::SetBond(CPOVCoordinate& Coordinate1, CPOVCoordinate& Coordinate2)
{
  bondcoordinate[1] = Coordinate1;
  bondcoordinate[2] = Coordinate2;
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CPOVFile
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CPOVFile::CPOVFile(QObject *parent, const char *name)
  : QObject(parent, name)
{
  // initialisations
  POVLight1 = new CPOVCoordinate(this, "povLight1POV");
  POVLight2 = new CPOVCoordinate(this, "povLight2POV");
  POVLight3 = new CPOVCoordinate(this, "povLight3POV");
  POVCameraRight = new CPOVCoordinate(this, "povCameraRightPOV");
  POVCameraUp = new  CPOVCoordinate(this, "povCameraUpPOV");
  POVCameraDirection = new CPOVCoordinate(this, "povCameraDirectionPOV");
  POVCameraLocation = new  CPOVCoordinate(this, "povCameraLocationPOV");
  POVCameraLookAt = new  CPOVCoordinate(this, "povCameraLookAtPOV");
  // enabling auto-delete will delete objects that are removed
  POVAtomList.setAutoDelete(TRUE);
  POVBondList.setAutoDelete(TRUE);
  // reset members;
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CPOVFile::~CPOVFile()
{
  // auto-delete is enabled!
  POVAtomList.clear();
  POVBondList.clear();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CPOVFile& CPOVFile::operator=(CPOVFile &POVFile)
{
  // clear
  Reset();
  // copy members
  POVHeader = POVFile.POVHeader;
  POVFoot = POVFile.POVFoot;
  POVTitle = POVFile.POVTitle;
  POVShadowless = POVFile.POVShadowless;
  POVLight1Enabled = POVFile.POVLight1Enabled;
  POVLight2Enabled = POVFile.POVLight2Enabled;
  POVLight3Enabled = POVFile.POVLight3Enabled;
  POVIncludeFile = POVFile.POVIncludeFile;
  *POVCameraRight = *(POVFile.POVCameraRight);
  *POVCameraUp = *(POVFile.POVCameraUp);
  *POVCameraDirection = *(POVFile.POVCameraDirection);
  *POVCameraLocation = *(POVFile.POVCameraLocation);
  *POVCameraLookAt = *(POVFile.POVCameraLookAt);
  *POVLight1 = *(POVFile.POVLight1);
  *POVLight2 = *(POVFile.POVLight2);
  *POVLight3 = *(POVFile.POVLight3);

  // POVAtomcoordinate-List
  POVAtomList.clear();
  CPOVAtom atomBuffer;
  if(POVFile.POVAtomList.first())
  {
    for(uint i = 0; i <= POVFile.POVAtomList.count() - 1; i++)
    {
      atomBuffer = *(CPOVAtom*)POVFile.POVAtomList.current();
      atom = new CPOVAtom;
      if(!atom)
      {
        Reset();
        return *this;
      }
      *atom = atomBuffer;
      POVAtomList.append(atom);
      POVFile.POVAtomList.next();
    }
  }
  // POVAtombond-List
  POVBondList.clear();
  CPOVBond bondBuffer;
  if(POVFile.POVBondList.first())
  {
    for(uint i = 0; i <= POVFile.POVBondList.count() - 1; i++)
    {
      bondBuffer = *(CPOVBond*)POVFile.POVBondList.current();
      bond = new CPOVBond;
      if(!bond)
      {
        Reset();
        return *this;
      }
      *bond = bondBuffer;
      POVBondList.append(bond);
      POVFile.POVBondList.next();
    }
  }
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CPOVFile::Reset(void)
{
  // reset lists  
  POVAtomList.clear();
  POVBondList.clear();
  // reset members;
  NumberOfAtoms = 0;
  NumberOfBonds = 0;
  POVHeader = "";
  POVFoot = "";
  POVTitle = "";
  POVShadowless = 0;
  POVLight1Enabled = 0;
  POVLight2Enabled = 0;
  POVLight3Enabled = 0;
  POVIncludeFile = "";
  POVCameraRight->Reset();
  POVCameraUp->Reset();
  POVCameraDirection->Reset();
  POVCameraLocation->Reset();
  POVCameraLookAt->Reset();
  POVLight1->Reset();
  POVLight1->Reset();
  POVLight1->Reset();
}

/////////////////////////////////////////////////////////////////////////////
// SaveFile
int CPOVFile::SaveFile(QString Path)
{
  // Initialisations
  CPOVAtom* atomBuffer;
  CPOVBond* bondBuffer;
  QString LineBuffer = "";
  int i = 0;
  QFile POVFile;
  QString Line = "";
  // try to open POV-file in write-only mode
  POVFile.setName(Path);
  if (POVFile.open(IO_WriteOnly))
  {
    // open file in write-only mode succesfull
    // write header
    if(POVHeader != "")
    {
      Line = "// " + POVHeader + "\r\n\r\n";
      POVFile.writeBlock(Line, Line.length());
    }
    // write Title:
    if(POVTitle != "")
    {
      Line = "// Title: " + POVTitle + "\r\n\r\n";
      POVFile.writeBlock(Line, Line.length());
    }
    // write default include-files
    Line = "#include \"colors.inc\"\r\n";
    Line += "#include \"textures.inc\"\r\n";
    Line += "#include \"shapes.inc\"\r\n";
    // write user defined include-file
    if(POVIncludeFile != "")
    {
      Line +=  "#include \"" + POVIncludeFile + "\"\r\n";
    }
    Line += "\r\n";
    POVFile.writeBlock(Line, Line.length());
    // write camera position
    Line = "// CAMERA\r\ncamera {\r\n";
    // camera right
    LineBuffer.setNum(POVCameraRight->GetX(), 'f');
    Line += " right     < " + LineBuffer + ", ";
    LineBuffer.setNum(POVCameraRight->GetY(), 'f');
    Line += LineBuffer + ", ";
    LineBuffer.setNum(POVCameraRight->GetZ(), 'f');
    Line += LineBuffer + " >\r\n";
    // camera up
    LineBuffer.setNum(POVCameraUp->GetX(), 'f');
    Line += " up        < " + LineBuffer + ", ";
    LineBuffer.setNum(POVCameraUp->GetY(), 'f');
    Line += LineBuffer + ", ";
    LineBuffer.setNum(POVCameraUp->GetZ(), 'f');
    Line += LineBuffer + " >\r\n";
    // camera direction
    LineBuffer.setNum(POVCameraDirection->GetX(), 'f');
    Line += " direction < " + LineBuffer + ", ";
    LineBuffer.setNum(POVCameraDirection->GetY(), 'f');
    Line += LineBuffer + ", ";
    LineBuffer.setNum(POVCameraDirection->GetZ(), 'f');
    Line += LineBuffer + " >\r\n";
    // camera location
    LineBuffer.setNum(POVCameraLocation->GetX(), 'f');
    Line += " location  < " + LineBuffer + ", ";
    LineBuffer.setNum(POVCameraLocation->GetY(), 'f');
    Line += LineBuffer + ", ";
    LineBuffer.setNum(POVCameraLocation->GetZ(), 'f');
    Line += LineBuffer + " >\r\n";
    // camera look at
    LineBuffer.setNum(POVCameraLookAt->GetX(), 'f');
    Line += " look_at   < " + LineBuffer + ", ";
    LineBuffer.setNum(POVCameraLookAt->GetY(), 'f');
    Line += LineBuffer + ", ";
    LineBuffer.setNum(POVCameraLookAt->GetZ(), 'f');
    Line += LineBuffer + " >\r\n}\r\n\r\n";
    POVFile.writeBlock(Line, Line.length());
    // write light positions
    if(POVLight1Enabled)
    {
      // switch on light 1
      Line = "// LIGHT 1\r\nlight_source {  < ";
      LineBuffer.setNum(POVLight1->GetX(), 'f');
      Line += LineBuffer + ", ";
      LineBuffer.setNum(POVLight1->GetY(), 'f');
      Line += LineBuffer + ", ";
      LineBuffer.setNum(POVLight1->GetZ(), 'f');
      Line += LineBuffer + " >\r\n color White\r\n";
      if(POVShadowless) Line += " shadowless\r\n";
      Line += "}\r\n\r\n";
      POVFile.writeBlock(Line, Line.length());
    }
    if(POVLight2Enabled)
    {
      // switch on light 2
      Line = "// LIGHT 2\r\nlight_source {  < ";
       LineBuffer.setNum(POVLight2->GetX(), 'f');
      Line += LineBuffer + ", ";
      LineBuffer.setNum(POVLight2->GetY(), 'f');
      Line += LineBuffer + ", ";
      LineBuffer.setNum(POVLight2->GetZ(), 'f');
      Line += LineBuffer + " >\r\n color White\r\n";
      if(POVShadowless) Line += " shadowless\r\n";
      Line += "}\r\n\r\n";
      POVFile.writeBlock(Line, Line.length());
    }
    if(POVLight3Enabled)
    {
      // switch on light 3
      Line = "// LIGHT 3\r\nlight_source {  < ";
       LineBuffer.setNum(POVLight3->GetX(), 'f');
      Line += LineBuffer + ", ";
      LineBuffer.setNum(POVLight3->GetY(), 'f');
      Line += LineBuffer + ", ";
      LineBuffer.setNum(POVLight3->GetZ(), 'f');
      Line += LineBuffer + " >\r\n color White\r\n";
      if(POVShadowless) Line += " shadowless\r\n";
      Line += "}\r\n\r\n";
      POVFile.writeBlock(Line, Line.length());
    }
    // white background
    Line = "// BACKGROUND\r\nbackground {\r\n color rgb < 1.00, 1.00, 1.00 >\r\n}\r\n\r\n";
    POVFile.writeBlock(Line, Line.length());
    // write atoms
    if(NumberOfAtoms > 0)
    {
      Line = "// ATOMS\r\n";
      POVFile.writeBlock(Line, Line.length());
      POVAtomList.first();
      for(i = 1; i <= NumberOfAtoms; i++)
      {
        Line = "sphere {  < ";
        if(!(atomBuffer = (CPOVAtom*)POVAtomList.current())) return FALSE;
        POVAtomList.next();
        // atom atoms
        Line += LineBuffer.setNum(atomBuffer->atomcoordinate.x, 'f') + ", ";
        Line += LineBuffer.setNum(atomBuffer->atomcoordinate.y, 'f') + ", ";
        Line += LineBuffer.setNum(atomBuffer->atomcoordinate.z, 'f') + " > ";
        Line += LineBuffer.setNum(atomBuffer->radius, 'f') + "\r\n";
        // atom texture
        if(atomBuffer->texture != "")
          Line += " texture { finish { " + atomBuffer->texture + " } }\r\n";
        else
          Line += " texture { finish { Phong_Shiny } }\r\n";
        // atom color
        if(atomBuffer->color != "")
        {
          Line += " pigment { rgb < ";
          Line += LineBuffer.setNum((float(atomBuffer->color.red()))/ 255, 'f') + ", ";
          Line += LineBuffer.setNum((float(atomBuffer->color.green()))/ 255, 'f') + ", ";
          Line += LineBuffer.setNum((float(atomBuffer->color.blue()))/ 255, 'f');
          Line += " > }\r\n}\r\n\r\n";
        }
        else
          Line += " pigment { rgb < 0.5, 0.5, 0.5 > }\r\n}\r\n\r\n";
        POVFile.writeBlock(Line, Line.length());
      }
    }
    // write bonds
    if(NumberOfBonds > 0)
    {
      Line = "// BONDS\r\n";
      POVFile.writeBlock(Line, Line.length());
      POVBondList.first();
      for(i = 1; i <= NumberOfBonds; i++)
      {

        Line = "cylinder {  < ";
        if(!(bondBuffer = (CPOVBond*)POVBondList.current())) return FALSE;
        POVBondList.next();
        // bond coordinates
        Line += LineBuffer.setNum(bondBuffer->bondcoordinate[1].GetX(), 'f') + ", ";
        Line += LineBuffer.setNum(bondBuffer->bondcoordinate[1].GetY(), 'f') + ", ";
        Line += LineBuffer.setNum(bondBuffer->bondcoordinate[1].GetZ(), 'f') + " >, < ";
        Line += LineBuffer.setNum(bondBuffer->bondcoordinate[2].GetX(), 'f') + ", ";
        Line += LineBuffer.setNum(bondBuffer->bondcoordinate[2].GetY(), 'f') + ", ";
        Line += LineBuffer.setNum(bondBuffer->bondcoordinate[2].GetZ(), 'f') + " > ";
        Line += LineBuffer.setNum(bondBuffer->radius, 'f') + "\r\n";
        // bond texture
        if(bondBuffer->texture != "")
          Line += " texture { finish { " + bondBuffer->texture + " } }\r\n";
        else
          Line += " texture { finish { Phong_Shiny } }\r\n";
        // bond color
        if(bondBuffer->color != "")
        {
          Line += " pigment { rgb < ";
          Line += LineBuffer.setNum((float(bondBuffer->color.red()))/ 255, 'f') + ", ";
          Line += LineBuffer.setNum((float(bondBuffer->color.green()))/ 255, 'f') + ", ";
          Line += LineBuffer.setNum((float(bondBuffer->color.blue()))/ 255, 'f');
          Line += " > }\r\n}\r\n\r\n";
        }
        else
          Line += " pigment { rgb < 0.5, 0.5, 0.5 > }\r\n}\r\n\r\n";
        POVFile.writeBlock(Line, Line.length());
      }
    }
    // write foot
    if(POVFoot != "")
    {
      Line = "// " + POVFoot + "\r\n\r\n";
      POVFile.writeBlock(Line, Line.length());
    }
    // end of file
    POVFile.close();
    return(TRUE);
  }
  else
  {
    // open file in write-only mode fails
    return(FALSE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// SetAtom
int CPOVFile::SetAtom(CPOVAtom* NewAtom)
{
  if(!(atom = new CPOVAtom))
  {
    Reset();
    return (FALSE);
  }
  NumberOfAtoms++;
  *atom = *NewAtom;
  POVAtomList.append(atom);
  return(TRUE);
}
  
/////////////////////////////////////////////////////////////////////////////
// SetBond
int CPOVFile::SetBond(CPOVBond* NewBond)
{
  if(!(bond = new CPOVBond))
  {
    Reset();
    return (FALSE);
  }
  NumberOfBonds++;
  *bond = *NewBond;
  POVBondList.append(bond);
  return(TRUE);
}  
