/***************************************************************************
                          vrmlfile.h  -  description
                             -------------------
    begin                : Sun Jul 9 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _VRMLFILE
#define _VRMLFILE

/* include files for QT */
#include <qobject.h>
#include <qstring.h>
#include <qfile.h>
#include <qcolor.h>
#include <qlist.h>

/* application specific includes */
#include <iostream>
#include <cmath>

/* class CVRMLCoordinate */
class CVRMLCoordinate : public QObject
{
  Q_OBJECT
  friend class CVRMLFile;

public:
  /* Constructor */
  CVRMLCoordinate(QObject *parent=0, const char *name=0);

  /* Destructor */
  ~CVRMLCoordinate();

  /* Operators */
  CVRMLCoordinate& operator=(CVRMLCoordinate &GVRMLCoordinate);

  /* Methods */
  void Reset(void);
  void Serialize();

  /* Get... */
  float GetX(void) {return (x);}
  float GetY(void) {return (y);}
  float GetZ(void) {return (z);}

  /* Set... */
  void SetX(float newx) {x = newx; return;}
  void SetY(float newy) {y = newy; return;}
  void SetZ(float newz) {z = newz; return;}

protected:
  /* Members */
  float x;
  float y;
  float z;
};

/* class CVRMLAtom */
class CVRMLAtom : public QObject
{
  Q_OBJECT
  friend class CVRMLFile;

public:
  /* Constructor */
  CVRMLAtom(QObject *parent=0, const char *name=0);

  /* Destructor */
  ~CVRMLAtom();

  /* Operators */
  CVRMLAtom& operator=(CVRMLAtom &CVRMLAtom);

  /* Methods */
  void Serialize();

  /* Get... */
  int GetLabel(void) {return (label);}
  QColor GetColor(void) {return (color);}
  float GetRadius(void) {return (radius);}
  float GetX(void) {return (atomcoordinate.GetX());}
  float GetY(void) {return (atomcoordinate.GetY());};
  float GetZ(void) {return (atomcoordinate.GetZ());}

  /* Set... */
  void SetLabel(int newlabel) {label = newlabel; return;}
  void SetColor(QColor newcolor) {color = newcolor; return;}
  void SetRadius(float newradius) {radius = newradius; return;}
  void SetX(float newx) {atomcoordinate.SetX(newx);}
  void SetY(float newy) {atomcoordinate.SetY(newy);}
  void SetZ(float newz) {atomcoordinate.SetZ(newz);}


protected:
  /* Members */
  int label;
  QColor color;
  float radius;
  CVRMLCoordinate atomcoordinate;
};

/* class CVRMLBond */
class CVRMLBond : public QObject
{
  Q_OBJECT
  friend class CVRMLFile;

public:
  /* Constructor */
  CVRMLBond(QObject *parent=0, const char *name=0);

  /* Destructor */
  ~CVRMLBond();

  /* Operators */
  CVRMLBond& operator=(CVRMLBond &VRMLBond);

  /* Methods */
  void Serialize();

  /* Get... */
  int GetLabel(void) {return (label);}
  QColor GetColor(void) {return (color);}
  float GetRadius(void) {return (radius);}
  void GetBond(CVRMLCoordinate& Coordinate1, CVRMLCoordinate& Coordinate2);
  float GetX(int atom);
  float GetY(int atom);
  float GetZ(int atom);

  /* Set... */
  void SetLabel(int newlabel) {label = newlabel; return;}
  void SetColor(QColor newcolor) {color = newcolor; return;}
  void SetRadius(float newradius) {radius = newradius; return;}
  void SetBond(CVRMLCoordinate& Coordinate1, CVRMLCoordinate& Coordinate2);
  int SetX(int atom, float newx);
  int SetY(int atom, float newy);
  int SetZ(int atom, float newz);

protected:
  /* Members */
  int label;
  QColor color;
  float radius;
  CVRMLCoordinate bondcoordinate[3];
};

/* class CVRMLFile */
class CVRMLFile : public QObject
{
  Q_OBJECT

public:
  /* constructor */
  CVRMLFile(QObject *parent=0, const char *name=0);

  /* destructor */
  ~CVRMLFile();

  /* Operators */
  CVRMLFile& operator=(CVRMLFile &VRMLFile);

  /* methods */
  void Reset(void);
  int SaveFile(QString Path);

  /* Get... */
  int GetNumberOfAtoms(void) {return VRMLAtomList.count();}
  int GetNumberOfBonds(void) {return VRMLBondList.count();}
  QString GetHeader(void) {return VRMLHeader;};
  QString GetFoot(void) {return VRMLFoot;};
  QString GetTitle(void) {return VRMLTitle;};
  int GetEnableTextureMapping(void) {return VRMLEnableTextureMapping;};
  QString GetImageTextureURL(void) {return VRMLImageTextureURL;};
  int GetEnableDiffuseColor(void) {return VRMLEnableDiffuseColor;};
  int GetEnableEmissiveColor(void) {return VRMLEnableEmissiveColor;};
  int GetEnableSpecularColor(void) {return VRMLEnableSpecularColor;};
  float GetDiffuseColorFactor(void) {return VRMLDiffuseColorFactor;};
  float GetEmissiveColorFactor(void) {return VRMLEmissiveColorFactor;};
  float GetSpecularColorFactor(void) {return VRMLSpecularColorFactor;};
  float GetShininess(void) {return VRMLShininess;};

  /* Set ... */
  int SetAtom(CVRMLAtom* NewAtom);
  int SetBond(CVRMLBond* NewBond);
  void SetHeader(QString newVRMLHeader) {VRMLHeader = newVRMLHeader; return;};
  void SetFoot(QString newVRMLFoot) {VRMLFoot = newVRMLFoot; return;};
  void SetTitle(QString newVRMLTitle) {VRMLTitle = newVRMLTitle; return;};
  void SetEnableTextureMapping(int newVRMLEnableTextureMapping) {VRMLEnableTextureMapping = newVRMLEnableTextureMapping; return;};
  void SetImageTextureURL(QString newVRMLImageTextureURL) {VRMLImageTextureURL = newVRMLImageTextureURL; return;};
  void SetEnableDiffuseColor(int newVRMLEnableDiffuseColor) {VRMLEnableDiffuseColor = newVRMLEnableDiffuseColor; return;};
  void SetEnableEmissiveColor(int newVRMLEnableEmissiveColor) {VRMLEnableEmissiveColor = newVRMLEnableEmissiveColor; return;};
  void SetEnableSpecularColor(int newVRMLEnableSpecularColor) {VRMLEnableSpecularColor = newVRMLEnableSpecularColor; return;};
  void SetDiffuseColorFactor(float newVRMLDiffuseColorFactor) {VRMLDiffuseColorFactor = newVRMLDiffuseColorFactor; return;};
  void SetEmissiveColorFactor(float newVRMLEmissiveColorFactor) {VRMLEmissiveColorFactor = newVRMLEmissiveColorFactor; return;};
  void SetSpecularColorFactor(float newVRMLSpecularColorFactor) {VRMLSpecularColorFactor = newVRMLSpecularColorFactor; return;};
  void SetShininess(float newVRMLShininess) {VRMLShininess = newVRMLShininess; return;};

protected:
  /* protected members */
  QList <CVRMLAtom> VRMLAtomList;
  QList <CVRMLBond> VRMLBondList;
  int NumberOfAtoms;
  int NumberOfBonds;
  CVRMLAtom *atom;
  CVRMLBond *bond;
  QString VRMLHeader;
  QString VRMLFoot;
  QString VRMLTitle;
  int VRMLEnableTextureMapping;
  QString VRMLImageTextureURL;
  int VRMLEnableDiffuseColor;
  int VRMLEnableEmissiveColor;
  int VRMLEnableSpecularColor;
  float VRMLDiffuseColorFactor;
  float VRMLEmissiveColorFactor;
  float VRMLSpecularColorFactor;
  float VRMLShininess;
};

#endif /* _VRMLFILE */
